#!/usr/bin/python
# -*- coding: utf-8 -*-

#############################################################################
#  Copyright & Licence:
#  ====================
#   
#  Copyright 2009-2017 gAGE/UPC & ESA 
#   
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#   
#  http://www.apache.org/licenses/LICENSE-2.0
#   
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#############################################################################

#############################################################################
# Copyright:  gAGE/UPC & ESA
# Project:    EDUNAV GNSS Lab Tool
# Authors:    Adria Rovira-Garcia (group of Astronomy and GEomatics - gAGE/UPC)
#             Pere Ramos-Bosch (group of Astronomy and GEomatics - gAGE/UPC)
# Curator:    Adria Rovira-Garcia ( gAGE/UPC )
# Developers: Deimos Ibáñez Segura ( gAGE/UPC )
#             Jesus Romero-Sanchez ( gAGE/UPC )
#             glab.gage @ upc.edu         
# File: gLAB_GUI.py
# Code Management Tool File Version: 4.0  Revision: 0
# Date: 2017/03/03
#############################################################################

#############################################################################
# MODULE DESCRIPTION
#
# Name: gLAB_GUI
# Language: python
#
# Purpose:
#  The purpose of this module is to show a Graphical User Interface.
#
# Function:
#  This program creates a GUI to allow selecting several options in order to 
#  execute the DPC (gLAB) and DAT (graph.py) with the selected options. It also
#  has some error checking routines, and warns the user of any error he might
#  have done.
# 
# Dependencies:
#  gLAB
#  graph.py
#
# Files modified:
#  Specified output file
#  .plot.lock
#
# Files read:
#  None
#
# Resources usage:
#  See Design Document
#############################################################################

#############################################################################
#       RELEASE_HISTORY
# -------------
#          gLAB v1.3.1
# Release: 2010/01/22
# Change Log: First public version.
# -------------
#          gLAB v1.3.2
# Release: 2010/02/23
# Change Log: General changes.
# -------------
#          gLAB v1.4.0
# Release: 2010/06/21
# Change Log: Redesign of Input tab: addition of DCB source and Receiver Type,
#                  and SINEX source files.
#             Processing Templates moved to a general position between the other buttons.
#             Show config button added.
#             New preprocess option: Flexible C1-P1 handling for better treatment of
#                  C1 and P1 measurements.
#             Filter's combo boxes of measurement configuration made available.
# -------------
#          gLAB v1.4.1
# Release: 2010/07/09
# Change Log: Further Redesign of Input tab.
# -------------
#          gLAB v1.4.2
# Release: 2010/07/31
# Change Log: Optional scroll-bars added to permit gLAB work in small screens.	  
#             .eph files are searched in addition to .sp3 in the input tab.            
# -------------
#          gLAB v1.4.3
# Release: 2010/08/31
# Change Log: Satellite elevation and azimuth addition in the Model message, which also
#                  are included in the analysis part to be plotted.
#             Improvement of the DCB / TGD Management.
#  -------------                  
#          gLAB v1.4.4
# Release: 2010/09/22
# Change Log: Further improvement of the (P1-C1) & (P1-P2) DCB / TGD configurations, display and tooltips.
#             Some general positioning and sizing adjustments.
#             Sanity Check added to the Save Configuration to File function.
#  -------------
#          gLAB v1.4.5
# Release: 2010/10/31
# Change Log: Dilution Of Precision extension: (GDOP, PDOP, TDOP, VDOP, HDOP).
#             Tropospheric correction option is extended to select independently the nominal and mapping used.
#	      Third row of Analysis tab templates implemented: Prefit residuals, Measurements Multipath/Noise,
#                  Dilution of Precision and Orbit and Clocks comparison.            
#             .dat files are searched in addition to .out in the analysis section.
#             SIGMA_INF split into different SIGMA_INF_DR and SIGMA_INF_DT: receiver coordinates and clock.  
#             GUI now is able to be executed from different folder, not only from where it is placed.
# -------------
#          gLAB v2.0.0
# Release: 2012/12/31
# Change Log: gLAB version released together with the Educational Book:
#             "GNSS Data Processing" (Vol. 1 and Vol. 2). ESA TM-23.
#             Authors: J. Sanz Subirana, J.M. Juan Zornoza and M. Hernandez-Pajares
#             ISBN: 978-92-9221-885-0 (two volumes)
#             ISSN: 1013-7076
#             December 2012.
# -------------
#          gLAB v2.2.0
# Release: 2014/09/22
# Change Log: Added buttons and options for BeiDou, IONEX, Fast-PPP and NeQuick ionosphere models.
#	      Added buttons and options for ESA's troposphere model.
#	      Added buttons and options for IONEX and Fast-PPP DCBs.
#	      Added buttons and options for using standard deviation data from IONEX or FPPP. 
#	      Fixed window clipping bug when scrollbars where activated (the bug was only on Windows).
#	      Workaround for python bug, which made that when a file was selected with double click,
#		the button which was on the window undearneath and on the same position where it 
#               had been clicked was activated (the bug was only on Windows).
# -------------      
#          gLAB v2.2.1
# Release: 2014/12/23
# Change Log: Fixed "file not found" error window infinite loop when plotting a file with a name with spaces or
#               when showing the output file and this file has been erased. 
#             Added tooltip for the "Use Sigma Iono" button.
# -----------
#          gLAB v2.2.2
# Release: 2015/03/02
# Change Log: Uncommented option to force selection of one frequency (so when we process with one frequency
#               only one frequency cycle-slip detector can be used, instead of using both for cycle-slip detection)
# -----------
#          gLAB v2.2.3
# Release: 2015/05/28
# Change Log: No changes in this file.
# -----------
#          gLAB v2.2.4
# Release: 2015/07/01
# Change Log: No changes in this file.
# -----------
#          gLAB v2.2.5
# Release: 2015/10/20
# Change Log: Fixed wx.Color error that crashed the GUI at start time in Linux.
#	      Fixed empty plots in Windows. It ocurred when the plot number selected was not the first one, had a condition 
#		and the previous plots were not used.
#	      After pressing the button "Clear", the selected plot was changed to number one, but the new plot parameters  
#		entered were saved in the plot number that was selected before pressing the "Clear" button. Now they are 
#		saved in the correct position.
#	      When a plot failed, the "Plot" button was not active again after 60 seconds due to a bug in graph.py. 
#		Now the button is active again when graph.py ends.
# -----------
#          gLAB v2.2.6
# Release: 2015/11/25 
# Change Log: Made the GUI size (in height) slighty higher in order to perfectly fit all objects under Windows.
#             Now the ESA/gAGE logo in the top part is resizable in to avoid issues in Windows.
# -----------
#          gLAB v2.2.7
# Release: 2016/03/01
# Change Log: Fixed SATDIFF condition not being set when applying "Orbit and Clock comparison" template.
#             Changed "Source File" value to "dif.out" in "Orbit and Clock comparison" template.
# -----------
#          gLAB v2.2.8
# Release: 2016/04/12
# Change Log: Fixed bug that prevented the user from selecting Klobuchar, BeiDou or NeQuick ionosphere models
#              when IONEX or FPPP models had been selected in the Modelling panel.
# -----------
#          gLAB v3.0.0
# Release: 2016/09/05
# Change Log: Redesign of Preprocess panel for handling new features of new Cycle Slip detectors.
#             Fixed parameter given to gLAB core that did not allow to use dual frequency cycle-slip detector
#               when "Single-Frequency" processing was selected in the "Filter" panel.
#             Fixed error when running gLAB processing or plots when paths or filenames contained special characters (e.g. accents).
#             Fixed "Run gLAB" button keeping disabled when there was an error when executing gLAB.
# -----------
#          gLAB v3.1.0
# Release: 2016/09/26
# Change Log: No changes in this file.
# -----------
#          gLAB v3.1.1
# Release: 2016/10/21
# Change Log: Changed SPP template now has all cycle-slip deactivated.
#             Changed LLI cycle-slip detector is activated by default in PPP template.
#             Added LLI cycle-slip detector is activated/deactivated when activating/deactivating carrier phase or smoothing in the filter.
#             Updated Melbourne-Wübbena cycle-slip detector help message and thresholds.
# -----------
#          gLAB v4.0.0
# Release: 2017/03/03
# Change Log: Added check for X and Y plot ranges in Analysis tab.
#             Fixed error when checking for a new version. When the current version was greater than the one in the website,
#              it showed the current one as the old one instead of the new one.
#             File '.plot.lock' is erased when the GUI starts to avoid synchronization problems between the graph.py and the GUI.
#             NOTE: This will be the last update for this GUI. A new GUI based in Qt will replace this one.             
# -----------
#       END_RELEASE_HISTORY
#############################################################################

import sys, os
if not hasattr(sys, 'frozen'):
    import wxversion
    wxversion.select(['2.8'])
import wx, wx.html
import time
import signal
import threading
import wx.lib.stattext
import wx.lib.buttons 
import string
import subprocess
import codecs
import urllib
import datetime
codecs.BOM_UTF8.decode("utf_8")


# Global Variables
global tooltips
tooltips = True
tooltipDelay = 0.8
tooltipXsize = 700
tooltipYsize = 50
global scrollbars
scrollbars = False
global RinexPosition
global WIN
global PYTHON
global DireName
global DireMain
global LastDirExecuted
global LastOutputExecuted
global LastConfigFile
global LastDirConfigFile
global Version
Version = "4.0"
Revision = "0"
global Xsize, Ysize
Xsize = 0 
Ysize = 0
global prevPlotFirstCondition
prevPlotFirstCondition = ''
global OutputPath
OutputPath = '' 
global FirstShow
FirstShow = True 

# Binaries Detection Variables: A possible paths list:
global gBinPaths
gBinPaths = [os.getcwd(),'PPWWDD','/usr/share/gLAB','/usr/bin/',sys.path[0]]

# Binaries Detection Variables: Auxiliary needed files list:
global gBinElements
gBinElements = ['Header.gif','MainLogo.gif','gAGE.png','gAGE.ico','graph.py','gLAB_linux']


##########################################################################
# Name        : isFloat
# Description : Check if a string contains a float inside a range
##########################################################################
def isFloat (str, min= -9e999, max=9e999):
    try:
        val = float(str)        
        if ((val >= min) & (val <= max)):
            return True
        else:
            return False
    except:
        if (str == "inf"):
            return True
        else:
            return False
    
##########################################################################
# Name        : isInt
# Description : Check if a string contains an integer inside a range
##########################################################################
def isInt (str, min= -9e999, max=9e999):
    try:
        val = int(str)
        if (str.find(".") != -1):
            return False
        if ((val >= min) & (val <= max)):
            return True
        else:
            return False
    except:
        return False

##########################################################################
# Name        : CalculusPage
# Description : This is the tab where the user can configure what the  
#               processing core will do. It has 5 different buttons
#               which conceptually divide the configuration
#
#               Inputs: Loads all necessary files, the ionosphere
#                 source and the receiver location
#               Preprocess: Selection of the PRN, cycle-slip detection, data
#                decimation and the elevation mask
#               Modelling: User can configure different corrections
#                 of the model
#               Filter: Selection of the different parameters for the
#                filter
#               Output: Selection of the displayed messages and the output 
#                file to be stored
##########################################################################
class CalculusPage(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)

        #Font settings for the boxed panels
        # font type: wx.DEFAULT, wx.DECORATIVE, wx.ROMAN, wx.SCRIPT, wx.SWISS, wx.MODERN
        # slant: wx.NORMAL, wx.SLANT or wx.ITALIC
        # weight: wx.NORMAL, wx.LIGHT or wx.BOLD
        LinedFont = wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL)
        LinedFontColour = (wx.BLUE)        


        #Create Page Two elements  
        global TabXSize, TabYSize, OutputPath
        XCalculusSize, YCalculusSize = TabXSize, TabYSize
        self.CalculusPagePanel = wx.Panel(self, -1, size=(XCalculusSize, YCalculusSize), style=wx.ALIGN_CENTER | wx.NO_BORDER)
#        self.CalculusPagePanel.SetBackgroundColour((150, 150, 145))    
    
        self.CalculusPageButtonsPanel = wx.Panel(self.CalculusPagePanel, -1)
        self.CalculusPageButtonsPanel.SetBackgroundColour((100, 100, 145))
        
        self.InputButton = wx.ToggleButton(self.CalculusPageButtonsPanel, -1, label="Input", style=wx.BU_EXACTFIT)
        self.PreprocesButton = wx.ToggleButton(self.CalculusPageButtonsPanel, -1, label="Preprocess", style=wx.BU_EXACTFIT)
        self.ModelButton = wx.ToggleButton(self.CalculusPageButtonsPanel, -1, label="Modelling", style=wx.BU_EXACTFIT)
        self.FilterButton = wx.ToggleButton(self.CalculusPageButtonsPanel, -1, label="Filter", style=wx.BU_EXACTFIT)
        self.OutputButton = wx.ToggleButton(self.CalculusPageButtonsPanel, -1, label="Output", style=wx.BU_EXACTFIT)

        
        CalculusPageButonSize = wx.Size(0.1 * XCalculusSize, 0.06 * YCalculusSize)
        self.InputButton.SetMinSize(CalculusPageButonSize), self.PreprocesButton.SetMinSize(CalculusPageButonSize)
        self.ModelButton.SetMinSize(CalculusPageButonSize), self.FilterButton.SetMinSize(CalculusPageButonSize)
        self.OutputButton.SetMinSize(CalculusPageButonSize)
                
        PageCalculusButtonBox = wx.BoxSizer(wx.HORIZONTAL)
        PageCalculusButtonBox.Add(self.InputButton, wx.ALIGN_CENTER_HORIZONTAL, wx.ALL, 10)
        PageCalculusButtonBox.Add(self.PreprocesButton, wx.ALIGN_CENTER_HORIZONTAL, wx.TOP | wx.BOTTOM | wx.RIGHT, 10)
        PageCalculusButtonBox.Add(self.ModelButton, wx.ALIGN_CENTER_HORIZONTAL, wx.TOP | wx.BOTTOM | wx.RIGHT, 10)
        PageCalculusButtonBox.Add(self.FilterButton, wx.ALIGN_CENTER_HORIZONTAL, wx.TOP | wx.BOTTOM, 10)
        PageCalculusButtonBox.Add(self.OutputButton, wx.ALIGN_CENTER_HORIZONTAL, wx.ALL, 10)
        PageCalculusButtonBox.SetMinSize(wx.Size(XCalculusSize, 0.10 * YCalculusSize))
        self.CalculusPageButtonsPanel.SetSizer(PageCalculusButtonBox)
        
        #SecondaryFrame
        self.SecondaryFrame = wx.Panel(self.CalculusPagePanel, -1)
        self.SecondaryFrame.SetSize(wx.Size(XCalculusSize, 0.82 * YCalculusSize))
        SecondaryFrameXSize, SecondaryFrameYSize = XCalculusSize, YCalculusSize
#       self.BackgroundLogo = wx.StaticBitmap( self.SecondaryFrame, -1, wx.Bitmap( gBinElements[1] ) ) #Original line
        start_image = wx.Image(gBinElements[1])
        start_image.Rescale(XCalculusSize, 0.82 * YCalculusSize)
        midd_image = wx.BitmapFromImage(start_image)
        self.BackgroundLogo = wx.StaticBitmap(self.SecondaryFrame, -1, midd_image)
	
	BackgroundLogoBox = wx.BoxSizer(wx.VERTICAL)
        BackgroundLogoBox.Add(self.BackgroundLogo)
        self.SecondaryFrame.SetSizer(BackgroundLogoBox)
	"""
        SecondaryFrameGrid = wx.GridBagSizer()
        SecondaryFrameGrid.Add(self.BackgroundLogo, (0, 0), (1, 1), wx.TOP ,5)
        self.SecondaryFrame.SetSizer(SecondaryFrameGrid)
        SecondaryFrameGrid.SetMinSize(wx.Size(SecondaryFrameXSize, SecondaryFrameYSize))
	"""

        # Save Config and Show Config Buttons
 	XTemplatesAndButtonsSize, TemplatesAndButtonsSize =  0.12 * XCalculusSize,  0.055 * YCalculusSize 

        self.SaveLoadButtonsPanel = wx.Panel(self.CalculusPagePanel, -1)
        self.SaveLoadButtonsPanel.SetSize(wx.Size(0.3 * XCalculusSize, 0.06 * YCalculusSize))
   
	self.SaveCfgButton = wx.Button(self.SaveLoadButtonsPanel, -1, label="&Save Config")
        self.SaveCfgButton.SetMinSize( wx.Size(XTemplatesAndButtonsSize, TemplatesAndButtonsSize ) )
	self.ShowConfigButton = wx.Button(self.SaveLoadButtonsPanel, -1, label="Show &Config")
        self.ShowConfigButton.SetMinSize( wx.Size(XTemplatesAndButtonsSize, TemplatesAndButtonsSize ) )

        self.SaveLoadButtonsBox = wx.BoxSizer (wx.HORIZONTAL)        
        self.SaveLoadButtonsBox.Add(self.SaveCfgButton)
        self.SaveLoadButtonsBox.Add(self.ShowConfigButton)
        self.SaveLoadButtonsPanel.SetSizer(self.SaveLoadButtonsBox)

	# Preconfigured Modes (templates): 
        self.PreconfigureModesPanel = wx.Panel(self.CalculusPagePanel, -1)        
	self.PreconfigureModesPanel.SetSize(wx.Size(0.3*XCalculusSize,0.06*YCalculusSize))
	XPreconfigurePanelSize, YPreconfigurePanelSize = self.PreconfigureModesPanel.GetSize()

        self.PreconfigureSPPB = wx.Button(self.PreconfigureModesPanel, -1, label='SPP Template')
        self.PreconfigureSPPB.SetMinSize( wx.Size(XTemplatesAndButtonsSize, TemplatesAndButtonsSize ) )
	self.PreconfigureSPPB.SetBackgroundColour(wx.Color(250, 250, 250))

        self.PreconfigurePPPB = wx.Button(self.PreconfigureModesPanel, -1, label='PPP Template') 
        self.PreconfigurePPPB.SetMinSize( wx.Size(XTemplatesAndButtonsSize, TemplatesAndButtonsSize ) )
	self.PreconfigurePPPB.SetBackgroundColour(wx.Color(250, 250, 250))

        PreconfigureButtonsBox = wx.GridBagSizer()
        PreconfigureButtonsBox.Add(self.PreconfigureSPPB, (0, 0),(1, 1), wx.LEFT , 130 )
        PreconfigureButtonsBox.Add(self.PreconfigurePPPB, (0, 1),(1, 1), wx.RIGHT, 130 )
        PreconfigureButtonsBox.SetMinSize(wx.Size( 0.95 * XPreconfigurePanelSize, 0.95 *YPreconfigurePanelSize))
        self.PreconfigureModesPanel.SetSizer(PreconfigureButtonsBox)
	        
        self.PreconfigureSPPB.Bind(wx.EVT_LEFT_DOWN, self.PreconfigureSPP, id=self.PreconfigureSPPB.GetId())
        self.PreconfigurePPPB.Bind(wx.EVT_LEFT_DOWN, self.PreconfigurePPP, id=self.PreconfigurePPPB.GetId())
 
        # Run and Show Output Buttons
        self.RunShowButtonsPanel = wx.Panel(self.CalculusPagePanel, -1)
        self.RunShowButtonsPanel.SetSize(wx.Size(0.3 * XCalculusSize, 0.06 * YCalculusSize))

        self.RunButton = wx.Button(self.RunShowButtonsPanel, -1, label="&Run gLAB")
        self.RunButton.SetMinSize( wx.Size(XTemplatesAndButtonsSize, TemplatesAndButtonsSize ) )
        self.ShowOutputButton = wx.Button(self.RunShowButtonsPanel, -1, label="Show &Output")
        self.ShowOutputButton.SetMinSize( wx.Size(XTemplatesAndButtonsSize, TemplatesAndButtonsSize ) )

        self.RunShowButtonsBox = wx.BoxSizer (wx.HORIZONTAL)
        self.RunShowButtonsBox.Add(self.RunButton)
        self.RunShowButtonsBox.Add(self.ShowOutputButton)
        self.RunShowButtonsPanel.SetSizer(self.RunShowButtonsBox)
	
	# Button Panels Layout
        self.PanelsBox = wx.BoxSizer (wx.HORIZONTAL) 
	self.PanelsBox.Add(self.SaveLoadButtonsPanel,0, wx.ALIGN_LEFT | wx.LEFT  | wx.TOP , 4)
	self.PanelsBox.Add(self.PreconfigureModesPanel,1.70, wx.ALIGN_CENTER| wx.TOP, 4)
	self.PanelsBox.Add(self.RunShowButtonsPanel,0, wx.ALIGN_RIGHT | wx.RIGHT| wx.TOP , 4)
	
        #Calculus Tab Layout 
        self.PageCalculusBox = wx.GridBagSizer()
        self.PageCalculusBox.Add(self.CalculusPageButtonsPanel, (0, 0), (1, 3))
        self.PageCalculusBox.Add(self.SecondaryFrame, (1, 0), (1, 3))
        self.PageCalculusBox.Add(self.PanelsBox, (2, 0), (1, 3), wx.EXPAND)
	"""
        self.PageCalculusBox.Add(self.SaveLoadButtonsPanel, (2,0), (1, 1), wx.ALIGN_LEFT | wx.TOP | wx.LEFT | wx.RIGHT,5 )
        self.PageCalculusBox.Add(self.PreconfigureModesPanel, (2, 1), (1, 1), wx.ALIGN_CENTER | wx.TOP  | wx.LEFT | wx.RIGHT, 5 )
        self.PageCalculusBox.Add(self.RunShowButtonsPanel, (2, 2), (1, 1), wx.ALIGN_RIGHT | wx.TOP | wx.LEFT | wx.RIGHT,5 )
	"""        
	self.PageCalculusBox.SetMinSize(wx.Size(0.95 * XCalculusSize, 0.99 * YCalculusSize))
        self.CalculusPagePanel.SetSizer(self.PageCalculusBox)

        self.SetSizer(self.PageCalculusBox)

        #Binding Buttons
        self.InputButton.Bind(wx.EVT_LEFT_DOWN, self.ShowInputs)
        self.PreprocesButton.Bind(wx.EVT_LEFT_DOWN, self.ShowPreProces)
        self.ModelButton.Bind(wx.EVT_LEFT_DOWN, self.ShowModes)
        self.FilterButton.Bind(wx.EVT_LEFT_DOWN, self.ShowFilters)
        self.OutputButton.Bind(wx.EVT_LEFT_DOWN, self.ShowOutput)
        self.SaveCfgButton.Bind(wx.EVT_LEFT_DOWN, self.SaveConfig)
        self.RunButton.Bind(wx.EVT_LEFT_DOWN, self.LaunchgLAB)
        self.ShowOutputButton.Bind(wx.EVT_LEFT_DOWN, self.ShowOutputFile)
        self.ShowConfigButton.Bind(wx.EVT_LEFT_DOWN, self.ShowConfigFile)
        

        
  #Panels linked on SecondaryFrame
     


        """
............................................................................................        
                INPUTS PANEL
............................................................................................                   
        """
    #Inputs Panel
        self.InputsPanel = wx.Panel(self.SecondaryFrame, -1, size=self.SecondaryFrame.GetSize())
        XInputsPanelSize, YInputsPanelSize = self.InputsPanel.GetSize()
      

        
      #Receiver Position:
        
        ReceiverPositionPanel = wx.Panel(self.InputsPanel, -1)
        ReceiverPositionPanel.SetSize(wx.Size(0.25 * XInputsPanelSize, 0.3 * YInputsPanelSize))
        XReceiverSize, YReceiverSize = ReceiverPositionPanel.GetSize()
	AprioryReceiverFirstSeparationSL = wx.StaticLine(parent=ReceiverPositionPanel, size=wx.Size(0.98 * XReceiverSize, 3))
        AprioryReceiverSecondSeparationSL = wx.StaticLine(parent=ReceiverPositionPanel, size=wx.Size(0.98 * XReceiverSize, 3))

        #Radio Box
        RadButtonReceiverPositionPanel = wx.Panel(ReceiverPositionPanel, -1)
        
        self.ReceiverPositionUseRinexRB = wx.RadioButton(RadButtonReceiverPositionPanel, -1, 'Use RINEX Position', style=wx.RB_GROUP)
        self.ReceiverPositionSpecifyRB = wx.RadioButton(RadButtonReceiverPositionPanel, -1, 'Specify')
        self.ReceiverPositionCalculateRB = wx.RadioButton(RadButtonReceiverPositionPanel, -1, 'Calculate')
        self.ReceiverPositionUseSinexRB = wx.RadioButton(RadButtonReceiverPositionPanel, -1, 'Use SINEX File')
        
        #RadButtonReceiverPositionBox = wx.BoxSizer(wx.HORIZONTAL)
        RadButtonReceiverPositionBox = wx.GridBagSizer()
        RadButtonReceiverPositionBox.Add(self.ReceiverPositionCalculateRB , (0, 0))
        RadButtonReceiverPositionBox.Add(self.ReceiverPositionUseRinexRB , (0, 1))
        RadButtonReceiverPositionBox.Add(self.ReceiverPositionSpecifyRB , (1, 0))
        RadButtonReceiverPositionBox.Add(self.ReceiverPositionUseSinexRB , (1, 1))
        RadButtonReceiverPositionPanel.SetSizer(RadButtonReceiverPositionBox)

        self.ReceiverPositionUseRinexRB.Bind(wx.EVT_RADIOBUTTON, self.SetReceiver)
        self.ReceiverPositionSpecifyRB.Bind(wx.EVT_RADIOBUTTON, self.SetReceiver)
        self.ReceiverPositionCalculateRB.Bind(wx.EVT_RADIOBUTTON, self.SetReceiver)
        self.ReceiverPositionUseSinexRB.Bind(wx.EVT_RADIOBUTTON, self.SetReceiver)

        #Specify Position
        self.SpecifyReceiverPositionPanel = wx.Panel(ReceiverPositionPanel, -1)
        XSpecifyReceiverPositionPanel, YSpecifyReceiverPositionPanel = self.SpecifyReceiverPositionPanel.GetSize()
        self.PosX = wx.TextCtrl(self.SpecifyReceiverPositionPanel, -1, '', size=wx.Size(0.8 * XReceiverSize, 0.05 * SecondaryFrameYSize))
        self.PosY = wx.TextCtrl(self.SpecifyReceiverPositionPanel, -1, '', size=wx.Size(0.8 * XReceiverSize, 0.05 * SecondaryFrameYSize))
        self.PosZ = wx.TextCtrl(self.SpecifyReceiverPositionPanel, -1, '', size=wx.Size(0.8 * XReceiverSize, 0.05 * SecondaryFrameYSize))
  
        self.PosX.Enable(False)
        self.PosY.Enable(False)
        self.PosZ.Enable(False)
 
        self.X = wx.lib.stattext.GenStaticText(self.SpecifyReceiverPositionPanel, -1, 'X [m]  : ')
        self.Y = wx.lib.stattext.GenStaticText(self.SpecifyReceiverPositionPanel, -1, 'Y [m]  : ')
        self.Z = wx.lib.stattext.GenStaticText(self.SpecifyReceiverPositionPanel, -1, 'Z [m]  : ')

        SpecifyReceiverPositionGrid = wx.GridBagSizer()
        SpecifyReceiverPositionGrid.Add(self.X, (0, 0), (1, 1), wx.ALIGN_CENTER | wx.TOP,10)
        SpecifyReceiverPositionGrid.Add(self.Y, (1, 0), (1, 1), wx.ALIGN_CENTER)
        SpecifyReceiverPositionGrid.Add(self.Z, (2, 0), (1, 1), wx.ALIGN_CENTER)
        SpecifyReceiverPositionGrid.Add(self.PosX, (0, 1), (1, 1), wx.TOP,10)
        SpecifyReceiverPositionGrid.Add(self.PosY, (1, 1), (1, 1))
        SpecifyReceiverPositionGrid.Add(self.PosZ, (2, 1), (1, 1))
        SpecifyReceiverPositionGrid.SetMinSize(wx.Size(XSpecifyReceiverPositionPanel, YSpecifyReceiverPositionPanel))
        
        self.SpecifyReceiverPositionPanel.SetSizer(SpecifyReceiverPositionGrid)
        
        #Use SINEX panel
        self.UseSinexReceiverPositionPanel = wx.Panel(ReceiverPositionPanel, -1) 
	self.UseSinexReceiverPositionPanel.SetSize(wx.Size(0.25 * XInputsPanelSize, 0.2 * YInputsPanelSize))   
        XUseSinexReceiverPositionPanel, YUseSinexReceiverPositionPanel = self.UseSinexReceiverPositionPanel.GetSize() 

        self.SinexTxt        = wx.lib.stattext.GenStaticText(self.UseSinexReceiverPositionPanel, -1, 'SINEX File :  ')   
        self.SinexTextCtrl   = wx.TextCtrl(self.UseSinexReceiverPositionPanel, -1, '', size=wx.Size(0.265 * XInputsPanelSize, 0.055* SecondaryFrameYSize), style=wx.TE_PROCESS_ENTER)
        self.SinexFileButton = wx.Button(self.UseSinexReceiverPositionPanel, 1, label='Examine')
	self.SinexPath = str('')

	SinexGrid = wx.GridBagSizer()
        SinexGrid.Add(self.SinexTxt , (0, 0), (1, 1), wx.ALIGN_LEFT | wx.TOP,10)
        SinexGrid.Add(self.SinexTextCtrl , (1, 0), (1, 1), wx.ALIGN_CENTER | wx.EXPAND | wx.ALL,2)
        SinexGrid.Add(self.SinexFileButton , (2, 0), (1, 1), wx.ALIGN_LEFT | wx.ALL,2)
        SinexGrid.SetMinSize(wx.Size(XUseSinexReceiverPositionPanel, YUseSinexReceiverPositionPanel))              
        self.UseSinexReceiverPositionPanel.SetSizer(SinexGrid)    

        self.SinexFileButton.Bind(wx.EVT_LEFT_DOWN, self.SaveSinexPathByButton)
        self.SinexTextCtrl.Bind(wx.EVT_TEXT, self.SaveSinexPathByTextCtrl)

        #Receiver Group
        ReceiverBox = wx.BoxSizer(wx.VERTICAL)
        ReceiverPositionPanel.SetSizer(ReceiverBox)    
        ReceiverPositionSB = wx.StaticBox(parent=ReceiverPositionPanel, label="A priori receiver position")
        ReceiverPositionSB.SetFont(LinedFont)
        ReceiverPositionSB.SetForegroundColour(LinedFontColour)        
        ReceiverPositionSizer = wx.StaticBoxSizer(ReceiverPositionSB, wx.VERTICAL)
        ReceiverPositionSizer.Add(RadButtonReceiverPositionPanel)
        ReceiverPositionSizer.Add(AprioryReceiverFirstSeparationSL, flag=wx.ALIGN_CENTER |  wx.TOP, border=10)
        ReceiverPositionSizer.Add(self.SpecifyReceiverPositionPanel)
        ReceiverPositionSizer.Add(AprioryReceiverSecondSeparationSL, flag=wx.ALIGN_CENTER |  wx.TOP, border=10)
	ReceiverPositionSizer.Add(self.UseSinexReceiverPositionPanel)
	ReceiverPositionSizer.SetMinSize(wx.Size(0.99 * XReceiverSize, 0.95 * YReceiverSize))
	ReceiverBox.Add(ReceiverPositionSizer)

     
     #Input Files
        InputFilesPanel = wx.Panel(self.InputsPanel, -1, size=self.SecondaryFrame.GetSize())
        InputFilesPanel.SetSize(wx.Size(0.5 * XCalculusSize, 0.6 * YCalculusSize))
        InputFilesPanelX, InputFilesPanelY = InputFilesPanel.GetSize()
        
        #First Third
        SelectInputFilesPanel = wx.Panel(InputFilesPanel, -1,)
        SelectInputFilesPanel.SetSize(wx.Size(InputFilesPanelX, 0.15 * InputFilesPanelY))
        XSelectInputFilesPanel,YSelectInputFilesPanel = SelectInputFilesPanel.GetSize()

        self.RinexObservationFileButton = wx.Button(SelectInputFilesPanel, 1, label='Examine')
        self.AntexObservationFileButton = wx.Button(SelectInputFilesPanel, 1, 'Examine')
        self.RinexObservation = wx.lib.stattext.GenStaticText(SelectInputFilesPanel, -1, ' RINEX Observation File :  ')
        self.AntexObservation = wx.lib.stattext.GenStaticText(SelectInputFilesPanel, -1, 'File :  ', style = wx.ALIGN_CENTER_VERTICAL)
	self.ShowAntexObservationCB = wx.CheckBox(SelectInputFilesPanel, -1 , 'Show ANTEX') 
        self.RinexObservation.SetMinSize(wx.Size(0.19 * XInputsPanelSize, 0.04 * YInputsPanelSize))
        self.AntexObservation.SetMinSize(wx.Size(0.10 * XInputsPanelSize, 0.04 * YInputsPanelSize))
        
        XTextCtrlInputsSize, YTextCtrlInputsSize = 0.35 * XInputsPanelSize, 0.05 * YInputsPanelSize
        
        self.TextCtrlRinexObservation = wx.TextCtrl(SelectInputFilesPanel, -1, '', size=wx.Size(XTextCtrlInputsSize, YTextCtrlInputsSize), style=wx.TE_PROCESS_ENTER)
        self.TextCtrlAntexObservation = wx.TextCtrl(SelectInputFilesPanel, -1, '', size=wx.Size(XTextCtrlInputsSize, YTextCtrlInputsSize), style=wx.TE_PROCESS_ENTER)

        InputFilesUpPanelGrid = wx.GridBagSizer(3,3)        
        InputFilesUpPanelGrid.Add(self.RinexObservation, (0, 0), (1, 3), wx.ALIGN_RIGHT | wx.ALIGN_CENTER | wx.LEFT, 5)
        InputFilesUpPanelGrid.Add(self.TextCtrlRinexObservation, (0, 3), (1, 9), wx.ALIGN_CENTER | wx.EXPAND |wx.LEFT, -10)
        InputFilesUpPanelGrid.Add(self.RinexObservationFileButton, (0, 12), (1, 1), wx.ALIGN_RIGHT)
   	InputFilesUpPanelGrid.Add(self.ShowAntexObservationCB, (1, 0), (1, 2), wx.ALIGN_LEFT | wx.ALIGN_CENTER | wx.LEFT,5)
        InputFilesUpPanelGrid.Add(self.AntexObservation, (1, 2), (1, 1), wx.ALIGN_RIGHT | wx.ALIGN_CENTER)
        InputFilesUpPanelGrid.Add(self.TextCtrlAntexObservation , (1, 3), (1, 9), wx.ALIGN_CENTER | wx.EXPAND | wx.LEFT,-10)
        InputFilesUpPanelGrid.Add(self.AntexObservationFileButton, (1, 12), (1, 1), wx.ALIGN_RIGHT)
        
        InputFilesUpPanelGrid.SetMinSize(wx.Size(0.95 * XSelectInputFilesPanel,0.95 * YSelectInputFilesPanel))
        SelectInputFilesPanel.SetSizer(InputFilesUpPanelGrid)

        self.RinexObservationFileButton.Bind(wx.EVT_LEFT_DOWN, self.SaveRinexPathByButton)
        self.AntexObservationFileButton.Bind(wx.EVT_LEFT_DOWN, self.SaveAntexPathByButton)
        self.TextCtrlRinexObservation.Bind(wx.EVT_TEXT, self.SaveRinexPathByTextCtrl)
        self.TextCtrlAntexObservation.Bind(wx.EVT_TEXT, self.SaveAntexPathByTextCtrl)
        self.ShowAntexObservationCB.Bind(wx.EVT_CHECKBOX, self.ShowAntex)  

        #Orbit and Clock Source
                  
        OrbitClockPanel = wx.Panel(InputFilesPanel, -1,)
        OrbitClockPanel.SetSize(wx.Size(InputFilesPanelX, 0.3 * InputFilesPanelY))
        
        self.OrbitClockPanelSpecifySource = wx.Panel(OrbitClockPanel, -1)
        self.OrbitClockPanelSpecifySource.SetSize(wx.Size(InputFilesPanelX, 0.3 * InputFilesPanelY))
        OrbitClockPanelX, OrbitClockPanelY = self.OrbitClockPanelSpecifySource.GetSize()
        
        OrbitClockRadioButtonsPanel = wx.Panel(self.OrbitClockPanelSpecifySource, -1,)
        
        self.BroadcastRB = wx.RadioButton(OrbitClockRadioButtonsPanel, -1, ' Broadcast ')
        self.SP3RB = wx.RadioButton(OrbitClockRadioButtonsPanel, -1, ' Precise (1 file) ')
        self.ClocksRinexOrbitSP3RB = wx.RadioButton(OrbitClockRadioButtonsPanel, -1, ' Precise (2 files) ')
        
        OrbitClockRadioButtonsBox = wx.BoxSizer(wx.HORIZONTAL)
        OrbitClockRadioButtonsPanel.SetSizer(OrbitClockRadioButtonsBox)

        OrbitClockRadioButtonsBox.Add(self.BroadcastRB)
        OrbitClockRadioButtonsBox.Add(self.SP3RB)
        OrbitClockRadioButtonsBox.Add(self.ClocksRinexOrbitSP3RB)
        
        self.OrbitClockGrid = wx.GridBagSizer()
        self.OrbitClockGrid.Add(OrbitClockRadioButtonsPanel, (0, 1), (1, 2), wx.EXPAND)
        
        self.Bind(wx.EVT_RADIOBUTTON, self.ShowOrbitClockSource, id=self.ClocksRinexOrbitSP3RB.GetId())      
        self.Bind(wx.EVT_RADIOBUTTON, self.ShowOrbitClockSource, id=self.BroadcastRB.GetId())
        self.Bind(wx.EVT_RADIOBUTTON, self.ShowOrbitClockSource, id=self.SP3RB.GetId())
        
        #Specify Orbit and Clock Source

        self.TextFirstRow = wx.lib.stattext.GenStaticText(self.OrbitClockPanelSpecifySource, -1, 'SP3  File :')
        self.TextFirstRow.SetMinSize(wx.Size(0.19 * XInputsPanelSize, 0.045 * YInputsPanelSize))
        self.TextCtrlFirstRow = wx.TextCtrl(self.OrbitClockPanelSpecifySource, -1, '', size=wx.Size(XTextCtrlInputsSize, YTextCtrlInputsSize), style=wx.TE_PROCESS_ENTER)
        self.FileButtonFirstRow = wx.Button(self.OrbitClockPanelSpecifySource, 1, label='Examine')
        self.TextSecondRow = wx.lib.stattext.GenStaticText(self.OrbitClockPanelSpecifySource, -1, 'Broadcast File :')
        self.TextSecondRow.SetMinSize(wx.Size(0.19 * XInputsPanelSize, 0.045 * YInputsPanelSize))
        self.TextCtrlSecondRow = wx.TextCtrl(self.OrbitClockPanelSpecifySource, -1, '', size=wx.Size(XTextCtrlInputsSize, YTextCtrlInputsSize), style=wx.TE_PROCESS_ENTER)
        self.FileButtonSecondRow = wx.Button(self.OrbitClockPanelSpecifySource, 1, label='Examine')
        
        self.OrbitClockGrid.Add(self.TextFirstRow , (1, 1), (1, 1), wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)
        self.OrbitClockGrid.Add(self.TextCtrlFirstRow , (1, 2), (1, 1), wx.ALIGN_CENTER | wx.EXPAND)
        self.OrbitClockGrid.Add(self.FileButtonFirstRow , (1, 3), (1, 1), wx.ALIGN_RIGHT | wx.LEFT , 3)
        self.OrbitClockGrid.Add(self.TextSecondRow , (2, 1), (1, 1), wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL | wx.TOP, 3)
        self.OrbitClockGrid.Add(self.TextCtrlSecondRow , (2, 2), (1, 1), wx.ALIGN_CENTER | wx.EXPAND| wx.TOP, 3)
        self.OrbitClockGrid.Add(self.FileButtonSecondRow , (2, 3) , (1, 1), wx.ALIGN_RIGHT| wx.TOP | wx.LEFT , 3)
        
        self.OrbitClockGrid.SetMinSize(wx.Size(OrbitClockPanelX, 0.95*OrbitClockPanelY))
        
        self.OrbitClockPanelSpecifySource.SetSizer(self.OrbitClockGrid)
       
       
        self.FileButtonFirstRow.Bind(wx.EVT_LEFT_DOWN, self.SaveOrbitsAndClockSourcePathByButton)
        self.FileButtonSecondRow.Bind(wx.EVT_LEFT_DOWN, self.SaveClocksRinexPathByButton)
        self.TextCtrlFirstRow.Bind(wx.EVT_TEXT, self.SaveOrbitsAndClockSourcePathByTextCtrl)
        self.TextCtrlSecondRow.Bind(wx.EVT_TEXT, self.SaveClocksRinexPathByTextCtrl)        
        

        self.BroadcastPath = u''
        self.SP3Path = u''
        self.OrbitsSP3Path = u''
        self.ClocksRinexPath = u''
        self.RinexPath = u''
        self.AntexPath = u''
        self.BroadcastIonosphereSourcePath = u''
        self.IonexIonosphereSourcePath = u''
	self.FPPPIonosphereSourcePath = u''
        
                
        OrbitClockBox = wx.BoxSizer(wx.VERTICAL)
        OrbitClockPanel.SetSizer(OrbitClockBox)
        OrbitClockSB = wx.StaticBox(parent=OrbitClockPanel, label="Orbit and Clock Source")
        OrbitClockSB.SetFont(LinedFont)
        OrbitClockSB.SetForegroundColour(LinedFontColour)            
        OrbitClockSizer = wx.StaticBoxSizer(OrbitClockSB, wx.VERTICAL)
        OrbitClockSizer.Add(self.OrbitClockPanelSpecifySource)
        OrbitClockSizer.SetMinSize(wx.Size(0.689 * XCalculusSize, 0.1 * YCalculusSize))
        OrbitClockBox.Add(OrbitClockSizer)
        

        
        #Ionosphere Source
        IonosphereSourcePanelOut = wx.Panel(InputFilesPanel, -1,)
        IonosphereSourcePanelOut.SetSize(wx.Size(InputFilesPanelX, 0.20 * InputFilesPanelY))
        IonosphereOutSizeX, IonosphereOutSizeY = IonosphereSourcePanelOut.GetSize()
        
        IonosphereSourcePanel = wx.Panel(IonosphereSourcePanelOut, -1)
        IonosphereSourcePanel.SetSize(wx.Size( IonosphereOutSizeX, 0.95 * IonosphereOutSizeY))
        IonosphereSizeX, IonosphereSizeY = IonosphereSourcePanel.GetSize()

	IonosphereSourceRBPanel = wx.Panel(IonosphereSourcePanel, -1)
        IonosphereSourceRBPanel.SetSize(wx.Size(IonosphereSizeX, 0.005 * IonosphereSizeY))
	IonosphereSourceRBPanelX, IonosphereSourceRBPanelY = IonosphereSourceRBPanel.GetSize()

	self.ShowIonosphereSourceCB = wx.CheckBox(IonosphereSourceRBPanel, -1 , 'Show ') 
        self.BroadcastSameRB = wx.RadioButton(IonosphereSourceRBPanel, -1, 'Broadcast (same as navigation) ')
        self.BroadcastSpecifyRB = wx.RadioButton(IonosphereSourceRBPanel, -1, 'Broadcast (specify)  ')
        self.IonexRB = wx.RadioButton(IonosphereSourceRBPanel, -1, 'IONEX ')
        self.FPPPRB = wx.RadioButton(IonosphereSourceRBPanel, -1, 'FPPP ')
        
        IonosphereSourceRadButtonGrid = wx.GridBagSizer()
        IonosphereSourceRadButtonGrid.Add(self.ShowIonosphereSourceCB, (0, 0), (1, 1) )
        IonosphereSourceRadButtonGrid.Add(self.BroadcastSameRB, (0, 1), (1, 1) )
        IonosphereSourceRadButtonGrid.Add(self.BroadcastSpecifyRB, (0, 2), (1, 1) ) 
        IonosphereSourceRadButtonGrid.Add(self.IonexRB, (0, 3), (1, 1))
	IonosphereSourceRadButtonGrid.Add(self.FPPPRB, (0, 4), (1, 1))
  	IonosphereSourceRadButtonGrid.SetMinSize(wx.Size( IonosphereSourceRBPanelX, 0.95 * IonosphereSourceRBPanelY)) 
        IonosphereSourceRBPanel.SetSizer(IonosphereSourceRadButtonGrid)

	self.ShowIonosphereSourceCB.Bind(wx.EVT_CHECKBOX, self.ShowIonosphereSource)  
        self.BroadcastSameRB.Bind(wx.EVT_RADIOBUTTON, self.ShowIonosphereSource)      
        self.BroadcastSpecifyRB.Bind(wx.EVT_RADIOBUTTON, self.ShowIonosphereSource)
        self.IonexRB.Bind(wx.EVT_RADIOBUTTON, self.ShowIonosphereSource)
	self.FPPPRB.Bind(wx.EVT_RADIOBUTTON, self.ShowIonosphereSource)
        
        
        SpecifyIonosphereFilePanel = wx.Panel(IonosphereSourcePanel, -1)
        SpecifyIonosphereFilePanel.SetSize(wx.Size(IonosphereSizeX, 0.01 * IonosphereSizeY))
        
	self.TextIonosphereSource = wx.lib.stattext.GenStaticText(SpecifyIonosphereFilePanel, -1, '      Broadcast File :      ')
        self.TextIonosphereSource.SetMinSize(wx.Size(0.20 * XInputsPanelSize, 0.05 * YInputsPanelSize))
        self.TextCtrlIonosphereSource = wx.TextCtrl(SpecifyIonosphereFilePanel, -1, size=wx.Size(XTextCtrlInputsSize, YTextCtrlInputsSize), style=wx.TE_PROCESS_ENTER)
        self.FileButtonIonosphereSource = wx.Button(SpecifyIonosphereFilePanel, 1, label='Examine')

        self.FileButtonIonosphereSource.Bind(wx.EVT_LEFT_DOWN, self.SaveIonosphereSourcePathByButton)
        self.TextCtrlIonosphereSource.Bind(wx.EVT_TEXT, self.SaveIonosphereSourcePathByTextCtrl)          
                          
        self.IonosphereSourceGrid = wx.GridBagSizer()
        self.IonosphereSourceGrid.Add(self.TextIonosphereSource , (0, 1), (1, 1), wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)
        self.IonosphereSourceGrid.Add(self.TextCtrlIonosphereSource, (0, 2), (1, 1), flag=wx.ALIGN_CENTER | wx.EXPAND)
        self.IonosphereSourceGrid.Add(self.FileButtonIonosphereSource , (0, 3), (1, 1), wx.ALIGN_RIGHT | wx.LEFT , 5 )       
        self.IonosphereSourceGrid.SetMinSize(wx.Size(IonosphereSizeX, 0.01 * IonosphereSizeY))  
        SpecifyIonosphereFilePanel.SetSizer(self.IonosphereSourceGrid)
               
        IonosphereFilePanelGroupGrid = wx.GridBagSizer()
        IonosphereFilePanelGroupGrid.Add(IonosphereSourceRBPanel, (0, 0), (1, 1),  wx.LEFT | wx.RIGHT | wx.TOP, 3 )
        IonosphereFilePanelGroupGrid.Add(SpecifyIonosphereFilePanel, (1, 0), (1, 1) ,  wx.LEFT | wx.RIGHT |wx.TOP, 3)
        IonosphereFilePanelGroupGrid.SetMinSize(wx.Size( 0.95 * IonosphereSizeX, 0.95 * IonosphereSizeY)) 
        IonosphereSourcePanel.SetSizer(IonosphereFilePanelGroupGrid)                             
               
        IonosphereSourceBox = wx.BoxSizer(wx.VERTICAL)
        IonosphereSourcePanelOut.SetSizer(IonosphereSourceBox)
        IonosphereSourceSB = wx.StaticBox(parent=IonosphereSourcePanelOut, label="Ionosphere Source (if activated)")
        IonosphereSourceSB.SetFont(LinedFont)
        IonosphereSourceSB.SetForegroundColour(LinedFontColour)        
        IonosphereSourceSizer = wx.StaticBoxSizer(IonosphereSourceSB)
        IonosphereSourceSizer.Add(IonosphereSourcePanel )
        IonosphereSourceBox.Add(IonosphereSourceSizer)
        IonosphereSourceSizer.SetMinSize(wx.Size(0.689 * XCalculusSize, 0.95 *  IonosphereOutSizeY))

        
        
        #Input Files Group
        InputFilesBox = wx.BoxSizer(wx.VERTICAL)
        InputFilesPanel.SetSizer(InputFilesBox)
        
        InputFilesSB = wx.StaticBox(parent=InputFilesPanel, label="Input Files")
        InputFilesSB.SetFont(LinedFont)
        InputFilesSB.SetForegroundColour(LinedFontColour)
        InputFilesSizer = wx.StaticBoxSizer(InputFilesSB, wx.VERTICAL)
        InputFilesSizer.Add(SelectInputFilesPanel)
        InputFilesSizer.Add(OrbitClockPanel)
        InputFilesSizer.Add(IonosphereSourcePanelOut )
        InputFilesBox.Add(InputFilesSizer)
        InputFilesSizer.SetMinSize(wx.Size(0.70 * XCalculusSize, 0.1 * YCalculusSize))

     #Auxiliary Files:        
	AuxiliaryFilesPanel = wx.Panel(self.InputsPanel, -1)
	AuxiliaryFilesPanel.SetSize(wx.Size(0.982 * XCalculusSize, 0.15 * YCalculusSize))
	XAuxiliaryFilesPanel, YAuxiliaryFilesPanelSize = AuxiliaryFilesPanel.GetSize()
	AuxiliaryFilesPanelCore = wx.Panel(AuxiliaryFilesPanel, -1)
	AuxiliaryFilesPanelCore.SetSize(wx.Size(0.95 * XAuxiliaryFilesPanel, 0.99 * YAuxiliaryFilesPanelSize))
	XAuxiliaryFilesPanelCoreSize, YAuxiliaryFilesPanelCoreSize = AuxiliaryFilesPanelCore.GetSize()

	XTxtCtrlAuxiliaryFilesPanel, YTxtCtrlAuxiliaryFilesPanelSize = 0.30 * XAuxiliaryFilesPanel, 0.40 * YAuxiliaryFilesPanelSize

	# P1 P2 Correction Panel
	P1P2Panel = wx.Panel(AuxiliaryFilesPanelCore, -1) 
	P1P2Panel.SetSize(wx.Size(0.30* XAuxiliaryFilesPanelCoreSize, 0.95 * YAuxiliaryFilesPanelCoreSize))
	XP1P2PanelSize, YP1P2PanelSize = P1P2Panel.GetSize()
#	P1P2Panel.SetBackgroundColour(wx.RED)

	P1P2PanelCore = wx.Panel(P1P2Panel, -1) 
	P1P2PanelCore.SetSize(wx.Size(0.99 * XP1P2PanelSize, 0.95 * YP1P2PanelSize))
	XP1P2PanelCore, YP1P2PanelCore = P1P2PanelCore.GetSize()

	self.ShowP1P2CB = wx.CheckBox(P1P2PanelCore, -1 , 'Show ', size = wx.Size(0.25 * XP1P2PanelCore, 0.3 * YP1P2PanelCore)  ) 
        self.P1P2SourceTxt  = wx.lib.stattext.GenStaticText(P1P2PanelCore, -1, 'DCB Source : ', size = wx.Size(0.45 * XP1P2PanelCore, 0.21 * YP1P2PanelCore) )   
	self.P1P2SourcesList = ['Broadcast (same as navigation)','Broadcast (specify)','Precise .DCB file','IONEX (same as ionosphere)','IONEX (specify)','FPPP (same as ionosphere)','FPPP (specify)']
        self.P1P2SourcesComboBox = wx.ComboBox(P1P2PanelCore, -1, value=self.P1P2SourcesList[0], choices=self.P1P2SourcesList, size = wx.Size(0.80 * XTxtCtrlAuxiliaryFilesPanel, YTxtCtrlAuxiliaryFilesPanelSize))
        self.P1P2SourcesComboBox.Enable(True)     
        self.P1P2FileTxt  = wx.lib.stattext.GenStaticText(P1P2PanelCore, -1, 'RINEX Nav. File : ')   
#        self.P1P2FileTxt.SetBackgroundColour(wx.RED)
        self.P1P2TextCtrl   = wx.TextCtrl(P1P2PanelCore, -1, '', style=wx.TE_PROCESS_ENTER, size = ( 0.80 * XTxtCtrlAuxiliaryFilesPanel, YTxtCtrlAuxiliaryFilesPanelSize ) )
        self.P1P2FileButton = wx.Button(P1P2PanelCore, 1, label='Examine',size = ( 0.30 * XTxtCtrlAuxiliaryFilesPanel, YTxtCtrlAuxiliaryFilesPanelSize ) )


	self.DCBSourceP1P2Path   = str('')
	self.IonexSourceP1P2Path = str('')
	self.FPPPSourceP1P2Path  = str('')
	self.TGDSourceP1P2Path   = str('')

        self.P1P2FileButton.Bind(wx.EVT_LEFT_DOWN, self.SaveP1P2PathByButton)
        self.P1P2TextCtrl.Bind(wx.EVT_TEXT, self.SaveP1P2PathByTextCtrl)
        self.ShowP1P2CB.Bind(wx.EVT_CHECKBOX, self.ShowP1P2DCBSource)
        self.P1P2SourcesComboBox.Bind(wx.EVT_COMBOBOX, self.ShowP1P2DCBSource)

        AuxReceiverTypePanelCoreGrid = wx.GridBagSizer()
        AuxReceiverTypePanelCoreGrid.Add(self.ShowP1P2CB, (0, 0),(1,1), wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL | wx.LEFT ,5 )          
        AuxReceiverTypePanelCoreGrid.Add(self.P1P2SourceTxt , (1, 0),(1,1), wx.ALIGN_CENTER  | wx.LEFT ,5 )
        AuxReceiverTypePanelCoreGrid.Add(self.P1P2SourcesComboBox, (1, 1),(1,1), wx.ALIGN_LEFT | wx.ALIGN_CENTER  |wx.BOTTOM ,5 )  
	AuxReceiverTypePanelCoreGrid.Add(self.P1P2FileTxt , (2, 0),(1,1), wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL)
        AuxReceiverTypePanelCoreGrid.Add(self.P1P2TextCtrl , (2, 1),(1,1), wx.ALIGN_LEFT  )
        AuxReceiverTypePanelCoreGrid.Add(self.P1P2FileButton , (2, 2),(1,1), wx.ALIGN_LEFT )
        P1P2PanelCore.SetSizer(AuxReceiverTypePanelCoreGrid)     
        AuxReceiverTypePanelCoreGrid.SetMinSize(wx.Size(0.905*XP1P2PanelCore, 0.95*YP1P2PanelCore))

        #P1P2 Core Group
        P1P2Box = wx.BoxSizer(wx.VERTICAL)
        P1P2Panel.SetSizer(P1P2Box)    
        P1P2SB = wx.StaticBox(parent=P1P2Panel, label="P1 - P2 Correction")
        P1P2SB.SetFont(LinedFont)
        P1P2SB.SetForegroundColour(LinedFontColour)        
        P1P2SBSizer = wx.StaticBoxSizer(P1P2SB, wx.VERTICAL)
        P1P2SBSizer.Add(P1P2PanelCore)
        P1P2SBSizer.SetMinSize(wx.Size( 0.99 * XP1P2PanelSize, 0.99 *  YP1P2PanelSize))
        P1P2Box.Add(P1P2SBSizer)
	

	#P1C1 Correction
        DCBSourcePanel = wx.Panel(AuxiliaryFilesPanelCore, -1) 
        DCBSourcePanel.SetSize(wx.Size(0.5 * XAuxiliaryFilesPanelCoreSize, 0.95 * YAuxiliaryFilesPanelCoreSize))
        XDCBSourceSize, YDCBSourceSize = DCBSourcePanel.GetSize()
	
	DCBSourcePanelCore = wx.Panel(DCBSourcePanel, -1)
        DCBSourcePanelCore.SetSize(wx.Size(0.95 * XDCBSourceSize, 0.80* YDCBSourceSize))  
	XDCBSourcePanelCore, YDCBSourcePanelCore = DCBSourcePanelCore.GetSize()
	
	#DCB Source RadButton  Panel
        DCBSourceRadButtonPanel = wx.Panel(DCBSourcePanelCore, -1)
        DCBSourceRadButtonPanel.SetSize(wx.Size(0.99 * XDCBSourcePanelCore, 0.10* YDCBSourcePanelCore))

	self.ShowP1C1CB = wx.CheckBox(DCBSourceRadButtonPanel, -1 , 'Show ')          
	self.ShowP1C1CB.Bind(wx.EVT_CHECKBOX, self.ShowP1C1)   
        DCBSourceRadButtonsBox = wx.BoxSizer(wx.HORIZONTAL)
        DCBSourceRadButtonPanel.SetSizer(DCBSourceRadButtonsBox)
        DCBSourceRadButtonsBox.Add(self.ShowP1C1CB)
   

	# P1-C1 DCB File & GPS Receiver Type File: 
        DCBSpecifySourcePanel = wx.Panel(DCBSourcePanelCore, -1) 
        DCBSpecifySourcePanel.SetSize(wx.Size(0.99 * XDCBSourcePanelCore, 0.90* YDCBSourcePanelCore)) 
        XDCBSpecifySourceSize, YDCBSpecifySourceSize = DCBSpecifySourcePanel.GetSize()

        self.P1C1FileTxt = wx.lib.stattext.GenStaticText(DCBSpecifySourcePanel, -1,'DCB  File :  ')
        self.P1C1FileTxt.SetSize(wx.Size(0.21 * XAuxiliaryFilesPanel, 0.25* YAuxiliaryFilesPanelSize))
        self.P1C1FileTextCtrl = wx.TextCtrl(DCBSpecifySourcePanel, -1, '',size=wx.Size(XTxtCtrlAuxiliaryFilesPanel, YTxtCtrlAuxiliaryFilesPanelSize), style=wx.TE_PROCESS_ENTER)
        self.P1C1FileButton = wx.Button(DCBSpecifySourcePanel, 1, label='Examine')

        self.ReceiverTypeTxt  = wx.lib.stattext.GenStaticText(DCBSpecifySourcePanel, -1, 'Rec Type File :' )   
        self.ReceiverTypeTxt.SetSize(wx.Size(0.21 * XAuxiliaryFilesPanel, 0.25* YAuxiliaryFilesPanelSize))
        self.ReceiverTypeTextCtrl = wx.TextCtrl(DCBSpecifySourcePanel, -1, '', size=wx.Size(XTxtCtrlAuxiliaryFilesPanel, YTxtCtrlAuxiliaryFilesPanelSize), style=wx.TE_PROCESS_ENTER)
        self.ReceiverTypeFileButton = wx.Button(DCBSpecifySourcePanel, 1, label='Examine')


	self.DCBSourceP1C1Path = str('')
	self.AuxReceiverTypePath = str('')

        self.P1C1FileButton.Bind(wx.EVT_LEFT_DOWN, self.SaveP1C1PathByButton)
        self.P1C1FileTextCtrl.Bind(wx.EVT_TEXT, self.SaveP1C1PathByTextCtrl)
        self.ReceiverTypeFileButton.Bind(wx.EVT_LEFT_DOWN, self.SaveReceiverTypePathByButton)
        self.ReceiverTypeTextCtrl.Bind(wx.EVT_TEXT, self.SaveReceiverTypePathByTextCtrl)

        AuxDCBSourcePanelCoreGrid = wx.GridBagSizer()
        AuxDCBSourcePanelCoreGrid.Add(self.P1C1FileTxt , (0, 0),(1,1), wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT )
        AuxDCBSourcePanelCoreGrid.Add(self.P1C1FileTextCtrl , (0, 1),(1,1), wx.ALIGN_CENTER_VERTICAL )
        AuxDCBSourcePanelCoreGrid.Add(self.P1C1FileButton , (0, 2),(1,1), wx.ALIGN_LEFT | wx.LEFT , 3 )
        AuxDCBSourcePanelCoreGrid.Add(self.ReceiverTypeTxt , (1, 0),(1,1), wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT  | wx.TOP, 3)
        AuxDCBSourcePanelCoreGrid.Add(self.ReceiverTypeTextCtrl , (1, 1),(1,1), wx.ALIGN_CENTER_VERTICAL  | wx.TOP, 3)
        AuxDCBSourcePanelCoreGrid.Add(self.ReceiverTypeFileButton , (1, 2),(1,1), wx.ALIGN_LEFT | wx.LEFT | wx.TOP, 3 )
        DCBSpecifySourcePanel.SetSizer(AuxDCBSourcePanelCoreGrid)     
        AuxDCBSourcePanelCoreGrid.SetMinSize(wx.Size( 0.95 * XDCBSpecifySourceSize, 0.95 * YDCBSpecifySourceSize))

        #DCB Source Core: 
        DCBSourcePanelCoreGrid = wx.GridBagSizer()
        DCBSourcePanelCoreGrid.Add(DCBSourceRadButtonPanel, (0, 0),(1,1), )
        DCBSourcePanelCoreGrid.Add(DCBSpecifySourcePanel , (1, 0),(1,1),  )
        DCBSourcePanelCore.SetSizer(DCBSourcePanelCoreGrid)     
        DCBSourcePanelCoreGrid.SetMinSize(wx.Size(XDCBSourcePanelCore, YDCBSourcePanelCore))
	
        #DCB Source Group
        DCBSourceBox = wx.BoxSizer(wx.VERTICAL)
        DCBSourcePanel.SetSizer(DCBSourceBox)    
        DCBSourcePanelSB = wx.StaticBox(parent=DCBSourcePanel, label="P1 - C1 Correction")
        DCBSourcePanelSB.SetFont(LinedFont)
        DCBSourcePanelSB.SetForegroundColour(LinedFontColour)        
        DCBSourcePanelSizer = wx.StaticBoxSizer(DCBSourcePanelSB, wx.VERTICAL)
        DCBSourcePanelSizer.Add(DCBSourcePanelCore)
        DCBSourcePanelSizer.SetMinSize(wx.Size(XDCBSourcePanelCore, YDCBSourcePanelCore))
        DCBSourceBox.Add(DCBSourcePanelSizer)
	


     	#Auxiliary Files Core:  
        AuxiliaryFilesCoreGrid = wx.GridBagSizer()
        AuxiliaryFilesCoreGrid.Add(DCBSourcePanel , (0, 0),(1,1),  wx.LEFT , 5)
        AuxiliaryFilesCoreGrid.Add(P1P2Panel , (0, 1),(1,1), wx.LEFT | wx.ALIGN_CENTER, 10)
        AuxiliaryFilesPanelCore.SetSizer(AuxiliaryFilesCoreGrid)     
        AuxiliaryFilesCoreGrid.SetMinSize(wx.Size( 0.99*XAuxiliaryFilesPanelCoreSize, 0.99*YAuxiliaryFilesPanelCoreSize ))


        #Auxiliary Files Group
        AuxiliaryFilesBox = wx.BoxSizer(wx.VERTICAL)
        AuxiliaryFilesPanel.SetSizer(AuxiliaryFilesBox)    
        AuxiliaryFilesSB = wx.StaticBox(parent=AuxiliaryFilesPanel, label="Auxiliary Files")
        AuxiliaryFilesSB.SetFont(LinedFont)
        AuxiliaryFilesSB.SetForegroundColour(LinedFontColour)        
        AuxiliaryFilesSizer = wx.StaticBoxSizer(AuxiliaryFilesSB, wx.VERTICAL)
        AuxiliaryFilesSizer.Add(AuxiliaryFilesPanelCore)
        AuxiliaryFilesSizer.SetMinSize(wx.Size(0.99*XAuxiliaryFilesPanel, 0.99*YAuxiliaryFilesPanelSize))
        AuxiliaryFilesBox.Add(AuxiliaryFilesSizer)


     # Input Panel Group
        
        self.InputPanelGrill = wx.GridBagSizer()
        self.InputPanelGrill.Add(InputFilesPanel, (0, 0), (1, 1), wx.TOP | wx.LEFT | wx.RIGHT, 5  )
        self.InputPanelGrill.Add(ReceiverPositionPanel, (0, 1), (5, 1), wx.TOP | wx.RIGHT | wx.LEFT, 5)
        self.InputPanelGrill.Add(AuxiliaryFilesPanel, (1, 0), (1, 5), wx.TOP | wx.RIGHT | wx.LEFT, 5 )
        self.InputPanelGrill.SetMinSize(wx.Size(XInputsPanelSize, YInputsPanelSize))        
        self.InputsPanel.SetSizer(self.InputPanelGrill)
        
     
        """
............................................................................................        
                PREPROCESS PANEL
............................................................................................                   
        """

        #Create PreprocesPanel elements   
        self.PreProcesPanel = wx.Panel(self.SecondaryFrame, -1, size=self.SecondaryFrame.GetSize())
        XPreProcesPanelSize, YPreProcesPanelSize = self.PreProcesPanel.GetSize()

      #Elevation Mask
        
        ElevationMaskPanel = wx.Panel(self.PreProcesPanel, -1)
        ElevationMaskPanel.SetSize(wx.Size(0.48 * XPreProcesPanelSize, 0.15 * YPreProcesPanelSize))
        XElevationMaskSize, YElevationMaskSize = ElevationMaskPanel.GetSize() 
        
           
        ElevationMaskPanelCore = wx.Panel(ElevationMaskPanel, -1)
        ElevationMaskPanelCore.SetSize(wx.Size(0.95 * XElevationMaskSize, 0.50 * YElevationMaskSize))
        XElevationMaskSizeCore, YElevationMaskSizeCore = ElevationMaskPanelCore.GetSize() 
        
        self.ElevationMaskText = wx.lib.stattext.GenStaticText(ElevationMaskPanelCore, -1 , 'Elevation Mask [Degrees]  ')
        self.ElevationMask = wx.TextCtrl(ElevationMaskPanelCore, -1, '5', size=wx.Size(0.15 * XElevationMaskSizeCore, 0.7 * YElevationMaskSizeCore))        
        
        SatelliteOptionsSL = wx.StaticLine(parent=ElevationMaskPanelCore, size=wx.Size(0.98 * XElevationMaskSizeCore, 3))
        
        self.DiscardEclipsedSatellitesCB = wx.CheckBox(ElevationMaskPanelCore, -1 , 'Discard satellites under eclipse condition')
        self.DiscardEclipsedSatellitesCB.SetValue(True)

        self.DiscardUnhealthySatellitesCB = wx.CheckBox(ElevationMaskPanelCore, -1 , 'Discard unhealthy satellites (Broadcast only)')
        self.DiscardUnhealthySatellitesCB.SetValue(True)
        
        #ElevationMaskBox = wx.BoxSizer(wx.HORIZONTAL)
        ElevationMaskGrid = wx.GridBagSizer()
        ElevationMaskGrid.Add(self.ElevationMaskText , (0, 0), (1, 1),wx.TOP | wx.LEFT , 3)
        ElevationMaskGrid.Add(self.ElevationMask , (0, 1), (1, 1),wx.TOP, 3)
        ElevationMaskGrid.Add(SatelliteOptionsSL , (1, 0), (1, 2), flag=wx.ALIGN_CENTER | wx.BOTTOM | wx.TOP, border=5)
        ElevationMaskGrid.Add(self.DiscardEclipsedSatellitesCB, (2, 0), (1, 2))
        ElevationMaskGrid.Add(self.DiscardUnhealthySatellitesCB, (3, 0), (1, 2))
        
        #ElevationMaskBox.Add( self.ElevationMaskText , flag=wx.ALIGN_CENTER )
        #ElevationMaskBox.Add( self.ElevationMask , flag=wx.ALIGN_CENTER )
        ElevationMaskPanelCore.SetSizer(ElevationMaskGrid)        
        
        
        #Elevation Mask Group
        ElevationMaskGroupBox = wx.BoxSizer(wx.VERTICAL)
        ElevationMaskPanel.SetSizer(ElevationMaskGroupBox)
        
        ElevationMaskGroupSB = wx.StaticBox(parent=ElevationMaskPanel, label="Satellite Options")
        ElevationMaskGroupSB.SetFont(LinedFont)
        ElevationMaskGroupSB.SetForegroundColour(LinedFontColour)
        ElevationMaskGroupSizer = wx.StaticBoxSizer(ElevationMaskGroupSB, wx.VERTICAL)
        ElevationMaskGroupSizer.Add(ElevationMaskPanelCore)
        
        ElevationMaskGroupSizer.SetMinSize(wx.Size(0.95 * XElevationMaskSize, 0.50 * YElevationMaskSize))
        ElevationMaskGroupBox.Add(ElevationMaskGroupSizer)      
        
      #Cycle-Slip Detection:
        
        CycleSlipDetectionPanel = wx.Panel(self.PreProcesPanel, -1)
        CycleSlipDetectionPanel.SetSize(wx.Size(0.459 * XPreProcesPanelSize, 0.3 * YPreProcesPanelSize))
        XCycleSlipDetectionPanel, YCycleSlipDetectionPanel = CycleSlipDetectionPanel.GetSize()          
           
        CycleSlipDetectionPanelCore = wx.Panel(CycleSlipDetectionPanel, -1)
        CycleSlipDetectionPanelCore.SetSize(wx.Size(0.95 * XCycleSlipDetectionPanel, 0.95 * YCycleSlipDetectionPanel))
        XCycleSlipDetectionCore, YCycleSlipDetectionCore = CycleSlipDetectionPanelCore.GetSize()       
              
        CycleSlipDetectionConfPanel = wx.Panel(CycleSlipDetectionPanelCore, -1)     
        CycleSlipDetectionConfPanel.SetSize(wx.Size(0.99 * XCycleSlipDetectionCore, 0.20 * YCycleSlipDetectionCore))
        XCycleSlipDetectionConfPanel, YCycleSlipDetectionConfPanel = CycleSlipDetectionConfPanel.GetSize() 

        self.DataGapText = wx.lib.stattext.GenStaticText(CycleSlipDetectionConfPanel, -1 , 'Data Gap [seconds]  ')
        self.DataGap = wx.TextCtrl(CycleSlipDetectionConfPanel, -1, '40', size=wx.Size(0.155 * XCycleSlipDetectionCore, 0.18 * YCycleSlipDetectionCore))
        self.EnableLLI = wx.CheckBox(CycleSlipDetectionConfPanel, -1, 'Loss of Lock Indicator (LLI)')
        self.EnableLICB = wx.CheckBox(CycleSlipDetectionConfPanel, -1, 'Geometry-free [Dual-frequency]')
        self.EnableMelbourneWubbenaCB = wx.CheckBox(CycleSlipDetectionConfPanel, -1, u'Melbourne-Wübbena [Dual-frequency]') #'\u00FC'stands for u with dieresis
        self.EnableC1L1DifferenceCB = wx.CheckBox(CycleSlipDetectionConfPanel, -1, 'L1-C1 difference [Single-frequency]')
        self.LIConfigButton = wx.Button(CycleSlipDetectionConfPanel, -1, 'Configure', style=wx.BU_EXACTFIT)
        self.MelbourneWubbenaConfigButton = wx.Button(CycleSlipDetectionConfPanel, -1, 'Configure', style=wx.BU_EXACTFIT)
        self.L1C1DifferenceConfigButton = wx.Button(CycleSlipDetectionConfPanel, -1, 'Configure', style=wx.BU_EXACTFIT)

        CycleSlipDetectionConfBox = wx.GridBagSizer()
        CycleSlipDetectionConfBox.Add(self.DataGapText, (0, 0), (1, 1), wx.TOP | wx.LEFT, 6)
        if WIN:
             CycleSlipDetectionConfBox.Add(self.DataGap, (0, 1), (1, 1), wx.LEFT, -120)
        else:
             CycleSlipDetectionConfBox.Add(self.DataGap, (0, 1), (1, 1), wx. wx.LEFT, -140)
        #CycleSlipDetectionConfBox.Add(self.DataGap, (0, 1), (1, 1), wx. wx.LEFT, -140)
        CycleSlipDetectionConfBox.Add(self.EnableLLI, (1, 0), (1,1), wx.ALIGN_CENTER_VERTICAL | wx.TOP, 6)
        CycleSlipDetectionConfBox.Add(self.EnableC1L1DifferenceCB, (2, 0), (1, 1), flag=wx.ALIGN_CENTER_VERTICAL)
        CycleSlipDetectionConfBox.Add(self.EnableMelbourneWubbenaCB, (3, 0), (1, 1),flag=wx.ALIGN_CENTER_VERTICAL)
        CycleSlipDetectionConfBox.Add(self.EnableLICB, (4, 0), (1, 1), flag=wx.ALIGN_CENTER_VERTICAL)
        CycleSlipDetectionConfBox.Add(self.L1C1DifferenceConfigButton, (2, 1), (1,1),wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_RIGHT | wx.LEFT, 20)
        CycleSlipDetectionConfBox.Add(self.MelbourneWubbenaConfigButton, (3, 1), (1,1),wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_RIGHT | wx.LEFT, 20)
        CycleSlipDetectionConfBox.Add(self.LIConfigButton, (4, 1), (1,1),wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_RIGHT | wx.LEFT, 20)
        
        
        CycleSlipDetectionConfPanel.SetSizer(CycleSlipDetectionConfBox)
        CycleSlipDetectionConfBox.SetMinSize(wx.Size(0.95 * XCycleSlipDetectionConfPanel, 0.95 *  YCycleSlipDetectionConfPanel)) 

      #SpecifyCSDPanel
        self.SpecifyCSDPanel = wx.Panel(CycleSlipDetectionPanelCore, -1)
	self.SpecifyCSDPanel.SetSize( wx.Size(0.9 * XCycleSlipDetectionCore, 0.5 * YCycleSlipDetectionCore )    ) 
	XSpecifyCSDPanel, YSpecifyCSDPanel = self.SpecifyCSDPanel.GetSize()   
        
        #SpecifyCSDPanel Core
        SpecifyCSDPanelCore = wx.Panel(self.SpecifyCSDPanel, -1, size=wx.Size(0.95 * XSpecifyCSDPanel, 0.95 * YSpecifyCSDPanel))     
	XSpecifyCSDPanelCore, YSpecifyCSDPanelCore = SpecifyCSDPanelCore.GetSize()
        
        
        self.SpecifyCycleSlipDetectionPanel = wx.Panel(SpecifyCSDPanelCore, -1)
	self.SpecifyCycleSlipDetectionPanel.SetSize( wx.Size(0.95 * XSpecifyCSDPanelCore, 0.95 * YSpecifyCSDPanelCore )    )  
	XSpecifyCycleSlipDetectionPanel, YSpecifyCycleSlipDetectionPanel = self.SpecifyCycleSlipDetectionPanel.GetSize()        

        self.MinimumThresholdText = wx.lib.stattext.GenStaticText(self.SpecifyCycleSlipDetectionPanel, -1, 'Minimum stdDev [m]: ')
        self.MinimumThreshold = wx.TextCtrl(self.SpecifyCycleSlipDetectionPanel, -1, size=wx.Size(0.2 * XElevationMaskSizeCore, 0.7 * YElevationMaskSizeCore), style=wx.TE_PROCESS_ENTER)        
        self.MaximumThresholdText = wx.lib.stattext.GenStaticText(self.SpecifyCycleSlipDetectionPanel, -1, 'Maximum Threshold [cycles]: ')        
        self.MaximumThreshold = wx.TextCtrl(self.SpecifyCycleSlipDetectionPanel, -1, size=wx.Size(0.2 * XElevationMaskSizeCore, 0.7 * YElevationMaskSizeCore), style=wx.TE_PROCESS_ENTER)
        self.TimeOrDeviationText = wx.lib.stattext.GenStaticText(self.SpecifyCycleSlipDetectionPanel, -1, 'Deviation/Threshold Proportion: ')
        self.TimeOrDeviation = wx.TextCtrl(self.SpecifyCycleSlipDetectionPanel, -1, size=wx.Size(0.2 * XElevationMaskSizeCore, 0.7 * YElevationMaskSizeCore), style=wx.TE_PROCESS_ENTER)
        self.NsamplesText = wx.lib.stattext.GenStaticText(self.SpecifyCycleSlipDetectionPanel, -1, 'N-sample: ')
        self.Nsamples = wx.TextCtrl(self.SpecifyCycleSlipDetectionPanel, -1, size=wx.Size(0.2 * XElevationMaskSizeCore, 0.7 * YElevationMaskSizeCore), style=wx.TE_PROCESS_ENTER)
   
        self.SpecifyCycleSlipDetectionGrid = wx.GridBagSizer()
        self.SpecifyCycleSlipDetectionGrid.Add(self.MinimumThresholdText, (0, 0), (1, 1), wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL | wx.TOP | wx.LEFT , 5)
        self.SpecifyCycleSlipDetectionGrid.Add(self.MinimumThreshold, (0, 1), (1, 1), wx.ALIGN_CENTER | wx.EXPAND | wx.TOP , 5)
        self.SpecifyCycleSlipDetectionGrid.Add(self.MaximumThresholdText , (1, 0), (1, 1), wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL | wx.LEFT , 5)
        self.SpecifyCycleSlipDetectionGrid.Add(self.MaximumThreshold , (1, 1), (1, 1), wx.ALIGN_CENTER | wx.EXPAND)
        self.SpecifyCycleSlipDetectionGrid.Add(self.TimeOrDeviationText , (2, 0), (1, 1), wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL | wx.LEFT , 5)
        self.SpecifyCycleSlipDetectionGrid.Add(self.TimeOrDeviation , (2, 1), (1, 1), wx.ALIGN_CENTER | wx.EXPAND)
        self.SpecifyCycleSlipDetectionGrid.Add(self.NsamplesText , (3, 0), (1, 1), wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL | wx.LEFT , 5)
        self.SpecifyCycleSlipDetectionGrid.Add(self.Nsamples , (3, 1), (1, 1), wx.ALIGN_CENTER | wx.EXPAND)
                                               
        self.SpecifyCycleSlipDetectionGrid.SetMinSize(wx.Size(0.95*XSpecifyCycleSlipDetectionPanel, 0.95*YSpecifyCycleSlipDetectionPanel))
        
        self.SpecifyCycleSlipDetectionPanel.SetSizer(self.SpecifyCycleSlipDetectionGrid)
        

      
        
        self.MinimumThreshold.Bind(wx.EVT_TEXT, self.SaveCycleSlipParameters)
        self.MaximumThreshold.Bind(wx.EVT_TEXT, self.SaveCycleSlipParameters)
        self.TimeOrDeviation.Bind(wx.EVT_TEXT, self.SaveCycleSlipParameters)
        self.Nsamples.Bind(wx.EVT_TEXT, self.SaveCycleSlipParameters)

        # LI
        self.MaximumJumpThreshold = '1'
        self.MaximumThresholdMetersLI = '0.08'
        self.TimeConstant = '60'
        self.NsamplesLI = '7'
        self.GeometricFreeCPCombinationShow = True

        # MW
        self.MinimumThresholdMetersMW = '0.8'
        self.WindowMW = '300'
        self.SlopeMW = '5'
        self.NsamplesMW = '2'
        self.MelbourneWubbenaShow = False
        
        # L1-C1
        self.MaximumThresholdC1L1Difference = '3'
        self.WindowC1L1Diference = '300'
        self.SlopeC1L1Diference = '5'
        self.NsamplesL1C1 = '2'
        self.C1L1DiferenceShow = False
        
        self.EnableLICB.SetValue(True)                
        self.EnableMelbourneWubbenaCB.SetValue(True)
        self.EnableC1L1DifferenceCB.SetValue(True)
        self.EnableLLI.SetValue(True)
        
        SpecifyCSDPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        SpecifyCSDPanelGroupBox.Add(self.SpecifyCycleSlipDetectionPanel)
        SpecifyCSDPanelCore.SetSizer(SpecifyCSDPanelGroupBox)

        #SpecifyCSDPanel Group 
        SpecifyCSDPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        self.SpecifyCSDPanel.SetSizer(SpecifyCSDPanelGroupBox)
        SpecifyCSDPanelGroupSB = wx.StaticBox(parent=self.SpecifyCSDPanel, label="Cycle-slip Detection Parameters")
        SpecifyCSDPanelGroupSB.SetFont(LinedFont)
        SpecifyCSDPanelGroupSB.SetForegroundColour(LinedFontColour)                  
        SpecifyCSDPanelGroupSizer = wx.StaticBoxSizer(SpecifyCSDPanelGroupSB, wx.VERTICAL)
        SpecifyCSDPanelGroupSizer.Add(SpecifyCSDPanelCore)
        SpecifyCSDPanelGroupSizer.SetMinSize(wx.Size(0.95 * XSpecifyCSDPanel, 0.95 * YSpecifyCSDPanel))
        SpecifyCSDPanelGroupBox.Add(SpecifyCSDPanelGroupSizer)
        
        #Cycle-Slip Core
        CycleSlipDetectionPanelCoreGroupGrid = wx.GridBagSizer()
        CycleSlipDetectionPanelCoreGroupGrid.Add(CycleSlipDetectionConfPanel, (0,0),(1,1))
        CycleSlipDetectionPanelCoreGroupGrid.Add(self.SpecifyCSDPanel, (1,0),(1,1), wx.ALIGN_LEFT | wx.LEFT , 20)

        CycleSlipDetectionPanelCore.SetSizer(CycleSlipDetectionPanelCoreGroupGrid)
        CycleSlipDetectionPanelCoreGroupGrid.SetMinSize(wx.Size(0.95 * XCycleSlipDetectionCore, 0.95 * YCycleSlipDetectionCore)) 
               
         
        #Cycle-Slip Detection Group
        CycleSlipDetectionGroupBox = wx.BoxSizer(wx.VERTICAL)
        CycleSlipDetectionPanel.SetSizer(CycleSlipDetectionGroupBox)
        CycleSlipDetectionGroupSB = wx.StaticBox(parent=CycleSlipDetectionPanel, label="Cycle-slip Detection")
        CycleSlipDetectionGroupSB.SetFont(LinedFont)
        CycleSlipDetectionGroupSB.SetForegroundColour(LinedFontColour)                  
        CycleSlipDetectionGroupSizer = wx.StaticBoxSizer(CycleSlipDetectionGroupSB, wx.VERTICAL)
        CycleSlipDetectionGroupSizer.Add(CycleSlipDetectionPanelCore)
        CycleSlipDetectionGroupSizer.SetMinSize(wx.Size(XCycleSlipDetectionPanel, YCycleSlipDetectionPanel))
        CycleSlipDetectionGroupBox.Add(CycleSlipDetectionGroupSizer)

        # Buttons bindings
        self.LIConfigButton.Bind(wx.EVT_BUTTON, self.ShowCycleSlipLI, id=self.LIConfigButton.GetId())
        self.MelbourneWubbenaConfigButton.Bind(wx.EVT_BUTTON, self.ShowCycleSlipMelbourneWubbena, id=self.MelbourneWubbenaConfigButton.GetId())
        self.L1C1DifferenceConfigButton.Bind(wx.EVT_BUTTON, self.ShowC1L1Diference, id=self.L1C1DifferenceConfigButton.GetId())




     #Station Data:
        
        StationDataPanel = wx.Panel(self.PreProcesPanel, -1)
        StationDataPanel.SetSize(wx.Size(0.522 * XPreProcesPanelSize, 0.13 * YPreProcesPanelSize))
        XStationDataSize, YStationDataSize = StationDataPanel.GetSize() 
        
           
        StationDataPanelCore = wx.Panel(StationDataPanel, -1)
        StationDataPanelCore.SetSize(wx.Size(0.95 * XStationDataSize, 0.50 * YStationDataSize))
        XStationDataSizeCore, YStationDataSizeCore = StationDataPanelCore.GetSize() 
        
        self.DataDecimationCB = wx.CheckBox(StationDataPanelCore, -1 , 'Data Decimation [s]  ')
        self.DataDecimation = wx.TextCtrl(StationDataPanelCore, -1, '300', size=wx.Size(0.15 * XStationDataSizeCore, 0.7 * YStationDataSizeCore))                   

        self.Bind(wx.EVT_CHECKBOX, self.ResetDataDecimationText, id=self.DataDecimationCB.GetId())
        self.Bind(wx.EVT_TEXT, self.AllowDataDecimation, id=self.DataDecimation.GetId())

        self.DataDecimationCB.SetValue(True)
        
        StationDataGrid = wx.GridBagSizer()
        StationDataGrid.Add(self.DataDecimationCB , (0, 0), (1, 1) )
        StationDataGrid.Add(self.DataDecimation ,  (0, 1), (1, 1) )
        StationDataGrid.SetMinSize(wx.Size(0.95 * XStationDataSizeCore, 0.95 * YStationDataSizeCore))
        StationDataPanelCore.SetSizer(StationDataGrid)        
           
        #Station Data Group
        StationDataGroupBox = wx.BoxSizer(wx.VERTICAL)
        StationDataPanel.SetSizer(StationDataGroupBox)

        StationDataGroupSB = wx.StaticBox(parent=StationDataPanel, label="Station Data")
        StationDataGroupSB.SetFont(LinedFont)
        StationDataGroupSB.SetForegroundColour(LinedFontColour)
        StationDataGroupSizer = wx.StaticBoxSizer(StationDataGroupSB, wx.VERTICAL)
        StationDataGroupSizer.Add(StationDataPanelCore)
        
        StationDataGroupSizer.SetMinSize(wx.Size(XStationDataSizeCore, YStationDataSizeCore))
        StationDataGroupBox.Add(StationDataGroupSizer)
        
     #GNSS Satellite Selection:
        
        GNSSSatelliteSelectionPanel = wx.Panel(self.PreProcesPanel, -1)
        GNSSSatelliteSelectionPanel.SetSize(wx.Size(0.48 * SecondaryFrameXSize, 0.6 * SecondaryFrameYSize))
        XGNSSSatelliteSelection, YGNSSSatelliteSelection = GNSSSatelliteSelectionPanel.GetSize() 
       
           
        GNSSSatelliteSelectionPanelCore = wx.Panel(GNSSSatelliteSelectionPanel, -1)
        GNSSSatelliteSelectionPanelCore.SetSize(wx.Size(0.99 * XGNSSSatelliteSelection, 0.8 * YGNSSSatelliteSelection))
        XGNSSSatelliteSelectionCore, YGNSSSatelliteSelectionCore = GNSSSatelliteSelectionPanelCore.GetSize() 
        
              
        GNSSSatelliteSelectionRBPanel = wx.Panel(GNSSSatelliteSelectionPanelCore, -1)
        self.EnableGPSRB = wx.RadioButton(GNSSSatelliteSelectionRBPanel, -1, 'GPS', style=wx.RB_GROUP)
        self.EnableGalileoRB = wx.RadioButton(GNSSSatelliteSelectionRBPanel, -1, 'Galileo')
        self.EnableGalileoRB.Enable(False)
        self.EnableGlonassRB = wx.RadioButton(GNSSSatelliteSelectionRBPanel, -1, 'GLONASS')
        self.EnableGlonassRB.Enable(False)
        
        GNSSSatelliteRadButtonBox = wx.BoxSizer(wx.HORIZONTAL)
        GNSSSatelliteRadButtonBox.Add(self.EnableGPSRB, flag=wx.ALIGN_CENTER)
        GNSSSatelliteRadButtonBox.Add(self.EnableGalileoRB, flag=wx.ALIGN_CENTER)
        GNSSSatelliteRadButtonBox.Add(self.EnableGlonassRB, flag=wx.ALIGN_CENTER)
        GNSSSatelliteSelectionRBPanel.SetSizer(GNSSSatelliteRadButtonBox)
        GNSSSatelliteRadButtonBox.SetMinSize(wx.Size(XGNSSSatelliteSelectionCore, 0.05 * YGNSSSatelliteSelectionCore)) 

        GNSSSatelliteEnableSelectionPanel = wx.Panel(GNSSSatelliteSelectionPanelCore, -1)
        self.ButtonAllEnabled = wx.Button(GNSSSatelliteEnableSelectionPanel, 1, label='All ')
        self.ButtonAllDisabled = wx.Button(GNSSSatelliteEnableSelectionPanel, 1, label='None')


        self.ButtonAllEnabled.Bind(wx.EVT_BUTTON, self.ShowPRNButtons, id=self.ButtonAllEnabled.GetId())      
        self.ButtonAllDisabled.Bind(wx.EVT_BUTTON, self.HidePRNButtons, id=self.ButtonAllDisabled.GetId())     
        
        GNSSSatelliteEnableSelectionBox = wx.BoxSizer(wx.HORIZONTAL)
        GNSSSatelliteEnableSelectionBox.Add(self.ButtonAllEnabled, flag=wx.ALIGN_CENTER)
        GNSSSatelliteEnableSelectionBox.Add(self.ButtonAllDisabled, flag=wx.ALIGN_CENTER)
        GNSSSatelliteEnableSelectionPanel.SetSizer(GNSSSatelliteEnableSelectionBox)
        GNSSSatelliteEnableSelectionBox.SetMinSize(wx.Size(0.95 * XGNSSSatelliteSelectionCore, 0.05 * YGNSSSatelliteSelectionCore)) 
        SpecifyGNSSSatellitePanel = wx.Panel(GNSSSatelliteSelectionPanelCore, -1)
        SpecifyGNSSSatellitePanel.SetSize(wx.Size(0.95 * XGNSSSatelliteSelectionCore, 0.3 * YGNSSSatelliteSelectionCore))
      
        self.ButtonPRN = []
        self.SpecifyGNSSSatelliteGrid = wx.GridBagSizer(wx.BU_EXACTFIT)
        
        for i in range(32):
            self.ButtonPRN.append(wx.lib.buttons.GenToggleButton(SpecifyGNSSSatellitePanel, 1, label='PRN ' + str(i + 1))) 
            self.ButtonPRN[i].SetValue(True)
            self.ButtonPRN[i].SetBackgroundColour((120, 255, 044))
            #self.SpecifyGNSSSatelliteGrid.Add(self.ButtonPRN[i] , ((i % 8) + 1, i / 8), (1, 1), wx.LEFT, 10)
            self.SpecifyGNSSSatelliteGrid.Add(self.ButtonPRN[i] , ((i % 7) + 1, i / 7), (1, 1), wx.LEFT, 7)
            self.ButtonPRN[i].Bind(wx.EVT_BUTTON, self.SetPRNButtonColor)
            
        self.SpecifyGNSSSatelliteGrid.SetMinSize(SpecifyGNSSSatellitePanel.GetSize()) 
        SpecifyGNSSSatellitePanel.SetSizer(self.SpecifyGNSSSatelliteGrid)
                  
        GNSSSatelliteSelectionPanelCoreGroupBox = wx.GridBagSizer()
        GNSSSatelliteSelectionPanelCoreGroupBox.Add(GNSSSatelliteSelectionRBPanel, (0, 0), (1, 1), wx.TOP | wx.RIGHT, 10 )
        GNSSSatelliteSelectionPanelCoreGroupBox.Add(GNSSSatelliteEnableSelectionPanel, (1, 0), (1, 1), wx.TOP | wx.RIGHT, 10 )
        #GNSSSatelliteSelectionPanelCoreGroupBox.Add(SpecifyGNSSSatellitePanel, (2, 0), (1, 1), wx.RIGHT | wx.BOTTOM , 15)
        GNSSSatelliteSelectionPanelCoreGroupBox.Add(SpecifyGNSSSatellitePanel, (2, 0), (1, 1), wx.BU_EXACTFIT)
        GNSSSatelliteSelectionPanelCore.SetSizer(GNSSSatelliteSelectionPanelCoreGroupBox)
        #GNSSSatelliteSelectionPanelCoreGroupBox.SetMinSize(wx.Size(0.95 * XCycleSlipDetectionCore, 0.5 * YCycleSlipDetectionCore)) 
              
         
        #Cycle-Slip Detection Group
        GNSSSatelliteSelectionGroupBox = wx.BoxSizer(wx.VERTICAL)
        GNSSSatelliteSelectionPanel.SetSizer(GNSSSatelliteSelectionGroupBox)
        GNSSSatelliteSelectionGroupSB = wx.StaticBox(parent=GNSSSatelliteSelectionPanel, label="GNSS Satellite Selection")
        GNSSSatelliteSelectionGroupSB.SetFont(LinedFont)
        GNSSSatelliteSelectionGroupSB.SetForegroundColour(LinedFontColour)             
        GNSSSatelliteSelectionGroupSizer = wx.StaticBoxSizer(GNSSSatelliteSelectionGroupSB, wx.VERTICAL)
        GNSSSatelliteSelectionGroupSizer.Add(GNSSSatelliteSelectionPanelCore)
        GNSSSatelliteSelectionGroupSizer.SetMinSize(GNSSSatelliteSelectionPanel.GetSize())
        GNSSSatelliteSelectionGroupBox.Add(GNSSSatelliteSelectionGroupSizer)        
        
      
        
        self.PreProcesPanelGrill = wx.GridBagSizer()
        self.PreProcesPanelGrill.Add(ElevationMaskPanel, (0, 0), (1, 1), wx.ALL , 20)
        self.PreProcesPanelGrill.Add(CycleSlipDetectionPanel, (1, 0), (1, 1),  wx.RIGHT | wx.LEFT, 20)
        self.PreProcesPanelGrill.Add(StationDataPanel, (0, 1), (1, 1), wx.TOP, 20)
        self.PreProcesPanelGrill.Add(GNSSSatelliteSelectionPanel, (1, 1), (8, 1), wx.TOP, -55)
        #self.PreProcesPanelGrill.Add(GNSSSatelliteSelectionPanel, (1, 1), (8, 1), wx.RIGHT | wx.LEFT, 2 | 15)
        
        self.PreProcesPanel.SetSizer(self.PreProcesPanelGrill)
        
        """
................................................................................................................................................        
                MODELLING PANEL
................................................................................................................................................                  
        """

    #Create Modelling Options Panel elements   
        self.ModelPanel = wx.Panel(self.SecondaryFrame, -1, size=self.SecondaryFrame.GetSize())
        XModesPanel, YModesPanel = self.ModelPanel.GetSize()
        
        #Modelling Options Group
        self.ModellingOptionsPanel = wx.Panel(self.ModelPanel, -1)
        self.ModellingOptionsPanel.SetSize(wx.Size(0.45 * XModesPanel, 0.8 * YModesPanel))
        XModellingOptionsPanel, YModellingOptionsPanel = self.ModellingOptionsPanel.GetSize()
        
        ModellingOptionsPanelCore = wx.Panel(self.ModellingOptionsPanel, -1)  
        ModellingOptionsPanelCore.SetSize(wx.Size(0.95 * XModellingOptionsPanel, 0.5 * YModellingOptionsPanel))
        XModellingOptionsPanelCore, YModellingOptionsPanelCore = ModellingOptionsPanelCore.GetSize()
        
        self.SatelliteClockOffsetCorrection = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'Satellite clock offset correction')
        self.SatelliteMovementFlightTime = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'Consider satellite movement during signal flight time')
        self.EarthRotationFlightTime = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'Consider Earth rotation during signal flight time')
        self.SatellitePhaseCenterCorrection = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'Satellite mass centre to antenna phase centre correction')
        self.ReceiverPhaseCenterCorrection = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'Receiver antenna phase centre correction')
        self.ReceiverRefPointCorrection = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'Receiver antenna reference point correction')
        self.RelativisticCorrection = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'Relativistic clock correction (orbit eccentricity)')
        self.IonosphericCorrectionCheckBox = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'Ionospheric correction ')
        self.TroposphericCorrectionCheckBox = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'Tropospheric correction ')
        self.P1P2CorrectionCheckBox = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'P1 - P2 correction ')
        self.P1C1CorrectionCheckBox = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'P1 - C1 correction ')
        self.WindUpCorrection = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'Wind up correction (Carrier phase only)')
        self.SolidTidesCorrection = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'Solid tides correction')
        self.GravitationalDelayCorrection = wx.CheckBox(ModellingOptionsPanelCore, -1 , 'Relativistic path range correction') 
        
        NominalTroposphericCorrection = ['UNB-3 Nominal', 'Simple Nominal']
        MappingTroposphericCorrection = ['Niell Mapping', 'Simple Mapping']
        IonosphericCorrection = ['Klobuchar (GPS)', 'Klobuchar (BeiDou)', 'IONEX', 'FPPP']
        P1P2Correction = ['RINEX Nav File', 'DCB File','IONEX File','FPPP File']
        P1C1Correction = ['Flexible', 'Strict']
	XCorrectionComboBoxes , YCorrectionComboBoxes = 150 , 25
        self.NominalTroposphericCorrectionComboBox = wx.ComboBox(ModellingOptionsPanelCore, -1, value=NominalTroposphericCorrection[0], choices=NominalTroposphericCorrection, style=wx.CB_READONLY, size=wx.Size(1.0 * XCorrectionComboBoxes , YCorrectionComboBoxes)) 
        self.MappingTroposphericCorrectionComboBox = wx.ComboBox(ModellingOptionsPanelCore, -1, value=MappingTroposphericCorrection[0], choices=MappingTroposphericCorrection, style=wx.CB_READONLY, size=wx.Size(1.0 * XCorrectionComboBoxes , YCorrectionComboBoxes))    
        self.IonosphericCorrectionComboBox = wx.ComboBox(ModellingOptionsPanelCore, -1, value=IonosphericCorrection[0], choices=IonosphericCorrection, style=wx.CB_READONLY, size=wx.Size(XCorrectionComboBoxes , YCorrectionComboBoxes))
       #self.IonosphericMaxRMSText = wx.lib.stattext.GenStaticText(ModellingOptionsPanelCore, -1, ' Max RMS [m] : ')
       #self.IonosphericMaxRMSText.SetMaxSize = ( wx.Size(0.5 * XCorrectionComboBoxes, YCorrectionComboBoxes)) 
       #self.IonosphericMaxRMSTextCtrl = wx.TextCtrl(ModellingOptionsPanelCore, -1,value='1', size=wx.Size(0.21 * XCorrectionComboBoxes, YCorrectionComboBoxes))

        self.P1P2CorrectionComboBox = wx.ComboBox(ModellingOptionsPanelCore, -1, value=P1P2Correction[0], choices=P1P2Correction, style=wx.CB_READONLY, size=wx.Size(XCorrectionComboBoxes , YCorrectionComboBoxes))      
        self.P1C1CorrectionComboBox = wx.ComboBox(ModellingOptionsPanelCore, -1, value=P1C1Correction[0], choices=P1C1Correction, style=wx.CB_READONLY, size=wx.Size(XCorrectionComboBoxes , YCorrectionComboBoxes))      

	self.IonosphericCorrectionCheckBox.Bind(wx.EVT_CHECKBOX, self.ShowIonosphericCorrectionCheckBox)
	self.IonosphericCorrectionComboBox.Bind(wx.EVT_COMBOBOX, self.SelectIonosphericCorrections)
        self.TroposphericCorrectionCheckBox.Bind(wx.EVT_CHECKBOX, self.ShowTroposphericCorrectionCheckBox)
        self.NominalTroposphericCorrectionComboBox.Bind(wx.EVT_COMBOBOX, self.SelectTroposphericMapping)
        self.P1P2CorrectionCheckBox.Bind(wx.EVT_CHECKBOX, self.ShowP1P2CorrectionCheckBox)
        self.P1P2CorrectionComboBox.Bind(wx.EVT_COMBOBOX, self.SelectP1P2Corrections)

        self.P1C1CorrectionCheckBox.Bind(wx.EVT_CHECKBOX, self.ShowP1C1CorrectionCheckBox)
        self.P1C1CorrectionComboBox.Bind(wx.EVT_COMBOBOX, self.ShowP1C1DueToModel)        

	self.ReceiverRefPointCorrection.Bind(wx.EVT_CHECKBOX, self.ShowReceiverRefPointCorrectionPanel)
        self.ReceiverPhaseCenterCorrection.Bind(wx.EVT_CHECKBOX, self.ShowReceiverPhaseCenterCorrectionPanel)        

        self.SatelliteClockOffsetCorrection.SetValue(True)
        self.SatelliteMovementFlightTime.SetValue(True)
        self.EarthRotationFlightTime.SetValue(True)
        self.SatellitePhaseCenterCorrection.SetValue(True)
        self.ReceiverPhaseCenterCorrection.SetValue(True)
        self.ReceiverRefPointCorrection.SetValue(False)
        self.RelativisticCorrection.SetValue(True)
        self.TroposphericCorrectionCheckBox.SetValue(True)

        self.P1P2CorrectionCheckBox.SetValue(True)
        self.WindUpCorrection.SetValue(True)
        self.SolidTidesCorrection.SetValue(True)
        self.GravitationalDelayCorrection.SetValue(True)
        
        VerticalAdjustment  = -4
        VerticalAdjustmentL = VerticalAdjustment + 1 
        ModellingOptionsPanelCoreBox = wx.GridBagSizer(wx.VERTICAL)
        ModellingOptionsPanelCoreBox.Add(self.SatelliteClockOffsetCorrection, (0, 0), (1, 2))
        ModellingOptionsPanelCoreBox.Add(self.SatelliteMovementFlightTime, (1, 0), (1, 2), wx.TOP , VerticalAdjustment)
        ModellingOptionsPanelCoreBox.Add(self.EarthRotationFlightTime, (2, 0), (1, 2), wx.TOP , VerticalAdjustment)
        ModellingOptionsPanelCoreBox.Add(self.SatellitePhaseCenterCorrection, (3, 0), (1, 2), wx.TOP , VerticalAdjustment)
        ModellingOptionsPanelCoreBox.Add(self.ReceiverPhaseCenterCorrection, (4, 0), (1, 2), wx.TOP , VerticalAdjustment)
        ModellingOptionsPanelCoreBox.Add(self.ReceiverRefPointCorrection, (5, 0), (1, 2), wx.TOP , VerticalAdjustment)
        ModellingOptionsPanelCoreBox.Add(self.RelativisticCorrection, (6, 0), (1, 2), wx.TOP , VerticalAdjustment)
        ModellingOptionsPanelCoreBox.Add(self.IonosphericCorrectionCheckBox, (7, 0), (1, 1), wx.ALIGN_CENTER_VERTICAL | wx.TOP , VerticalAdjustmentL )
	IonosphericSizerGroupBox = wx.BoxSizer(wx.HORIZONTAL)
	IonosphericSizerGroupBox.Add(self.IonosphericCorrectionComboBox, wx.LEFT , 10 )
	#IonosphericSizerGroupBox.Add(self.IonosphericMaxRMSText, wx.ALIGN_LEFT , 5 )
	#IonosphericSizerGroupBox.Add(self.IonosphericMaxRMSTextCtrl, wx.ALIGN_LEFT , 5 )
	ModellingOptionsPanelCoreBox.Add(IonosphericSizerGroupBox, (7, 1), (1, 1), wx.TOP , VerticalAdjustmentL)
        #ModellingOptionsPanelCoreBox.Add(self.IonosphericCorrectionComboBox, (7, 1), (1, 2), wx.TOP , VerticalAdjustmentL)
        ModellingOptionsPanelCoreBox.Add(self.TroposphericCorrectionCheckBox, (8, 0), (1, 1), wx.ALIGN_CENTER_VERTICAL| wx.TOP , VerticalAdjustmentL )
        TroposhericSizerGroupBox = wx.BoxSizer(wx.HORIZONTAL)
        TroposhericSizerGroupBox.Add(self.NominalTroposphericCorrectionComboBox, wx.LEFT , 10 )
        TroposhericSizerGroupBox.Add(self.MappingTroposphericCorrectionComboBox, wx.LEFT , 10 )
        ModellingOptionsPanelCoreBox.Add(TroposhericSizerGroupBox, (8, 1), (1, 1), wx.TOP , VerticalAdjustmentL)
#        ModellingOptionsPanelCoreBox.Add(self.MappingTroposphericCorrectionComboBox, (8, 1), (1, 1), wx.TOP , VerticalAdjustment)
#        ModellingOptionsPanelCoreBox.Add(self.NominalTroposphericCorrectionComboBox, (8, 2), (1, 1), wx.TOP , VerticalAdjustment)
        ModellingOptionsPanelCoreBox.Add(self.P1P2CorrectionCheckBox, (9, 0), (1, 1), wx.ALIGN_CENTER_VERTICAL| wx.TOP , VerticalAdjustmentL )
        ModellingOptionsPanelCoreBox.Add(self.P1P2CorrectionComboBox, (9, 1), (1, 2), wx.TOP , VerticalAdjustmentL)
        ModellingOptionsPanelCoreBox.Add(self.P1C1CorrectionCheckBox, (10, 0), (1, 1), wx.ALIGN_CENTER_VERTICAL| wx.TOP , VerticalAdjustmentL )
        ModellingOptionsPanelCoreBox.Add(self.P1C1CorrectionComboBox, (10, 1), (1, 2), wx.TOP , VerticalAdjustmentL)
        ModellingOptionsPanelCoreBox.Add(self.WindUpCorrection, (11, 0), (1, 2), wx.TOP , VerticalAdjustment)
        ModellingOptionsPanelCoreBox.Add(self.SolidTidesCorrection, (12, 0), (1, 2), wx.TOP , VerticalAdjustment)
        ModellingOptionsPanelCoreBox.Add(self.GravitationalDelayCorrection, (13, 0), (1, 2), wx.TOP , VerticalAdjustment)

        ModellingOptionsPanelCoreBox.SetMinSize(wx.Size(0.95 * XModellingOptionsPanelCore, 0.9 * YModellingOptionsPanelCore))
        ModellingOptionsPanelCore.SetSizer(ModellingOptionsPanelCoreBox)
        
        
        #Modelling Options Group
        ModellingOptionsPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        self.ModellingOptionsPanel.SetSizer(ModellingOptionsPanelGroupBox)
        ModellingOptionsPanelGroupSB = wx.StaticBox(parent=self.ModellingOptionsPanel, label="Modelling Options")
        ModellingOptionsPanelGroupSB.SetFont(LinedFont)
        ModellingOptionsPanelGroupSB.SetForegroundColour(LinedFontColour)             
        ModellingOptionsPanelGroupSizer = wx.StaticBoxSizer(ModellingOptionsPanelGroupSB, wx.VERTICAL)
        ModellingOptionsPanelGroupSizer.Add(ModellingOptionsPanelCore, wx.LEFT | wx.RIGHT, 5)
        ModellingOptionsPanelGroupSizer.SetMinSize(wx.Size(0.75 * XModellingOptionsPanel, 0.5 * YModellingOptionsPanel))
        ModellingOptionsPanelGroupBox.Add(ModellingOptionsPanelGroupSizer)
        
      #PreciseProductsDI Panel
        self.PreciseProductsDIPanel = wx.Panel(self.ModelPanel, -1, size=wx.Size(0.4 * XModesPanel, 0.20 * YModesPanel))
        XPreciseProductsDIPanel, YPreciseProductsDIPanel = self.PreciseProductsDIPanel.GetSize()
        
        #PreciseProductsDI Panel Core
        PreciseProductsDIPanelCore = wx.Panel(self.PreciseProductsDIPanel, -1, size=wx.Size(0.95 * XPreciseProductsDIPanel, 0.95 * YPreciseProductsDIPanel))
        XPreciseProductsDIPanelCore, YPreciseProductsDIPanelCore = PreciseProductsDIPanelCore.GetSize()

        self.OrbitInterpolationDegreeText = wx.lib.stattext.GenStaticText(PreciseProductsDIPanelCore, -1, 'Orbit Interpolation Degree : ')
        self.OrbitInterpolationDegreeTextCtrl = wx.TextCtrl(PreciseProductsDIPanelCore, -1, value='9', size=wx.Size(0.2 * XPreciseProductsDIPanelCore, 0.35 * YPreciseProductsDIPanelCore), style=wx.TE_PROCESS_ENTER)
        
        self.ClockInterpolationDegreeText = wx.lib.stattext.GenStaticText(PreciseProductsDIPanelCore, -1, 'Clock Interpolation Degree : ')
        self.ClockInterpolationDegreeTextCtrl = wx.TextCtrl(PreciseProductsDIPanelCore, -1, value='0', size=wx.Size(0.2 * XPreciseProductsDIPanelCore, 0.35 * YPreciseProductsDIPanelCore), style=wx.TE_PROCESS_ENTER)
        
        PreciseProductsDIPanelCoreBox = wx.GridBagSizer(wx.VERTICAL)
        PreciseProductsDIPanelCoreBox.Add(self.OrbitInterpolationDegreeText, (0, 0), (1, 1), wx.ALIGN_CENTER | wx.TOP, 10)
        PreciseProductsDIPanelCoreBox.Add(self.OrbitInterpolationDegreeTextCtrl, (0, 1), (1, 1), wx.TOP, 10)
        PreciseProductsDIPanelCoreBox.Add(self.ClockInterpolationDegreeText, (1, 0), (1, 1), wx.ALIGN_CENTER)
        PreciseProductsDIPanelCoreBox.Add(self.ClockInterpolationDegreeTextCtrl, (1, 1), (1, 1), wx.TOP)

        PreciseProductsDIPanelCore.SetSizer(PreciseProductsDIPanelCoreBox)
        PreciseProductsDIPanelCoreBox.SetMinSize(size=wx.Size(0.95 * XPreciseProductsDIPanelCore, 0.95 * YPreciseProductsDIPanelCore))

        #PreciseProductsDI Panel Group 
        PreciseProductsDIPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        self.PreciseProductsDIPanel.SetSizer(PreciseProductsDIPanelGroupBox)
        PreciseProductsDIPanelGroupSB = wx.StaticBox(parent=self.PreciseProductsDIPanel, label="Precise Products Data Interpolation")
        PreciseProductsDIPanelGroupSB.SetFont(LinedFont)
        PreciseProductsDIPanelGroupSB.SetForegroundColour(LinedFontColour)                 
        PreciseProductsDIPanelGroupSizer = wx.StaticBoxSizer(PreciseProductsDIPanelGroupSB, wx.VERTICAL)
        PreciseProductsDIPanelGroupSizer.Add(PreciseProductsDIPanelCore, wx.LEFT | wx.RIGHT, 5)
        PreciseProductsDIPanelGroupSizer.SetMinSize(wx.Size(0.95 * XPreciseProductsDIPanel, 0.95 * YPreciseProductsDIPanel))
        PreciseProductsDIPanelGroupBox.Add(PreciseProductsDIPanelGroupSizer)

      #AntennaPhaseCenter Panel 
        self.AntennaPhaseCenterPanel = wx.Panel(self.ModelPanel, -1, size=wx.Size(0.4 * XModesPanel, 0.32 * YModesPanel))
        XAntennaPhaseCenterPanel, YAntennaPhaseCenterPanel = self.AntennaPhaseCenterPanel.GetSize()
        
        #AntennaPhaseCenter Panel Core
        AntennaPhaseCenterPanelCore = wx.Panel(self.AntennaPhaseCenterPanel, -1, size=wx.Size(0.95 * XAntennaPhaseCenterPanel, 0.95 * YAntennaPhaseCenterPanel))
        XAntennaPhaseCenterPanelCore, YAntennaPhaseCenterPanelCore = AntennaPhaseCenterPanelCore.GetSize()
        
        self.ReadAntennaPhaseCenterRB = wx.RadioButton(AntennaPhaseCenterPanelCore, -1, 'Read from ANTEX', style=wx.RB_GROUP)
        self.SpecifyAntennaPhaseCenterRB = wx.RadioButton(AntennaPhaseCenterPanelCore, -1, 'Specify')
        
 
        SpecifyReadAntennaPhaseCenterBox = wx.BoxSizer(wx.HORIZONTAL)
        SpecifyReadAntennaPhaseCenterBox.Add(self.SpecifyAntennaPhaseCenterRB)
        SpecifyReadAntennaPhaseCenterBox.Add(self.ReadAntennaPhaseCenterRB)
        
        self.SpecifyAntennaPhaseCenterRB.Bind(wx.EVT_RADIOBUTTON, self.ShowNorthEastUpPanel)
        self.ReadAntennaPhaseCenterRB.Bind(wx.EVT_RADIOBUTTON, self.ShowNorthEastUpPanel)
        
        self.NorthEastUpPanel = wx.Panel(AntennaPhaseCenterPanelCore, -1, size=wx.Size(0.95 * XAntennaPhaseCenterPanelCore, 0.7 * YAntennaPhaseCenterPanelCore))
        XNorthEastUpPanel, YNorthEastUpPanel = self.NorthEastUpPanel.GetSize()
        
        self.F1AntennaPhaseCenterRB = wx.RadioButton(self.NorthEastUpPanel, -1, 'F1')
        self.F2AntennaPhaseCenterRB = wx.RadioButton(self.NorthEastUpPanel, -1, 'F2')
        
        self.F1AntennaPhaseCenterRB.Bind(wx.EVT_RADIOBUTTON, self.StoreUpdateAntennaPhaseCenter)
        self.F2AntennaPhaseCenterRB.Bind(wx.EVT_RADIOBUTTON, self.StoreUpdateAntennaPhaseCenter)      
        
        
        self.NorthPhaseCenterText = wx.lib.stattext.GenStaticText(self.NorthEastUpPanel, -1, 'North [m] : ')
        self.NorthPhaseCenterTextCtrl = wx.TextCtrl(self.NorthEastUpPanel, -1, size=wx.Size(0.2 * XNorthEastUpPanel, 0.25 * YNorthEastUpPanel), style=wx.TE_PROCESS_ENTER)
        
        
        self.EastPhaseCenterText = wx.lib.stattext.GenStaticText(self.NorthEastUpPanel, -1, 'East [m] : ')
        self.EastPhaseCenterTextCtrl = wx.TextCtrl(self.NorthEastUpPanel, -1, size=wx.Size(0.2 * XNorthEastUpPanel, 0.25 * YNorthEastUpPanel), style=wx.TE_PROCESS_ENTER)
        

        self.UpPhaseCenterText = wx.lib.stattext.GenStaticText(self.NorthEastUpPanel, -1, 'Up [m] : ')
        self.UpPhaseCenterTextCtrl = wx.TextCtrl(self.NorthEastUpPanel, -1, size=wx.Size(0.2 * XNorthEastUpPanel, 0.25 * YNorthEastUpPanel), style=wx.TE_PROCESS_ENTER)
        

        self.NorthPhaseCenterTextCtrl.Bind(wx.EVT_TEXT, self.SaveNorthPhaseCenterByTextCtrl)
        self.EastPhaseCenterTextCtrl.Bind(wx.EVT_TEXT, self.SaveEastPhaseCenterByTextCtrl)
        self.UpPhaseCenterTextCtrl.Bind(wx.EVT_TEXT, self.SaveUpPhaseCenterByTextCtrl)

        self.NorthPhaseCenterF1 = '0'
        self.EastPhaseCenterF1 = '0'
        self.UpPhaseCenterF1 = '0'
        
        self.NorthPhaseCenterF2 = '0'
        self.EastPhaseCenterF2 = '0'
        self.UpPhaseCenterF2 = '0'
                
        self.F1AntennaPhaseCenterRB.SetValue(True)
        self.NorthPhaseCenterTextCtrl.Clear()
        self.EastPhaseCenterTextCtrl.Clear()
        self.UpPhaseCenterTextCtrl.Clear()

        self.NorthPhaseCenterTextCtrl.WriteText('0')
        self.EastPhaseCenterTextCtrl.WriteText('0')
        self.UpPhaseCenterTextCtrl.WriteText('0')

        
        NorthEastUpPanelGrid = wx.GridBagSizer()
        NorthEastUpPanelGrid.Add(self.F1AntennaPhaseCenterRB, (0, 0), (1, 1), wx.ALIGN_CENTER | wx.TOP, 5)
        NorthEastUpPanelGrid.Add(self.F2AntennaPhaseCenterRB, (0, 1), (1, 1), wx.TOP, 5)
        NorthEastUpPanelGrid.Add(self.NorthPhaseCenterText, (1, 0), (1, 1), wx.ALIGN_CENTER_VERTICAL| wx.TOP, 5)
        NorthEastUpPanelGrid.Add(self.NorthPhaseCenterTextCtrl, (1, 1), (1, 1), wx.TOP, 5)
        NorthEastUpPanelGrid.Add(self.EastPhaseCenterText, (2, 0), (1, 1), wx.ALIGN_CENTER_VERTICAL| wx.TOP, 5)
        NorthEastUpPanelGrid.Add(self.EastPhaseCenterTextCtrl, (2, 1), (1, 1), wx.TOP, 5)
        NorthEastUpPanelGrid.Add(self.UpPhaseCenterText, (3, 0), (1, 1), wx.ALIGN_CENTER_VERTICAL| wx.TOP, 5)
        NorthEastUpPanelGrid.Add(self.UpPhaseCenterTextCtrl, (3, 1), (1, 1), wx.TOP, 5)           
        

        self.NorthEastUpPanel.SetSizer(NorthEastUpPanelGrid)
        NorthEastUpPanelGrid.SetMinSize(size=wx.Size(0.95 * XNorthEastUpPanel, 0.950 * YNorthEastUpPanel))
        
        #AntennaPhaseCenter Panel Core Layout
        
        AntennaPhaseCenterPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        AntennaPhaseCenterPanelGroupBox.Add(SpecifyReadAntennaPhaseCenterBox)
        AntennaPhaseCenterPanelGroupBox.Add(self.NorthEastUpPanel)
        AntennaPhaseCenterPanelCore.SetSizer(AntennaPhaseCenterPanelGroupBox)
        AntennaPhaseCenterPanelGroupBox.SetMinSize(size=wx.Size(0.95 * XAntennaPhaseCenterPanelCore, 0.950 * YAntennaPhaseCenterPanelCore))
        
        
        #AntennaPhaseCenter Panel Group 
        AntennaPhaseCenterPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        self.AntennaPhaseCenterPanel.SetSizer(AntennaPhaseCenterPanelGroupBox)
        AntennaPhaseCenterPanelGroupSB = wx.StaticBox(parent=self.AntennaPhaseCenterPanel, label="Receiver Antenna Phase Centre Correction")
        AntennaPhaseCenterPanelGroupSB.SetFont(LinedFont)
        AntennaPhaseCenterPanelGroupSB.SetForegroundColour(LinedFontColour)         
        AntennaPhaseCenterPanelGroupSizer = wx.StaticBoxSizer(AntennaPhaseCenterPanelGroupSB, wx.VERTICAL)
        AntennaPhaseCenterPanelGroupSizer.Add(AntennaPhaseCenterPanelCore, wx.ALL, 5)
        AntennaPhaseCenterPanelGroupSizer.SetMinSize(wx.Size(0.95 * XAntennaPhaseCenterPanel, 0.95 * YAntennaPhaseCenterPanel))
        AntennaPhaseCenterPanelGroupBox.Add(AntennaPhaseCenterPanelGroupSizer)
        
      #Antenna Reference Point Panel
        self.AntennaReferencePointPanel = wx.Panel(self.ModelPanel, -1, size=wx.Size(0.4 * XModesPanel, 0.28 * YModesPanel))
        XAntennaReferencePointPanel, YAntennaReferencePointPanel = self.AntennaReferencePointPanel.GetSize()
        
        #AntennaReferencePoint Panel Core
        AntennaReferencePointPanelCore = wx.Panel(self.AntennaReferencePointPanel, -1, size=wx.Size(0.95 * XAntennaReferencePointPanel, 0.95 * YAntennaReferencePointPanel))
        XAntennaReferencePointPanelCore, YAntennaReferencePointPanelCore = AntennaReferencePointPanelCore.GetSize()


        self.SpecifyAntennaReferencePointRB = wx.RadioButton(AntennaReferencePointPanelCore, -1, 'Specify', style=wx.RB_GROUP)
        self.ReadAntennaReferencePointRB = wx.RadioButton(AntennaReferencePointPanelCore, -1, 'Read from RINEX')
 
        SpecifyReadAntennaReferencePointBox = wx.BoxSizer(wx.HORIZONTAL)
        SpecifyReadAntennaReferencePointBox.Add(self.SpecifyAntennaReferencePointRB)
        SpecifyReadAntennaReferencePointBox.Add(self.ReadAntennaReferencePointRB)
        
        self.SpecifyAntennaReferencePointRB.Bind(wx.EVT_RADIOBUTTON, self.ShowNorthEastUpReferencePointPanel)
        self.ReadAntennaReferencePointRB.Bind(wx.EVT_RADIOBUTTON, self.ShowNorthEastUpReferencePointPanel)
        
        self.NorthEastUpReferencePointPanel = wx.Panel(AntennaReferencePointPanelCore, -1, size=wx.Size(0.95 * XAntennaReferencePointPanelCore, 0.75 * YAntennaReferencePointPanelCore))
        XNorthEastUpReferencePointPanel, YNorthEastUpReferencePointPanel = self.NorthEastUpReferencePointPanel.GetSize()
        
        self.NorthReferencePointText = wx.lib.stattext.GenStaticText(self.NorthEastUpReferencePointPanel, -1, 'North [m] : ')
        self.NorthReferencePointTextCtrl = wx.TextCtrl(self.NorthEastUpReferencePointPanel, -1, value='0', size=wx.Size(0.2 * XNorthEastUpReferencePointPanel, 0.25 * YNorthEastUpReferencePointPanel), style=wx.TE_PROCESS_ENTER)
        
        self.EastReferencePointText = wx.lib.stattext.GenStaticText(self.NorthEastUpReferencePointPanel, -1, 'East [m] : ')
        self.EastReferencePointTextCtrl = wx.TextCtrl(self.NorthEastUpReferencePointPanel, -1, value='0', size=wx.Size(0.2 * XNorthEastUpReferencePointPanel, 0.25 * YNorthEastUpReferencePointPanel), style=wx.TE_PROCESS_ENTER)

        self.UpReferencePointText = wx.lib.stattext.GenStaticText(self.NorthEastUpReferencePointPanel, -1, 'Up [m] : ')
        self.UpReferencePointTextCtrl = wx.TextCtrl(self.NorthEastUpReferencePointPanel, -1, value='0', size=wx.Size(0.2 * XNorthEastUpReferencePointPanel, 0.25 * YNorthEastUpReferencePointPanel), style=wx.TE_PROCESS_ENTER)
        
        NorthEastUpReferencePointPanelGrid = wx.GridBagSizer()
        NorthEastUpReferencePointPanelGrid.Add(self.NorthReferencePointText, (0, 0), (1, 1), wx.ALIGN_CENTER_VERTICAL | wx.TOP, 5)
        NorthEastUpReferencePointPanelGrid.Add(self.NorthReferencePointTextCtrl, (0, 1), (1, 1),  wx.TOP, 5)
        NorthEastUpReferencePointPanelGrid.Add(self.EastReferencePointText, (1, 0), (1, 1), wx.ALIGN_CENTER_VERTICAL| wx.TOP, 5)
        NorthEastUpReferencePointPanelGrid.Add(self.EastReferencePointTextCtrl, (1, 1), (1, 1), wx.TOP, 5)
        NorthEastUpReferencePointPanelGrid.Add(self.UpReferencePointText, (2, 0), (1, 1), wx.ALIGN_CENTER_VERTICAL| wx.TOP, 5)
        NorthEastUpReferencePointPanelGrid.Add(self.UpReferencePointTextCtrl, (2, 1), (1, 1), wx.TOP, 5)           

        self.NorthEastUpReferencePointPanel.SetSizer(NorthEastUpReferencePointPanelGrid)
        NorthEastUpPanelGrid.SetMinSize(size=wx.Size(0.95 * XNorthEastUpReferencePointPanel, 0.950 * YNorthEastUpReferencePointPanel))
        
        #AntennaReferencePoint Panel Core Layout
        
        AntennaReferencePointPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        AntennaReferencePointPanelGroupBox.Add(SpecifyReadAntennaReferencePointBox)
        AntennaReferencePointPanelGroupBox.Add(self.NorthEastUpReferencePointPanel)
        AntennaReferencePointPanelCore.SetSizer(AntennaReferencePointPanelGroupBox)
        AntennaReferencePointPanelGroupBox.SetMinSize(size=wx.Size(0.95 * XAntennaReferencePointPanelCore, 0.550 * YAntennaReferencePointPanelCore))

        #AntennaReferencePoint Panel Group 
        AntennaReferencePointPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        self.AntennaReferencePointPanel.SetSizer(AntennaReferencePointPanelGroupBox)
        AntennaReferencePointPanelGroupSB = wx.StaticBox(parent=self.AntennaReferencePointPanel, label="Receiver Antenna Reference Point Correction")
        AntennaReferencePointPanelGroupSB.SetFont(LinedFont)
        AntennaReferencePointPanelGroupSB.SetForegroundColour(LinedFontColour)               
        AntennaReferencePointPanelGroupSizer = wx.StaticBoxSizer(AntennaReferencePointPanelGroupSB, wx.VERTICAL)
        AntennaReferencePointPanelGroupSizer.Add(AntennaReferencePointPanelCore, wx.ALL, 5)
        AntennaReferencePointPanelGroupSizer.SetMinSize(wx.Size(0.95 * XAntennaReferencePointPanel, 0.95 * YAntennaReferencePointPanel))
        AntennaReferencePointPanelGroupBox.Add(AntennaReferencePointPanelGroupSizer)
               
    #Modes Layout
        self.ModesPanelGrill = wx.GridBagSizer()
        self.ModesPanelGrill.Add(self.ModellingOptionsPanel, (0, 0), (4, 1), wx.ALL, 10)
        self.ModesPanelGrill.Add(self.PreciseProductsDIPanel, (0, 1), (1, 1), wx.ALL, 10)
        self.ModesPanelGrill.Add(self.AntennaPhaseCenterPanel, (1, 1), (1, 1), wx.RIGHT | wx.LEFT, 10)
        self.ModesPanelGrill.Add(self.AntennaReferencePointPanel, (2, 1), (1, 1), wx.ALL, 10)
        self.ModelPanel.SetSizer(self.ModesPanelGrill)        

                                   
        """
............................................................................................        
                OUTPUT PANEL
............................................................................................                   
        """

    #Create OUTPUT Panel elements   
        self.OutputPanel = wx.Panel(self.SecondaryFrame, -1, size=self.SecondaryFrame.GetSize())
        XOutputPanel, YOutputPanel = 0.97 * SecondaryFrameXSize, 0.97 * SecondaryFrameYSize
        #Messages Group
        OutputMessagesPanel = wx.Panel(self.OutputPanel, -1, size=wx.Size(0.3 * XOutputPanel, 0.6 * YOutputPanel))
	#OutputMessagesPanel.SetBackgroundColour((120, 255, 044))
        XOutputMessagesPanel, YOutputMessagesPanel = OutputMessagesPanel.GetSize()
        
        OutputMessagesPanelCore = wx.Panel(OutputMessagesPanel, -1, size=wx.Size(0.99 * XOutputMessagesPanel, 0.95 * YOutputMessagesPanel))
        XOutputMessagesPanelCore, YOutputMessagesPanelCore = OutputMessagesPanelCore.GetSize()
        
        self.InfoMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print INFO Messages')
        self.InfoMessage.SetValue(True)
        self.CSMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print CS (Cycle-Slip) Messages')
        self.CSMessage.SetValue(True)
        self.InputMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print INPUT Messages')
        self.InputMessage.SetValue(True)
        self.ModelMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print MODEL Messages')
        self.ModelMessage.SetValue(True)
        self.EpochSatMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print EPOCHSAT Messages')
        self.EpochSatMessage.SetValue(True)
        self.PreFitMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print PREFIT Messages')
        self.PreFitMessage.SetValue(True)
        self.PostFitMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print POSTFIT Messages')
        self.PostFitMessage.SetValue(True)
        self.FilterMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print FILTER Messages')
        self.FilterMessage.SetValue(True)
        self.OutputMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print OUTPUT Messages')
        self.OutputMessage.SetValue(True)
        self.SatDifMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print SATDIFF Messages')
        self.SatStatMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print SATSTAT Messages')
        self.SatStatTotMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print SATSTATTOT Messages')
        self.SatMEASMessage = wx.CheckBox(OutputMessagesPanelCore, -1 , 'Print MEAS Message')
        
        self.SatDifMessage.Disable()
        self.SatStatMessage.Disable()
        self.SatStatTotMessage.Disable()
        self.SatDifMessage.Hide()
        self.SatStatMessage.Hide()
        self.SatStatTotMessage.Hide()
        
        OutputMessagesPanelCoreBox = wx.GridBagSizer(wx.VERTICAL)
        OutputMessagesPanelCoreBox.Add(self.InfoMessage, (0, 0), (1, 1))
        OutputMessagesPanelCoreBox.Add(self.CSMessage, (1, 0), (1, 1))
        OutputMessagesPanelCoreBox.Add(self.InputMessage, (2, 0), (1, 1))
        OutputMessagesPanelCoreBox.Add(self.SatMEASMessage, (3, 0), (1, 1))
        OutputMessagesPanelCoreBox.Add(self.ModelMessage, (4, 0), (1, 1))
        OutputMessagesPanelCoreBox.Add(self.EpochSatMessage, (5, 0), (1, 1))
        OutputMessagesPanelCoreBox.Add(self.PreFitMessage, (6, 0), (1, 1))
        OutputMessagesPanelCoreBox.Add(self.PostFitMessage, (7, 0), (1, 1))
        OutputMessagesPanelCoreBox.Add(self.FilterMessage, (8, 0), (1, 1))
        OutputMessagesPanelCoreBox.Add(self.OutputMessage, (9, 0), (1, 1))
        OutputMessagesPanelCoreBox.Add(self.SatDifMessage, (10, 0), (1, 1))
        OutputMessagesPanelCoreBox.Add(self.SatStatMessage, (11, 0), (1, 1))
        OutputMessagesPanelCoreBox.Add(self.SatStatTotMessage, (12, 0), (1, 1))

        #OutputMessagesPanelCoreBox = wx.BoxSizer(wx.VERTICAL)
        #OutputMessagesPanelCoreBox.Add(self.InfoMessage)
        #OutputMessagesPanelCoreBox.Add(self.CSMessage)
        #OutputMessagesPanelCoreBox.Add(self.InputMessage)
        #OutputMessagesPanelCoreBox.Add(self.ModelMessage)
        #OutputMessagesPanelCoreBox.Add(self.EpochSatMessage)
        #OutputMessagesPanelCoreBox.Add(self.PreFitMessage)
        #OutputMessagesPanelCoreBox.Add(self.PostFitMessage)
        #OutputMessagesPanelCoreBox.Add(self.FilterMessage)
        #OutputMessagesPanelCoreBox.Add(self.OutputMessage)
        #OutputMessagesPanelCoreBox.Add(self.SatDifMessage)
        #OutputMessagesPanelCoreBox.Add(self.SatStatMessage)
        #OutputMessagesPanelCoreBox.Add(self.SatStatTotMessage)
        
        OutputMessagesPanelCoreBox.SetMinSize(wx.Size(0.95 * XOutputMessagesPanelCore, 0.95 * YOutputMessagesPanelCore))
        OutputMessagesPanelCore.SetSizer(OutputMessagesPanelCoreBox)
        
        
        #Messages Group
        OutputMessagesPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        OutputMessagesPanel.SetSizer(OutputMessagesPanelGroupBox)
        OutputMessagesPanelGroupSB = wx.StaticBox(parent=OutputMessagesPanel, label="Messages")
        OutputMessagesPanelGroupSB.SetFont(LinedFont)
        OutputMessagesPanelGroupSB.SetForegroundColour(LinedFontColour)          
        OutputMessagesPanelGroupSizer = wx.StaticBoxSizer(OutputMessagesPanelGroupSB, wx.VERTICAL)
        OutputMessagesPanelGroupSizer.Add(OutputMessagesPanelCore)
        OutputMessagesPanelGroupSizer.SetMinSize(wx.Size(0.5 * XOutputMessagesPanel, 0.5 * YOutputMessagesPanel))
        OutputMessagesPanelGroupBox.Add(OutputMessagesPanelGroupSizer)
        
    #Output Destination Panel
        OutputDestinationPanel = wx.Panel(self.OutputPanel, -1, size=wx.Size(0.6 * XOutputPanel, 0.05 * YOutputPanel))
        XOutputDestinationPanel, YOutputDestinationPanel = OutputDestinationPanel.GetSize()
        
        OutputDestinationPanelCore = wx.Panel(OutputDestinationPanel, -1, size=wx.Size(0.95 * XOutputDestinationPanel, 0.95 * YOutputDestinationPanel))
        XOutputDestinationPanelCore, YOutputDestinationPanelCore = OutputDestinationPanelCore.GetSize()
        
        OutputDestinationFileButton = wx.Button(OutputDestinationPanelCore, 1, label='Examine')
        self.OutputDestinationText = wx.lib.stattext.GenStaticText(OutputDestinationPanelCore, -1, 'Output File :  ')
        OutputPath = 'gLAB.out'
        self.TextCtrlOutputDestination = wx.TextCtrl(OutputDestinationPanelCore, -1, OutputPath, size=wx.Size(0.4 * XOutputPanel, 0.05 * YOutputMessagesPanelCore), style=wx.TE_PROCESS_ENTER)
        self.OutputDestinationPath = OutputPath
        
        OutputDestinationFileButton.Bind(wx.EVT_LEFT_DOWN, self.SaveDestinationPathByButton)
        self.TextCtrlOutputDestination.Bind(wx.EVT_TEXT, self.SaveDestinationPathByTextCtrl)
        
        OutputDestinationPanelCoreBox = wx.GridBagSizer(wx.HORIZONTAL)
        OutputDestinationPanelCore.SetSizer(OutputDestinationPanelCoreBox)
        OutputDestinationPanelCoreBox.Add(self.OutputDestinationText, (0, 0), (1, 1), wx.ALIGN_LEFT | wx.ALIGN_CENTER )
        OutputDestinationPanelCoreBox.Add(self.TextCtrlOutputDestination, (0, 1), (1, 1), wx.ALIGN_CENTER | wx.EXPAND)
        OutputDestinationPanelCoreBox.Add(OutputDestinationFileButton , (0, 2), (1, 1), wx.ALIGN_RIGHT) 
        OutputDestinationPanelCoreBox.SetMinSize(size=wx.Size(0.95 * XOutputDestinationPanelCore, 0.99 * YOutputDestinationPanelCore))

        #Output Group
        OutputDestinationPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        OutputDestinationPanel.SetSizer(OutputDestinationPanelGroupBox)
        OutputDestinationPanelGroupSB = wx.StaticBox(parent=OutputDestinationPanel, label="Output Destination")
        OutputDestinationPanelGroupSB.SetFont(LinedFont)
        OutputDestinationPanelGroupSB.SetForegroundColour(LinedFontColour)          
        OutputDestinationPanelGroupSizer = wx.StaticBoxSizer(OutputDestinationPanelGroupSB, wx.VERTICAL)
        OutputDestinationPanelGroupSizer.Add(OutputDestinationPanelCore, wx.ALL, 5)
        OutputDestinationPanelGroupSizer.SetMinSize(wx.Size(0.5 * XOutputDestinationPanel, 0.5 * YOutputDestinationPanel))
        OutputDestinationPanelGroupBox.Add(OutputDestinationPanelGroupSizer)

        
        
    #Output Layout
        self.OutputPanelGrill = wx.GridBagSizer()
        self.OutputPanelGrill.Add(OutputDestinationPanel, (0, 0), (1, 1), wx.ALL, 15) 
        self.OutputPanelGrill.Add(OutputMessagesPanel, (1, 0), (1, 1), wx.LEFT, 15)
        self.OutputPanel.SetSizer(self.OutputPanelGrill)
       
        """
................................................................................................................................................        
                FILTER PANEL
................................................................................................................................................                  
        """       
        
  #Create FILTER Panel elements   
        self.FilterPanel = wx.Panel(self.SecondaryFrame, -1, size=self.SecondaryFrame.GetSize())
        XFilterPanel, YFilterPanel = 0.97 * SecondaryFrameXSize, 0.97 * SecondaryFrameYSize
        
     #Navigation Panel
        NavigationModePanel = wx.Panel(self.FilterPanel, -1, size=wx.Size(0.25 * XFilterPanel, 0.1 * YFilterPanel))
        XNavigationModePanel, YNavigationModePanel = NavigationModePanel.GetSize()
        
        NavigationModePanelCore = wx.Panel(NavigationModePanel, -1, size=wx.Size(0.95 * XNavigationModePanel, 0.95 * YNavigationModePanel))
        XNavigationModePanelCore, YNavigationModePanelCore = NavigationModePanelCore.GetSize()
        
        self.PPPRB = wx.RadioButton(NavigationModePanelCore, -1, 'PPP', style=wx.RB_GROUP)
        self.StandaloneRB = wx.RadioButton(NavigationModePanelCore, -1, 'Standalone')
        NavigationPanelCoreBox = wx.BoxSizer(wx.VERTICAL)
        NavigationModePanelCore.SetSizer(NavigationPanelCoreBox)
        NavigationPanelCoreBox.Add(self.PPPRB)
        NavigationPanelCoreBox.Add(self.StandaloneRB)
        NavigationPanelCoreBox.SetMinSize(size=wx.Size(0.95 * XNavigationModePanelCore, 0.95 * YNavigationModePanelCore))
        
        self.PPPRB.Bind(wx.EVT_RADIOBUTTON, self.SetConfigurationTexts)
        self.StandaloneRB.Bind(wx.EVT_RADIOBUTTON, self.SetConfigurationTexts)
        
        
        #Navigation Group
        NavigationPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        NavigationModePanel.SetSizer(NavigationPanelGroupBox)
        NavigationModePanelGroupSB = wx.StaticBox(parent=NavigationModePanel, label="Navigation Mode")
        NavigationModePanelGroupSB.SetFont(LinedFont)
        NavigationModePanelGroupSB.SetForegroundColour(LinedFontColour)              
        NavigationModePanelGroupSizer = wx.StaticBoxSizer(NavigationModePanelGroupSB, wx.VERTICAL)
        NavigationModePanelGroupSizer.Add(NavigationModePanelCore, wx.ALL, 5)
        NavigationModePanelGroupSizer.SetMinSize(wx.Size(0.75 * XNavigationModePanel, 0.5 * YNavigationModePanel))
        NavigationPanelGroupBox.Add(NavigationModePanelGroupSizer)
        
     #Troposphere Panel
        TropospherePanel = wx.Panel(self.FilterPanel, -1, size=wx.Size(0.25 * XFilterPanel, 0.1 * YFilterPanel))
        XTropospherePanel, YTropospherePanel = TropospherePanel.GetSize()
        
        TropospherePanelCore = wx.Panel(TropospherePanel, -1, size=wx.Size(0.95 * XTropospherePanel, 0.95 * YTropospherePanel))
        XTropospherePanelCore, YTropospherePanelCore = TropospherePanelCore.GetSize()
        
        self.EstimateTroposphereCB = wx.CheckBox(TropospherePanelCore, -1 , 'Estimate Troposphere', style=wx.ALIGN_LEFT)
        self.EstimateTroposphereCB.SetValue(True)
        self.EstimateTroposphereCB.Bind(wx.EVT_CHECKBOX, self.ShowTroposphereCheckBox)
        
	TropospherePanelCoreGrid = wx.GridBagSizer(wx.VERTICAL)
        TropospherePanelCore.SetSizer(TropospherePanelCoreGrid)
        TropospherePanelCoreGrid.Add(self.EstimateTroposphereCB, (0, 0), (1, 1), wx.TOP, 11)
        TropospherePanelCoreGrid.SetMinSize(size=wx.Size(0.95 * XTropospherePanelCore, 0.95 * YTropospherePanelCore))   

        #Troposphere Group
        TropospherePanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        TropospherePanel.SetSizer(TropospherePanelGroupBox)
        TropospherePanelGroupSB = wx.StaticBox(parent=TropospherePanel, label="Troposphere")
        TropospherePanelGroupSB.SetFont(LinedFont)
        TropospherePanelGroupSB.SetForegroundColour(LinedFontColour)          
        TropospherePanelGroupSizer = wx.StaticBoxSizer(TropospherePanelGroupSB, wx.VERTICAL)
        TropospherePanelGroupSizer.Add(TropospherePanelCore, wx.ALL, 5)
        TropospherePanelGroupSizer.SetMinSize(wx.Size(0.75 * XTropospherePanel, 0.5 * YTropospherePanel))
        TropospherePanelGroupBox.Add(TropospherePanelGroupSizer)
        
     #Ionosphere Panel
        IonospherePanel = wx.Panel(self.FilterPanel, -1, size=wx.Size(0.25 * XFilterPanel, 0.1 * YFilterPanel))
        XIonospherePanel, YIonospherePanel = IonospherePanel.GetSize()
        IonospherePanelCore = wx.Panel(IonospherePanel, -1, size=wx.Size(0.95 * XIonospherePanel, 0.95 * YIonospherePanel))
        XIonospherePanelCore, YIonospherePanelCore = IonospherePanelCore.GetSize()
        
        self.UseSigmaIonoCB = wx.CheckBox(IonospherePanelCore, -1 , 'Use Sigma Iono', style=wx.ALIGN_LEFT)
        self.UseSigmaIonoCB.SetValue(True)
       # self.UseSigmaIono.Bind(wx.EVT_CHECKBOX, self.ShowIonosphereCheckBox)
        
        IonospherePanelCoreGrid = wx.GridBagSizer(wx.VERTICAL)
        IonospherePanelCore.SetSizer(IonospherePanelCoreGrid)
        IonospherePanelCoreGrid.Add(self.UseSigmaIonoCB, (0, 0), (1, 1), wx.TOP, 11)
        IonospherePanelCoreGrid.SetMinSize(size=wx.Size(0.95 * XIonospherePanelCore, 0.95 * YIonospherePanelCore))
        
        #Ionosphere Group
        IonospherePanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        IonospherePanel.SetSizer(IonospherePanelGroupBox)
        IonospherePanelGroupSB = wx.StaticBox(parent=IonospherePanel, label="Ionosphere")
        IonospherePanelGroupSB.SetFont(LinedFont)
        IonospherePanelGroupSB.SetForegroundColour(LinedFontColour)
        IonospherePanelGroupSizer = wx.StaticBoxSizer(IonospherePanelGroupSB, wx.VERTICAL)
        IonospherePanelGroupSizer.Add(IonospherePanelCore, wx.ALL, 5)
        IonospherePanelGroupSizer.SetMinSize(wx.Size(0.75 * XIonospherePanel, 0.5 * YIonospherePanel))
        IonospherePanelGroupBox.Add(IonospherePanelGroupSizer)

     #Available Frequencies Panel
        AvailableFrequenciesPanel = wx.Panel(self.FilterPanel, -1, size=wx.Size(0.25 * XFilterPanel, 0.1 * YFilterPanel))
        XAvailableFrequenciesPanel, YAvailableFrequenciesPanel = AvailableFrequenciesPanel.GetSize()
        
        AvailableFrequenciesPanelCore = wx.Panel(AvailableFrequenciesPanel, -1, size=wx.Size(0.95 * XAvailableFrequenciesPanel, 0.95 * YAvailableFrequenciesPanel))
        XAvailableFrequenciesPanelCore, YAvailableFrequenciesPanelCore = AvailableFrequenciesPanelCore.GetSize()

        self.DualFrequencyRB = wx.RadioButton(AvailableFrequenciesPanelCore, -1, 'Dual-Frequency', style=wx.RB_GROUP)
        self.SingleFrequencyRB = wx.RadioButton(AvailableFrequenciesPanelCore, -1, 'Single-Frequency')
        
        self.DualFrequencyRB.Bind(wx.EVT_RADIOBUTTON, self.SetConfigurationTexts)
        self.SingleFrequencyRB.Bind(wx.EVT_RADIOBUTTON, self.SetConfigurationTexts)
        
        AvailableFrequenciesPanelCoreBox = wx.BoxSizer(wx.VERTICAL)
        AvailableFrequenciesPanelCore.SetSizer(AvailableFrequenciesPanelCoreBox)
        AvailableFrequenciesPanelCoreBox.Add(self.SingleFrequencyRB)
        AvailableFrequenciesPanelCoreBox.Add(self.DualFrequencyRB)
        AvailableFrequenciesPanelCoreBox.SetMinSize(size=wx.Size(0.95 * XAvailableFrequenciesPanelCore, 0.95 * YAvailableFrequenciesPanelCore))
        
                
        #Available Frequencies Group
        AvailableFrequenciesPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        AvailableFrequenciesPanel.SetSizer(AvailableFrequenciesPanelGroupBox)
        AvailableFrequenciesPanelGroupSB = wx.StaticBox(parent=AvailableFrequenciesPanel, label="Available Frequencies")
        AvailableFrequenciesPanelGroupSB.SetFont(LinedFont)
        AvailableFrequenciesPanelGroupSB.SetForegroundColour(LinedFontColour)              
        AvailableFrequenciesPanelGroupSizer = wx.StaticBoxSizer(AvailableFrequenciesPanelGroupSB, wx.VERTICAL)
        AvailableFrequenciesPanelGroupSizer.Add(AvailableFrequenciesPanelCore, wx.ALL, 5)
        AvailableFrequenciesPanelGroupSizer.SetMinSize(wx.Size(0.75 * XAvailableFrequenciesPanel, 0.5 * YAvailableFrequenciesPanel))
        AvailableFrequenciesPanelGroupBox.Add(AvailableFrequenciesPanelGroupSizer)        
        
     
     #Receiver Kinematics Panel
        ReceiverKinematicsPanel = wx.Panel(self.FilterPanel, -1, size=wx.Size(0.25 * XFilterPanel, 0.1 * YFilterPanel))
        XReceiverKinematicsPanel, YReceiverKinematicsPanel = ReceiverKinematicsPanel.GetSize()
        
        ReceiverKinematicsPanelCore = wx.Panel(ReceiverKinematicsPanel, -1, size=wx.Size(0.95 * XReceiverKinematicsPanel, 0.95 * YReceiverKinematicsPanel))
        XReceiverKinematicsPanelCore, YReceiverKinematicsPanelCore = ReceiverKinematicsPanelCore.GetSize()

        self.StaticReceiverRB = wx.RadioButton(ReceiverKinematicsPanelCore, -1, 'Static', style=wx.RB_GROUP)
        self.KinematicReceiverRB = wx.RadioButton(ReceiverKinematicsPanelCore, -1, 'Kinematic')
        
        self.StaticReceiverRB.Bind(wx.EVT_RADIOBUTTON, self.SetParametersTexts)
        self.KinematicReceiverRB.Bind(wx.EVT_RADIOBUTTON, self.SetParametersTexts)
        
        ReceiverKinematicsPanelCoreCoreBox = wx.BoxSizer(wx.VERTICAL)
        ReceiverKinematicsPanelCore.SetSizer(ReceiverKinematicsPanelCoreCoreBox)
        ReceiverKinematicsPanelCoreCoreBox.Add(self.StaticReceiverRB)
        ReceiverKinematicsPanelCoreCoreBox.Add(self.KinematicReceiverRB)
        ReceiverKinematicsPanelCoreCoreBox.SetMinSize(size=wx.Size(0.95 * XReceiverKinematicsPanelCore, 0.95 * YReceiverKinematicsPanelCore))
        
        #Receiver Kinematics Group            
        ReceiverKinematicsPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        ReceiverKinematicsPanel.SetSizer(ReceiverKinematicsPanelGroupBox)
        ReceiverKinematicsPanelGroupSB = wx.StaticBox(parent=ReceiverKinematicsPanel, label="Receiver Kinematics")
        ReceiverKinematicsPanelGroupSB.SetFont(LinedFont)
        ReceiverKinematicsPanelGroupSB.SetForegroundColour(LinedFontColour)           
        ReceiverKinematicsPanelGroupSizer = wx.StaticBoxSizer(ReceiverKinematicsPanelGroupSB, wx.VERTICAL)
        ReceiverKinematicsPanelGroupSizer.Add(ReceiverKinematicsPanelCore, wx.ALL, 5)
        ReceiverKinematicsPanelGroupSizer.SetMinSize(wx.Size(0.75 * XReceiverKinematicsPanel, 0.5 * YReceiverKinematicsPanel))
        ReceiverKinematicsPanelGroupBox.Add(ReceiverKinematicsPanelGroupSizer)        
                
     #Other options Filter Panel
        OtherOptionsFilterPanel = wx.Panel(self.FilterPanel, -1, size=wx.Size(0.25 * XFilterPanel, 0.05 * YFilterPanel))
        XOtherOptionsFilterPanel, YOtherOptionsFilterPanel = ReceiverKinematicsPanel.GetSize()

        OtherOptionsFilterPanelCore = wx.Panel(OtherOptionsFilterPanel, -1, size=wx.Size(0.95 * XOtherOptionsFilterPanel, 0.95 * YOtherOptionsFilterPanel))
        XOtherOptionsFilterPanelCore, YOtherOptionsFilterPanelCore = OtherOptionsFilterPanelCore.GetSize()
        self.BackwardCB = wx.CheckBox(OtherOptionsFilterPanelCore, -1, 'Backward filtering')

        OtherOptionsFilterPanelCoreGrid = wx.GridBagSizer()
        OtherOptionsFilterPanelCoreGrid.Add(self.BackwardCB, (0, 0), (1, 1), wx.TOP, 6)
        OtherOptionsFilterPanelCore.SetSizer(OtherOptionsFilterPanelCoreGrid)
        OtherOptionsFilterPanelCoreGrid.SetMinSize(size=wx.Size(0.95 * XOtherOptionsFilterPanelCore, 0.95 * YOtherOptionsFilterPanelCore))
                
      #Other options Group
        OtherOptionsFilterPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        OtherOptionsFilterPanel.SetSizer(OtherOptionsFilterPanelGroupBox)
        OtherOptionsFilterPanelGroupSB = wx.StaticBox(parent=OtherOptionsFilterPanel, label="Other options")
        OtherOptionsFilterPanelGroupSB.SetFont(LinedFont)
        OtherOptionsFilterPanelGroupSB.SetForegroundColour(LinedFontColour)           
        OtherOptionsFilterPanelGroupSizer = wx.StaticBoxSizer(OtherOptionsFilterPanelGroupSB, wx.VERTICAL)
        OtherOptionsFilterPanelGroupSizer.Add(OtherOptionsFilterPanelCore, wx.ALL, 11)
        OtherOptionsFilterPanelGroupSizer.SetMinSize(wx.Size(0.75 * XOtherOptionsFilterPanel, 0.5 * YOtherOptionsFilterPanel))
        OtherOptionsFilterPanelGroupBox.Add(OtherOptionsFilterPanelGroupSizer)    
        
      #Measurements Panel        
        MeasurementsPanel = wx.Panel(self.FilterPanel, -1, size=wx.Size(0.8 * XFilterPanel, 0.20 * YFilterPanel))
        XMeasurementsPanel, YMeasurementsPanel = MeasurementsPanel.GetSize()
        
        MeasurementsPanelCore = wx.Panel(MeasurementsPanel, -1, size=wx.Size(0.95 * XMeasurementsPanel, 0.95 * YMeasurementsPanel))
        XMeasurementsPanelCore , YMeasurementsPanelCore = MeasurementsPanelCore.GetSize()
        
       #Selection 
        SelectionPanel = wx.Panel(MeasurementsPanelCore, -1, size=wx.Size(0.99 * XMeasurementsPanelCore, 0.1 * XMeasurementsPanelCore))
        XSelectionPanel, YSelectionPanel = SelectionPanel.GetSize()
        
        SelectionPanelCore = wx.Panel(SelectionPanel, -1, size=wx.Size(0.99 * XSelectionPanel, 0.99 * YSelectionPanel))
        XSelectionPanelCore, YSelectionPanelCore = SelectionPanelCore.GetSize()
        
        self.PseudoRangeCarrierPhaseRB = wx.RadioButton(SelectionPanelCore, -1, 'Pseudorange + Carrier phase', style=wx.RB_GROUP)
        self.PseudoRangeCarrierPhaseRB.Bind(wx.EVT_RADIOBUTTON, self.ShowPseudoRangeCarrierPhase)
        self.PseudoRangeRB = wx.RadioButton(SelectionPanelCore, -1, 'Pseudorange')
        self.PseudoRangeRB.Bind(wx.EVT_RADIOBUTTON, self.ShowPseudoRangeCarrierPhase)
        self.PseudoRangeSmoothingCB = wx.CheckBox(SelectionPanelCore, -1 , 'Pseudorange Smoothing [epochs]')  
        self.PseudoRangeSmoothingTC = wx.TextCtrl(SelectionPanelCore, -1, size=wx.Size(0.07 * XSelectionPanelCore, 0.35 * YSelectionPanelCore), value='10')        
        self.PseudoRangeSmoothingCB.Bind(wx.EVT_CHECKBOX, self.ShowPseudoRangeSmoothingTC)
        
        
        SelectionPanelCoreGrid = wx.GridBagSizer()
        SelectionPanelCore.SetSizer(SelectionPanelCoreGrid)
        SelectionPanelCoreGrid.Add(self.PseudoRangeRB, (0, 0), (1, 1), wx.TOP | wx.BOTTOM, 5)
        SelectionPanelCoreGrid.Add(self.PseudoRangeSmoothingCB, (0, 1), (1, 1), wx.TOP | wx.BOTTOM, 5)
        SelectionPanelCoreGrid.Add(self.PseudoRangeSmoothingTC, (0, 2), (1, 1), wx.LEFT | wx.TOP | wx.BOTTOM, 5)
        SelectionPanelCoreGrid.Add(self.PseudoRangeCarrierPhaseRB, (1, 0), (1, 1), wx.BOTTOM, 5)
        SelectionPanelCoreGrid.SetMinSize(size=wx.Size(0.99 * XSelectionPanelCore, 0.99 * YSelectionPanelCore))        

        
        #Selection Group
        SelectionPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        SelectionPanel.SetSizer(SelectionPanelGroupBox)
        SelectionPanelGroupSB = wx.StaticBox(parent=SelectionPanel, label="Selection")
        SelectionPanelGroupSB.SetFont(LinedFont)
        SelectionPanelGroupSB.SetForegroundColour(LinedFontColour)                 
        SelectionPanelGroupSizer = wx.StaticBoxSizer(SelectionPanelGroupSB, wx.VERTICAL)
        SelectionPanelGroupSizer.Add(SelectionPanelCore, wx.ALL, 5)
        SelectionPanelGroupSizer.SetMinSize(wx.Size(0.75 * XSelectionPanel, 0.5 * YSelectionPanel))
        SelectionPanelGroupBox.Add(SelectionPanelGroupSizer)               
        
       #Configuration  
        ConfigurationPanel = wx.Panel(MeasurementsPanelCore, -1, size=wx.Size(0.99 * XMeasurementsPanelCore, 0.45 * YMeasurementsPanelCore))
        XConfigurationPanel, YConfigurationPanel = ConfigurationPanel.GetSize()
 
        ConfigurationPanelCore = wx.Panel(ConfigurationPanel, -1, size=wx.Size(0.99 * XConfigurationPanel, 0.95 * YConfigurationPanel))
        XConfigurationPanelCore, YConfigurationPanelCore = ConfigurationPanelCore.GetSize()

#	it was: self.PseudoRangeComboBoxList = ['C1C', 'PC']
#	        self.PseudoRangeCarrierPhaseBoxList = ['G1C', 'LC', 'L1P']
#	it is now:
#	        self.PseudoRangeComboBoxList = ['C1C','C2C','C1P','C2P','PC','PCC']
#	        self.PseudoRangeCarrierPhaseBoxList = ['L1C','L1P','L2C','L2P','G1C', 'LC']
#	it will be:
#	        self.PseudoRangeComboBoxListSingleFreq = ['C1C','C2C','C1P','C2P']
#	        self.PseudoRangeComboBoxListDoubleFreq = ['PC']
#	        self.PseudoRangeCarrierPhaseBoxListSingleFreq = ['L1C','L1P','L2C','L2P']
#	        self.PseudoRangeCarrierPhaseBoxListDoubleFreq = ['G1C', 'LC']

        self.PseudoRangeComboBoxListSingleFreq = ['C1C','C2C','C1P','C2P']
        self.PseudoRangeComboBoxListDoubleFreq = ['PC']
        self.PseudoRangeCarrierPhaseBoxListSingleFreq = ['L1C','L1P','L2C','L2P','G1C','G1P']
        self.PseudoRangeCarrierPhaseBoxListDoubleFreq = ['LC']

        self.PseudoRangeComboBoxList = [' ']
        self.PseudoRangeComboBox = wx.ComboBox(ConfigurationPanelCore, -1, value=self.PseudoRangeComboBoxList[0], choices=self.PseudoRangeComboBoxList, size=wx.Size(80, 25))
        self.PseudoRangeComboBox.Enable(True)     


        self.PseudoRangeCarrierPhaseBoxList = [' ']
        self.PseudoRangeCarrierPhaseComboBox = wx.ComboBox(ConfigurationPanelCore, -1, value=self.PseudoRangeComboBoxList[0], choices=self.PseudoRangeCarrierPhaseBoxList, size=wx.Size(80, 25))        
        self.PseudoRangeCarrierPhaseComboBox.Enable(True)   
        
        self.FixedWeightPseudoRangeCarrierPhaseRB = wx.RadioButton(ConfigurationPanelCore, -1, 'Fixed StdDev [m]', style=wx.RB_GROUP)
        self.FixedWeightPseudoRangeCarrierPhaseRB.SetValue(True)
        self.ElevationWeightPseudoRangeCarrierPhaseRB = wx.RadioButton(ConfigurationPanelCore, -1, 'Elevation StdDev [m]')
        self.ElevationWeightPseudoRangeCarrierPhaseRB.SetValue(False)
       
        self.FixedWeightPseudoRangeRB = wx.RadioButton(ConfigurationPanelCore, -1, 'Fixed StdDev [m]', style=wx.RB_GROUP)
        self.FixedWeightPseudoRangeRB.SetValue(True)      
        self.ElevationWeightPseudoRangeRB = wx.RadioButton(ConfigurationPanelCore, -1, 'Elevation StdDev [m]')
        self.ElevationWeightPseudoRangeRB.SetValue(False)
        
        self.FixedWeightPseudoRangeCarrierPhaseTC = wx.TextCtrl(ConfigurationPanelCore, -1, size=wx.Size(0.08 * XConfigurationPanelCore, 0.6 * YConfigurationPanelCore))
        self.FixedWeightPseudoRangeTC = wx.TextCtrl(ConfigurationPanelCore, -1, size=wx.Size(0.08 * XConfigurationPanelCore, 0.6 * YConfigurationPanelCore)) 
        self.FixedWeightPseudoRangeTC.SetValue('1')    
        self.FixedWeightPseudoRangeCarrierPhaseTC.SetValue('0.01')
        
        self.AElevationWeightPseudoRangeCarrierPhaseTC = wx.TextCtrl(ConfigurationPanelCore, -1, size=wx.Size(0.1 * XConfigurationPanelCore, 0.6 * YConfigurationPanelCore))
        self.BElevationWeightPseudoRangeCarrierPhaseTC = wx.TextCtrl(ConfigurationPanelCore, -1, size=wx.Size(0.1 * XConfigurationPanelCore, 0.6 * YConfigurationPanelCore))
        self.CElevationWeightPseudoRangeCarrierPhaseTC = wx.TextCtrl(ConfigurationPanelCore, -1, size=wx.Size(0.1 * XConfigurationPanelCore, 0.6 * YConfigurationPanelCore))
        
        self.AElevationWeightPseudoRangeTC = wx.TextCtrl(ConfigurationPanelCore, -1, size=wx.Size(0.1 * XConfigurationPanelCore, 0.6 * YConfigurationPanelCore))
        self.BElevationWeightPseudoRangeTC = wx.TextCtrl(ConfigurationPanelCore, -1, size=wx.Size(0.1 * XConfigurationPanelCore, 0.6 * YConfigurationPanelCore))
        self.CElevationWeightPseudoRangeTC = wx.TextCtrl(ConfigurationPanelCore, -1, size=wx.Size(0.1 * XConfigurationPanelCore, 0.6 * YConfigurationPanelCore))
        
        
        ConfigurationPanelCoreGrid = wx.GridBagSizer()
        ConfigurationPanelCore.SetSizer(ConfigurationPanelCoreGrid)
        
        ConfigurationPanelCoreGrid.Add(self.PseudoRangeComboBox , (0, 0), (1, 1), wx.TOP | wx.BOTTOM, 5)
        ConfigurationPanelCoreGrid.Add(self.FixedWeightPseudoRangeRB , (0, 2), (1, 1), wx.ALIGN_CENTER_VERTICAL, wx.TOP , 5)
        ConfigurationPanelCoreGrid.Add(self.FixedWeightPseudoRangeTC , (0, 3), (1, 1), wx.ALIGN_CENTER_VERTICAL, wx.TOP , 5)
        ConfigurationPanelCoreGrid.Add(self.ElevationWeightPseudoRangeRB , (0, 5), (1, 1), wx.ALIGN_CENTER_VERTICAL, wx.TOP , 5) 
        ConfigurationPanelCoreGrid.Add(self.AElevationWeightPseudoRangeTC , (0, 6), (1, 1), wx.ALIGN_CENTER_VERTICAL, wx.TOP , 5) 
        ConfigurationPanelCoreGrid.Add(self.BElevationWeightPseudoRangeTC , (0, 7), (1, 1), wx.ALIGN_CENTER_VERTICAL, wx.TOP , 5) 
        ConfigurationPanelCoreGrid.Add(self.CElevationWeightPseudoRangeTC , (0, 8), (1, 1), wx.ALIGN_CENTER_VERTICAL, wx.TOP , 5) 
        
        ConfigurationPanelCoreGrid.Add(self.PseudoRangeCarrierPhaseComboBox , (1, 0), (1, 1), wx.TOP | wx.BOTTOM, 5)
        ConfigurationPanelCoreGrid.Add(self.FixedWeightPseudoRangeCarrierPhaseRB , (1, 2), (1, 1), wx.ALIGN_CENTER_VERTICAL, wx.TOP | wx.BOTTOM, 5)
        ConfigurationPanelCoreGrid.Add(self.FixedWeightPseudoRangeCarrierPhaseTC , (1, 3), (1, 1), wx.ALIGN_CENTER_VERTICAL, wx.TOP | wx.BOTTOM, 5)
        ConfigurationPanelCoreGrid.Add(self.ElevationWeightPseudoRangeCarrierPhaseRB , (1, 5), (1, 1), wx.ALIGN_CENTER_VERTICAL, wx.TOP | wx.BOTTOM, 5)
        ConfigurationPanelCoreGrid.Add(self.AElevationWeightPseudoRangeCarrierPhaseTC , (1, 6), (1, 1), wx.ALIGN_CENTER_VERTICAL, wx.TOP | wx.BOTTOM, 5) 
        ConfigurationPanelCoreGrid.Add(self.BElevationWeightPseudoRangeCarrierPhaseTC , (1, 7), (1, 1), wx.ALIGN_CENTER_VERTICAL, wx.TOP | wx.BOTTOM, 5) 
        ConfigurationPanelCoreGrid.Add(self.CElevationWeightPseudoRangeCarrierPhaseTC , (1, 8), (1, 1), wx.ALIGN_CENTER_VERTICAL, wx.TOP | wx.BOTTOM, 5)  
              
        ConfigurationPanelCoreGrid.SetMinSize(size=wx.Size(0.99 * XConfigurationPanelCore, 0.95 * YConfigurationPanelCore))
        
        
        #Configuration Group
        
        ConfigurationPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        ConfigurationPanel.SetSizer(ConfigurationPanelGroupBox)
        ConfigurationPanelGroupSB = wx.StaticBox(parent=ConfigurationPanel, label="Measurement configuration and noise")
        ConfigurationPanelGroupSB.SetFont(LinedFont)
        ConfigurationPanelGroupSB.SetForegroundColour(LinedFontColour)          
        ConfigurationPanelGroupSizer = wx.StaticBoxSizer(ConfigurationPanelGroupSB, wx.VERTICAL)
        ConfigurationPanelGroupSizer.Add(ConfigurationPanelCore, wx.ALL, 5)
        ConfigurationPanelGroupSizer.SetMinSize(wx.Size(0.99 * XConfigurationPanel, 0.95 * YConfigurationPanel))
        ConfigurationPanelGroupBox.Add(ConfigurationPanelGroupSizer)          
        

        MeasurementsPanelCoreGrid =  wx.GridBagSizer()
        MeasurementsPanelCore.SetSizer(MeasurementsPanelCoreGrid)
        MeasurementsPanelCoreGrid.Add(SelectionPanel,(0, 0), (1, 1), wx.TOP , 10)
        MeasurementsPanelCoreGrid.Add(ConfigurationPanel,(1, 0), (1, 1), wx.TOP | wx.BOTTOM, 10)
        MeasurementsPanelCoreGrid.SetMinSize(size=wx.Size(0.95 * XAvailableFrequenciesPanelCore, 0.95 * YAvailableFrequenciesPanelCore))
        

        #Measurements Group
        MeasurementsPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        MeasurementsPanel.SetSizer(MeasurementsPanelGroupBox)
        MeasurementsPanelGroupSB = wx.StaticBox(parent=MeasurementsPanel, label="Measurements")
        MeasurementsPanelGroupSB.SetFont(LinedFont)
        MeasurementsPanelGroupSB.SetForegroundColour(LinedFontColour)         
        MeasurementsPanelGroupSizer = wx.StaticBoxSizer(MeasurementsPanelGroupSB, wx.VERTICAL)
        MeasurementsPanelGroupSizer.Add(MeasurementsPanelCore, wx.ALL, 5)
        MeasurementsPanelGroupSizer.SetMinSize(wx.Size(0.75 * XMeasurementsPanel, 0.5 * YMeasurementsPanel))
        MeasurementsPanelGroupBox.Add(MeasurementsPanelGroupSizer)
        
      #Parameters Panel
        ParametersPanel = wx.Panel(self.FilterPanel, -1, size=wx.Size(1.01 * XFilterPanel, 0.50 * YFilterPanel))
        XParametersPanel, YParametersPanel = ParametersPanel.GetSize()
        
        ParametersPanelCore = wx.Panel(ParametersPanel, -1, size=wx.Size(0.95 * XParametersPanel, 0.95 * YParametersPanel))
        XParametersPanelCore , YParametersPanelCore = ParametersPanelCore.GetSize()
        
        self.PhiText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, 'Phi')
        self.NoiseText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, 'Q')
        self.PoText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, 'Po')
        self.PositionText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, 'Coordinates : ') 
        self.PositionNoiseUnitsText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, u'[m²]') # UTF-8 encoded form of [m^2]
        self.PositionPoUnitsText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, u'[m²]') # UTF-8 encoded form of [m^2]
        
        self.ReceiverText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, 'Receiver Clock : ')
        self.ReceiverNoiseUnitsText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, u'[m²]') # UTF-8 encoded form of [m^2]
        self.ReceiverPoUnitsText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, u'[m²]') # UTF-8 encoded form of [m^2]
        self.TroposphereText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, 'Troposphere : ')
        self.TroposphereNoiseUnitsText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, u'[m²/h]') # UTF-8 encoded form of [m^2/h]
        self.TropospherePoUnitsText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, u'[m²]') # UTF-8 encoded form of [m^2]
        
        self.PhaseAmbiguitiesText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, 'Phase Ambiguities : ')
        self.PhaseAmbiguitiesNoiseUnitsText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, u'[m²]') # UTF-8 encoded form of [m^2]
        self.PhaseAmbiguitiesPoUnitsText = wx.lib.stattext.GenStaticText(ParametersPanelCore, -1, u'[m²]') # UTF-8 encoded form of [m^2]
        
        self.PositionPhiTC = wx.TextCtrl(ParametersPanelCore, -1, size=wx.Size(0.07 * XParametersPanelCore, 0.1 * YParametersPanelCore), style=wx.TE_CENTER, value='1')
        self.PositionNoiseTC = wx.TextCtrl(ParametersPanelCore, -1, size=wx.Size(0.07 * XParametersPanelCore, 0.1 * YParametersPanelCore), style=wx.TE_CENTER, value='0')
        self.PositionPoTC = wx.TextCtrl(ParametersPanelCore, -1, size=wx.Size(0.07 * XParametersPanelCore, 0.1 * YParametersPanelCore), style=wx.TE_CENTER, value='1e8') 
        self.ReceiverClockPhiTC = wx.TextCtrl(ParametersPanelCore, -1, size=wx.Size(0.07 * XParametersPanelCore, 0.1 * YParametersPanelCore), style=wx.TE_CENTER, value='0') 
        self.ReceiverClockNoiseTC = wx.TextCtrl(ParametersPanelCore, -1, size=wx.Size(0.07 * XParametersPanelCore, 0.1 * YParametersPanelCore), style=wx.TE_CENTER, value='9e10') 
        self.ReceiverClockPoTC = wx.TextCtrl(ParametersPanelCore, -1, size=wx.Size(0.07 * XParametersPanelCore, 0.1 * YParametersPanelCore), style=wx.TE_CENTER, value='9e10') 
        self.TroposherePhiTC = wx.TextCtrl(ParametersPanelCore, -1, size=wx.Size(0.07 * XParametersPanelCore, 0.1 * YParametersPanelCore), style=wx.TE_CENTER, value='1') 
        self.TroposhereNoiseTC = wx.TextCtrl(ParametersPanelCore, -1, size=wx.Size(0.07 * XParametersPanelCore, 0.1 * YParametersPanelCore), style=wx.TE_CENTER, value='1e-4') 
        self.TroposherePoTC = wx.TextCtrl(ParametersPanelCore, -1, size=wx.Size(0.07 * XParametersPanelCore, 0.1 * YParametersPanelCore), style=wx.TE_CENTER, value='0.25')
        self.PhaseAmbiguitiesPhiTC = wx.TextCtrl(ParametersPanelCore, -1, size=wx.Size(0.07 * XParametersPanelCore, 0.1 * YParametersPanelCore), style=wx.TE_CENTER, value='1') 
        self.PhaseAmbiguitiesNoiseTC = wx.TextCtrl(ParametersPanelCore, -1, size=wx.Size(0.07 * XParametersPanelCore, 0.1 * YParametersPanelCore), style=wx.TE_CENTER, value='0')         
        self.PhaseAmbiguitiesPoTC = wx.TextCtrl(ParametersPanelCore, -1, size=wx.Size(0.07 * XParametersPanelCore, 0.1 * YParametersPanelCore), style=wx.TE_CENTER, value='400') 

        ParametersPanelCoreGrid = wx.GridBagSizer()
        ParametersPanelCore.SetSizer(ParametersPanelCoreGrid)
        
        ParametersPanelCoreGrid.Add(self.PhiText , (0, 2), (1, 1) , wx.ALIGN_CENTER | wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.NoiseText , (0, 4), (1, 1) , wx.ALIGN_CENTER | wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.PoText , (0, 6), (1, 1) , wx.ALIGN_CENTER | wx.LEFT, 15)
        
        ParametersPanelCoreGrid.Add(self.PositionText , (1, 0), (1, 2) , wx.ALIGN_CENTER_VERTICAL)
        ParametersPanelCoreGrid.Add(self.PositionPhiTC , (1, 2), (1, 1), wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.PositionNoiseTC , (1, 4), (1, 1), wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.PositionNoiseUnitsText , (1, 5), (1, 1), wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 3)
        ParametersPanelCoreGrid.Add(self.PositionPoTC , (1, 6), (1, 1), wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.PositionPoUnitsText , (1, 7), (1, 1), wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 3)

        
        ParametersPanelCoreGrid.Add(self.ReceiverText , (2, 0), (1, 2) , wx.ALIGN_CENTER_VERTICAL)
        ParametersPanelCoreGrid.Add(self.ReceiverClockPhiTC , (2, 2), (1, 1), wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.ReceiverClockNoiseTC , (2, 4), (1, 1), wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.ReceiverNoiseUnitsText , (2, 5), (1, 1), wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 3)
        ParametersPanelCoreGrid.Add(self.ReceiverClockPoTC , (2, 6), (1, 1), wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.ReceiverPoUnitsText , (2, 7), (1, 1), wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 3)
        
        ParametersPanelCoreGrid.Add(self.TroposphereText , (3, 0), (1, 2) , wx.ALIGN_CENTER_VERTICAL)
        ParametersPanelCoreGrid.Add(self.TroposherePhiTC , (3, 2), (1, 1), wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.TroposhereNoiseTC , (3, 4), (1, 1), wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.TroposphereNoiseUnitsText , (3, 5), (1, 1), wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 3)
        ParametersPanelCoreGrid.Add(self.TroposherePoTC , (3, 6), (1, 1), wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.TropospherePoUnitsText , (3, 7), (1, 1), wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 3)
        
        ParametersPanelCoreGrid.Add(self.PhaseAmbiguitiesText , (4, 0), (1, 2) , wx.ALIGN_CENTER_VERTICAL)
        ParametersPanelCoreGrid.Add(self.PhaseAmbiguitiesPhiTC , (4, 2), (1, 1), wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.PhaseAmbiguitiesNoiseTC , (4, 4), (1, 1), wx.LEFT, 15)        
        ParametersPanelCoreGrid.Add(self.PhaseAmbiguitiesNoiseUnitsText , (4, 5), (1, 1), wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 3)
        ParametersPanelCoreGrid.Add(self.PhaseAmbiguitiesPoTC , (4, 6), (1, 1), wx.LEFT, 15)
        ParametersPanelCoreGrid.Add(self.PhaseAmbiguitiesPoUnitsText , (4, 7), (1, 1), wx.ALIGN_CENTER_VERTICAL | wx.LEFT, 3)
                 
        ParametersPanelCoreGrid.SetMinSize(size=wx.Size(0.95 * XAvailableFrequenciesPanelCore, 0.95 * YAvailableFrequenciesPanelCore))

        #Parameters Group
        ParametersPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        ParametersPanel.SetSizer(ParametersPanelGroupBox)
        ParametersPanelGroupSB = wx.StaticBox(parent=ParametersPanel, label="Parameters")
        ParametersPanelGroupSB.SetFont(LinedFont)
        ParametersPanelGroupSB.SetForegroundColour(LinedFontColour)                    
        ParametersPanelGroupSizer = wx.StaticBoxSizer(ParametersPanelGroupSB, wx.VERTICAL)
        ParametersPanelGroupSizer.Add(ParametersPanelCore, wx.ALL, 5)
        ParametersPanelGroupSizer.SetMinSize(wx.Size(0.75 * XParametersPanel, 0.5 * YParametersPanel))
        ParametersPanelGroupBox.Add(ParametersPanelGroupSizer)        
        
        #Filter Layout
        self.FilterPanelGrill = wx.GridBagSizer()
        self.FilterPanelGrill.Add(MeasurementsPanel, (0, 0), (3, 1), wx.ALL, 10)
        self.FilterPanelGrill.Add(AvailableFrequenciesPanel, (0, 1), (1, 1), wx.ALL, 10)     
        self.FilterPanelGrill.Add(TropospherePanel, (1, 1), (1, 1), wx.ALL, 10)  
        self.FilterPanelGrill.Add(IonospherePanel, (2, 1), (1, 1), wx.ALL, 10)  
        self.FilterPanelGrill.Add(ReceiverKinematicsPanel, (3, 1), (1, 1), wx.ALL, 10)        
        self.FilterPanelGrill.Add(OtherOptionsFilterPanel,(4, 1),(1, 1), wx.ALL, 10)
        self.FilterPanelGrill.Add(ParametersPanel, (3, 0), (2, 1),  wx.LEFT | wx.RIGHT, 10) 
#        self.FilterPanelGrill.Add(NavigationModePanel,(4,1),(1,1),wx.ALL,5 )  
        
        NavigationModePanel.Hide()
        self.FilterPanel.SetSizer(self.FilterPanelGrill)

        
    '''
    Definition of functions:
    '''
    
##########################################################################
# Name        : StoreUpdateAntennaPhaseCenter
# Description : This function stores the north, east, up phase centre 
#               of the receiver antenna
##########################################################################
    def StoreUpdateAntennaPhaseCenter(self, event):
        if (self.F1AntennaPhaseCenterRB.GetValue() == True):
            AuxNorth = self.NorthPhaseCenterF1
            AuxEast = self.EastPhaseCenterF1
            AuxUp = self.UpPhaseCenterF1
        elif (self.F2AntennaPhaseCenterRB.GetValue() == True):
            AuxNorth = self.NorthPhaseCenterF2
            AuxEast = self.EastPhaseCenterF2
            AuxUp = self.UpPhaseCenterF2
            
        self.NorthPhaseCenterTextCtrl.Clear()
        self.EastPhaseCenterTextCtrl.Clear() 
        self.UpPhaseCenterTextCtrl.Clear()
        
        self.NorthPhaseCenterTextCtrl.WriteText(AuxNorth)
        self.EastPhaseCenterTextCtrl.WriteText(AuxEast) 
        self.UpPhaseCenterTextCtrl.WriteText(AuxUp)
        
##########################################################################
# Name        : SaveNorthPhaseCenterByTextCtrl
# Description : This function stores the North phase centre of the antenna
#               when its text control is modified 
##########################################################################
    def SaveNorthPhaseCenterByTextCtrl(self, event):
        if (self.F1AntennaPhaseCenterRB.GetValue() == True):
            self.NorthPhaseCenterF1 = self.NorthPhaseCenterTextCtrl.GetValue()
        elif (self.F2AntennaPhaseCenterRB.GetValue() == True):
            self.NorthPhaseCenterF2 = self.NorthPhaseCenterTextCtrl.GetValue()         

##########################################################################
# Name        : SaveEastPhaseCenterByTextCtrl
# Description : This function stores the East phase centre of the antenna
#               when its text control is modified 
##########################################################################
    def SaveEastPhaseCenterByTextCtrl(self, event):
        if (self.F1AntennaPhaseCenterRB.GetValue() == True):
            self.EastPhaseCenterF1 = self.EastPhaseCenterTextCtrl.GetValue()
        elif (self.F2AntennaPhaseCenterRB.GetValue() == True):
            self.EastPhaseCenterF2 = self.EastPhaseCenterTextCtrl.GetValue()              
            
##########################################################################
# Name        : SaveUpPhaseCenterByTextCtrl
# Description :  This function stores the Up phase centre of the antenna
#               when its text control is modified
##########################################################################
    def SaveUpPhaseCenterByTextCtrl(self, event):
        if (self.F1AntennaPhaseCenterRB.GetValue() == True):
            self.UpPhaseCenterF1 = self.UpPhaseCenterTextCtrl.GetValue()
        elif (self.F2AntennaPhaseCenterRB.GetValue() == True):
            self.UpPhaseCenterF2 = self.UpPhaseCenterTextCtrl.GetValue()  
            
##########################################################################
# Name        : ShowNorthEastUpReferencePointPanel
# Description : This function shows the panel where the reference point 
#               of the receiver antenna can be set 
##########################################################################
    def ShowNorthEastUpReferencePointPanel(self, event=None):
        if (self.SpecifyAntennaReferencePointRB.GetValue() == True):
            self.NorthEastUpReferencePointPanel.Show()  
        elif (self.ReadAntennaReferencePointRB.GetValue() == True):
            self.NorthEastUpReferencePointPanel.Hide()  

##########################################################################
# Name        : ShowNorthEastUpPanel
# Description : This function shows the panel where the North East Up 
#               phase centre values of the receiver antenna can be set
#		if the ANTEX file is needed, it is shown in the inputs panel
##########################################################################
    def ShowNorthEastUpPanel(self, event=None):
        if (self.SpecifyAntennaPhaseCenterRB.GetValue() == True):
            self.NorthEastUpPanel.Show()
            self.ShowAntexObservationCB.SetValue(False)  
        elif (self.ReadAntennaPhaseCenterRB.GetValue() == True):
            self.NorthEastUpPanel.Hide()  
	    if (self.ReceiverPhaseCenterCorrection.GetValue() == True):
            	self.ShowAntexObservationCB.SetValue(True)
	    else:
		self.ShowAntexObservationCB.SetValue(False)  
        self.ShowAntexFunc()

##########################################################################
# Name        : ShowReceiverPhaseCenterCorrectionPanel
# Description : This function shows the panel where the phase centre
#               of the receiver antenna can be set or read from ANTEX
##########################################################################
    def ShowReceiverPhaseCenterCorrectionPanel(self, event=None):
        if (self.ReceiverPhaseCenterCorrection.GetValue() == True):
            self.AntennaPhaseCenterPanel.Show()  
        else:
            self.AntennaPhaseCenterPanel.Hide() 
        self.ShowNorthEastUpPanel()   

##########################################################################
# Name        : ShowReceiverRefPointCorrectionPanel
# Description : This function shows the panel where the reference point
#               of the receiver antenna can be set or read from RINEX
##########################################################################
    def ShowReceiverRefPointCorrectionPanel(self, event=None):
        if (self.ReceiverRefPointCorrection.GetValue() == True):
            self.AntennaReferencePointPanel.Show() 
        else:
            self.AntennaReferencePointPanel.Hide()   
            
##########################################################################
# Name        : SetPRNButtonColor
# Description : This sets the colour of the button for the different  
#               PRN: green if active, red if disabled 
##########################################################################
    def SetPRNButtonColor(self, event):
         for i in range(32):
            if (self.ButtonPRN[i].GetValue() == True):
                self.ButtonPRN[i].SetBackgroundColour((120, 255, 044))
            elif (self.ButtonPRN[i].GetValue() == False):
                self.ButtonPRN[i].SetBackgroundColour((230, 050, 043))   
        
##########################################################################
# Name        : ShowOutputFile
# Description : Launches an editor to show the output file
##########################################################################
    def ShowOutputFile (self, event):
        os.chdir (LastDirExecuted)
        filename = LastOutputExecuted
        if (filename == ""):
            dial = wx.MessageDialog(None, 'Output file was defined as standard output', 'Problem opening output', wx.OK | wx.ICON_HAND)
            dial.ShowModal()
        elif(os.path.isfile(filename) == False):
            dial = wx.MessageDialog(None, 'Output file ' + filename + ' not found', 'File not found', wx.OK | wx.ICON_HAND)
            dial.ShowModal()
        else:
            if WIN:
		auxstr = u''
                auxstr += 'notepad.exe "'
                auxstr += filename + '" '
                self.popenrun = subprocess.Popen( auxstr.encode('iso-8859-1') )
            else:
                if (os.path.isfile("/usr/bin/gedit") == True):
                    editor = "/usr/bin/gedit"
                elif (os.path.isfile("/usr/bin/kedit") == True):
                    editor = "/usr/bin/kedit"
                else:
                    editor = ""
                    self.ShowAlert('Alert', 'No editor found, I cannot open the file')
                
                if (editor != ""):
                    self.popenrun = subprocess.Popen([editor, filename])
        os.chdir (DireMain)  
        #event.Skip()

##########################################################################
# Name        : ShowConfigFile
# Description : Launches an editor to show the configuration file
##########################################################################
    global DireName
    global LastConfigFile
    global LastDirConfigFile

    def ShowConfigFile (self, event):
        os.chdir (LastDirConfigFile)
        filename = LastConfigFile

        if (filename == ""):
            dial = wx.MessageDialog(None, 'No configuration file was defined', 'Problem opening configuration file', wx.OK | wx.ICON_HAND)
            dial.ShowModal()
        elif(os.path.isfile(filename) == False):
            dial = wx.MessageDialog(None, 'Configuration file ' + filename + ' not found', 'File not found', wx.OK | wx.ICON_HAND)
            dial.ShowModal()
        else:
            if WIN:
		auxstr = u''
                auxstr += 'notepad.exe "'
                auxstr += filename + '" '
                self.popenrun = subprocess.Popen( auxstr.encode('iso-8859-1') )
            else:
                if (os.path.isfile("/usr/bin/gedit") == True):
                    editor = "/usr/bin/gedit"
                elif (os.path.isfile("/usr/bin/kedit") == True):
                    editor = "/usr/bin/kedit"
                else:
                    editor = ""
                    self.ShowAlert('Alert', 'No editor found, I cannot open the file')
                
                if (editor != ""):
                    self.popenrun = subprocess.Popen([editor, filename])
        os.chdir (DireMain)  
        event.Skip()



##########################################################################
# Name        : GenerateParameterList
# Description : This parses all the different options collected in the GUI 
#               and generates a list with the parameters that could be sent
#               to the core processing module
#               An error checking is also included in some of the input 
#               parameters
##########################################################################        
    def GenerateParameterList(self, gLABParameters):
        global LastDirExecuted
        global LastOutputExecuted
        global DireMain
        global DireName
        
        #gLABParameters = []
        errorstr = ""
        warningstr = ""
        
        # Change directory to the DireName (in order to provide an output directory if no absolute file has been provided in the output)
        os.chdir(DireName)
        
        '''INPUTS'''
        if (self.RinexPath != ""):
            gLABParameters.extend(['-input:obs', self.RinexPath])
            if (os.path.isfile(self.RinexPath) == False):
                errorstr += "INPUT: RINEX Observation file not found.\n"
        else:
            errorstr += "INPUT: RINEX Observation file is a required input.\n"
            
        if (self.AntexPath != ""):
            gLABParameters.extend(['-input:ant', self.AntexPath])
            if (os.path.isfile(self.AntexPath) == False):
                errorstr += "INPUT: ANTEX file not found.\n"
        else:
            if ((self.SP3RB.GetValue() == True) | (self.ClocksRinexOrbitSP3RB.GetValue() == True)):
                if (self.SatellitePhaseCenterCorrection.GetValue()==True | (self.ReceiverPhaseCenterCorrection.GetValue()==True & self.ReadAntennaPhaseCenterRB.GetValue()==True)):
                    warningstr += "INPUT: Precise orbit and clock source is specified, but no ANTEX is given. gLAB will not be able to process the data, only to generate INPUT and CS messages. In order to process the data, please include an ANTEX file or switch to Broadcast orbit and clock source.\n"
        
        if ((self.BroadcastRB.GetValue()) == True):
            if (self.BroadcastPath != ""):
                gLABParameters.extend(['-input:nav', self.BroadcastPath])
                if (os.path.isfile(self.BroadcastPath) == False):
                    errorstr += "INPUT: RINEX Broadcast navigation file not found.\n"
            else:
                errorstr += "INPUT: RINEX Broadcast navigation file is a required input if 'Broadcast' orbit and clock source is specified.\n"
                 
        elif ((self.SP3RB.GetValue()) == True):
            if (self.SP3Path != ""):
                gLABParameters.extend(['-input:sp3', self.SP3Path])
                if (os.path.isfile(self.SP3Path) == False):
                    errorstr += "INPUT: sp3 file not found.\n"
            else:
                errorstr += "INPUT: sp3 file is a required input if 'Precise (1 file)' orbit and clock source is specified.\n"
            
        elif ((self.ClocksRinexOrbitSP3RB.GetValue()) == True):
            if (self.OrbitsSP3Path != ""):
                gLABParameters.extend(['-input:orb', self.OrbitsSP3Path])
                if (os.path.isfile(self.OrbitsSP3Path) == False):
                    errorstr += "INPUT: sp3 orbit file not found.\n"
            else:
                errorstr += "INPUT: sp3 orbit file is a required input if 'Precise (2 files)' orbit and clock source is specified.\n"
            if (self.ClocksRinexPath != ""):
                gLABParameters.extend(['-input:clk', self.ClocksRinexPath])
                if (os.path.isfile(self.ClocksRinexPath) == False):
                    errorstr += "INPUT: RINEX clocks file not found.\n"
            else:
                errorstr += "INPUT: RINEX clocks file is a required input if 'Precise (2 files)' orbit and clock source is specified.\n"
                
        if ((self.BroadcastSpecifyRB.GetValue()) == True):
            if (self.BroadcastIonosphereSourcePath != ''):
                if(self.IonosphericCorrectionComboBox.GetValue() == "Klobuchar (GPS)"):
                    gLABParameters.extend(['-input:klb', self.BroadcastIonosphereSourcePath])
                    if (os.path.isfile(self.BroadcastIonosphereSourcePath) == False):
                        errorstr += "INPUT: Broadcast navigation file for ionosphere not found.\n"
                if(self.IonosphericCorrectionComboBox.GetValue() == "Klobuchar (BeiDou)"):
                    gLABParameters.extend(['-input:bei', self.BroadcastIonosphereSourcePath])
                    if (os.path.isfile(self.BroadcastIonosphereSourcePath) == False):
                        errorstr += "INPUT: Broadcast navigation file for ionosphere not found.\n"
        
        if ((self.IonexRB.GetValue()) == True):
            if (self.IonexIonosphereSourcePath != ''):
                gLABParameters.extend(['-input:inx', self.IonexIonosphereSourcePath])
                if (os.path.isfile(self.IonexIonosphereSourcePath) == False):
                    errorstr += "INPUT: IONEX file for ionosphere not found.\n"

        if ((self.FPPPRB.GetValue()) == True):
            if (self.FPPPIonosphereSourcePath != ''):
                gLABParameters.extend(['-input:fpp', self.FPPPIonosphereSourcePath])
                if (os.path.isfile(self.FPPPIonosphereSourcePath) == False):
                    errorstr += "INPUT: FPPP file for ionosphere not found.\n"

        if ((self.ReceiverPositionUseRinexRB.GetValue()) == True):
            gLABParameters.extend(['-pre:setrecpos', 'RINEX'])

        elif ((self.ReceiverPositionSpecifyRB.GetValue()) == True):
            gLABParameters.extend(['-pre:setrecpos', self.PosX.GetValue(), self.PosY.GetValue(), self.PosZ.GetValue()])
            if ((self.PosX.GetValue() == "") | (self.PosY.GetValue() == "") | (self.PosZ.GetValue() == "")):
                errorstr += "INPUT: XYZ receiver position missing.\n"
            if ((isFloat(self.PosX.GetValue()) == False) | (isFloat(self.PosY.GetValue()) == False) | (isFloat(self.PosZ.GetValue()) == False)):
                errorstr += "INPUT: XYZ receiver position coordinates must be numbers.\n"
        
        elif ((self.ReceiverPositionCalculateRB.GetValue()) == True):
            gLABParameters.extend(['-pre:setrecpos', 'calculate'])
        
	elif ((self.ReceiverPositionUseSinexRB.GetValue()) == True): 
            if (self.SinexPath != ""):
		gLABParameters.extend(['-pre:setrecpos', 'SINEX'])
                gLABParameters.extend(['-input:snx', self.SinexPath])
                if (os.path.isfile(self.SinexPath) == False):
                    errorstr += "INPUT: SINEX file not found.\n"
            else:
                errorstr += "INPUT: SINEX file is a required input if 'Use SINEX File' as 'a priori receiver position' is specified.\n"

	# Check if processing is done with PC & LC: no need to correct DCBs because it is cancelled by definition. 
	NeedToDCBs = True
	if ( (self.PseudoRangeComboBox.GetStringSelection() != '' and self.PseudoRangeComboBox.GetValue() == 'PC' ) or (self.PseudoRangeCarrierPhaseComboBox.GetStringSelection() != '' and self.PseudoRangeCarrierPhaseComboBox.GetValue() == 'LC') ): NeedToDCBs = False


        if ( self.P1P2CorrectionCheckBox.GetValue() == True):
		if ( self.P1P2CorrectionComboBox.GetValue() == 'RINEX Nav File' ):

		    if ( self.P1P2SourcesComboBox.GetValue() == 'Broadcast (same as navigation)' ):
			gLABParameters.extend(['-model:dcb:p1p2', 'RINEX'])
		        gLABParameters.extend(['-input:dcb', self.BroadcastPath])
		        if (os.path.isfile(self.BroadcastPath) == False):
		            errorstr += "INPUT: RINEX Broadcast navigation file used as (P1 - P2) DCB source not found.\n"


		    elif ( self.P1P2SourcesComboBox.GetValue() == 'Broadcast (specify)' ):
		        if (self.TGDSourceP1P2Path != ''):
			    gLABParameters.extend(['-model:dcb:p1p2', 'RINEX'])
		            gLABParameters.extend(['-input:dcb', self.TGDSourceP1P2Path])
		            if (os.path.isfile(self.TGDSourceP1P2Path) == False):
		                errorstr += "INPUT: RINEX Broadcast navigation file used as (P1 - P2) DCB source is not found.\n"
		        else:
		            if ( NeedToDCBs == True ): errorstr += "INPUT: RINEX Broadcast navigation file is a required input if P1 - P2 Correction [In the Modelling Section] is set and 'Broadcast (specify)' DCB source is specified [In the Input Section].\n"


		elif ( self.P1P2CorrectionComboBox.GetValue() == 'DCB File' ):

		    if ( self.DCBSourceP1P2Path != '' ):
	   	        gLABParameters.extend(['-model:dcb:p1p2', 'DCB'])
		        gLABParameters.extend(['-input:dcb', self.DCBSourceP1P2Path])
		        if (os.path.isfile(self.DCBSourceP1P2Path) == False):
		            errorstr += "INPUT: ( P1 - P2 ) .DCB file used as (P1 - P2) DCB source not found.\n"
		    else:
		        if ( NeedToDCBs == True ): errorstr += "INPUT: ( P1 - P2 ) .DCB file is a required input if P1 - P2 Correction [In the Modelling Section] is set and 'Precise  .DCB files' is specified [In the Input Section].\n"

		elif ( self.P1P2CorrectionComboBox.GetValue() == 'IONEX File' ):

                    if ( self.P1P2SourcesComboBox.GetValue() == 'IONEX (same as ionosphere)' ):
                        gLABParameters.extend(['-model:dcb:p1p2', 'IONEX'])
                        gLABParameters.extend(['-input:dcb', self.IonexIonosphereSourcePath])
                        if (os.path.isfile(self.IonexIonosphereSourcePath) == False):
                            errorstr += "INPUT: IONEX file used as (P1 - P2) DCB source not found.\n"

                    elif ( self.P1P2SourcesComboBox.GetValue() == 'IONEX (specify)' ):
			if ( self.IonexSourceP1P2Path != '' ):
                            gLABParameters.extend(['-model:dcb:p1p2', 'IONEX'])
		            gLABParameters.extend(['-input:dcb', self.IonexSourceP1P2Path])
                            if (os.path.isfile(self.IonexSourceP1P2Path) == False):
                                errorstr += "INPUT: IONEX file used as (P1 - P2) DCB source not found.\n"
                        else:
		            if ( NeedToDCBs == True ): errorstr += "INPUT: ( P1 - P2 ) IONEX file is a required input if P1 - P2 Correction [In the Modelling Section] is set and 'IONEX (specify)' is specified [In the Input Section].\n"

		elif ( self.P1P2CorrectionComboBox.GetValue() == 'FPPP File' ):

                    if ( self.P1P2SourcesComboBox.GetValue() == 'FPPP (same as ionosphere)' ):
                        gLABParameters.extend(['-model:dcb:p1p2', 'FPPP'])
                        gLABParameters.extend(['-input:dcb', self.FPPPIonosphereSourcePath])
                        if (os.path.isfile(self.FPPPIonosphereSourcePath) == False):
                            errorstr += "INPUT: FPPP file used as (P1 - P2) DCB source not found.\n"

                    elif ( self.P1P2SourcesComboBox.GetValue() == 'FPPP (specify)' ):
                        if ( self.FPPPSourceP1P2Path != '' ):
                            gLABParameters.extend(['-model:dcb:p1p2', 'FPPP'])
                            gLABParameters.extend(['-input:dcb', self.FPPPSourceP1P2Path])
                            if (os.path.isfile(self.FPPPSourceP1P2Path) == False):
                                errorstr += "INPUT: FPPP file used as (P1 - P2) DCB source not found.\n"
                        else:
		            if ( NeedToDCBs == True ): errorstr += "INPUT: ( P1 - P2 ) FPPP file is a required input if P1 - P2 Correction [In the Modelling Section] is set and 'FPPP (specify)' is specified [In the Input Section].\n"


        else:
	   	gLABParameters.extend(['--model:dcb:p1p2'])

        if ( self.P1C1CorrectionCheckBox.GetValue() == True ): 
		if ( self.P1C1CorrectionComboBox.GetValue() == 'Flexible' ):
	   	    gLABParameters.extend(['-model:dcb:p1c1', 'flexible'])

		elif ( self.P1C1CorrectionComboBox.GetValue() == 'Strict' ):
	   	    gLABParameters.extend(['-model:dcb:p1c1', 'strict'])		   

		    if (self.AuxReceiverTypePath != ''):
		        gLABParameters.extend(['-input:rec', self.AuxReceiverTypePath])
		        if (os.path.isfile(self.AuxReceiverTypePath) == False):
		            errorstr += "INPUT: GPS Receiver Types file is not found.\n"
		    else:
		        errorstr += "INPUT: GPS Receiver Types file is a required input if Strict P1 - C1 Correction [In the Modelling Section] is set.\n"


		    if (self.DCBSourceP1C1Path != ''):
			gLABParameters.extend(['-input:dcb', self.DCBSourceP1C1Path])
			if (os.path.isfile(self.DCBSourceP1C1Path) == False):
			    errorstr += "INPUT: ( P1 - C1 ) .DCB file used as DCB source not found.\n"
		    else:
			errorstr += "INPUT: ( P1 - C1 ) .DCB file is a required input if Strict P1 - C1 Correction [In the Modelling Section] is set. \n"
        else:
	   	gLABParameters.extend(['--model:dcb:p1c1'])


        '''PREPROCESS'''
        
        if (self.DataDecimationCB.GetValue() == True): 
            gLABParameters.extend(['-pre:dec', self.DataDecimation.GetValue()])
            if (isInt(self.DataDecimation.GetValue(), 0) == False):
                errorstr += "PREPROCESS: Decimation must be a positive integer number.\n"
        else:
            gLABParameters.extend(['-pre:dec', '0'])      

        gLABParameters.extend(['-pre:elevation', self.ElevationMask.GetValue()])
        if (isFloat(self.ElevationMask.GetValue(), -90, 90) == False):
            errorstr += "PREPROCESS: Elevation mask must be a number between -90 and 90.\n"
      
        if (self.DiscardEclipsedSatellitesCB.GetValue() == True):
            gLABParameters.extend(['-pre:eclipse'])
        else:
            gLABParameters.extend(['--pre:eclipse'])

        if (self.DiscardUnhealthySatellitesCB.GetValue() == True):
            gLABParameters.extend(['-model:satellitehealth'])
        else:
            gLABParameters.extend(['--model:satellitehealth'])


        if (self.EnableLLI.GetValue() == True):
            gLABParameters.extend(['-pre:cs:lli'])
        else:
            gLABParameters.extend(['--pre:cs:lli'])


        gLABParameters.extend(['-pre:cs:datagap', self.DataGap.GetValue()])
        if (isFloat(self.DataGap.GetValue(), 0.5) == False):
            errorstr += "PREPROCESS: Data gap must be a positive number with a value of at least 0.5.\n"
        
        
        if (self.EnableLICB.GetValue() == True):       
            gLABParameters.extend(['-pre:cs:li'])
            gLABParameters.extend(['-pre:cs:li:maxjump', self.MaximumJumpThreshold])
            if (isFloat(self.MaximumJumpThreshold, 0) == False):
                errorstr += "PREPROCESS: Geometry-free CP Combination: Maximum Jump Threshold must be >= 0.\n"
            gLABParameters.extend(['-pre:cs:li:max', self.MaximumThresholdMetersLI])
            if (isFloat(self.MaximumThresholdMetersLI, 0) == False):
                errorstr += "PREPROCESS: Geometry-free CP Combination: Maximum Threshold must be >= 0.\n"
            gLABParameters.extend(['-pre:cs:li:t0', self.TimeConstant])
            if (isInt(self.TimeConstant, 1) == False):
                errorstr += "PREPROCESS: Geometry-free CP Combination: Time constant must be an integer with a value of at least 1.\n"
            gLABParameters.extend(['-pre:cs:li:samples', self.NsamplesLI])
            if (isInt(self.NsamplesLI, 3, 20) == False):
                errorstr += "PREPROCESS: Geometry-free CP Combination: Minimum samples must be an integer number between 3 and 20.\n"
        else:
            gLABParameters.extend(['--pre:cs:li'])
            

        if (self.EnableMelbourneWubbenaCB.GetValue() == True):  
            gLABParameters.extend(['-pre:cs:bw'])
            gLABParameters.extend(['-pre:cs:bw:sigmin', self.MinimumThresholdMetersMW])
            if (isFloat(self.MinimumThresholdMetersMW, 0.8) == False):
                errorstr += "PREPROCESS: Melbourne-Wubbena Combination: Minimum stdDev must be >= 0.8.\n"
            gLABParameters.extend(['-pre:cs:bw:timewindow', self.WindowMW])
            if (isInt(self.WindowMW, 1, 600) == False):
                errorstr += "PREPROCESS: Melbourne-Wubbena Combination: Window must be an integer number between 1 and 600.\n"
            gLABParameters.extend(['-pre:cs:bw:kfactor', self.SlopeMW])
            if (isFloat(self.SlopeMW, 0.000001) == False):
                errorstr += "PREPROCESS: Melbourne-Wubbena Combination: Deviation/Threshold Proportion must be strictly positive.\n"
            gLABParameters.extend(['-pre:cs:bw:samples', self.NsamplesMW])
            if (isInt(self.NsamplesMW, 2) == False):
                errorstr += "PREPROCESS: Melbourne-Wubbena Combination: Minimum samples must be an integer with a value of at least 2.\n"
        else:
            gLABParameters.extend(['--pre:cs:bw'])

        if (self.EnableC1L1DifferenceCB.GetValue() == True):  
            gLABParameters.extend(['-pre:cs:l1c1'])
            gLABParameters.extend(['-pre:cs:l1c1:init', self.MaximumThresholdC1L1Difference])
            if (isFloat(self.MaximumThresholdC1L1Difference, 0) == False):
                errorstr += "PREPROCESS: L1-C1 Difference: Initial stdDev must be >=0.\n"
            gLABParameters.extend(['-pre:cs:l1c1:timewindow', self.WindowC1L1Diference])
            if (isInt(self.WindowC1L1Diference, 1, 600) == False):
                errorstr += "PREPROCESS: L1-C1 Difference: Window must be an integer number between 1 and 600.\n"
            gLABParameters.extend(['-pre:cs:l1c1:kfactor', self.SlopeC1L1Diference])
            if (isFloat(self.SlopeC1L1Diference, 0.000001) == False):
                errorstr += "PREPROCESS: L1-C1 Difference: Slope must be strictly positive.\n"
            gLABParameters.extend(['-pre:cs:l1c1:samples', self.NsamplesL1C1])
            if (isInt(self.NsamplesL1C1, 0) == False):
                errorstr += "PREPROCESS: L1-C1 Difference: Minimum samples must be an integer with a value of at least 0.\n"
        else:
            gLABParameters.extend(['--pre:cs:l1c1'])

        for i in range(32):
            if (self.ButtonPRN[i].GetValue() == False):
                gLABParameters.extend(['-pre:sat', 'G' + str(i + 1)])
            
        '''MODELLING'''
        if ((self.SatelliteClockOffsetCorrection.GetValue()) == True):
            gLABParameters.extend(['-model:satclocks']) 

        else:
            gLABParameters.extend(['--model:satclocks']) 
            
        if ((self.SatelliteMovementFlightTime.GetValue()) == True):
            gLABParameters.extend(['-model:satmovinflight']) 
        else:
            gLABParameters.extend(['--model:satmovinflight']) 
             
        if ((self.EarthRotationFlightTime.GetValue()) == True):
            gLABParameters.extend(['-model:earthrotinflight']) 
        else:
            gLABParameters.extend(['--model:earthrotinflight']) 
                        
        if ((self.SatellitePhaseCenterCorrection.GetValue()) == True):
            gLABParameters.extend(['-model:satphasecenter']) 
        else:
            gLABParameters.extend(['--model:satphasecenter']) 
        
             
        if ((self.RelativisticCorrection.GetValue()) == True):
            gLABParameters.extend(['-model:relclock']) 
        else:
            gLABParameters.extend(['--model:relclock']) 
                    
        if ((self.IonosphericCorrectionCheckBox.GetValue()) == True):
            #gLABParameters.extend(['-model:iono', self.IonosphericCorrectionComboBox.GetValue() ])
            # If using Klobuchar and precise products and Broadcast (specify) and No broadcast file specified
            if (self.IonosphericCorrectionComboBox.GetValue() == "Klobuchar (GPS)"):
                gLABParameters.extend(['-model:iono', 'Klobuchar' ])
                if ((self.BroadcastIonosphereSourcePath == "") & (self.BroadcastSpecifyRB.GetValue() == True)): 
                    errorstr += "MODEL/INPUT: Klobuchar (GPS) ionospheric correction activated, but no ionosphere source is found. Please include one in Input section.\n"
            # If using BeiDou and precise products and Broadcast (specify) and No broadcast file specified
            elif (self.IonosphericCorrectionComboBox.GetValue() == "Klobuchar (BeiDou)"):
                gLABParameters.extend(['-model:iono', 'BeiDou' ])
                if ((self.BroadcastIonosphereSourcePath == "") & (self.BroadcastSpecifyRB.GetValue() == True)): 
                    errorstr += "MODEL/INPUT: Klobuchar (BeiDou) ionospheric correction activated, but no ionosphere source is found. Please include one in Input section.\n"
            # If using IONEX and and No IONEX file specified
            elif (self.IonosphericCorrectionComboBox.GetValue() == "IONEX"):
                gLABParameters.extend(['-model:iono', 'IONEX' ])
                if (self.IonexIonosphereSourcePath == "" ): 
                    errorstr += "MODEL/INPUT: IONEX ionospheric correction activated, but no ionosphere source is found. Please include one in Input section.\n"
            # If using FPPP and precise products and Broadcast (specify) and No broadcast file specified
            elif (self.IonosphericCorrectionComboBox.GetValue() == "FPPP"):
                gLABParameters.extend(['-model:iono', 'FPPP' ])
                if (self.FPPPIonosphereSourcePath == ""): 
                    errorstr += "MODEL/INPUT: FPPP ionospheric correction activated, but no ionosphere source is found. Please include one in Input section.\n"
		#if (isFloat(self.IonosphericMaxRMSTextCtrl.GetValue(), 0.000001) == False):  
                #    errorstr += "MODEL/INPUT: Maximum RMS must be strictly positive.\n"
		#else:
		    #gLABParameters.extend(['-model:ionoRMS', self.IonosphericMaxRMSTextCtrl.GetValue()])
        else:
            gLABParameters.extend(['--model:iono'])
             
        if ((self.TroposphericCorrectionCheckBox.GetValue()) == True):
            gLABParameters.extend(['-model:trop'])
            NominalTroposphericCorrection = ['UNB3', 'Simple']
            MappingTroposphericCorrection = ['Niell', 'Simple']
            gLABParameters.extend(['-model:trop:nominal', NominalTroposphericCorrection[ self.NominalTroposphericCorrectionComboBox.GetSelection() ]  ])
            gLABParameters.extend(['-model:trop:mapping', MappingTroposphericCorrection[ self.MappingTroposphericCorrectionComboBox.GetSelection() ]  ])
        else:
            gLABParameters.extend(['--model:trop'])
            
	"""                         
        if ((self.P1P2CorrectionCheckBox.GetValue()) == True ):
	    if (  self.P1P2CorrectionComboBox.GetStringSelection() == 'RINEX Nav' ):
	    	gLABParameters.extend(['-model:tgd','brdc'])  

            elif( self.P1P2CorrectionComboBox.GetStringSelection() == 'DCB Files'  and NeedToDCBs == True ):
             	gLABParameters.extend(['-model:tgd', 'dcb']) 
        else:
            gLABParameters.extend(['--model:tgd'])  
	"""

        if ((self.WindUpCorrection.GetValue()) == True):
            gLABParameters.extend(['-model:windup'])  
        else:
            gLABParameters.extend(['--model:windup'])  
            
        if ((self.SolidTidesCorrection.GetValue()) == True):
            gLABParameters.extend(['-model:solidtides']) 
        else:
            gLABParameters.extend(['--model:solidtides']) 
                   
        if ((self.GravitationalDelayCorrection.GetValue()) == True):
            gLABParameters.extend(['-model:relpath'])  
        else:
            gLABParameters.extend(['--model:relpath'])  

        if (self.PPPRB.GetValue() == True):
            gLABParameters.extend(['-model:orbit:deg', self.OrbitInterpolationDegreeTextCtrl.GetValue() ])
            if (isInt(self.OrbitInterpolationDegreeTextCtrl.GetValue(), 0, 19) == False):
                errorstr += "MODEL: Orbit interpolation degree must be an integer >=0 and <20.\n"
            gLABParameters.extend(['-model:clock:deg', self.ClockInterpolationDegreeTextCtrl.GetValue() ])
            if (isInt(self.ClockInterpolationDegreeTextCtrl.GetValue(), 0, 19) == False):
                errorstr += "MODEL: Clock interpolation degree must be an integer >=0 and <20.\n"
        
        if (self.ReceiverPhaseCenterCorrection.GetValue() == True):
            
            if ((self.SpecifyAntennaPhaseCenterRB.GetValue()) == True):        
                gLABParameters.extend(['-model:recphasecenter', '1', self.NorthPhaseCenterF1, self.EastPhaseCenterF1, self.UpPhaseCenterF1  ])                
                gLABParameters.extend(['-model:recphasecenter', '2', self.NorthPhaseCenterF2, self.EastPhaseCenterF2, self.UpPhaseCenterF2 ])
                if ((isFloat(self.NorthPhaseCenterF1) == False) | (isFloat(self.EastPhaseCenterF1) == False) | (isFloat(self.UpPhaseCenterF1) == False)):
                    errorstr += "MODEL: Receiver Antenna Phase Centre (F1) values must be numbers.\n"  
                if ((isFloat(self.NorthPhaseCenterF2) == False) | (isFloat(self.EastPhaseCenterF2) == False) | (isFloat(self.UpPhaseCenterF2) == False)):                    
                    errorstr += "MODEL: Receiver Antenna Phase Centre (F2) values must be numbers.\n" 

            elif(self.ReadAntennaPhaseCenterRB.GetValue() == True):
                gLABParameters.extend(['-model:recphasecenter', 'ANTEX' ]) 
                if (self.AntexPath == ""):
                    errorstr += "MODEL/INPUT: Receiver Antenna Phase Centre source is set as ANTEX, but no ANTEX file is found. Please include one in Input section.\n"
        else:
            gLABParameters.extend(['--model:recphasecenter'])


        if (self.ReceiverRefPointCorrection.GetValue() == True):
            if ((self.SpecifyAntennaReferencePointRB.GetValue()) == True):        
                gLABParameters.extend(['-model:arp', self.NorthReferencePointTextCtrl.GetValue(), self.EastReferencePointTextCtrl.GetValue(), self.UpReferencePointTextCtrl.GetValue()])    
                if ((isFloat(self.NorthReferencePointTextCtrl.GetValue()) == False) | (isFloat(self.EastReferencePointTextCtrl.GetValue()) == False) | (isFloat(self.UpReferencePointTextCtrl.GetValue()) == False)):                    
                    errorstr += "MODEL: Receiver Antenna Reference Point values must be numbers.\n" 
            elif(self.ReadAntennaReferencePointRB.GetValue() == True):
                gLABParameters.extend(['-model:arp', 'RINEX'])       
        else:
            gLABParameters.extend(['--model:arp'])
     
        '''FILTER'''
        
        #if ((self.SingleFrequencyRB.GetValue()) == True):
        #    '''gLABParameters.append(self.SingleFrequencyRB)  '''
        #    gLABParameters.extend(['-pre:availf', 'G1'])
        #elif ((self.DualFrequencyRB.GetValue()) == True):
        #    '''gLABParameters.append(self.DualFrequencyRB)  '''
        #    gLABParameters.extend(['-pre:availf', 'G12'])
        
        if ((self.UseSigmaIonoCB.GetValue()) == True):
            gLABParameters.extend(['-filter:SigmaIono'])
        else:
            gLABParameters.extend(['--filter:SigmaIono'])
        
        if ((self.EstimateTroposphereCB.GetValue()) == True):
            gLABParameters.extend(['-filter:trop']) 
            gLABParameters.extend(['-filter:phi:trop', self.TroposherePhiTC.GetValue()])
            if (isFloat(self.TroposherePhiTC.GetValue(), 0) == False):
                errorstr += "FILTER: Filter Parameters: Troposphere Phi must be >=0.\n"
            gLABParameters.extend(['-filter:q:trop', self.TroposhereNoiseTC.GetValue()])
            if (isFloat(self.TroposhereNoiseTC.GetValue(), 0) == False):
                errorstr += "FILTER: Filter Parameters: Troposphere Noise must be >=0.\n"
            gLABParameters.extend(['-filter:p0:trop', self.TroposherePoTC.GetValue()])
            if (isFloat(self.TroposherePoTC.GetValue(), 0) == False):
                errorstr += "FILTER: Filter Parameters: Troposphere Po must be >=0.\n"
        else:
            gLABParameters.extend(['--filter:trop'])

        if ((self.StaticReceiverRB.GetValue()) == True):
            gLABParameters.extend(['-filter:nav', 'static'])
            
        elif ((self.KinematicReceiverRB.GetValue()) == True):
            gLABParameters.extend(['-filter:nav', 'kinematic'])
        
        # Backward / Forward filtering type
        if (self.BackwardCB.GetValue()==True):
            gLABParameters.extend(['-filter:backward'])
        else:
            gLABParameters.extend(['--filter:backward'])
        
        gLABParameters.extend(['-filter:phi:dr', self.PositionPhiTC.GetValue() ])  
        if (isFloat(self.PositionPhiTC.GetValue(), 0) == False):
                errorstr += "FILTER: Filter Parameters: Coordinates Phi must be >=0.\n"
        gLABParameters.extend(['-filter:q:dr', self.PositionNoiseTC.GetValue()]) 
        if (isFloat(self.PositionNoiseTC.GetValue(), 0) == False):
                errorstr += "FILTER: Filter Parameters: Coordinates Noise must be >=0.\n"        
        gLABParameters.extend(['-filter:p0:dr', self.PositionPoTC.GetValue() ])  
        if (isFloat(self.PositionPoTC.GetValue(), 0) == False):
                errorstr += "FILTER: Filter Parameters: Coordinates Po must be >=0.\n"
        
        gLABParameters.extend(['-filter:phi:clk', self.ReceiverClockPhiTC.GetValue()])
        if (isFloat(self.ReceiverClockPhiTC.GetValue(), 0) == False):
                errorstr += "FILTER: Filter Parameters: Receiver Clock Phi must be >=0.\n"
        gLABParameters.extend(['-filter:q:clk', self.ReceiverClockNoiseTC.GetValue()])  
        if (isFloat(self.ReceiverClockNoiseTC.GetValue(), 0) == False):
                errorstr += "FILTER: Filter Parameters: Receiver Clock Noise must be >=0.\n"
        gLABParameters.extend(['-filter:p0:clk', self.ReceiverClockPoTC.GetValue()])
        if (isFloat(self.ReceiverClockPoTC.GetValue(), 0) == False):
                errorstr += "FILTER: Filter Parameters: Receiver Clock Po must be >=0.\n"
        
        
        if ((self.ElevationWeightPseudoRangeRB.GetValue()) == True):
            gLABParameters.extend(['-filter:elevweight', '1', self.AElevationWeightPseudoRangeTC.GetValue(), self.BElevationWeightPseudoRangeTC.GetValue(), self.CElevationWeightPseudoRangeTC.GetValue() ])  
            if ((isFloat(self.AElevationWeightPseudoRangeTC.GetValue(), 0) == False) | (isFloat(self.BElevationWeightPseudoRangeTC.GetValue(), 0) == False) | (isFloat(self.CElevationWeightPseudoRangeTC.GetValue(), 0.000001) == False)):
                errorstr += "FILTER: Elevation-dependant standard deviation (pseudorange) values must be positive for a and b, and strictly positive for c.\n"
                
        elif ((self.FixedWeightPseudoRangeRB.GetValue()) == True):
            gLABParameters.extend(['-filter:fixedweight', '1', self.FixedWeightPseudoRangeTC.GetValue()])  
            if (isFloat(self.FixedWeightPseudoRangeTC.GetValue(), 0.000001) == False):
                errorstr += "FILTER: Fixed standard deviation (pseudorange) must be strictly positive.\n"

        if ((self.PseudoRangeRB.GetValue()) == True):
            gLABParameters.extend(['-filter:meas', 'pseudorange'])
            gLABParameters.extend(['-filter:select', '1', self.PseudoRangeComboBox.GetValue()])
                
        elif ((self.PseudoRangeCarrierPhaseRB.GetValue()) == True):
            gLABParameters.extend(['-filter:meas', 'carrierphase'])
            gLABParameters.extend(['-filter:select', '2', self.PseudoRangeComboBox.GetValue(), self.PseudoRangeCarrierPhaseComboBox.GetValue()])
            if ((self.SingleFrequencyRB.GetValue() == True) & (self.EnableLICB.GetValue() == True)):
                warningstr += "FILTER/PREPROCESS: Single-frequency carrier phase measurements are used, but Geometry-free CP Combination is marked to be used to detect cycle-slips. This combination requires dual-frequency data so it will not be used. You can disable it in the PREPROCESS section to avoid this message.\n"
            if ((self.SingleFrequencyRB.GetValue() == True) & (self.EnableMelbourneWubbenaCB.GetValue() == True)):
                warningstr += "FILTER/PREPROCESS: Single-frequency carrier phase measurements are used, but Melbourne-Wübbena is marked to be used to detect cycle-slips. This combination requires dual-frequency data so it will not be used. You can disable it in the PREPROCESS section to avoid this message.\n"
            if ((self.SingleFrequencyRB.GetValue() == True) & (self.EnableC1L1DifferenceCB.GetValue() == False)):
                warningstr += "FILTER/PREPROCESS: Single-frequency carrier phase measurements are used, but no single-frequency cycle-slip detector has been activated. To do so, activate L1-C1 difference as cycle-slip detector in the PREPROCESS section.\n"
            if ((self.DualFrequencyRB.GetValue() == True) & (self.EnableMelbourneWubbenaCB.GetValue() == False) & (self.EnableLICB.GetValue() == False)):
                warningstr += "FILTER/PREPROCESS: Dual-frequency carrier phase measurements are used, but no dual-frequency cycle-slip detector has been activated. To do so, activate the Geometry-free CP Combination and/or the Melbourne-Wübbena Combination as cycle-slip detector in the PREPROCESS section.\n"
                 
            gLABParameters.extend(['-filter:phi:amb', self.PhaseAmbiguitiesPhiTC.GetValue()])
            if (isFloat(self.PhaseAmbiguitiesPhiTC.GetValue(), 0) == False):
                errorstr += "FILTER: Filter Parameters: Phase ambiguities Phi must be >=0.\n"
            gLABParameters.extend(['-filter:q:amb', self.PhaseAmbiguitiesNoiseTC.GetValue()])  
            if (isFloat(self.PhaseAmbiguitiesNoiseTC.GetValue(), 0) == False):
                errorstr += "FILTER: Filter Parameters: Phase ambiguities Noise must be >=0.\n"
            gLABParameters.extend(['-filter:p0:amb', self.PhaseAmbiguitiesPoTC.GetValue()])
            if (isFloat(self.PhaseAmbiguitiesPoTC.GetValue(), 0) == False):
                errorstr += "FILTER: Filter Parameters: Phase ambiguities Po must be >=0.\n"
                
            if ((self.ElevationWeightPseudoRangeCarrierPhaseRB.GetValue()) == True):
                gLABParameters.extend(['-filter:elevweight', '2', self.AElevationWeightPseudoRangeCarrierPhaseTC.GetValue(), self.BElevationWeightPseudoRangeCarrierPhaseTC.GetValue(), self.CElevationWeightPseudoRangeCarrierPhaseTC.GetValue() ])
                if ((isFloat(self.AElevationWeightPseudoRangeCarrierPhaseTC.GetValue(), 0) == False) | (isFloat(self.BElevationWeightPseudoRangeCarrierPhaseTC.GetValue(), 0) == False) | (isFloat(self.CElevationWeightPseudoRangeCarrierPhaseTC.GetValue(), 0.000001) == False)):
                    errorstr += "FILTER: Elevation-dependant standard deviation (carrier phase) values must be positive for a and b, and strictly positive for c.\n"
                
            elif ((self.FixedWeightPseudoRangeCarrierPhaseRB.GetValue()) == True):
                gLABParameters.extend(['-filter:fixedweight', '2', self.FixedWeightPseudoRangeCarrierPhaseTC.GetValue()])                  
                if (isFloat(self.FixedWeightPseudoRangeCarrierPhaseTC.GetValue(), 0.000001) == False):
                    errorstr += "FILTER: Fixed standard deviation (carrier phase) must be strictly positive.\n"
        
        if ((self.PseudoRangeSmoothingCB.GetValue()) == True):
            gLABParameters.extend(['-pre:smooth', self.PseudoRangeSmoothingTC.GetValue()])
            if (isInt(self.PseudoRangeSmoothingTC.GetValue(), 0) == False):
                errorstr += "FILTER: Pseudorange smoothing epochs must be a positive integer.\n"
            if ((self.PseudoRangeComboBox.GetValue() == 'PC') ):
                gLABParameters.extend(['-pre:smoothMeas', '1', 'LC'])
            elif ((self.PseudoRangeComboBox.GetValue() == 'C1C') | (self.PseudoRangeComboBox.GetValue() == 'C1P')):
                gLABParameters.extend(['-pre:smoothMeas', '1', 'L1P'])
            elif ((self.PseudoRangeComboBox.GetValue() == 'C2C') | (self.PseudoRangeComboBox.GetValue() == 'C2P')):
                gLABParameters.extend(['-pre:smoothMeas', '1', 'L2P'])
        else:
            gLABParameters.extend(['-pre:smooth', '0'])


        '''OUTPUT'''
        if (self.OutputDestinationPath != " "):
            gLABParameters.extend(['-output:file', self.OutputDestinationPath]) 
            if (self.OutputDestinationPath == ""):
                warningstr += "OUTPUT: No output file is given, all the output of gLAB will be directed to the standard output.\n"
        
        if ((self.InfoMessage.GetValue()) == True):
            gLABParameters.extend(['-print:info'])   
        else: 
            gLABParameters.extend(['--print:info'])   
                       
        if ((self.CSMessage.GetValue()) == True):
            gLABParameters.extend(['-print:cycleslips'])   
        else: 
            gLABParameters.extend(['--print:cycleslips'])   
            
        if ((self.InputMessage.GetValue()) == True):
            gLABParameters.extend(['-print:input'])   
        else: 
            gLABParameters.extend(['--print:input'])   

        if ((self.ModelMessage.GetValue()) == True):
            gLABParameters.extend(['-print:model']) 
        else: 
            gLABParameters.extend(['--print:model']) 
   
        if ((self.EpochSatMessage.GetValue()) == True):
            gLABParameters.extend(['-print:satellites']) 
        else: 
            gLABParameters.extend(['--print:satellites']) 
                
        if ((self.PreFitMessage.GetValue()) == True):
            gLABParameters.extend(['-print:prefit']) 
        else: 
            gLABParameters.extend(['--print:prefit']) 
   
        if ((self.PostFitMessage.GetValue()) == True):
            gLABParameters.extend(['-print:postfit'])  
        else: 
            gLABParameters.extend(['--print:postfit'])  
    
        if ((self.FilterMessage.GetValue()) == True):
            gLABParameters.extend(['-print:filter']) 
        else: 
            gLABParameters.extend(['--print:filter']) 
          
        if ((self.OutputMessage.GetValue()) == True):
            gLABParameters.extend(['-print:output'])  
        else: 
            gLABParameters.extend(['--print:output'])  

        if ((self.SatDifMessage.GetValue()) == True):
            gLABParameters.extend(['-print:satdiff'])  
        else: 
            gLABParameters.extend(['--print:satdiff'])  
   
        if ((self.SatStatMessage.GetValue()) == True):
            gLABParameters.extend(['-print:satstat'])  
        else: 
            gLABParameters.extend(['--print:satstat'])  
            
        if ((self.SatStatTotMessage.GetValue()) == True):
            gLABParameters.extend(['-print:satstattot'])  
        else: 
            gLABParameters.extend(['--print:satstattot'])  

        if ((self.SatMEASMessage.GetValue()) == True):
            gLABParameters.extend(['-print:meas'])  
        else: 
            gLABParameters.extend(['--print:meas'])  
	
    	os.chdir(DireMain)
        return (errorstr, warningstr)
    
##########################################################################
# Name        : LaunchgLAB
# Description : This parses all the different options collected in the GUI 
#               to the core processing module of the program 
#               An error checking is also included in some of the input 
#               parameters
##########################################################################
    def LaunchgLAB(self, event):
        global LastDirExecuted
        global LastOutputExecuted
        global DireMain
        global DireName
        
        gLABParameters = []
        errorstr = ""
        warningstr = ""
        
        errorstr, warningstr = self.GenerateParameterList(gLABParameters)

        # Change directory to the DireName (in order to provide an output directory if no absolute file has been provided in the output)
        os.chdir(DireName)
        
        self.EnableCalculusButtons(0, 0)                               
        # Show error alert, if any
        if (errorstr != ""):
            showstr = "gLAB has found the following errors, please correct them before processing again:\n\n" + errorstr
            dial = wx.MessageDialog(None, showstr, 'Errors found', wx.OK | wx.ICON_HAND)
            dial.ShowModal()
            self.EnableCalculusButtons(1, 0)
        else:  
            if (warningstr != ""):
                showstr = "gLAB has found the following warnings:\n\n" + warningstr + "\nDo you want to execute gLAB anyway?"
                dial = wx.MessageDialog(None, showstr, 'Warning', wx.YES_NO | wx.NO_DEFAULT)
                ret = dial.ShowModal()
            else:
                ret = wx.ID_YES
            if ret == wx.ID_YES:
                # Executing gLAB
                LastDirExecuted = DireName
                LastOutputExecuted = self.OutputDestinationPath
                if WIN:

		    """
		    auxstr = u''
                    auxstr += DireMain + '\gLAB.exe '
		    """
		    auxstr = u''
		    auxstr += '"'
		    auxstr += DireMain.encode('utf-8') 
                    auxstr += '\gLAB.exe" '

                    for i in gLABParameters:
                        auxstr = auxstr + '"' + i + '" '
		    try:
                        self.popenrun = subprocess.Popen( auxstr.encode('iso-8859-1') )
		    except:
                        self.EnableCalculusButtons(1, 0)
                else:
                    RunParametersList = [gBinElements[5]]
                    RunParametersList.extend(gLABParameters)
		    try:
                        self.popenrun = subprocess.Popen(RunParametersList)
		    except:
                        self.EnableCalculusButtons(1, 0)

                ThreadRun = threading.Timer(0, function=self.WaitRun)
                ThreadRun.start()
            else:
                self.EnableCalculusButtons(1, 0)
        os.chdir(DireMain)
        
##########################################################################
# Name        : EnableCalculusButtons
# Description : This function simply activates/deactivates the buttons of
#               the calculus tab
##########################################################################
    def EnableCalculusButtons(self, run=1, output=1):
        if (run):
            self.RunButton.Enable()
        else:
            self.RunButton.Disable()
            
        if (output):
            self.ShowOutputButton.Enable()
        else:
            self.ShowOutputButton.Disable()

###########################################################################
# Name        : WaitRun
# Description : This functions checks if the core processing has finished. 
#               Once calculations are finished, it enables the run and 
#               show output buttons 
###########################################################################
    def WaitRun (self):
        errorstr = ''
        
        try:
            self.popenrun.wait()
        except:
            pass
        
        if (self.popenrun.returncode != 1):
            os.chdir(DireName)
            if (self.OutputDestinationPath == ""):
                errorstr = 'Output is not directed to a file, cannot determine the cause of the error.'
            elif (os.path.isfile(self.OutputDestinationPath) == False):
                errorstr = 'Output file not found, cannot determine the cause of the error.'
            else:
                f = open(self.OutputDestinationPath, 'r')
                try:
                    f.seek(-1000, 2) # Go to the 1000th byte before the end (ERROR message will be the last on the output)
                except:
                    
                    f.seek(0, 0)
                for line in f:
                    if (line[0:6] == "ERROR "):
                        errorstr = line
                f.close()
            
            if (errorstr == ''):
                errorstr = 'Undetermined error source.'
            showstr = 'Problem executing gLAB Processing Data Core (DPC), the following error have been found:\n\n' + errorstr
            dial = wx.MessageDialog(None, showstr, 'Errors found', wx.OK | wx.ICON_HAND)
            wx.CallAfter(dial.ShowModal)
            os.chdir(DireMain)

        wx.CallAfter(self.EnableCalculusButtons)
##########################################################################
# Name        : ShowPseudoRangeSmoothingTC
# Description : Shows or hides the text control that gets the number of  
#               epochs to be smoothed in the PseudoRange
##########################################################################
    def ShowPseudoRangeSmoothingTC(self, event=None):
        if (self.PseudoRangeSmoothingCB.GetValue() == True):
            self.PseudoRangeSmoothingTC.Show()
            self.EnableLLI.SetValue(True)
            if (self.SingleFrequencyRB.GetValue() == True):
                self.EnableC1L1DifferenceCB.SetValue(True)
                self.EnableLICB.SetValue(False)
                self.EnableMelbourneWubbenaCB.SetValue(False)
            if (self.DualFrequencyRB.GetValue() == True):
                self.EnableC1L1DifferenceCB.SetValue(False)
                self.EnableLICB.SetValue(True)
                self.EnableMelbourneWubbenaCB.SetValue(True)
        else:
            self.PseudoRangeSmoothingTC.Hide()
            if ((self.PseudoRangeCarrierPhaseRB.GetValue()) == False):
                self.EnableLLI.SetValue(False)
                if (self.SingleFrequencyRB.GetValue() == True):
                    self.EnableC1L1DifferenceCB.SetValue(False)
                if (self.DualFrequencyRB.GetValue() == True):
                    self.EnableLICB.SetValue(False)
                    self.EnableMelbourneWubbenaCB.SetValue(False)
        
        
##########################################################################
# Name        : SetParametersTexts
# Description : Some default parameters when the Static or the Kinematic  
#               receiver modes are selected 
##########################################################################  
    def SetParametersTexts(self, event=None):
            
        if (self.StaticReceiverRB.GetValue() == True):
            self.PositionPhiTC.SetValue('1') 
            self.PositionNoiseTC.SetValue('0')
            
        if (self.KinematicReceiverRB.GetValue() == True):
            self.PositionPhiTC.SetValue('0') 
            self.PositionNoiseTC.SetValue('1e8')

##########################################################################
# Name        : ShowConfigurationTexts
# Description : Some default parameters when the PPP or the Standalone
#               Navigation modes are selected, and if single or dual
#               frequencies are selected 
#        
#	        self.PseudoRangeComboBoxList = ['C1C', 'PC']
#	        self.PseudoRangeCarrierPhaseBoxList = ['G1C', 'LC', 'L1P']
#
#	        self.PseudoRangeComboBoxList = ['C1C','C2C','C1P','C2P','PC','PCC']
#	        self.PseudoRangeCarrierPhaseBoxList = ['L1C','L1P','L2C','L2P','G1C', 'LC']
##########################################################################  
    def ShowConfigurationTexts (self):


        #Pseudo Range & Carrier Phase Combo Boxes List Management:
        self.PseudoRangeComboBox.SetSelection(0) 
        self.PseudoRangeCarrierPhaseComboBox.SetSelection(0) 
        self.PseudoRangeComboBox.Clear()
        self.PseudoRangeCarrierPhaseComboBox.Clear()

        if (self.SingleFrequencyRB.GetValue() == True):
            if (self.PseudoRangeSmoothingCB.GetValue() == True or self.PseudoRangeCarrierPhaseRB.GetValue() == True):
                self.EnableC1L1DifferenceCB.SetValue(True)
                self.EnableLICB.SetValue(False)
                self.EnableMelbourneWubbenaCB.SetValue(False)
            self.PseudoRangeComboBox.AppendItems(self.PseudoRangeComboBoxListSingleFreq);
            self.PseudoRangeCarrierPhaseComboBox.AppendItems(self.PseudoRangeCarrierPhaseBoxListSingleFreq);
            self.PseudoRangeComboBox.SetSelection(0) 
            self.PseudoRangeCarrierPhaseComboBox.SetSelection(0) 

        elif (self.DualFrequencyRB.GetValue() == True):
            if (self.PseudoRangeSmoothingCB.GetValue() == True or self.PseudoRangeCarrierPhaseRB.GetValue() == True):
                self.EnableC1L1DifferenceCB.SetValue(False)
                self.EnableLICB.SetValue(True)
                self.EnableMelbourneWubbenaCB.SetValue(True)
            self.PseudoRangeComboBox.AppendItems(self.PseudoRangeComboBoxListDoubleFreq);
            self.PseudoRangeCarrierPhaseComboBox.AppendItems(self.PseudoRangeCarrierPhaseBoxListDoubleFreq);
            self.PseudoRangeComboBox.SetSelection(0) 
            self.PseudoRangeCarrierPhaseComboBox.SetSelection(0)        
        
        if (self.PPPRB.GetValue() == True):
            if (self.SingleFrequencyRB.GetValue() == True):
                self.PseudoRangeComboBox.SetValue('C1C') 
                self.PseudoRangeCarrierPhaseComboBox.SetValue('G1C') 
                self.FixedWeightPseudoRangeTC.SetValue('100')    
                self.FixedWeightPseudoRangeCarrierPhaseTC.SetValue('0.5')
            
            if (self.DualFrequencyRB.GetValue() == True):
                self.PseudoRangeComboBox.SetValue('PC')
                self.PseudoRangeCarrierPhaseComboBox.SetValue('LC')
                self.FixedWeightPseudoRangeTC.SetValue('1')    
                self.FixedWeightPseudoRangeCarrierPhaseTC.SetValue('0.01')
            
            # If PPP is set and if any precise input is not selected (from Input section), select "Precise (1 file)"    
            if (self.BroadcastRB.GetValue() == True):
                self.SP3RB.SetValue(True)
                self.ShowOrbitClockSourceFunc()
                
        if (self.StandaloneRB.GetValue() == True):            
            if (self.SingleFrequencyRB.GetValue() == True):
                self.PseudoRangeComboBox.SetValue('C1C') 
                self.PseudoRangeCarrierPhaseComboBox.SetValue('L1P') 
                self.FixedWeightPseudoRangeTC.SetValue('1')    
                self.FixedWeightPseudoRangeCarrierPhaseTC.SetValue('0.01') 
            
            if (self.DualFrequencyRB.GetValue() == True):
                self.PseudoRangeComboBox.SetValue('PC') 
                self.PseudoRangeCarrierPhaseComboBox.SetValue('LC') 
                self.FixedWeightPseudoRangeTC.SetValue('1')    
                self.FixedWeightPseudoRangeCarrierPhaseTC.SetValue('0.01')     
                
            # If Standalone is set and if broadcast input is not selected (from Input section), select "Broadcast"    
            if (self.BroadcastRB.GetValue() == False):
                self.BroadcastRB.SetValue(True)
                self.ShowOrbitClockSourceFunc()

##########################################################################
# Name        : SetConfigurationTexts
# Description : Handling function for radiobuttons of the Filter section 
##########################################################################  
    def SetConfigurationTexts(self, event):
        self.ShowConfigurationTexts()
        event.Skip()
                
                   
##########################################################################
# Name        : ShowPseudoRangeCarrierPhase
# Description : The options that are showed or hidden when the measurements
#               selection are chosen from just Pseudorange or 
#               Pseudorange + carrier phase
########################################################################## 
    def ShowPseudoRangeCarrierPhase(self, event=None):
         
        if ((self.PseudoRangeRB.GetValue()) == True):
            if ((self.PseudoRangeSmoothingCB.GetValue()) == False):
                self.EnableLLI.SetValue(False)
                self.EnableC1L1DifferenceCB.SetValue(False)
                self.EnableLICB.SetValue(False)
                self.EnableMelbourneWubbenaCB.SetValue(False)
            self.PhaseAmbiguitiesText.Hide() 
            self.PhaseAmbiguitiesPhiTC.Hide()
            self.PhaseAmbiguitiesNoiseTC.Hide()
            self.PhaseAmbiguitiesNoiseUnitsText.Hide()     
            self.PhaseAmbiguitiesPoTC.Hide()
            self.PhaseAmbiguitiesPoUnitsText.Hide()   
        else: 
            self.PhaseAmbiguitiesText.Show() 
            self.PhaseAmbiguitiesPhiTC.Show()
            self.PhaseAmbiguitiesNoiseTC.Show()
            self.PhaseAmbiguitiesNoiseUnitsText.Show()     
            self.PhaseAmbiguitiesPoTC.Show()
            self.PhaseAmbiguitiesPoUnitsText.Show()   
        
        if ((self.PseudoRangeCarrierPhaseRB.GetValue()) == True):
            self.EnableLLI.SetValue(True)
            if (self.SingleFrequencyRB.GetValue() == True):
                self.EnableC1L1DifferenceCB.SetValue(True)
                self.EnableLICB.SetValue(False)
                self.EnableMelbourneWubbenaCB.SetValue(False)
            if (self.DualFrequencyRB.GetValue() == True):
                self.EnableC1L1DifferenceCB.SetValue(False)
                self.EnableLICB.SetValue(True)
                self.EnableMelbourneWubbenaCB.SetValue(True)
            self.PseudoRangeComboBox.Show()
            self.FixedWeightPseudoRangeRB.Show()
            self.FixedWeightPseudoRangeTC.Show()
            self.PseudoRangeCarrierPhaseComboBox.Show()
            self.FixedWeightPseudoRangeCarrierPhaseRB.Show()
            self.FixedWeightPseudoRangeCarrierPhaseTC.Show()
            
            self.ElevationWeightPseudoRangeCarrierPhaseRB.Show()
            self.AElevationWeightPseudoRangeCarrierPhaseTC.Show()
            self.BElevationWeightPseudoRangeCarrierPhaseTC.Show()
            self.CElevationWeightPseudoRangeCarrierPhaseTC.Show()
            
        else: 
            self.PseudoRangeCarrierPhaseComboBox.Hide()
            self.FixedWeightPseudoRangeCarrierPhaseRB.Hide()
            self.FixedWeightPseudoRangeCarrierPhaseTC.Hide()
            
            self.ElevationWeightPseudoRangeCarrierPhaseRB.Hide()
            self.AElevationWeightPseudoRangeCarrierPhaseTC.Hide()
            self.BElevationWeightPseudoRangeCarrierPhaseTC.Hide()
            self.CElevationWeightPseudoRangeCarrierPhaseTC.Hide()

##########################################################################
# Name        : ShowTroposphereCheckBox
# Description : Phi, Q and Po can be hidden or shown depending if the user 
#               estimates or not the troposphere 
########################################################################## 
    def ShowTroposphereCheckBox(self, event=None):
        if ((self.EstimateTroposphereCB.GetValue()) == True):
            self.TroposphereText.Show() 
            self.TroposherePhiTC.Show()
            self.TroposhereNoiseTC.Show()
            self.TroposphereNoiseUnitsText.Show()     
            self.TroposherePoTC.Show()
            self.TropospherePoUnitsText.Show()  
        else: 
            self.TroposphereText.Hide() 
            self.TroposherePhiTC.Hide()
            self.TroposhereNoiseTC.Hide()
            self.TroposphereNoiseUnitsText.Hide()
            self.TroposphereNoiseUnitsText.Hide()     
            self.TroposherePoTC.Hide()
            self.TropospherePoUnitsText.Hide()     
     
    
##########################################################################
# Name        : ShowTroposphericCorrectionCheckBox
# Description : This functions hides or shows the tropospheric correction 
#               in the modeling options, enabling it or not 
##########################################################################    
    def ShowTroposphericCorrectionCheckBox(self, event):
        if ((self.TroposphericCorrectionCheckBox.GetValue()) == True):
            self.MappingTroposphericCorrectionComboBox.Show() 
            self.NominalTroposphericCorrectionComboBox.Show() 
        else: 
            self.MappingTroposphericCorrectionComboBox.Hide()  
            self.NominalTroposphericCorrectionComboBox.Hide() 

##########################################################################
# Name        : ShowP1P2CorrectionCheckBox
# Description : This functions hides or shows the TGD correction 
#               in the modelling options, enabling it or not 
##########################################################################    
    def ShowP1P2CorrectionCheckBox(self, event):
        if ((self.P1P2CorrectionCheckBox.GetValue()) == True):
            self.P1P2CorrectionComboBox.Show() 
        else: 
            self.P1P2CorrectionComboBox.Hide()  

##########################################################################
# Name        : 
# Description : This functions hides or shows the P1C1 DCB correction 
#               in the modelling options, enabling it or not 
##########################################################################    
    def ShowP1C1CorrectionCheckBox(self, event):
        if ( self.P1C1CorrectionCheckBox.GetValue() == True):
            self.P1C1CorrectionComboBox.Show() 
        else: 
            self.P1C1CorrectionComboBox.Hide()  

##########################################################################
# Name        : ShowIonosphericCorrectionCheckBox
# Description : This functions hides or shows the ionospheric correction 
#               in the modelling options, enabling it or not
##########################################################################    
    def ShowIonosphericCorrectionCheckBox(self, event):
        if ((self.IonosphericCorrectionCheckBox.GetValue()) == True):
            self.IonosphericCorrectionComboBox.Show() 
	    #if ((self.IonosphericCorrectionComboBox.GetValue()) == "FPPP" ):
            #    self.IonosphericMaxRMSText.Show() 
            #    self.IonosphericMaxRMSTextCtrl.Show() 
	    #else:
            #    self.IonosphericMaxRMSText.Hide() 
            #    self.IonosphericMaxRMSTextCtrl.Hide() 
	    self.ShowIonosphereSourceCB.SetValue(True)
	    
        else:
            self.IonosphericCorrectionComboBox.Hide()  
            #self.IonosphericMaxRMSText.Hide() 
            #self.IonosphericMaxRMSTextCtrl.Hide() 
	    self.ShowIonosphereSourceCB.SetValue(False)

        self.ShowIonosphereSourceFunc() 


##########################################################################
# Name        : SelectIonosphericCorrections
# Description : A call to Show Ionosphere SelectIonosphericCorrections 
#		Function 
##########################################################################    
    def SelectIonosphericCorrections(self, event):
        self.SelectIonosphericCorrectionsFunc()
        event.Skip()

##########################################################################
# Name        : SelectIonosphericCorrectionsFunc
# Description : The user can select different Iono Corrections, here are
#               are managed different options.
##########################################################################      
    def SelectIonosphericCorrectionsFunc(self):
        if ( self.IonosphericCorrectionComboBox.GetSelection() == 0):
            if ( self.BroadcastSameRB.Enabled == False ):
		self.BroadcastSpecifyRB.SetValue(True)
		self.IonexRB.SetValue(False)
		self.FPPPRB.SetValue(False)
	    elif (self.BroadcastSpecifyRB.GetValue() == True):
	        self.BroadcastSameRB.SetValue(False)
		self.IonexRB.SetValue(False)
		self.FPPPRB.SetValue(False)
	    else:
	        self.BroadcastSameRB.SetValue(True)
		self.IonexRB.SetValue(False)
		self.FPPPRB.SetValue(False)
	    self.UseSigmaIonoCB.SetValue(False)
	    #self.IonosphericMaxRMSText.Hide()
            #self.IonosphericMaxRMSTextCtrl.Hide()
        elif ( self.IonosphericCorrectionComboBox.GetSelection() == 1):
            if ( self.BroadcastSameRB.Enabled == False ):
                self.BroadcastSpecifyRB.SetValue(True)
                self.IonexRB.SetValue(False)
                self.FPPPRB.SetValue(False)
	    elif (self.BroadcastSpecifyRB.GetValue() == True):
	        self.BroadcastSameRB.SetValue(False)
		self.IonexRB.SetValue(False)
	    else:
	        self.BroadcastSameRB.SetValue(True)
		self.IonexRB.SetValue(False)
		self.FPPPRB.SetValue(False)
	    self.UseSigmaIonoCB.SetValue(False)
	    #self.IonosphericMaxRMSText.Hide()
            #self.IonosphericMaxRMSTextCtrl.Hide()
        elif ( self.IonosphericCorrectionComboBox.GetSelection() == 2):
            self.BroadcastSameRB.SetValue(False)
            self.BroadcastSpecifyRB.SetValue(False)
            self.IonexRB.SetValue(True)
            self.FPPPRB.SetValue(False)
	    self.UseSigmaIonoCB.SetValue(True)
	    #self.IonosphericMaxRMSText.Hide()
            #self.IonosphericMaxRMSTextCtrl.Hide()
        elif ( self.IonosphericCorrectionComboBox.GetSelection() == 3):
            self.BroadcastSameRB.SetValue(False)
            self.BroadcastSpecifyRB.SetValue(False)
            self.IonexRB.SetValue(False)
            self.FPPPRB.SetValue(True)
	    self.UseSigmaIonoCB.SetValue(True)
	    #self.IonosphericMaxRMSText.Show()
            #self.IonosphericMaxRMSTextCtrl.Show()
	self.ShowIonosphereSourceFunc()

##########################################################################
# Name        : SelectTroposphericMapping
# Description : A call to Show SelectTroposphericMapping 
#               Function 
##########################################################################    
    def SelectTroposphericMapping(self, event):
	self.SelectTroposphericMappingFunc()
	event.Skip()

##########################################################################
# Name        : SelectTroposphericMappingFunc
# Description : The user can select different Tropospheric models, when
#               ESA's model is selected, change to Niell mapping 
##########################################################################      
    def SelectTroposphericMappingFunc(self):  
	if ( self.NominalTroposphericCorrectionComboBox.GetSelection() == 2):
	    self.MappingTroposphericCorrectionComboBox.SetSelection(0)


##########################################################################
# Name        : ResetDataDecimationText
# Description : This functions hides or shows the data decimation text 
#               control in the PREPROCESS options, enabling it or not
##########################################################################    
    def ResetDataDecimationText(self, event=None):
        if ((self.DataDecimationCB.GetValue()) == False):
            self.DataDecimation.Hide() 
        else:
            self.DataDecimation.Show()
##########################################################################
# Name        : AllowDataDecimation
# Description : This functions sets the data decimation check button false 
#               if there is not data decimation seconds
##########################################################################    
    def AllowDataDecimation(self, event):
        self.DataDecimationCB.SetValue(True) 
        
        if ((self.DataDecimation.IsEmpty()) == True and (self.DataDecimationCB.GetValue()) == True):
            self.DataDecimationCB.SetValue(False)  

##########################################################################
# Name        : HideCSDPanel
# Description : Hide the Cycle-Slip detection parameters panel
##########################################################################        
    def HideCSDPanel(self):
        self.SpecifyCSDPanel.Hide()
        self.C1L1DiferenceShow = False
        self.MelbourneWubbenaShow = False
        self.GeometricFreeCPCombinationShow = False
        self.MelbourneWubbenaConfigButton.SetBackgroundColour(wx.NullColour)
        self.LIConfigButton.SetBackgroundColour(wx.NullColour)
        self.L1C1DifferenceConfigButton.SetBackgroundColour(wx.NullColour)
            
##########################################################################
# Name        : ShowCycleSlipLI
# Description : This loads all different options for the Geometric Free 
#               CP combination, in the Cycle-Slip Detection parameters panel 
##########################################################################
    def ShowCycleSlipLI(self, event=False):
        if (self.SpecifyCSDPanel.IsShown() & self.GeometricFreeCPCombinationShow == True):
            self.HideCSDPanel()
        else:
            self.SpecifyCSDPanel.Show()
            self.MinimumThresholdText.Show()
            self.MinimumThreshold.Show()
            self.NsamplesText.Show()
            self.Nsamples.Show()
            
            self.MinimumThresholdText.SetLabel('Maximum Jump Threshold [m]: ')
            self.MaximumThresholdText.SetLabel('Maximum Threshold [m]: ')  
            self.TimeOrDeviationText.SetLabel('Time Constant [s]: ')
            self.NsamplesText.SetLabel('N-sample [epochs]: ')
            
            aux1 = self.MaximumJumpThreshold 
            aux2 = self.MaximumThresholdMetersLI  
            aux3 = self.TimeConstant
            aux4 = self.NsamplesLI
                
            self.GeometricFreeCPCombinationShow = True
            self.MelbourneWubbenaShow = False
            self.C1L1DiferenceShow = False
            self.MelbourneWubbenaConfigButton.SetBackgroundColour(wx.NullColour)
            self.LIConfigButton.SetBackgroundColour(wx.Color(245, 245, 245, 255))
            self.L1C1DifferenceConfigButton.SetBackgroundColour(wx.NullColour)
            
            self.MinimumThreshold.Clear()
            self.MaximumThreshold.Clear()         
            self.TimeOrDeviation.Clear()
            self.Nsamples.Clear()
            
            self.MinimumThreshold.WriteText(str(aux1))
            self.MaximumThreshold.WriteText(str(aux2))        
            self.TimeOrDeviation.WriteText(str(aux3))
            self.Nsamples.WriteText(str(aux4))
            
##########################################################################
# Name        : ShowCycleSlipMelbourneWubbena
# Description : This loads all different options for the Melbourne Wübenna
#               in the Cycle-Slip Detection parameters panel
##########################################################################        
    def ShowCycleSlipMelbourneWubbena(self, event):
        if (self.SpecifyCSDPanel.IsShown() & self.MelbourneWubbenaShow == True):
            self.HideCSDPanel()
        else:
            self.SpecifyCSDPanel.Show()
            self.MaximumThresholdText.Show()
            self.MaximumThreshold.Show()
            self.NsamplesText.Show()
            self.Nsamples.Show()
        
            self.MinimumThresholdText.SetLabel('Minimum stdDev [m]: ')
            self.MaximumThresholdText.SetLabel('Window [s]: ')  
            self.TimeOrDeviationText.SetLabel('k-factor: ')
            self.NsamplesText.SetLabel('N-sample [epochs]: ')
    
            aux1 = self.MinimumThresholdMetersMW    
            aux2 = self.WindowMW 
            aux3 = self.SlopeMW
            aux4 = self.NsamplesMW
            
            self.MelbourneWubbenaShow = True
            self.GeometricFreeCPCombinationShow = False
            self.C1L1DiferenceShow = False
            self.MelbourneWubbenaConfigButton.SetBackgroundColour(wx.Color(245, 245, 245, 255))
            self.LIConfigButton.SetBackgroundColour(wx.NullColour)
            self.L1C1DifferenceConfigButton.SetBackgroundColour(wx.NullColour)
            
            self.MinimumThreshold.Clear()
            self.MaximumThreshold.Clear()         
            self.TimeOrDeviation.Clear()
            self.Nsamples.Clear()
            
            self.MinimumThreshold.WriteText(str(aux1))
            self.MaximumThreshold.WriteText(str(aux2))        
            self.TimeOrDeviation.WriteText(str(aux3))
            self.Nsamples.WriteText(str(aux4))
            
##########################################################################
# Name        : ShowC1L1Diference
# Description : This loads all different options for the L1 - C1
#               in the Cycle-Slip Detection parameters panel
##########################################################################
    def ShowC1L1Diference(self, event):
        if (self.SpecifyCSDPanel.IsShown() & self.C1L1DiferenceShow == True):
            self.HideCSDPanel()
        else:
            self.SpecifyCSDPanel.Show()
            self.MaximumThresholdText.Show()
            self.MinimumThresholdText.Show()
            self.NsamplesText.Show()
            self.Nsamples.Show()
        
            self.MinimumThresholdText.SetLabel('Initial stdDev [m]: ')
            self.MaximumThresholdText.SetLabel('Window [s]: ')  
            self.TimeOrDeviationText.SetLabel('k-factor: ')
            self.NsamplesText.SetLabel('N-sample [epochs]: ')
    
            aux1 = self.MaximumThresholdC1L1Difference
            aux2 = self.WindowC1L1Diference
            aux3 = self.SlopeC1L1Diference
            aux4 = self.NsamplesL1C1
            
            self.MelbourneWubbenaShow = False
            self.GeometricFreeCPCombinationShow = False
            self.C1L1DiferenceShow = True
            self.MelbourneWubbenaConfigButton.SetBackgroundColour(wx.NullColour)
            self.LIConfigButton.SetBackgroundColour(wx.NullColour)
            self.L1C1DifferenceConfigButton.SetBackgroundColour(wx.Color(245, 245, 245, 255))
            
            self.MinimumThreshold.Clear()
            self.MaximumThreshold.Clear()         
            self.TimeOrDeviation.Clear()
            self.Nsamples.Clear()
            
            self.MinimumThreshold.WriteText(str(aux1))
            self.MaximumThreshold.WriteText(str(aux2))        
            self.TimeOrDeviation.WriteText(str(aux3))
            self.Nsamples.WriteText(str(aux4))
   
##########################################################################
# Name        : SaveCycleSlipParameters
# Description : When the radiobuttons are pressed, the information is  
#               stored and the new configuration is load 
##########################################################################
    def SaveCycleSlipParameters(self, event):
        if (self.GeometricFreeCPCombinationShow == True):
            self.MaximumJumpThreshold = self.MinimumThreshold.GetValue()
            self.MaximumThresholdMetersLI = self.MaximumThreshold.GetValue()
            self.TimeConstant = self.TimeOrDeviation.GetValue()
            self.NsamplesLI = self.Nsamples.GetValue()
            
        elif (self.MelbourneWubbenaShow == True):          
            self.MinimumThresholdMetersMW = self.MinimumThreshold.GetValue()
            self.WindowMW = self.MaximumThreshold.GetValue()
            self.SlopeMW = self.TimeOrDeviation.GetValue()
            self.NsamplesMW = self.Nsamples.GetValue()

        elif (self.C1L1DiferenceShow == True):       
            self.MaximumThresholdC1L1Difference = self.MinimumThreshold.GetValue()
            self.WindowC1L1Diference = self.MaximumThreshold.GetValue()
            self.SlopeC1L1Diference = self.TimeOrDeviation.GetValue()
            self.NsamplesL1C1 = self.Nsamples.GetValue()


##########################################################################
# Name        : ConfigureCalculusGUI
# Description : This changes all the GUI for a specific configuration 
# 		ConfigureList  = [ Broadcast, PreList, CSList, ModelList, FilterList, OutputList ]:
#		Broadcast      = [ true vs false ]
#		PreList        = [ DataDecimation, ElevationMask, DiscardEclipsedSatellitesCB, DiscardUnhealthySatellitesCB ]
#		CSList         = [ EnableLICB, EnableMelbourneWubbenaCB, EnableC1L1DifferenceCB, EnableLLI ]
#       ModelList      = [ SatelliteClockOffsetCorrection, SatelliteMovementFlightTime, EarthRotationFlightTime, SatellitePhaseCenterCorrection,
#                          ReceiverPhaseCenterCorrection,ReceiverRefPointCorrection, RelativisticCorrection, IonosphericCorrectionComboBox, 
#		                   NominalTroposphericCorrectionComboBox,MappingTroposphericCorrectionComboBox, P1P2Correction, P1C1Correction, WindUpCorrection, 
#		                   SolidTidesCorrection, GravitationalDelayCorrection ] 
#		FilterList     = [ UseSigmaIono, EstimateTroposphereCB, SingleFrequencyRB vs DualFrequencyRB, StaticReceiverRB vs KinematicReceiverRB, 
#				           PseudoRangeRB vs PseudoRangeCarrierPhaseRB, PseudoRangeSmoothingCB, PseudoRangeSmoothingTC, BackwardCB ]
#		OutputList     = [ InfoMessage, CSMessage , InputMessage , SatMEASMessage, ModelMessage, EpochSatMessage, PreFitMessage, PostFitMessage,
#				           FilterMessage, OutputMessage ]
##########################################################################      
    def ConfigureCalculusGUI(self, Broadcast, PreList, CSList, ModelList, FilterList, OutputList):
        #Preconfigure Input panel
    	if (Broadcast == True):
            self.ShowAntexObservationCB.SetValue(False)
            self.ShowAntexFunc()
            self.BroadcastRB.SetValue(True)
            self.ShowOrbitClockSourceFunc()
    	else:
            self.ShowAntexObservationCB.SetValue(True)
            self.ShowAntexFunc()
            self.SP3RB.SetValue(True)
            self.ShowOrbitClockSourceFunc()

	self.ShowIonosphereSourceCB.SetValue(True)
        self.ShowIonosphereSourceFunc()

        self.ReceiverPositionUseRinexRB.SetValue(True)
        self.SetReceiverFunc()

        self.ShowP1P2CB.SetValue(False)
        self.ShowP1P2DCBSourceFunc()
        self.ShowP1C1CB.SetValue(False)
        self.ShowP1C1Func()
		    
    	#Preconfigure Preprocess panel
    	if (PreList[0] > 0):
    	    self.DataDecimationCB.SetValue(True)
    	    self.DataDecimation.SetValue(str(PreList[0]))
    	else:
    	    self.dataDecimationCB.SetValue(False)
    	self.ElevationMask.SetValue(str(PreList[1]))
    	self.DiscardEclipsedSatellitesCB.SetValue(PreList[2])
    	self.DiscardUnhealthySatellitesCB.SetValue(PreList[3])
    	self.ResetDataDecimationText()
        self.GeometricFreeCPCombinationShow = CSList[0]

        if (CSList[0] == 1):
            self.EnableLICB.SetValue(True)
            self.MaximumJumpThreshold = '1'
            self.MaximumThresholdMetersLI = '0.08'
            self.TimeConstant = '60'
            self.NsamplesLI = '7'
            self.MelbourneWubbenaShow = CSList[1]
        else:
            self.EnableLICB.SetValue(False)
        if (CSList[1] == 1):
            self.EnableMelbourneWubbenaCB.SetValue(True)
            self.MinimumThresholdMetersMW = '0.8'
            self.WindowMW = '300'
            self.SlopeMW = '5'
            self.NsamplesMW = '2'
        else:
            self.EnableMelbourneWubbenaCB.SetValue(False)
        if (CSList[2] == 1):
            self.EnableC1L1DifferenceCB.SetValue(True)
            self.MaximumThresholdC1L1Difference = '3'
            self.WindowC1L1Diference = '300'
            self.SlopeC1L1Diference = '5'
            self.NsamplesL1C1 = '2'
        else:
            self.EnableC1L1DifferenceCB.SetValue(False)
            self.HideCSDPanel()
            self.ShowPRNButtons(None)
        if (CSList[3] == 1):
            self.EnableLLI.SetValue(True)
        else:
            self.EnableLLI.SetValue(False)
   	

    	#Preconfigure Modelling panel
    	self.SatelliteClockOffsetCorrection.SetValue(ModelList[0])
    	self.SatelliteMovementFlightTime.SetValue(ModelList[1])
    	self.EarthRotationFlightTime.SetValue(ModelList[2])
    	self.SatellitePhaseCenterCorrection.SetValue(ModelList[3])
    	self.ReceiverPhaseCenterCorrection.SetValue(ModelList[4])
    	if (ModelList[4] == True):
    	    self.ReadAntennaPhaseCenterRB.SetValue(True)
    	    self.ShowNorthEastUpPanel()
    	self.ShowReceiverPhaseCenterCorrectionPanel()
    	self.ReceiverRefPointCorrection.SetValue(ModelList[5])
        if (ModelList[5] == True):
            self.ReadAntennaReferencePointRB.SetValue(True)
            self.ShowNorthEastUpReferencePointPanel()
    	self.ShowReceiverRefPointCorrectionPanel()
    	self.RelativisticCorrection.SetValue(ModelList[6])
    	found = self.IonosphericCorrectionComboBox.SetStringSelection(ModelList[7])
    	self.IonosphericCorrectionCheckBox.SetValue(found)
    	self.ShowIonosphericCorrectionCheckBox(None)
	self.NominalTroposphericCorrectionComboBox.SetStringSelection(ModelList[8])
    	found = self.MappingTroposphericCorrectionComboBox.SetStringSelection(ModelList[9])
    	self.TroposphericCorrectionCheckBox.SetValue(found)
    	self.ShowTroposphericCorrectionCheckBox(None)
    	
	found = self.P1P2CorrectionComboBox.SetStringSelection(ModelList[10])
    	self.P1P2CorrectionCheckBox.SetValue(found)
    	self.ShowP1P2CorrectionCheckBox(None)

	found = self.P1C1CorrectionComboBox.SetStringSelection(ModelList[11])
    	self.P1C1CorrectionCheckBox.SetValue(found)
    	self.ShowP1C1CorrectionCheckBox(None)
    	self.ShowP1C1DueToModelFunc()

    	self.WindUpCorrection.SetValue(ModelList[12])
    	self.SolidTidesCorrection.SetValue(ModelList[13])
    	self.GravitationalDelayCorrection.SetValue(ModelList[14])
    	if (Broadcast == False):
    	    self.OrbitInterpolationDegreeTextCtrl.Clear()
    	    self.OrbitInterpolationDegreeTextCtrl.WriteText('10')
    	    self.ClockInterpolationDegreeTextCtrl.Clear()
    	    self.ClockInterpolationDegreeTextCtrl.WriteText('0')

    	#Preconfigure Filter panel
    	self.UseSigmaIonoCB.SetValue(FilterList[0])
    	self.EstimateTroposphereCB.SetValue(FilterList[1])
    	self.ShowTroposphereCheckBox()
    	if (FilterList[2] == 1):
    	    self.SingleFrequencyRB.SetValue(True)
    	elif (FilterList[2] == 2):
            self.DualFrequencyRB.SetValue(True)
        self.ShowConfigurationTexts()
        if (FilterList[3] == 1):
            self.StaticReceiverRB.SetValue(True)
        elif (FilterList[3] == 2):
            self.KinematicReceiverRB.SetValue(True)
        self.SetParametersTexts()
        if (FilterList[4] == 1):
            self.PseudoRangeRB.SetValue(True)
        elif (FilterList[4] == 2):
            self.PseudoRangeCarrierPhaseRB.SetValue(True)
        self.ShowPseudoRangeCarrierPhase()
        self.PseudoRangeSmoothingCB.SetValue(FilterList[5])
        self.ShowPseudoRangeSmoothingTC()
        if (FilterList[6] != 0):
            self.PseudoRangeSmoothingTC.Clear()
            self.PseudoRangeSmoothingTC.WriteText(str(FilterList[6]))
        self.FixedWeightPseudoRangeCarrierPhaseRB.SetValue(True)
        self.FixedWeightPseudoRangeRB.SetValue(True)
        self.BackwardCB.SetValue(FilterList[7])

    	#Preconfigure Output panel
    	self.InfoMessage.SetValue(OutputList[0])
    	self.CSMessage.SetValue(OutputList[1])
    	self.InputMessage.SetValue(OutputList[2])
    	self.SatMEASMessage.SetValue(OutputList[3])
    	self.ModelMessage.SetValue(OutputList[4])
    	self.EpochSatMessage.SetValue(OutputList[5])
    	self.PreFitMessage.SetValue(OutputList[6])
    	self.PostFitMessage.SetValue(OutputList[7])
    	self.FilterMessage.SetValue(OutputList[8])
    	self.OutputMessage.SetValue(OutputList[9])

##########################################################################
# Name        : PreconfigureSPP
# Description : A call to PreconfigureSPP Function 
##########################################################################  
    def PreconfigureSPP(self, event):
	self.PreconfigureSPPFunc()    	
        event.Skip()

##########################################################################
# Name        : PreconfigureSPPFunc
# Description : This configures all the GUI for a regular SPP
#               processing
# 		ConfigureList  = [ Broadcast, PreList, CSList, ModelList, FilterList, OutputList ]:
#		Broadcast      = [ true vs false ]
#		PreList        = [ DataDecimation, ElevationMask, DiscardEclipsedSatellitesCB, DiscardUnhealthySatellitesCB ]
#		CSList         = [ EnableLICB, EnableMelbourneWubbenaCB, EnableC1L1DifferenceCB, EnableLLI ]
#	    	ModelList      = [ SatelliteClockOffsetCorrection, SatelliteMovementFlightTime, EarthRotationFlightTime, SatellitePhaseCenterCorrection,
#				ReceiverPhaseCenterCorrection,ReceiverRefPointCorrection, RelativisticCorrection, IonosphericCorrectionComboBox, 
#				NominalTroposphericCorrectionComboBox,MappingTroposphericCorrectionComboBox, P1P2Correction, P1C1Correction, WindUpCorrection, 
#				SolidTidesCorrection, GravitationalDelayCorrection ] 
#		FilterList     = [ UseSigmaIono, EstimateTroposphereCB, SingleFrequencyRB vs DualFrequencyRB, StaticReceiverRB vs KinematicReceiverRB, 
#				PseudoRangeRB vs PseudoRangeCarrierPhaseRB, PseudoRangeSmoothingCB, PseudoRangeSmoothingTC, BackwardCB ]
#		OutputList     = [ InfoMessage, CSMessage , InputMessage , SatMEASMessage, ModelMessage, EpochSatMessage, PreFitMessage, PostFitMessage,
#				 FilterMessage, OutputMessage ]
##########################################################################
    def PreconfigureSPPFunc(self):
        self.ConfigureCalculusGUI(True, [300, 5, False, True], [0, 0, 0, 0], [True, True, True, False, False, False, True, 'Klobuchar (GPS)', 'UNB-3 Nominal','Simple Mapping', 'RINEX Nav File','Flexible', False, False, False], [True, 0, 1, 2, 1, False, 100, False], [True, True, True, False, True, True, True, True, True, True])


##########################################################################
# Name        : PreconfigurePP
# Description : A call to PreconfigurePPP Function 
##########################################################################  
    def PreconfigurePPP(self, event):
	self.PreconfigurePPPFunc()    	
        event.Skip()

##########################################################################
# Name        : PreconfigurePPPFunc
# Description : This configures all the GUI for a regular PPP
#               processing
# 		ConfigureList  = [ Broadcast, PreList, CSList, ModelList, FilterList, OutputList ]:
#		Broadcast      = [ true vs false ]
#		PreList        = [ DataDecimation, ElevationMask, DiscardEclipsedSatellitesCB, DiscardUnhealthySatellitesCB ]
#		CSList         = [ EnableLICB, EnableMelbourneWubbenaCB, EnableC1L1DifferenceCB, EnableLLI ]
#	    	ModelList      = [ SatelliteClockOffsetCorrection, SatelliteMovementFlightTime, EarthRotationFlightTime, SatellitePhaseCenterCorrection,
#				ReceiverPhaseCenterCorrection,ReceiverRefPointCorrection, RelativisticCorrection, IonosphericCorrectionComboBox, 
#				NominalTroposphericCorrectionComboBox,MappingTroposphericCorrectionComboBox, P1P2Correction, P1C1Correction, WindUpCorrection, 
#				SolidTidesCorrection, GravitationalDelayCorrection ] 
#		FilterList     = [ UseSigmaIono, EstimateTroposphereCB, SingleFrequencyRB vs DualFrequencyRB, StaticReceiverRB vs KinematicReceiverRB, 
#				PseudoRangeRB vs PseudoRangeCarrierPhaseRB, PseudoRangeSmoothingCB, PseudoRangeSmoothingTC, BackwardCB ]
#		OutputList     = [ InfoMessage, CSMessage , InputMessage , SatMEASMessage, ModelMessage, EpochSatMessage, PreFitMessage, PostFitMessage,
#				 FilterMessage, OutputMessage ]
##########################################################################
    def PreconfigurePPPFunc(self):
        self.ConfigureCalculusGUI(False, [300, 5, True, False], [1, 1, 0, 1], [True, True, True, True, True, True, True, '', 'Simple Nominal', 'Niell Mapping', 'False','Flexible', True, True, True], [False, 1, 2, 1, 2, False, 10, False], [True, True, True, False, True, True, True, True, True, True])

##########################################################################
# Name        : ShowPRNButtons
# Description : This activates all the PRN, setting its value in TRUE 
#               this is visually seen in painting them in green colour 
##########################################################################
    def ShowPRNButtons(self, event):
        for i in range(32):
            self.ButtonPRN[i].SetValue(True)
            self.ButtonPRN[i].SetBackgroundColour((120, 255, 044))
            
##########################################################################
# Name        : HidePRNButtons
# Description : This deactivates all the PRN, setting its value in TRUE 
#               this is visually seen in painting them in red colour 
##########################################################################
    def HidePRNButtons(self, event):
        for i in range(32):
            self.ButtonPRN[i].SetValue(False)
            self.ButtonPRN[i].SetBackgroundColour((230, 050, 043))
            
##########################################################################
# Name        : HideButtons
# Description : This hides the calculus tab buttons
##########################################################################
    def HideButtons (self):
        self.RunButton.Hide() 
        self.ShowOutputButton.Hide()    
        self.SaveCfgButton.Hide()
        self.ShowConfigButton.Hide()   
        self.PreconfigureSPPB.Hide()
        self.PreconfigurePPPB.Hide()

##########################################################################
# Name        : ShowButtons
# Description : This activates the calculus tab buttons
##########################################################################
    def ShowButtons (self):
        self.RunButton.Show() 
        self.ShowOutputButton.Show()
        self.SaveCfgButton.Show()
        self.ShowConfigButton.Show()
        self.PreconfigureSPPB.Show()
        self.PreconfigurePPPB.Show() 


##########################################################################
# Name        : SaveConfig
# Description : Saves a configuration file to an archive opening a dialogue
#               to let user choose the destination
##########################################################################
    def SaveConfig (self, event):
        global DireName
        global LastConfigFile
        global LastDirConfigFile
        
	gLABParameters = []
        errorstr, warningstr = self.GenerateParameterList(gLABParameters)

        # Show error alert, if any
        if (errorstr != ""):
            showstr = "gLAB has found the following errors, please correct them before generating the configuration file again:\n\n" + errorstr
            dial = wx.MessageDialog(None, showstr, 'Errors found', wx.OK | wx.ICON_HAND)
            dial.ShowModal()

        else:
		if WIN:
		    wildcard = ".CFG files (*.cfg)|*.cfg*|All files|*.*"
		else:
		    wildcard = ".CFG files (*.cfg)|*.[c,C][f,F][g,G]*|All files|*"
		dlg = wx.FileDialog(self, "Choose destination configuration file", DireName, "gLAB.cfg", wildcard, wx.SAVE)
		if dlg.ShowModal() == wx.ID_OK:
		    path = dlg.GetPath()
		    FileName = dlg.GetFilename()
		    DireName = dlg.GetDirectory()
		    FilePath = os.path.join(DireName, path)
		    LastConfigFile = FilePath
		    LastDirConfigFile = DireName
		    self.SaveConfigFile(FilePath)
		    self.ShowConfigButton.Enable()
		dlg.Destroy() 

##########################################################################
# Name        : SaveConfigFile
# Description : Saves a configuration file to an specified archive
##########################################################################
    def SaveConfigFile (self, FilePath):
        gLABParameters = []
        line = u'' 
        strINP = u''
        strPRE = u''
        strMOD = u''
        strFIL = u''
        strOUT = u''
        errorstr, warningstr = self.GenerateParameterList(gLABParameters)

        gLABParameters.extend(['-'])

        for parameter in gLABParameters:
	    if (parameter!=''):
                if (parameter[0] == '-'):
                    if (line != ""):
                        if (line[1] == '-'):
                            type = line[2:5]
                        else:
                            type = line[1:4]
                        if (type == 'inp'):
                            strINP += line + '\n'
                        elif (type == 'pre'):
                            strPRE += line + '\n'
                        elif (type == 'mod'):
                            strMOD += line + '\n'
                        elif (type == 'fil'):
                            strFIL += line + '\n'
                        elif (type == 'pri' or type == 'out'):
                            strOUT += line + '\n'
                    line = parameter
                else:
                    line += ' ' + parameter



        try:
	    now = datetime.datetime.now()
            f = open(FilePath, 'w')
            commentline = '###################################################\n'
#            f.write('%s#\n#     gLAB self-generated config file version: %s.%s\n#     at: %02d:%02d %02d/%02d/%04d\n#\n%s\n' % (commentline,Version,Revision,now.hour,now.minute,now.day,now.month,now.year,commentline))
            f.write('%s#\n#     gLAB - Version: %s.%s \n#     This is a self-generated configuration file. \n#     Created at: %02d:%02d on %02d/%02d/%04d\n#\n%s\n' % (commentline,Version,Revision,now.hour,now.minute,now.day,now.month,now.year,commentline))
            f.write('\n' + commentline + '#     INPUT section\n' + commentline + '\n')
            f.write( strINP.encode( 'utf-8' ) )
            f.write('\n' + commentline + '#     PREPROCESS section\n' + commentline + '\n')
            f.write( strPRE.encode( 'utf-8' ) )
            f.write('\n' + commentline + '#     MODELLING section\n' + commentline + '\n')
            f.write( strMOD.encode( 'utf-8' ) )
            f.write('\n' + commentline + '#     FILTER section\n' + commentline + '\n')
            f.write( strFIL.encode( 'utf-8' ) )
            f.write('\n' + commentline + '#     OUTPUT section\n' + commentline + '\n')
            f.write( strOUT.encode( 'utf-8' ) )
            f.write('\n' + commentline + '#     End of self-generated parameters\n' + commentline)
            f.close()
        except:
            dial = wx.MessageDialog(None, 'Cannot write configuration file ' + FilePath, 'Problem opening file', wx.OK | wx.ICON_HAND)
            dial.ShowModal()

        

##########################################################################
# Name        : InitializeControls
# Description : This function is called just once when the program is 
#               initiating, this hides some objects after the 
#               interface has calculated their position, size... (layout) 
##########################################################################
    def InitializeControls(self):
        # Hide controls                     
        self.SecondaryFrame.SetSizer(self.FilterPanelGrill)
        self.SecondaryFrame.Layout()                             
        self.SecondaryFrame.SetSizer(self.OutputPanelGrill)
        self.SecondaryFrame.Layout()
        self.SecondaryFrame.SetSizer(self.PreProcesPanelGrill)
        self.SecondaryFrame.Layout()    
        self.SecondaryFrame.SetSizer(self.ModesPanelGrill)
        self.SecondaryFrame.Layout()              
        self.SecondaryFrame.SetSizer(self.InputPanelGrill)
        self.SecondaryFrame.Layout()  
                        
	# Input Section:
        self.ShowAntexObservationCB.SetValue(False)
        self.ShowAntexFunc()

        self.BroadcastRB.SetValue(True)
        self.ShowOrbitClockSourceFunc()
      
        self.ReceiverPositionUseRinexRB.SetValue(True)
        self.SetReceiverFunc()
          
        self.ShowP1P2CB.SetValue(False)
        self.ShowP1P2DCBSourceFunc()
        self.ShowP1C1CB.SetValue(False)
        self.ShowP1C1Func()  

        
	# Preprocess Section: Cycle-slip detection
        self.HideCSDPanel()
        	
	# Modelling Section:
	self.ReceiverPhaseCenterCorrection.SetValue(False)
	self.ShowReceiverPhaseCenterCorrectionPanel()

        self.IonosphericCorrectionComboBox.Hide() 
        self.PseudoRangeSmoothingTC.Hide()    
        
        self.AntennaReferencePointPanel.Hide()   
        self.NorthEastUpPanel.Hide() 
        
        self.HideButtons()
        self.HideAll()
        

	# Filter Section:
        self.ShowConfigurationTexts()

        # Set tooltips
        self.SetTooltipDictionary() 
        self.SetTooltipBinds()
        # Thread lock
        self.threadLock = threading.RLock()
        # Tooltip creation
        self.ToolTipFrame = WindowHTML(None)
        self.formerSize = wx.Size(tooltipXsize, tooltipYsize)

	# Apply the SPP template:
	self.PreconfigureSPPFunc()


##########################################################################
# Name        : SetReceiver
# Description : This function hides or shows the specify receiver panel 
#               the user can manually introduce a position if active.
#               This function takes care of introduced events.
##########################################################################
    def SetReceiver(self, event):
        self.SetReceiverFunc()
        event.Skip()

##########################################################################
# Name        : SetReceiverFunc
# Description : This function call hides or shows the SINEX receiver panel 
#               the user can also manually introduce a position if active 
##########################################################################    
    def SetReceiverFunc(self):
        if ((self.ReceiverPositionSpecifyRB.GetValue()) == False):
	    self.PosX.Enable(False)
	    self.PosY.Enable(False)
	    self.PosZ.Enable(False)


        if ((self.ReceiverPositionSpecifyRB.GetValue()) == True):
	    self.PosX.Enable(True)
	    self.PosY.Enable(True)
	    self.PosZ.Enable(True)

        if ((self.ReceiverPositionUseSinexRB.GetValue()) == False):
            #self.UseSinexReceiverPositionPanel.Hide()
	    self.SinexTextCtrl.Enable(False)
	    self.SinexFileButton.Enable(False)
        if ((self.ReceiverPositionUseSinexRB.GetValue()) == True):
            #self.UseSinexReceiverPositionPanel.Show()
	    self.SinexTextCtrl.Enable(True)
	    self.SinexFileButton.Enable(True)


    
##########################################################################
# Name        : SaveDestinationPathByButton
# Description : The user can select the path to save the output from the 
#               dialog that will be opened when pressed the Examine button 
##########################################################################
    def SaveDestinationPathByButton(self, event):
        global DireName
        
        if WIN:
            wildcard = ".OUT files (*.out)|*.out*|All files|*.*"
        else:
            wildcard = ".OUT files (*.out)|*.[o,O][u,U][t,T]*|All files|*"
        dlg = wx.FileDialog(self, "Choose destination file for output messages", DireName, ".out", wildcard, wx.SAVE)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            FileName = dlg.GetFilename()
            DireName = dlg.GetDirectory()
            Aux = os.path.join(DireName, path)
            self.TextCtrlOutputDestination.Clear()
            self.TextCtrlOutputDestination.WriteText(Aux)
            global OutputPath
            OutputPath = self.OutputDestinationPath
            
        dlg.Destroy()    

        
##########################################################################
# Name        : SaveDestinationPathByTextCtrl
# Description : This function gets the Output path from the user when 
#               its text control is modified 
##########################################################################
    def SaveDestinationPathByTextCtrl(self, event):
        self.OutputDestinationPath = self.TextCtrlOutputDestination.GetValue()
        global OutputPath
        OutputPath = self.OutputDestinationPath

############################################################################
# Name        : SaveReceiverTypePathByButton
# Description : The user can select the path to open the GPS Receiver Type 
#		file from the dialogue opened when pressed the Examine button 
############################################################################
    def SaveReceiverTypePathByButton(self, event):
        global DireName
        
        if WIN:
            wildcard = "GPS_Receiver_Types files|GPS_Receiver_Types*|All files|*.*"
        else:
            wildcard = "GPS_Receiver_Types files|GPS_Receiver_Types*![.gz]|All files|*"
        dlg = wx.FileDialog(self, "Choose GPS Receiver Types file", DireName, "", wildcard, wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            FileName = dlg.GetFilename()
            DireName = dlg.GetDirectory()
            Aux = os.path.join(DireName, path)
            self.ReceiverTypeTextCtrl.Clear()
            self.ReceiverTypeTextCtrl.WriteText(Aux)            
        dlg.Destroy()    



##########################################################################
# Name        : SaveReceiverTypePathByTextCtrl
# Description : This function gets the GPS Receiver Type path from the user
#               when its text control is modified 
##########################################################################
    def SaveReceiverTypePathByTextCtrl(self, event):
        self.AuxReceiverTypePath = self.ReceiverTypeTextCtrl.GetValue()


        
############################################################################
# Name        : SaveRinexPathByButton
# Description : The user can select the path to open the RINEX file from the 
#               dialogue that will be opened when pressed the Examine button 
############################################################################
    def SaveRinexPathByButton(self, event):
        global DireName
        
        if WIN:
            wildcard = "RINEX files (*.??o)|*.??o*|All files|*.*"
        else:
            wildcard = "RINEX files (*.??o)|*.[0-9][0-9][o,O]*|All files|*"
        dlg = wx.FileDialog(self, "Choose RINEX Observation file", DireName, "", wildcard, wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            FileName = dlg.GetFilename()
            DireName = dlg.GetDirectory()
            Aux = os.path.join(DireName, path)
            self.TextCtrlRinexObservation.Clear()
            self.TextCtrlRinexObservation.WriteText(Aux)            
        dlg.Destroy()    

    
##########################################################################
# Name        : SaveRinexPathByTextCtrl
# Description : This function gets the RINEX path from the user when 
#               its text control is modified 
##########################################################################
    def SaveRinexPathByTextCtrl(self, event):
        self.RinexPath = self.TextCtrlRinexObservation.GetValue()

############################################################################
# Name        : SaveSinexPathByButton
# Description : The user can select the path to open the SINEX file from the 
#               dialogue that will be opened when pressed the Examine button 
############################################################################
    def SaveSinexPathByButton(self, event):
        global DireName
        
        if WIN:
            wildcard = "SINEX files (*.snx)|*.snx*|All files|*.*"
        else:
            wildcard = "SINEX files (*.??snx)|*.[s,S][n,N][x,X]*|All files|*"
        dlg = wx.FileDialog(self, "Choose SINEX file", DireName, "", wildcard, wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            FileName = dlg.GetFilename()
            DireName = dlg.GetDirectory()
            Aux = os.path.join(DireName, path)
            self.SinexTextCtrl.Clear()
            self.SinexTextCtrl.WriteText(Aux)            
        dlg.Destroy()    


##########################################################################
# Name        : SaveSinexPathByTextCtrl
# Description : This function gets the SINEX path from the user when 
#               its text control is modified 
##########################################################################
    def SaveSinexPathByTextCtrl(self, event):
        self.SinexPath = self.SinexTextCtrl.GetValue()

        
############################################################################
# Name        : SaveAntexPathByButton
# Description : The user can select the path to load the ANTEX file from the 
#               dialogue that will be opened when pressed the Examine button 
############################################################################
    def SaveAntexPathByButton(self, event):
        global DireName
        
        if WIN: 
            wildcard = "ANTEX files (*.atx)|*.atx*|All files|*.*"
        else:
            wildcard = "ANTEX files (*.atx)|*.[a,A][t,T][x,X]*|All files|*"
        dlg = wx.FileDialog(self, "Choose ANTEX file", DireName, "", wildcard, wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            FileName = dlg.GetFilename()
            DireName = dlg.GetDirectory()
            Aux = os.path.join(DireName, path)
            self.TextCtrlAntexObservation.Clear()   
            self.TextCtrlAntexObservation.WriteText(Aux)
        dlg.Destroy() 

##########################################################################
# Name        : SaveAntexPathByTextCtrl
# Description : This function gets the ANTEX path from the user when 
#               its text control is modified 
##########################################################################
    def SaveAntexPathByTextCtrl(self, event):
        self.AntexPath = self.TextCtrlAntexObservation.GetValue()

##########################################################################
# Name        : ShowAntex
# Description : This function is a call to the ShowAntexFunc function
#               This function takes care of introduced events.
##########################################################################
    def ShowAntex(self, event):
        self.ShowAntexFunc()
        event.Skip()

##########################################################################
# Name        : ShowAntexFunc
# Description : This function shows or hides the ANTEX line where the path 
#		from the user when its text control is modified 
##########################################################################
    def ShowAntexFunc(self):
        if ((self.ShowAntexObservationCB.GetValue()) == True):
	    self.AntexObservation.Show()
	    self.TextCtrlAntexObservation.Show()
	    self.AntexObservationFileButton.Show()
	else:
	    self.AntexObservation.Hide()
	    self.TextCtrlAntexObservation.Hide()
	    self.AntexObservationFileButton.Hide()

##########################################################################
# Name        : SaveOrbitsAndClockSourcePathByButton
# Description : The user can select the path to load the Orbits and clock
#               source from the dialogue that will be opened when it is
#               pressed the Examine button 
##########################################################################   
    def SaveOrbitsAndClockSourcePathByButton(self, event):
        global DireName
        if ((self.BroadcastRB.GetValue()) == True):
            FrameName = "Choose Broadcast file"          
            if WIN:
                wcd = "Broadcast files (*.??n)|*.??n*|All files|*.*"
            else:
                wcd = "Broadcast files (*.??n)|*.[0-9][0-9][n,N]*|All files|*"
        if ((self.SP3RB.GetValue()) == True):
            FrameName = "Choose SP3 file"
            if WIN:
                wcd = "SP3 files (*.sp3)|*.sp3*|Ephemeris files (*.eph)|*.eph*|All files|*.*"
            else:
                wcd = "SP3 files (*.sp3)|*.[s,S][p,P]3*|Ephemeris files (*.eph)|*.[e,E][p,P][h,H]*|All files|*"
        if ((self.ClocksRinexOrbitSP3RB.GetValue()) == True):
            FrameName = "Choose Orbits SP3 file"
            if WIN:
                wcd = "SP3 files (*.sp3)|*.sp3*|Ephemeris files (*.eph)|*.eph*|All files|*.*"
            else:
                wcd = "SP3 files (*.sp3)|*.[s,S][p,P]3*|Ephemeris files (*.eph)|*.[e,E][p,P][h,H]*|All files|*"
        
        dlg = wx.FileDialog(self, FrameName, DireName, "", wcd, wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            FileName = dlg.GetFilename()
            DireName = dlg.GetDirectory()
            name = os.path.join(DireName, path)
            if ((self.BroadcastRB.GetValue()) == True): 
                self.BroadcastPath = os.path.join(DireName, path)
            if ((self.SP3RB.GetValue()) == True):
                self.SP3Path = os.path.join(DireName, path)
            if ((self.ClocksRinexOrbitSP3RB.GetValue()) == True):
                self.OrbitsSP3Path = os.path.join(DireName, path)

            self.TextCtrlFirstRow.Clear()
            self.TextCtrlFirstRow.WriteText(name)            
        dlg.Destroy()
    

##########################################################################
# Name        : SaveOrbitsAndClockSourcePathByTextCtrl
# Description : This function gets the orbits and clock source path from 
#               the user when its text control is modified 
########################################################################## 
    def SaveOrbitsAndClockSourcePathByTextCtrl(self, event):
        if ((self.BroadcastRB.GetValue()) == True):
            self.BroadcastPath = self.TextCtrlFirstRow.GetValue()   
    
        if ((self.SP3RB.GetValue()) == True):
            self.SP3Path = self.TextCtrlFirstRow.GetValue()
             
        if ((self.ClocksRinexOrbitSP3RB.GetValue()) == True):
            self.OrbitsSP3Path = self.TextCtrlFirstRow.GetValue()    

 
##########################################################################
# Name        : SaveP1P2PathByButton
# Description : The user can select the path to load the DCB files
#               source from the dialogue that will be opened when it is
#               pressed the Examine button 
##########################################################################   
    def SaveP1P2PathByButton(self, event):
        global DireName

        if ( self.P1P2SourcesComboBox.GetSelection() == 1 ):
            FrameName = "Choose RINEX Navigation Broadcast file"          
            if WIN:
                wcd = "Broadcast files (*.??n)|*.??n*|All files|*.*"
            else:
                wcd = "Broadcast files (*.??n)|*.[0-9][0-9][n,N]*|All files|*"

        if ( self.P1P2SourcesComboBox.GetSelection() == 2 ):
            FrameName = "Choose P1-P2 DCB file"
            if WIN:
                wcd = "DCB files (*.dcb)|*.DCB*|All files|*.*"
            else:
                wcd = "DCB files (*.dcb)|*.[d,D][c,C][b,B]*|All files|*"
        
        if ( self.P1P2SourcesComboBox.GetSelection() == 4 ):
            FrameName = "Choose IONEX file"
            if WIN:
        
                wcd = "IONEX files (*.??i)|*.??i*|All files|*.*"
            else:
                wcd = "IONEX files (*.??i)|*.[0-9][0-9][i,I]*|All files|*"

        if ( self.P1P2SourcesComboBox.GetSelection() == 6 ):
            FrameName = "Choose FPPP file"
            if WIN:
                wcd = "FPPP files (*.ion)|*.ion*|All files|*.*"
            else:
                wcd = "FPPP files (*.ion)|*.ion*|All files|*"
        
        dlg = wx.FileDialog(self, FrameName, DireName, "", wcd, wx.OPEN)

        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            FileName = dlg.GetFilename()
            DireName = dlg.GetDirectory()
            Aux = os.path.join(DireName, path)
            self.P1P2TextCtrl.Clear()
            self.P1P2TextCtrl.WriteText(Aux)        

	    if (self.P1P2SourcesComboBox.GetSelection() == 2 and FileName[0] == "P" and FileName[1] == "1" and FileName[2] == "C" and FileName[3] == "1"):
		showstr = "P1C1 DCB File introduced, instead of P1P2 File."
		dial = wx.MessageDialog(None, showstr, 'Warning', wx.OK | wx.YES_DEFAULT )
	        dial.SetIcon(wx.Icon( gBinElements[2] , wx.BITMAP_TYPE_PNG))
		ret = dial.ShowModal()
	    else:
	        ret = wx.ID_YES
	    

        dlg.Destroy()

##########################################################################
# Name        : SaveP1P2PathByTextCtrl
# Description : This function gets the DCB source path from the user when
#               its text control is modified 
########################################################################## 
    def SaveP1P2PathByTextCtrl(self, event):
        if ( self.P1P2SourcesComboBox.GetSelection() == 1 ):
            self.TGDSourceP1P2Path = self.P1P2TextCtrl.GetValue()   
    
        if ( self.P1P2SourcesComboBox.GetSelection() == 2 ):
            self.DCBSourceP1P2Path = self.P1P2TextCtrl.GetValue()

        if ( self.P1P2SourcesComboBox.GetSelection() == 4 ):
            self.IonexSourceP1P2Path = self.P1P2TextCtrl.GetValue()

        if ( self.P1P2SourcesComboBox.GetSelection() == 6 ):
            self.FPPPSourceP1P2Path = self.P1P2TextCtrl.GetValue()

##########################################################################
# Name        : SaveP1C1PathByButton
# Description : The user can select the path to load the C1-P1 DCB file 
#               source from the dialogue that will be opened when it is
#               pressed the Examine button 
##########################################################################
    def SaveP1C1PathByButton(self, event):
        global DireName
        
        if WIN:
            wildcard = "DCB files (*.dcb)|*.DCB*|All files|*.*"
        else:
            wildcard = "DCB files (*.dcb)|*.[d,D][c,C][b,B]*|All files|*"
        dlg = wx.FileDialog(self, "Choose P1-C1 DCB file", DireName, "", wildcard, wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            FileName = dlg.GetFilename()
            DireName = dlg.GetDirectory()
            Aux = os.path.join(DireName, path)
            self.P1C1FileTextCtrl.Clear()
            self.P1C1FileTextCtrl.WriteText(Aux)

	    if (FileName[0] == "P" and FileName[1] == "1" and FileName[2] == "P" and FileName[3] == "2"):
		showstr = "P1P2 DCB File introduced, instead of P1C1 File."
		dial = wx.MessageDialog(None, showstr, 'Warning', wx.OK | wx.YES_DEFAULT )
	        dial.SetIcon(wx.Icon( gBinElements[2] , wx.BITMAP_TYPE_PNG))
		ret = dial.ShowModal()
	    else:
	        ret = wx.ID_YES
        dlg.Destroy() 


##########################################################################
# Name        : SaveP1C1PathByTextCtrl
# Description : This function gets the P1-C1 DCB source path from the user when
#               its text control is modified  
########################################################################## 
    def SaveP1C1PathByTextCtrl(self, event):
            self.DCBSourceP1C1Path = self.P1C1FileTextCtrl.GetValue()


 
##########################################################################
# Name        : SaveClocksRinexPathByButton
# Description : The user can select the path to load the RINEX file from the 
#               dialogue that will be opened when pressed the Examine button 
##########################################################################
    def SaveClocksRinexPathByButton(self, event):
        global DireName
        
        if WIN:
            wildcard = "Clock files (*.clk)|*.clk*|All files|*.*"
        else:
            wildcard = "Clock files (*.clk)|*.[c,C][l,L][k,K]*|All files|*"
        dlg = wx.FileDialog(self, "Choose Clocks Rinex file", DireName, "", wildcard, wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            FileName = dlg.GetFilename()
            DireName = dlg.GetDirectory()
            Aux = os.path.join(DireName, path)
            self.TextCtrlSecondRow.Clear()
            self.TextCtrlSecondRow.WriteText(Aux)
        dlg.Destroy() 

        
##########################################################################
# Name        : SaveClocksRinexPathByTextCtrl
# Description : This function gets the RINEX path from the user when 
#               its text control is modified 
########################################################################## 
    def SaveClocksRinexPathByTextCtrl(self, event):
        self.ClocksRinexPath = self.TextCtrlSecondRow.GetValue() 


##########################################################################
# Name        : ShowP1C1DueToModel
# Description : A configuration pre-call to ShowP1C1DueToModel Function 
##########################################################################    
    def ShowP1C1DueToModel(self, event):
	self.ShowP1C1DueToModelFunc()
        event.Skip()  

##########################################################################
# Name        : ShowP1C1DueToModel
# Description : A configuration pre-call to ShowP1C1DueToModel Function 
##########################################################################    
    def ShowP1C1DueToModelFunc(self):
    	self.ShowP1C1CB.SetValue(False)

        if ( self.P1C1CorrectionCheckBox.GetValue() == True ): 
		if ( self.P1C1CorrectionComboBox.GetValue() == 'Strict' ):
		    self.ShowP1C1CB.SetValue(True)

	self.ShowP1C1Func()
 

    """
##########################################################################
# Name        : ShowAuxDueToStrictDCB
# Description : A call to Show Ionosphere ShowDCBSource Function 
##########################################################################    
    def ShowP1C1DueToModel(self, event):
        if (self.StrictDCBsCB.GetValue() == True):
		self.ShowP1C1CB.SetValue(True)
		self.ShowP1P2CB.SetValue(True)
		self.P1P2SourcesComboBox.SetSelection(2)
	      	self.P1P2SourcesComboBox.Enable(False)
	else:
	      	self.P1P2SourcesComboBox.Enable(True)
		self.ShowP1C1CB.SetValue(False)
		self.ShowP1P2CB.SetValue(False)
	
	self.ShowP1C1Func()
	self.ShowP1P2DCBSourceFunc()
        event.Skip()  
    """ 
    
##########################################################################
# Name        : ShowP1C1
# Description : A call to Show P1C1 Function 
##########################################################################    
    def ShowP1C1(self, event):
        self.ShowP1C1Func()
        event.Skip()    

##########################################################################
# Name        : ShowP1C1Func
# Description : The P1C1 panel management.
##########################################################################        
    def ShowP1C1Func(self):
        if (self.ShowP1C1CB.GetValue() == True):
		self.P1C1FileTxt.Show()
		self.P1C1FileTextCtrl.Show()
		self.P1C1FileButton.Show()

		self.ReceiverTypeTxt.Show()
		self.ReceiverTypeTextCtrl.Show()
		self.ReceiverTypeFileButton.Show()
	else:
		self.P1C1FileTxt.Hide()
		self.P1C1FileTextCtrl.Hide()
		self.P1C1FileButton.Hide()

		self.ReceiverTypeTxt.Hide()
		self.ReceiverTypeTextCtrl.Hide()
		self.ReceiverTypeFileButton.Hide()

    
##########################################################################
# Name        : ShowP1P2DCBSource
# Description : A call to Show Ionosphere ShowP1P2DCBSource Function 
##########################################################################    
    def ShowP1P2DCBSource(self, event):
        self.ShowP1P2DCBSourceFunc()
        event.Skip()    

##########################################################################
# Name        : ShowP1P2DCBSourceFunc
# Description : The user can select the path to load the Orbits and Clocks
#               file from the dialogue that will be opened when it is
#               pressed the Examine button 
##########################################################################        
    def ShowP1P2DCBSourceFunc(self):
        self.P1P2SourceTxt.Hide()
        self.P1P2SourcesComboBox.Hide()
        self.P1P2FileTxt.Hide()
        self.P1P2TextCtrl.Hide()
        self.P1P2FileButton.Hide()


        if ( self.ShowP1P2CB.GetValue() == True):	
	    self.P1P2SourceTxt.Show()
	    self.P1P2SourcesComboBox.Show()
	else:
	    self.P1P2SourceTxt.Hide()
	    self.P1P2SourcesComboBox.Hide()


        if ( self.P1P2SourcesComboBox.GetSelection() == 0 ):
	    self.P1P2CorrectionComboBox.SetStringSelection('RINEX Nav File')

        if ( self.P1P2SourcesComboBox.GetSelection() == 1 ):
            self.P1P2FileTxt.SetLabel('RINEX Nav. File : ')
            Aux = self.TGDSourceP1P2Path
     
            self.P1P2TextCtrl.Clear()
            self.P1P2TextCtrl.WriteText(Aux)
	    self.P1P2CorrectionComboBox.SetStringSelection('RINEX Nav File')

	    if ( self.ShowP1P2CB.GetValue()  == True):   
		self.P1P2FileTxt.Show()
		self.P1P2TextCtrl.Show()
		self.P1P2FileButton.Show()
    
        if ( self.P1P2SourcesComboBox.GetSelection() == 2 ):
            self.P1P2FileTxt.SetLabel('      DCB File : ')
            Aux = self.DCBSourceP1P2Path 

            self.P1P2TextCtrl.Clear()
            self.P1P2TextCtrl.WriteText(Aux) 
	    self.P1P2CorrectionComboBox.SetStringSelection('DCB File')

	    if ( self.ShowP1P2CB.GetValue()  == True):   
		self.P1P2FileTxt.Show()
		self.P1P2TextCtrl.Show()
		self.P1P2FileButton.Show()

        if ( self.P1P2SourcesComboBox.GetSelection() == 3 ):
            self.P1P2CorrectionComboBox.SetStringSelection('IONEX File')	

        if ( self.P1P2SourcesComboBox.GetSelection() == 4 ):
            self.P1P2FileTxt.SetLabel('   IONEX File : ')
            Aux = self.IonexSourceP1P2Path

            self.P1P2TextCtrl.Clear()
            self.P1P2TextCtrl.WriteText(Aux)
            self.P1P2CorrectionComboBox.SetStringSelection('IONEX File')

            if ( self.ShowP1P2CB.GetValue()  == True):
                self.P1P2FileTxt.Show()
                self.P1P2TextCtrl.Show()
                self.P1P2FileButton.Show()

        if ( self.P1P2SourcesComboBox.GetSelection() == 5 ):
            self.P1P2CorrectionComboBox.SetStringSelection('FPPP File')	

        if ( self.P1P2SourcesComboBox.GetSelection() == 6 ):
            self.P1P2FileTxt.SetLabel('     FPPP File : ')
            Aux = self.FPPPSourceP1P2Path

            self.P1P2TextCtrl.Clear()
            self.P1P2TextCtrl.WriteText(Aux)
            self.P1P2CorrectionComboBox.SetStringSelection('FPPP File')

            if ( self.ShowP1P2CB.GetValue()  == True):
                self.P1P2FileTxt.Show()
                self.P1P2TextCtrl.Show()
                self.P1P2FileButton.Show()

    
##########################################################################
# Name        : SelectP1P2Corrections
# Description : A call to Show Ionosphere SelectP1P2Corrections Function 
##########################################################################    
    def SelectP1P2Corrections(self, event):
        self.SelectP1P2CorrectionsFunc()
        event.Skip()   

##########################################################################
# Name        : SelectP1P2CorrectionsFunc
# Description : The user can select different TGD Corrections, here are
# 		are managed different options.
##########################################################################        
    def SelectP1P2CorrectionsFunc(self):
	if ( self.P1P2CorrectionComboBox.GetSelection() == 0):
                self.ShowP1P2CB.SetValue(False)
        	self.P1P2SourcesComboBox.SetSelection(0)
		self.ShowP1P2DCBSourceFunc()
	elif ( self.P1P2CorrectionComboBox.GetSelection() == 1): 
                self.ShowP1P2CB.SetValue(True)
        	self.P1P2SourcesComboBox.SetSelection(2)
		self.ShowP1P2DCBSourceFunc()
	elif ( self.P1P2CorrectionComboBox.GetSelection() == 2): 
                self.ShowP1P2CB.SetValue(True)
        	self.P1P2SourcesComboBox.SetSelection(3)
		self.ShowP1P2DCBSourceFunc()
	elif ( self.P1P2CorrectionComboBox.GetSelection() == 3): 
                self.ShowP1P2CB.SetValue(True)
        	self.P1P2SourcesComboBox.SetSelection(5)
		self.ShowP1P2DCBSourceFunc()

##########################################################################
# Name        : ShowOrbitClockSource
# Description : A call to Show Ionosphere ShowOrbitClockSource Function 
##########################################################################    
    def ShowOrbitClockSource(self, event):
        self.ShowOrbitClockSourceFunc()
        event.Skip()    

##########################################################################
# Name        : ShowOrbitClockSource
# Description : The user can select the path to load the Orbits and Clocks
#               file from the dialogue that will be opened when it is
#               pressed the Examine button 
##########################################################################        
    def ShowOrbitClockSourceFunc(self):
        if ((self.BroadcastRB.GetValue()) == True):
            self.TextFirstRow.SetLabel('RINEX Navigation File :')
            Aux = self.BroadcastPath
            
            self.TextCtrlFirstRow.Clear()
            self.TextCtrlFirstRow.WriteText(Aux)
            
            self.TextSecondRow.Hide()
            self.FileButtonSecondRow.Hide()
            self.TextCtrlSecondRow.Hide()
            
            # Change Data Interpolation visibility
            self.PreciseProductsDIPanel.Hide()
            
            # Change Navigation mode of Filter section
            if (self.StandaloneRB.GetValue() == False):
                self.StandaloneRB.SetValue(True)
                self.ShowConfigurationTexts()
            
            # Change Ionosphere Source status
            self.BroadcastSameRB.Enable()
            self.BroadcastSameRB.SetValue(True)
            self.ShowIonosphereSourceFunc()

            # Set P1-P2 DCB configuration:   	    
	    self.P1P2SourcesComboBox.SetSelection(0)
	    self.ShowP1P2DCBSourceFunc()
            
        if ((self.SP3RB.GetValue()) == True):
            self.TextFirstRow.SetLabel('SP3 File :')
            Aux = self.SP3Path
            
            self.TextCtrlFirstRow.Clear()
            self.TextCtrlFirstRow.WriteText(Aux)
            
            self.TextSecondRow.Hide()
            self.FileButtonSecondRow.Hide()
            self.TextCtrlSecondRow.Hide()
            
            # Change Data Interpolation visibility
            self.PreciseProductsDIPanel.Show()
            
            # Change Navigation mode of Filter section
            if (self.PPPRB.GetValue() == False):
                self.PPPRB.SetValue(True)
                self.ShowConfigurationTexts()
                
            # Change Ionosphere Source status
            self.BroadcastSpecifyRB.SetValue(True)
            self.BroadcastSameRB.Disable()
            self.ShowIonosphereSourceFunc()
	 
            # Set P1-P2 DCB configuration:   	    
	    self.P1P2SourcesComboBox.SetSelection(1)
	    self.ShowP1P2DCBSourceFunc()
	    """   
            self.PreciseDCBSourceRB.SetValue(True)
	    self.ShowDCBSourceFunc()
            self.BrodcastNavigationDCBSourceRB.Disable()
            """


        if ((self.ClocksRinexOrbitSP3RB.GetValue()) == True):
            self.TextFirstRow.SetLabel('SP3 File :')
            self.TextSecondRow.SetLabel('CLK File :')
            
            aux1 = self.OrbitsSP3Path 
            self.TextCtrlFirstRow.Clear()
            self.TextCtrlFirstRow.WriteText(aux1)
            
            aux2 = self.ClocksRinexPath
            self.TextCtrlSecondRow.Clear()
            self.TextCtrlSecondRow.WriteText(aux2)
            
            self.TextSecondRow.Show()
            self.TextCtrlSecondRow.Show()
            self.FileButtonSecondRow.Show()   
            
            # Change Data Interpolation visibility
            self.PreciseProductsDIPanel.Show()
            
            # Change Navigation mode of Filter section
            if (self.PPPRB.GetValue() == False):
                self.PPPRB.SetValue(True)
                self.ShowConfigurationTexts()
            
            # Change Ionosphere Source status
            self.BroadcastSpecifyRB.SetValue(True)
            self.BroadcastSameRB.Disable()
            self.ShowIonosphereSourceFunc()

            # Set P1-P2 DCB configuration:   	    
	    self.P1P2SourcesComboBox.SetSelection(1)
	    self.ShowP1P2DCBSourceFunc()
            """	    
            self.PreciseDCBSourceRB.SetValue(True)
	    self.ShowDCBSourceFunc()
            self.BrodcastNavigationDCBSourceRB.Disable()
            """
##########################################################################
# Name        : ShowIonosphereSource
# Description : A call to Show Ionosphere Source Function 
##########################################################################
    def ShowIonosphereSource(self, event):
        self.ShowIonosphereSourceFunc()
        event.Skip()

##########################################################################
# Name        : ShowIonosphereSourceFunc
# Description : User can use Broadcast, IONEX or FPPP ionosphere source 
##########################################################################    
    def ShowIonosphereSourceFunc(self):   
        self.BroadcastSpecifyRB.Hide()
        self.BroadcastSameRB.Hide()
        self.IonexRB.Hide()
        self.FPPPRB.Hide()
        self.TextIonosphereSource.Hide()
        self.TextCtrlIonosphereSource.Hide()
        self.FileButtonIonosphereSource.Hide()   

        if (self.ShowIonosphereSourceCB.GetValue() == True):	
	      	self.BroadcastSpecifyRB.Show()
		self.BroadcastSameRB.Show()
        	self.IonexRB.Show()
		self.FPPPRB.Show()
	else:
	      	self.BroadcastSpecifyRB.Hide()
		self.BroadcastSameRB.Hide()
        	self.IonexRB.Hide()
        	self.FPPPRB.Hide()


        if ((self.BroadcastSpecifyRB.GetValue()) == True):
            self.TextIonosphereSource.SetLabel('RINEX Navigation File : ')
            
            aux = self.BroadcastIonosphereSourcePath
            self.TextCtrlIonosphereSource.Clear()
            self.TextCtrlIonosphereSource.WriteText(aux)

            if (self.ShowIonosphereSourceCB.GetValue() == True):           
		self.TextIonosphereSource.Show()
		self.TextCtrlIonosphereSource.Show()
		self.FileButtonIonosphereSource.Show()   


        if ( (self.IonexRB.GetValue())==True ):
            self.TextIonosphereSource.SetLabel('IONEX File : ')
            
            aux5=self.IonexIonosphereSourcePath
            self.TextCtrlIonosphereSource.Clear()
            self.TextCtrlIonosphereSource.WriteText(str(aux5))
           
            if (self.ShowIonosphereSourceCB.GetValue() == True): 
	        self.TextIonosphereSource.Show()
                self.TextCtrlIonosphereSource.Show()
                self.FileButtonIonosphereSource.Show()         

        if ( (self.FPPPRB.GetValue())==True ):
            self.TextIonosphereSource.SetLabel('FPPP File : ')

            aux6=self.FPPPIonosphereSourcePath
            self.TextCtrlIonosphereSource.Clear()
            self.TextCtrlIonosphereSource.WriteText(str(aux6))

            if (self.ShowIonosphereSourceCB.GetValue() == True):   
            	self.TextIonosphereSource.Show()
            	self.TextCtrlIonosphereSource.Show()
            	self.FileButtonIonosphereSource.Show()

	#Select here the same DCB source as the iono source in Input and Modelling panel
        if (self.BroadcastSameRB.GetValue() == True or self.BroadcastSpecifyRB.GetValue() == True ):
                self.P1P2SourcesComboBox.SetSelection(0)
		self.P1P2CorrectionComboBox.SetSelection(0)

        if (self.IonexRB.GetValue() == True):
                self.P1P2SourcesComboBox.SetSelection(3)
		self.P1P2CorrectionComboBox.SetSelection(2)


        if (self.FPPPRB.GetValue() == True):
                self.P1P2SourcesComboBox.SetSelection(5)
		self.P1P2CorrectionComboBox.SetSelection(3)

        self.ShowP1P2DCBSourceFunc()

        #In case IONEX or FPPP is selected, also change in Modelling panel the ionospheric correction
        if (self.IonexRB.GetValue() == True):
		self.IonosphericCorrectionComboBox.SetSelection(2)

        if (self.FPPPRB.GetValue() == True):
		self.IonosphericCorrectionComboBox.SetSelection(3)

    
##################################################################################
# Name        : SaveIonosphereSourcePathByButton
# Description : The user can select the path to load the Ionosphere file from the 
#               dialogue that will be opened when pressed the Examine button 
##################################################################################
    def SaveIonosphereSourcePathByButton(self, event):
        global DireName
        if ((self.BroadcastSpecifyRB.GetValue()) == True):
            FrameName = "Choose Broadcast file"
            if WIN:
                wcd = "Broadcast files (*.??n)|*.??n*|All files|*.*"
            else:
                wcd = "Broadcast files (*.??n)|*.[0-9][0-9][n,N]*|All files|*"
        
	if ((self.IonexRB.GetValue()) == True):
            FrameName = "Choose IONEX file"
            if WIN:
                wcd = "IONEX files (*.??i)|*.??i*|All files|*.*"
            else:
                wcd = "IONEX files (*.??i)|*.[0-9][0-9][i,I]*|All files|*"

        if ((self.FPPPRB.GetValue()) == True):
            FrameName = "Choose FPPP file"
            if WIN:
                wcd = "FPPP files (*.ion)|*.ion*|All files|*.*"
            else:
                wcd = "FPPP files (*.ion)|*.ion*|All files|*"
    
        dlg = wx.FileDialog(self, FrameName, DireName, "", wcd, wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            FileName = dlg.GetFilename()
            DireName = dlg.GetDirectory()
            
            if ((self.BroadcastSpecifyRB.GetValue()) == True):
                self.BroadcastIonosphereSourcePath = os.path.join(DireName, path)
                aux = self.BroadcastIonosphereSourcePath
                self.TextCtrlIonosphereSource.Clear()
                self.TextCtrlIonosphereSource.WriteText(aux)

            if ((self.IonexRB.GetValue()) == True):
                self.IonexIonosphereSourcePath = os.path.join(DireName, path)
                aux = self.IonexIonosphereSourcePath
                self.TextCtrlIonosphereSource.Clear()
                self.TextCtrlIonosphereSource.WriteText(aux)

            if ((self.FPPPRB.GetValue()) == True):
                self.FPPPIonosphereSourcePath = os.path.join(DireName, path)
                aux = self.FPPPIonosphereSourcePath
                self.TextCtrlIonosphereSource.Clear()
                self.TextCtrlIonosphereSource.WriteText(aux)

        dlg.Destroy()    
    
##########################################################################
# Name        : SaveIonosphereSourcePathByTextCtrl
# Description : This function gets the Ionosphere path from the user when 
#               its text control is modified 
##########################################################################
    def SaveIonosphereSourcePathByTextCtrl(self, event):
       if ((self.BroadcastSpecifyRB.GetValue()) == True):
            self.BroadcastIonosphereSourcePath = self.TextCtrlIonosphereSource.GetValue()
       if ( (self.IonexRB.GetValue())==True ):
            self.IonexIonosphereSourcePath = self.TextCtrlIonosphereSource.GetValue()
       if ( (self.FPPPRB.GetValue())==True ):
            self.FPPPIonosphereSourcePath = self.TextCtrlIonosphereSource.GetValue()

            
##########################################################################
# Name        : ShowFilters
# Description : This functions show the Filter panel 
##########################################################################
    def ShowFilters(self, event):     
        self.HideAll()
        self.FilterPanel.Show()
        
        global FirstShow
        if FirstShow == True: 
            self.SetFirstGLABUse() 
                  
        event.Skip()

##########################################################################
# Name        : ShowModes
# Description : This functions show the Modes panel 
##########################################################################
    def ShowModes(self, event):       
        self.HideAll()
        self.ModelPanel.Show()
        
        global FirstShow
        if FirstShow == True: 
            self.SetFirstGLABUse() 
                
        event.Skip()
              
##########################################################################
# Name        : ShowPreProces
# Description : This functions show the Preprocess panel 
##########################################################################
    def ShowPreProces(self, event):      
        self.HideAll()
        self.PreProcesPanel.Show()
        
        global FirstShow
        if FirstShow == True: 
            self.SetFirstGLABUse() 
            
        event.Skip()
        
##########################################################################
# Name        : ShowInputs
# Description : This functions show the Input panel 
##########################################################################
    def ShowInputs(self, event):
        self.HideAll()
        self.InputsPanel.Show()
        
        global FirstShow
        if FirstShow == True: 
            self.SetFirstGLABUse() 
              
        event.Skip()
        
##########################################################################
# Name        : ShowOutput
# Description : This functions show the Output panel 
##########################################################################
    def ShowOutput(self, event):
        self.HideAll()
        self.OutputPanel.Show()
        
        global FirstShow
        if FirstShow == True: 
            self.SetFirstGLABUse() 
                
        event.Skip()

##########################################################################
# Name        : SetFirstGLABUse
# Description : This hides the Welcome logo and shows the run button when
#               user first uses the calculus tab
##########################################################################    
    def SetFirstGLABUse(self):
        self.BackgroundLogo.Hide()
        self.ShowButtons()
        self.EnableCalculusButtons(1, 0)
	self.ShowConfigButton.Disable()
        
        global FirstShow
        FirstShow = False    
       
##########################################################################
# Name        : HideAll
# Description : This hides all the panels linked to the buttons of the  
#               calculus tab and resets the buttons 
##########################################################################
    def HideAll(self):
        self.PreProcesPanel.Hide()
        self.InputsPanel.Hide()
        self.OutputPanel.Hide()
        self.FilterPanel.Hide()
        self.ModelPanel.Hide()
                
        self.InputButton.SetValue(False)
        self.PreprocesButton.SetValue(False)
        self.ModelButton.SetValue(False)
        self.FilterButton.SetValue(False)
        self.OutputButton.SetValue(False)


        """
............................................................................................        
                TOOLTIPS
............................................................................................                   
        """
##########################################################################
# Name        : SetTooltipDictionary
# Description : In this function, the text for the calculus tab tooltips 
#               is stored in a dictionary
##########################################################################
    def SetTooltipDictionary(self):
        self.TooltipTextDictionary = {
                  #Inputs Tooltips:
                          self.InputButton.GetId():u"<b>Input section</b><br><br>This section provides all the configuration options to select the input files for gLAB.<br>",
                          
                      #Preconfigure modes
                          self.PreconfigureSPPB.GetId():u"<b>SPP default configuration</b><br><br>Selects all the options to perform a regular Standard Point Positioning (SPP) processing. Only the following items are necessary for a complete run:<br><ul><li>RINEX observation file from a station.</li><li>broadcast navigation message file for the same day.</li></ul><br><br><b><font color='#C00000'>NOTE:</font></b> By default, the <b>decimation is configured at 300 seconds</b> and all output messages are displayed.<br>If the data decimation is disconnected, the program will require a lot of time writing the output file when all messages are activated (with measurements at 1 second sampling rate). In this case, it is recommended to print only the INFO and OUTPUT messages.<br>",
                          self.PreconfigurePPPB.GetId():u"<b>PPP default configuration</b><br><br>Selects all the options to perform a regular Precise Point Positioning (PPP) processing for the computation of precise coordinates. Only the following items are necessary for a complete run:<br><ul><li>RINEX observation file from a station.</li><li>ANTEX file for precise satellite and receiver antenna data.</li><li>SP3 file for the same day.</li></ul>",
                          
                      #Receiver Position:
                          self.ReceiverPositionUseRinexRB.GetId():u"<b>Use RINEX position as a priori receiver position</b><br><br>Use the RINEX header record <b><i>APPROX POSITION XYZ</i></b> as the receiver a priori position. This is used to linearize the filter and to obtain the values for the models. Besides some fields of the <b>OUTPUT</b> message are given as difference to this a priori position.<br>",
                          self.ReceiverPositionSpecifyRB.GetId():u"<b>Specify a priori receiver position</b><br><br>Specify in XYZ the receiver a priori position. This is used to linearize the filter and to obtain the values for the models. Besides some fields of the <b>OUTPUT</b> message are given as difference to this a priori position.<br>",
                          self.ReceiverPositionCalculateRB.GetId():u"<b>Calculate a priori receiver position</b><br><br>With this option, no a priori receiver position is required to be given, as the program will automatically calculate it. This is especially useful in moving receivers (trajectories) or when the approximate receiver position is not known. The differential fields of the <b>OUTPUT</b> message will be zero, as no reference is given.<br>",
                          self.ReceiverPositionUseSinexRB.GetId():u"<b>Use SINEX position as a priori receiver position</b><br><br>Match the observation RINEX header record <b><i>MARKER NAME</i></b> with the marker position present in the SINEX file. This is used to linearize the filter and to obtain the values for the models. Besides some fields of the <b>OUTPUT</b> message are given as difference to this a priori position.<br>",
                            
                      #Input Files:
                          self.RinexObservation.GetId():u"<b>RINEX observation file</b><br><br>Selects the input RINEX observation file. This file contains the GNSS measurements for a given station. It can understand RINEX formats 2.11 and 3.00.<br>",
                          self.AntexObservation.GetId():u"<b>ANTEX file</b><br><br>Selects the input ANTEX file. This file contains information on the antenna phase centre of the GNSS satellites and ground stations.<br>",
                          self.SP3RB.GetId():u"<b>Precise orbit and clocks (1 file)</b><br><br>Selects as orbit and clock products source an <b>SP3</b> file. This file contains the orbits and clocks of the satellite at specific times. The orbit and clock interpolation degrees in the <i>Modelling</i> section affects the computation of orbit and clocks in this mode.<br>",
                          self.BroadcastRB.GetId():u"<b>Broadcast orbits and clocks</b><br><br>Selects as orbit and clock product source the <b>navigation message</b>. This message is included inside the GPS signal. For processing purposes, you can select the input file where the navigation message is stored in navigation RINEX format.<br>",
                          self.ClocksRinexOrbitSP3RB.GetId():u"<b>Precise orbit and clocks (2 files)</b><br><br>Selects as orbit and clock products source an <b>SP3</b> file (for orbits) and a <b>clk</b> file (for clocks). These files contains the orbits and clocks of the satellite at specific times. The use of a <b>clk</b> file provides a higher rate of the clocks, and thence a higher rate processing steps. The orbit and clock interpolation degrees in the <i>Modelling</i> section affects the computation of orbit and clocks in this mode.<br>",
                          self.BroadcastSameRB.GetId():u"<b>Broadcast (same as navigation)</b><br><br>Use, as ionosphere source, the broadcast file from the <i>Orbit and Clock Source</i> section. Be careful to process in <i>Broadcast</i> mode.",
                          self.BroadcastSpecifyRB.GetId():u"<b>Broadcast (specify)</b><br><br>Specify a file to obtain the Klobuchar (GPS) correction from.",
                          self.IonexRB.GetId():u"<b>IONEX</b><br><br>Specify a file to obtain the IONEX correction.",
                          self.FPPPRB.GetId():u"<b>FPPP</b><br><br>Specify a file to obtain the FPPP correction.",

                     #Auxiliary Files:			 
        		  self.P1C1FileTxt.GetId():u"<b>P1-C1 DCB File: </b><br><br>Specify a file with the P1-C1 Differential Code Bias <b>DCB</b> corrections. This file is generated by the International GNSS Service (<b>IGS</b>) and gives an accurate monthly estimation of the (P1-C1) bias for all Satellites. <br><br> Because the <b><font color='#C00000'> code generation depends on the Receiver Type,</font></b> this receiver-related information has to be given (see next field). <br><br>P1-C1 DCB files can be found at: <br> <font color='#0000FF'> ftp://ftp.unibe.ch/aiub/CODE/</font>",

			  self.ReceiverTypeTxt.GetId():u"<b>Receiver Type File: </b><br><br>Specify a receiver type file. This file details naming conventions for IGS equipment, in-site logs, RINEX headers, and SINEX files. <br><br> These naming conventions include manufacturer's name, model and code of: Receivers, Antennas, Radomes, and Antenna+Radome. Additional and historical equipment names also appear. <br><br> Because codes are generated differently depending the Receiver Type, C1-P1 biases need to be removed as follows: <ul><li><b> Type 0 </b> : C1-P1 classification unknown. </li> <li><b> Type 1 </b> : Cross correlated receiver, reporting: <ul> <li> <br> C1 </li><li> P2 = C1 + (Y<sub>2</sub> - Y<sub>1</sub>)</li> <li> <b><font color='#C00000'>  Needs Corrections in C1->C1+b and P2->P2+b' </font></b>  </li> </ul> </li>  <li><b> Type 2 </b> : C1 reported in place of P1: <ul> <li> <b><font color='#C00000'>  Needs Corrections in C1->C1+b  </font></b> </li> </ul> </li> <li><b> Type 3 </b> : L1,L2,P1,P2 reported as consistent set: <ul><li> <b><font color='#C00000'>  No bias removal needed. </li> </ul>  </font></b> </li></ul> <br><br>GPS Receiver Types files can be found at: <br> <font color='#0000FF'> ftp://sideshow.jpl.nasa.gov/pub/gipsy_products/gipsy_params/</font>",
	
			  self.P1P2SourceTxt.GetId():u"<b>Source of DCB Corrections: </b><br><br> Select the P1-P2 Differential Code Bias (<b>DCB</b>) corrections source. ",

                          self.P1P2SourcesComboBox.GetId():u"<b>Source of DCB Corrections: </b><br><br> Select the P1-P2 Differential Code Bias (<b>DCB</b>) corrections source. User can select three different types of sources: <ul><li>The same <b>RINEX Navigation</b> file used to calculate satellite orbits and clocks (i.e, satellite positions and satellite clock offsets).</li> <li>A different <b>RINEX Navigation</b> file than the one is used to calculate satellite orbits and clocks. </li><li><b>Precise .DCB</b> file where accurate estimation of the (P1-P2) bias is given. </li></ul> ",

                          self.P1P2FileTxt.GetId():u"<b>File of DCB Corrections: </b><br><br> Depending on the Differential Code Bias (<b>DCB</b>) corrections source, two different files can be upload here: <ul><li>A <b>RINEX Navigation</b> file where the (P2-P1) bias are given as the Total Group Delay (<b>TGD</b>) located in the 3<sup>rd</sub> field of the 7<sup>th</sub> row of the broadcast ephemerides. This estimation is generated by the Control Segment that monitorises all satellites.</li><li>A <b>Precise .DCB</b> file where the International GNSS Service (<b>IGS</b>) gives an accurate estimation of the (P1-P2) bias. This file contains a monthly estimation of the (P1-P2) Differential Code Bias (<b>DCB</b>) for all Satellites. These DCB files can be found at: <font color='#0000FF'> ftp://ftp.unibe.ch/aiub/CODE/</font> </li></ul> <br><br><br><b><font color='#C00000'>About TGDs & DCBs:</font></b><br><br>The relation between the DCBs of the precise files and the TGDs broadcast in the GPS navigation message is:<br> TGD<sub>GPS Broadcast Message</sub> = -&alpha;<sub>1</sub> / ( &alpha;<sub>2</sub> - &alpha;<sub>1</sub> ) · DCB<sub>File</sub><br> Where &alpha;<sub>i</sub> = 40.3 / f<sub>i</sub><sup>2</sup>",

                  #Preprocess Tooltips:
                          self.PreprocesButton.GetId():u"<b>Preprocess section</b><br><br>This section provides all the configuration options to preprocess the input data. In particular, it allows changing the decimation rate, the elevation mask, the cycle-slip detection, and selecting individual satellites for the processing.<br>",

                      #Data Decimation:
                          self.DataDecimationCB.GetId():u"<b>Data decimation</b><br><br>Mark this option to decimate the input data at the specified rate [in seconds]. If this option is unchecked, every time an epoch is found in the input RINEX observation file, all the processing takes place. If this option is checked, the data is decimated and not even modelled. Even in decimated data, all the epochs are used for cycle-slip detection, and arc length computations, but the process is stopped just before the modelling.<br>This option is meant to be used to reduce computation time.<br>",


                      #FlexibleC1P1 Handling:
#                          self.FlexibleC1P1HandlingCB.GetId():u"<b>Flexible C1/P1 Handling</b><br><br>Mark this option to set a flexible C1 to P1 conversions.<br>This is used for receivers that provide C1 but do not provide P1 (or the contrary). With this option set, gLAB will use whichever C1 or P1 measurement is available in the receiver.<br><br> <b><font color='#C00000'>NOTE:</font></b> <br> <ul><li>The DCB file (C1 -P1) is used when available in Flexible DCB mode. (User can provide this file in the Input Section). </li> <li>In Strict DCB Mode, the Data Processing Core (DPC) stops if DCB file (C1 - P1) is not provided.</li></ul> ",

                      #StrictDCBs Handling:
#                          self.StrictDCBsCB.GetId():u"<b>Strict DCB's Handling</b><br><br>Mark this option to set a strict TGDs treatment. This option <b>requires</b> both files to be provided:<ul><li>The *.DCB files: TGD data is required for all the code measurements.</li><li>The receiver file type: If the receiver is not classified in its proper type no processing can be done. </li></ul>  <br><br> Disabling this option it will allow using only P1-C1 and thence correcting only C1, leaving P1 and P2 unmodified. This can be useful when navigating with PC (using P1 and P2 measurements will remove all TGD without the need of explicitly correcting it) or PCC (correcting only the C1's TGDs)<br><br><b><font color='#C00000'>NOTE:</font></b> While this option is active, the Auxiliary Files [In the Input Section] are displayed and only precise *.DCB files can be set as DCB source (RINEX Nav files are not allowed).<br>",

                      #Satellite Options:
                          self.ElevationMaskText.GetId():u"<b>Elevation mask</b><br><br>The elevation mask parameter is used to discard all the satellites below the specified elevation. Low elevation satellites should be discarded for geodetic processing as they may contain increased errors due to low signal-to-noise ratio and multipath.<br>",
                          self.DiscardEclipsedSatellitesCB.GetId():u"<b>Discard satellites under eclipse condition</b><br><br>This parameter allows to activate the discard of satellites if they are under eclipse conditions:<ul><li>They do not have direct visibility of the Sun or</li><li>They have been in the former condition at some time of the last 30 minutes.</li></ul>",

                          self.DiscardUnhealthySatellitesCB.GetId():u"<b>Discard unhealthy satellites </b><br><br>This parameter allows to discard satellites based upon the healthy flag of the broadcast <b>navigation message.</b> <br>",

                      #Cycle-Slip:
                          self.DataGapText.GetId():u"<b>Data Gap</b><br><br>This parameter sets the maximum period of time (in seconds) allowed between two consecutive data. Therefore, a larger period of time will declare cycle-slip.",
                          self.EnableLLI.GetId():u"<b>Loss of Lock Indicator (LLI)</b><br><br>The LLI parameter is included in the RINEX file and when it is odd; i.e. 1, 3, 5 or 7, indicates that the measurement may contain cycle-slip.",
                          self.EnableLICB.GetId():u"<b>Geometry-free carrier phase combination</b><br><br>This cycle-slip detector for dual-frequency receivers uses only carrier phase measurements. The detection is based on fitting a second-degree polynomial with the the geometry-free combination within a sliding window of <b><i>N-sample</i></b>. Note that this combination may be affected by ionosphere. A cycle-slip is declared when any of the two following statements (OR) is fulfilled:<blockquote>1. If the expected value is larger than the measured one by more than a specific threshold (<i>Th</i>) <b>AND</b> also larger than 2 times the residual of the LMS used to fit the second-degree polynomial.<br>The threshold is obtained as:<br><i>Th</i> = <b><i>maximum threshold</i></b> / (1 + exp( -<i>Data Gap</i> / <b><i>Time constant</i></b> ))<br><br>2. The jump between two consecutive geometry-free combinations (LI) is larger than the <b><i>maximum jump threshold</i></b> (<i>th</i>).<br>That is,<br>| <i>LI<sub>t</sub></i> - <i>LI<sub>t-1</sub></i> | > <i>th</i></blockquote><i><b>N-sample</i></b> is the number of epochs used to fit the second-dregree polynomial.<br><br><b><font color='#C00000'><b>Notice</b></font> that the geometry-free cycle-slip detector, if enabled, will evaluate the signals from the epoch <i><b>N-sample</i></b> + 1. Furthermore, the first <i>n</i> epochs of every satellite's arc length will be discarded, where<br><i>n</i> = maximum( <i>N-sample of L1-C1 [if enabled]</i>, <i>N-sample of Melbourne-Wübbena [if enabled]</i>, <i>N-sample of Geometry-free [if enabled]</i> )<br>",
                          self.EnableMelbourneWubbenaCB.GetId():u"<b>Melbourne-Wübbena combination</b><br><br>This cycle-slip detector for dual-frequency receivers uses the Melbourne-Wübbena combination (geometry-free ionosphere-free). This combination uses pseudorange measurements, and thence, can be very affected by high noise and multipath effects. This combination is basically a constant with noise and jumps due to the cycle-slips. In this sense, the mean, the standard deviation and a mobile mean over the sliding <b><i>window</i></b> (last seconds) is computed. A cycle-slip is declared when three conditions are fulfilled:<blockquote>1. The mean is compared with the measured value, and it must be larger than a specified threshold. The threshold depends on the standard deviation from the last cycle-slip, and is computed as: <i>Th</i> = <b><i>k-factor</i></b> · <i>stdDev</i><br><br>2. The mobile mean over the <b><i>window</i></b> is compared with the measured value, and it must be larger than the wave length of the Melbourne-Wübbena combination (~0.8 m).<br><br>3. The standard deviation must be equal or larger than the <b><i>minimum stdDev</i></b> (configurable parameter) set as default to the wave length of the Melbourne-Wübbena combination (~0.8 m).</blockquote><br><b><i>N-sample</i></b> is the number of epochs needed to stabilize both mean and stdDev, thus, it shall be >= 2.<br><br><font color='#C00000'><b>Notice</b></font> that the Melbourne-Wübbena cycle-slip detector, if enabled, will evaluate the signals from the epoch <i><b>N-sample</i></b> + 1. Furthermore, the first <i>n</i> epochs of every satellite's arc length will be discarded, where<br><i>n</i> = maximum( <i>N-sample of L1-C1 [if enabled]</i>, <i>N-sample of Melbourne-Wübbena [if enabled]</i>, <i>N-sample of Geometry-free [if enabled]</i> )<br>",
                          #self.EnableMelbourneWubbenaCB.GetId():u"<b>Melbourne-Wübbena combination</b><br><br>This cycle-slip detector for dual-frequency receivers uses the Melbourne-Wübbena combination (geometry-free ionosphere-free). This combination uses pseudorange measurements, and thence, can be very affected by high noise and multipath effects. This combination is basically a constant with noise and jumps due to the cycle-slips. In this sense, the mean, the standard deviation and a mobile mean over the sliding <b><i>window</i></b> (last seconds) is computed. A cycle-slip is declared when three conditions are fulfilled:<blockquote>1. The mean is compared with the measured value, and it must be larger than a specified threshold. The threshold depends on the standard deviation from the last cycle-slip, and is computed as: <i>Th</i> = <b><i>k-factor</i></b> · <i>stdDev</i><br><br>2. The mobile mean over the <b><i>window</i></b> is compared with the measured value, and it must be larger than the wave length of the Melbourne-Wübbena combination (~0.8 m).<br><br>3. The standard deviation must be equal or larger than the <b><i>minimum stdDev</i></b> (configurable parameter) set as default to the wave length of the Melbourne-Wübbena combination (~0.8 m).</blockquote><br><font color='#C00000'><b>Notice</b></font> that the Melbourne-Wübbena cycle-slip detector, if enabled, will evaluate the signals from the first epoch, unless the geometry-free cycle-slip detector (dual-frequency detector too) was activated, in such case, the Melbourne-Wübbena detector will not be able to detect/declare cycle-slips until reaching the N-sample + 1 selected in the geometry-free detector.<br>",
                          self.EnableC1L1DifferenceCB.GetId():u"<b>L1-C1 difference</b><br><br>This cycle-slip detector for single-frequency receivers uses the difference between L1 and C1 (L1P and C1C). This difference contains basically noise coming from C1, sudden jumps coming from cycle-slips, and a ionospheric divergence with time, due to the different effects that the ionosphere causes in carrier phase and pseudorange measurements. This detector computes the mean and standard deviation of L1-C1 along the epochs, proving a window to limit the divergence. The expected mean value is compared against the obtained one, and if it is larger than a specific threshold, a cycle-slip is declared.<br>The threshold is obtained as:<br><i>Th</i> = <b>k-factor</b>·<i>stdDev</i><br>The sigma is inflated with an <b>initial stdDev</b> during the first iteration and after every cycle-slip with the aim to have a sigma and to be able to use the first epoch.<br><i><b>N-sample</i></b> is the number of epochs needed to stabilize both mean and stdDev, thus, it shall be >= 2. Smaller values result into using the <b>initial stdDev</b>.<br>Being <b>k</b>, <b>initial stdDev</b>, <b>window</b> size and the <b>N-sample</b> configurable parameters.<br><br><b><font color='#C00000'><b>Notice</b></font> that the L1-C1 difference cycle-slip detector, if enabled, will evaluate the signals from the epoch <i><b>N-sample</i></b> + 1. Furthermore, the first <i>n</i> epochs of every satellite's arc length will be discarded, where<br><i>n</i> = maximum( <i>N-sample of L1-C1 [if enabled]</i>, <i>N-sample of Melbourne-Wübbena [if enabled]</i>, <i>N-sample of Geometry-free [if enabled]</i> )<br>",
                          #self.EnableC1L1DifferenceCB.GetId():u"<b>L1-C1 difference</b><br><br>This cycle-slip detector for single-frequency receivers uses the difference between L1 and C1 (L1P and C1C). This difference contains basically noise coming from C1, sudden jumps coming from cycle-slips, and a ionospheric divergence with time, due to the different effects that the ionosphere causes in carrier phase and pseudorange measurements. This detector computes the mean and standard deviation of L1-C1 along the epochs, proving a window to limit the divergence. The expected mean value is compared against the obtained one, and if it is larger than a specific threshold, a cycle-slip is declared.<br>The threshold is obtained as:<br><i>Th</i> = <b>k-factor</b>·<i>stdDev</i><br>The sigma is inflated with an <b>initial stdDev</b> during the first iteration and after every cycle-slip with the aim to have a sigma and to be able to use the first epoch.<br>Being <b>k</b>, <b>initial stdDev</b> and the <b>window</b> size configurable parameters.<br><font color='#C00000'><b>Notice</b></font> that the L1-C1 difference cycle-slip detector, if enabled, will evaluate the signals from the first epoch, unless the geometry-free cycle-slip detector (dual-frequency detector) was activated, in such case, the L1-C1 difference cycle-slip detector will not be able to detect/declare cycle-slips until reaching the N-sample + 1 selected in the geometry-free detector.<br>",

                      #PRN Selection:
                          self.EnableGPSRB.GetId():u"<b>GPS satellite selector</b><br><br>The buttons allow to individually select/deselect each satellite for processing. A deselected satellite shall not be taken into account when processing. <font color=green><b>Green</b></font> color marks selected satellites and <font color='#E6322B'><b>Red</b></font> deselected satellites.<br>",
                          self.EnableGalileoRB.GetId():u"<b>Galileo satellite selector</b><br>",
                          self.EnableGlonassRB.GetId():u"<b>GLONASS satellite selector</b><br>",

                  #Modelling Tooltips:                          
                          self.ModelButton.GetId() :u"<b>Modelling section</b><br><br>This section provides the configuration options to set/unset each individual model that is used by gLAB.<br>",

                      #Modelling Options:
                          self.SatelliteClockOffsetCorrection.GetId():u"<b>Satellite clock offset correction</b><br><br>" "The satellite clock offsets correspond to the clock synchronism errors of the satellite clocks in relation to the GNSS system time scale. These errors depend heavily on the type of oscillator of the satellite and are quite unpredictable. They can only be obtained by some kind of estimation. The typical source for estimations of these errors are the own <b>navigation message</b>, or some kind of external estimation, such as <b>SP3</b> files. The effect of these clock offsets can reach up to <font color=red>hundreds of kilometres</font>.<br>",
                          self.SatelliteMovementFlightTime.GetId():u"<b>Consider satellite movement during signal flight time<font></b><br><br>Due to the distance between satellites and receivers (between 20000 and 30000 Km for GPS), the signal travel time is not despicable (about 70 ms for GPS). Thence the receiver is obtaining the measurement after it has been emitted by the satellite. This fact should be taken into consideration, as the position of the satellite must be computed in the transmission time, not in the reception time. This effect can impact on the measurements up to <font color=red>hundreds of metres</font>.<br>",
                          self.EarthRotationFlightTime.GetId():u"<b>Consider Earth rotation during signal flight time<font></b><br><br>Besides the satellite movement during signal flight time, the Earth also moves [rotates]. If this effect is not taken into consideration, an error of about <font color=red>30 m</font> in the east direction would be seen.<br>",
                          self.SatellitePhaseCenterCorrection.GetId():u"<b>Satellite mass centre to antenna phase centre correction<font></b><br><br>Each data source of satellite orbits (in general the <b>navigation message</b> or an <b>SP3</b> file) provides these orbits in its specific reference. In particular, the <b>SP3</b> files provide the positions of the satellite referred to its mass centre (which is different than the antenna phase centres). In order to properly correct the GNSS measurements with the satellite position, a correction between these two centres must be done. The positions extracted from the <b>navigation message</b> do not require any additional corrections, as they are referred to the antenna phase centre.<br>Usually these corrections can be obtained from an ANTEX file.<br>This error can be up to <font color=red>1-2 m</font>.<br>",
                          self.ReceiverPhaseCenterCorrection.GetId():u"<b>Receiver antenna phase centre correction<font></b><br><br>Normally the positions of the stations are given in relation to the base of the station. The difference between this point and the antenna phase centre should be taken into account. This effect depends on the frequency, and can reach up to <font color=red>some decimetres</font>.<br>",
                          self.ReceiverRefPointCorrection.GetId():u"<b>Antenna Reference Point correction<font></b><br><br>Additionally to the Antenna Phase Centre, the position of a station can be given in relation to a specific point (such as a geodetically positioned point in the ground). This correction allows to give a specific correction to the position in North/East/Up components.<br>",
                          self.RelativisticCorrection.GetId():u"<b>Relativistic clock correction (orbit eccentricity)<font></b><br><br>The rate of advance of two identical clocks, one placed in the satellite and the other on the terrestrial surface, will differ due to the difference of the gravitational potential (general relativity) and to the relative speed between them (special relativity). The special relativity difference can be broken into:<br><ul><li>A constant component that only depends on the nominal value of the semi-major axis of the satellite orbit, which is adjusted modifying the clock oscillating frequency of the satellite<br></li><li>A periodical component due to the orbit eccentricity (that must be adjusted by the user receiver) equal to:<br><i>rel</i> = 2*(<b>r</b>·<b>v</b>)/c</li></ul><br>This effect can reach up to <font color=red>13 m</font>.<br>",
                          self.IonosphericCorrectionCheckBox.GetId():u"<b>Ionospheric correction</font></b><br><br>The ionosphere is the zone of the terrestrial atmosphere that extends itself from about 60 km until more than 2000 km high. Due to the interaction with free electrons, electromagnetic signals that go through it suffer a delay/advancement in relation to the propagation in a vacuum. This effect is a dispersive effect (depend on the frequency), and can be removed in multi-frequency receivers (with a specific combination of measurements). The ionosphere is hard to model, and the Klobuchar (GPS) model (the one defined in the GPS/SPS-SS and available in the <b>navigation message</b>) can only reduce its impact between a 50% and a 60%.<br><br>The ionosphere effect can reach up to <font color=red>50 m</font> in turbulent ionospheric environments.<br>",

#                          self.TroposphericCorrectionCheckBox.GetId():u"<b>Tropospheric correction</font></b><br><br>At the frequency which the GPS signal is emitted, the troposphere behaves like a non dispersive media, being its effect independent of the frequency. The tropospheric delay can be modelled in an approximate way (about 90%-95%) using the following expression:<br>Tij = d<i><font size=-2>dry</font></i>·m<i><font size=-2>dry</font></i>(elev) + d<i><font size=-2>wet</font></i> · m<i><font size=-2>wet</font></i>(elev)<br>where d<i><font size=-2>dry</font></i> corresponds to the vertical delay due to the dry component of the troposphere and d<i>wet</i> corresponds to the vertical delay associated with the wet component (due to the water vapour of the atmosphere), being:<br>d<i><font size=-2>dry</font></i> = 2.3 · exp(−0.116 · 10<sup>−3</sup> · H)<br>d<i><font size=-2>wet</font></i> = 0.1<br>where H is the height over the ellipsoid. Finally m<i><font size=-2>dry</font></i>(elev) and m<i><font size=-2>wet</font></i>(elev) are the slant factors in order to project the vertical delay in the direction of the satellite observation for the dry and wet components. Two mappings can be chosen to compute these m(elev):<br><ul><li>A simple mapping (SBAS like)</li><li>The mapping of Niell</li></ul>The main part of the troposphere which has not been properly modelled (about 10%) corresponds mainly to the wet component. The effect of troposphere can range up to <font color=red>10 m</font>.<br>",

                          self.TroposphericCorrectionCheckBox.GetId():u"<b>Tropospheric correction</font></b><br><br>Troposphere is the atmospheric layer placed between Earth’s surface and an altitude of about 60 kilometres. The effect of the troposphere on the GNSS signals appears as an extra delay in the measurement of the signal travelling from the satellite to receiver. This delay depends on the temperature, pressure, humidity as well as the transmitter and receiver antennas location. Because at the frequency which the GPS signal is emitted the troposphere behaves like a non dispersive media, its effect independent of the frequency. The tropospheric delay can be modelled in an approximate way (about 90%-95%) using the following expression:<br><br> T ( elev ) = T<i><sub>dry</sub></i> · M<i><sub>dry</sub></i>( elev ) + T<i><sub>wet</sub></i> · M<i><sub>wet</sub></i>( elev )<br><br>Where it is separately computed:<br><ul><li> <b>Tropospheric Nominals</b>: T<i><sub>dry</sub></i> and  T<i><sub>wet</sub></i>: The Tropospheric dry and wet nominal component. It can be divided in hydrostatic, i.e., Dry gases (mainly N<sub>2</sub> and O<sub>2</sub> ), and wet, i.e., Water vapour. </li> <li> <b>Tropospheric Mappings</b>: M<i><sub>dry</sub></i>( elev ) and M<i><sub>wet</sub></i>( elev ) : The Tropospheric dry and wet mappings. The tropospheric delay depends on the signal path through the neutral atmosphere, and thence, can be modelled as a function of the satellite elevation angle. </li></ul> <br><br> The effect of troposphere can range up to <font color=red>10 m</font>.<br>",


                          self.NominalTroposphericCorrectionComboBox.GetId():u"<b>Tropospheric Nominal</font></b><br><br>Two different tropospheric nominals are already implemented in gLAB :<br> <br><ul><li> <b> Simple Nominal </b>: This approach allows a huge simplification of the model for the vertical delays because it does not require any surface meteorological data. Nominal values are computed as follows: <ul><li> T<i><sub>dry</sub></i> = 2.3 · exp(−0.116 · 10<sup>−3</sup> · H) </li> <li> T<i><sub>wet</sub></i> = 0.1</li></ul> Where <b> H </b> is the receiver height over the ellipsoid. </li> <br> <li><b> UNB-3 Nominal</b>: Where T<i><sub>dry</sub></i> and T<i><sub>wet</sub></i> are calculated from the receiver’s height and estimates of five meteorological parameters: pressure [P (mbar)], temperature [T (K)], water vapour pressure [e (mbar)], temperature ”lapse” rate [β (o K/m)] and water vapour ”lapse rate” [λ (dimensionless)] according to procedure described in [RTCA-MOPS, 2006]. </li></ul> ",

                          self.MappingTroposphericCorrectionComboBox.GetId(): u"<b>Tropospheric Mappings</font></b><br><br>Two different tropospheric mappings are already implemented in gLAB :<br> <br><ul><li> <b> Simple Mapping </b>:  It uses a common mapping for both components:<br> <ul><li>  M<i><sub>dry</sub></i>( elev ) = M<i><sub>wet</sub></i>( elev ) =  M ( elev ) = 1.001 / sqrt ( 0.002001 + sin<sup>2</sup>( elev) ) </li></ul>The obliquity factor M ( elev ) is described in [Black and Eisner, 1984]. It is valid for satellite elevation angles over 5 degrees. </li> <br> <li><b> Niell Mapping </b>: It does not require surface meteorology measurements, but provides comparable accuracy and precision to others that requires such data is presented in [Niell, 1996]. This mapping only use the receiver geographic location and measurement time as inputs. Niell’s mapping involves two different obliquity factors, for the M<i><sub>dry</sub></i>( elev ) and M<i><sub>wet</sub></i>( elev ) components. </li></ul> ",


                          self.P1P2CorrectionCheckBox.GetId():u"<b>P1 - P2 Correction</font></b><br><br>Differential Code Biases (<b>DCB</b>) are the delays due to electronic, antennas and cables of receiver and transmitter devices which directly affect the measurements with a bias. This effect depends on the frequency and can be corrected using the information extracted from:<br><br><ul><li>A <b>RINEX Navigation</b> file where the (P2-P1) bias are given as the Total Group Delay (<b>TGD</b>) located in the 3<sup>rd</sub> field of the 7<sup>th</sub> row of the broadcast ephemerides. This estimation is generated by the Control Segment that monitorises all satellites.</li><br><li>A <b>Precise .DCB</b> file where the International GNSS Service (<b>IGS</b>) gives an accurate estimation of the (P1-P2) bias. This file contains a monthly estimation of the (P1-P2) Differential Code Bias (<b>DCB</b>) for all Satellites. </li></ul> <br><br>These P1-P2 DCBs are cancelled in the ionospheric-free combination. This effect can reach up to <font color=red>the meter level</font>.<br><br><br><b><font color='#C00000'>About TGDs & DCBs:</font></b><br><br>The relation between the DCBs of the precise files and the TGDs broadcast in the GPS navigation message is:<br> TGD<sub>GPS Broadcast Message</sub> = -&alpha;<sub>1</sub> / ( &alpha;<sub>2</sub> - &alpha;<sub>1</sub> ) · DCB<sub>File</sub><br> Where &alpha;<sub>i</sub> = 40.3 / f<sub>i</sub><sup>2</sup>",

                          self.P1C1CorrectionCheckBox.GetId():u"<b>P1 - C1 Correction</b><br><br>Differential Code Biases (<b>DCB</b>) are the delays due to electronic, antennas and cables of receiver and transmitter devices which directly affect the measurements with a bias. <br><br> Mark this option to correct C1 - P1 bias. Because the code generation depends on the Receiver Type, this receiver-related information has to be given together with the corrections.  gLAB works in two different modes: <br> <br><ul><li><b>Flexible : </b> gLAB will use whichever C1 or P1 measurement is available in the receiver,<b><font color='#C00000'> without correcting DCB</font></b>. If both code measures are available, P1 will be used as default.  <br>Used when receiver provides C1 but P1 is missing (or viceversa). </li> <br><li><b> Strict : </b> gLAB Data Processing Core (DPC) <b><font color='#C00000'> stops if the receiver cannot be matched </font></b>with the Receiver Type File or <b><font color='#C00000'>both</font></b> files are not provided (in the Input Section): <ul><li> P1-C1 DCB File : Containing the P1-C1 DCB corrections.</li> <li> Receiver Type File : To identify how codes are generated in the receiver, and how to remove these C1-P1 biases. </li> </ul> </li></ul> <br><br> <b><font color='#C00000'>NOTE:</font></b> When this correction is unset, gLAB DPC will not identify P1 = C1, and will only detect cycle-slips with P1 if this code is provided by the receiver. In the case there receiver does not provide P1, no cycle-slip detection can be done, nor PC combination is computed.",


                          self.WindUpCorrection.GetId():u"<b>Wind up correction</font></b><br><br>The wind up only appears in carrier phase measurements and is due to the rotation of the Line-of-sight vector in relation to the antenna. The wind-up has an accumulative effect, and for fixed antennas can reach up to <font color=red>half the wave length</font> of the measurement.<br>",
                          self.SolidTidesCorrection.GetId():u"<b>Solid tides correction</font></b><br><br>The attraction of Sun and Moon and the an-elasticity of the Earth's mantle causes variations to the positions of ground receivers. This effect can reach up to <font color=red>some decimetres</font>.<br>",
                          self.GravitationalDelayCorrection.GetId():u"<b>Relativistic path range correction</font></b><br><br>As introduced in the <i>Relativistic clock correction</i> section, the difference of the gravitational potential (general relativity) affects the measurement. This is a small effect that has an elevation dependence, and has a total effect of about <font color=red>4 cm</font>.<br>",

                      #Precise Products Data Interpolation:  
                          self.OrbitInterpolationDegreeText.GetId():u"<b>Orbit interpolation degree</font></b><br><br>This value is only valid for orbits obtained from <b>SP3</b> files, as they require an interpolation. By default, the interpolation is done with a polynomial of degree 9, but this value can be adjusted with this parameter. Excessively low values would hardly impact the precision of the position obtained.<br>",
                          self.ClockInterpolationDegreeText.GetId():u"<b>Clock interpolation degree</font></b><br><br>This value is only valid for clocks obtained from <b>SP3</b> or <b>clk</b> files. By default, no interpolation is done (\"degree\" 0), but you can chose to activate the interpolation by providing a number different than 0. Due to the unpredictability of clocks, and its non-smoothed nature, the interpolation of low sampling rate clocks (i.e., t>30 secs) would strongly affect the precision of the clocks obtained. Only clocks with sampling rate higher than 1/30s should be interpolated with a polynomial of degree 1.<br>",

                      #Receiver Antenna Phase Center  
                          self.ReadAntennaPhaseCenterRB.GetId():u"<b>Read receiver Antenna Phase Centre from ANTEX</b><br><br>Read the Phase Centre data of the receiver from the ANTEX file specified in the <b>INPUT</b> section. It tries to obtain the name of the antenna using the RINEX header record <i>ANT # / TYPE</i>, and seeks for that name in the ANTEX file. If it is not found, it gives an error, and asks to disconnect this option.<br>",
                          self.SpecifyAntennaPhaseCenterRB.GetId():u"<b>Specify receiver Antenna Phase Centre</b><br><br>Specify in North/East/Up components the receiver antenna phase centre. Different values can be specified for different frequencies.<br>",
                          self.F1AntennaPhaseCenterRB.GetId():u"Receiver antenna phase centre at frequency F1<br>",
                          self.F2AntennaPhaseCenterRB.GetId():u"Receiver antenna phase centre at frequency F2<br>",

                      #Receiver Antenna Reference Point  
                          self.SpecifyAntennaReferencePointRB.GetId():u"<b>Specify receiver Antenna Reference Point</b><br><br>Specify in North/East/Up components the receiver Antenna Reference Point.<br>",
                          self.ReadAntennaReferencePointRB.GetId():u"<b>Read receiver Antenna Reference Point from RINEX</b><br><br>Read the receiver Antenna Reference Point from the RINEX file. It seeks for the <i>ANTENNA: DELTA H/E/N</i> RINEX header record.<br>",

                  #Filter Tooltips:                                   
                          self.FilterButton.GetId():u"<b>Filter section</b><br><br>This section provides all the configuration options to specify the behaviour of the Kalman Filter. In particular, the selection of measurement and the parameters to be estimated can be chosen in this section.<br>",

                       #Navigation Mode
                           self.PPPRB.GetId():u"<b>Precise Point Positioning (PPP)</b><br><br>Sets the navigation mode to <b>PPP</b>. When <i>Pseudorange+Carrier phase</i> and <i>Dual-Frequency</i> are selected, the following options are used as default:<br><ul><li><b>Filter Measurements:</b> Pc (weight=1m) and Lc (weight=1cm)</li><li><b>Ambiguities estimation:</b> Connected </ul><br>When <i>Pseudorange</i> and <i>Dual-Frequency<i> are selected:<br><ul><li><b>Filter Measurements:</b> Pc (weight=1m)</li><li><b>Ambiguities estimation:</b> Disconnected</li></ul><br>When <i>Pseudorange+Carrier phase</i> and <i>Single-Frequency</i> are selected:<br><ul><li><b>Filter Measurements:</b> C1C (weight=2m) [C1] and G1C (weight=30cm) [GRAPHIC from C1 and L1]</li><li><b>Ambiguities estimation:</b> Connected</li></ul><br>When <i>Pseudorange</i> and <i>Single-Frequency</i> are selected:<br><ul><li><b>Filter Measurements:</b> C1C (weight=2m) [C1]</li><li><b>Ambiguities estimation:</b> Disconnected</li></ul><b><font color='#C00000'>IMPORTANT:</font></b> When <b>PPP</b> is selected gLAB requires the use of any of the types of precise products source (selected in the <i>Input</i> section).<br>",
                           self.StandaloneRB.GetId():u"<b>GPS Standalone positioning</b><br><br>Sets the navigation mode to <b>standalone</b>. When <i>Pseudorange+Carrier phase</i> and <i>Dual-Frequency</i> are selected, the following options are used as default:<br><ul><li><b>Filter Measurements:</b> Pc (weight=1m) and Lc (weight=1cm)</li><li><b>Ambiguities estimation:</b> Connected </ul><br>When <i>Pseudorange</i> and <i>Dual-Frequency<i> are selected:<br><ul><li><b>Filter Measurements:</b> Pc (weight=1m)</li><li><b>Ambiguities estimation:</b> Disconnected</li></ul><br>When <i>Pseudorange+Carrier phase</i> and <i>Single-Frequency</i> are selected:<br><ul><li><b>Filter Measurements:</b> C1C (weight=5m) [C1] and L1P (weight=10cm) [L1]</li><li><b>Ambiguities estimation:</b> Connected</li></ul><br>When <i>Pseudorange</i> and <i>Single-Frequency</i> are selected:<br><ul><li><b>Filter Measurements:</b> C1C (weight=5m) [C1]</li><li><b>Ambiguities estimation:</b> Disconnected</li></ul><br><b><font color='#C00000'>IMPORTANT:</font></b> When <b>standalone</b> is selected gLAB requires the use of the <b>navigation message</b> as the products source (selected in the <i>Input</i> section).<br>",
                       #Troposphere
                           self.EstimateTroposphereCB.GetId():u"<b>Estimate Troposphere</b><br><br>Activates the estimation of the troposphere. This estimation tries to remove the part of the wet troposphere not removed by the nominal modelling (see <i>Modelling</i> section). The estimation depends on the model chosen to compute the m<i><font size=-2>wet</font></i>(elev) function (Niell should be chosen for better results).<br><b><font color='#C00000'>IMPORTANT:</font></b> A reliable troposphere estimation can only be obtained in the following options:<br><ul><li><b>Navigation Mode:</b> PPP</li><li><b>Measurements:</b> Pseudorange + Carrier phase</li><li><b>Available frequencies:</b> Dual-Frequency</li></ul><br>Outside this specific case, troposphere estimation should be disabled.<br>",
                       #Ionosphere
			   self.UseSigmaIonoCB.GetId():u"<b>Use Sigma Ionosphere</b><br><br>Add the ionosphere correction sigma in the measurement weight.<br><br><b><font color='#C00000'>NOTE:</font></b> This option will only have effect when using IONEX or F-PPP ionosphere models.<br>",
                       #Available Frequencies
                           self.SingleFrequencyRB.GetId():u"<b>Single-frequency receiver</b><br><br>Use this option to configure the navigation filter to single-frequency. Discarding all the measurements in the F2. For conditions with the rest of the values of this window, see <i>SPP or PPP Template</i> tooltips.<br>",
                           self.DualFrequencyRB.GetId():u"<b>Dual-frequency receiver</b><br><br>Use this option to process the measurements for both frequencies (F1 and F2) in the navigation filter. For conditions with the rest of the values of this window, see <i>SPP or PPP Template</i> tooltips.<br>",

                       #Receiver Kinematics
                           self.StaticReceiverRB.GetId():u"<b>Static receiver</b><br><br>Select this option to do a processing supposing that the receiver is static. This modifies the filter parameters <b>Phi</b> (propagation) and <b>Q</b> (process noise) for the coordinates estimation to:<br><ul><li>Phi = 1</li><li>Q = 0</li></ul><br>",
                           self.KinematicReceiverRB.GetId():u"<b>Kinematic receiver</b><br><br>Select this option to do a processing supposing that the receiver is in movement. This modifies the filter parameters <b>Phi</b> (propagation) and <b>Q</b> (process noise) for the coordinates estimation to:<br><ul><li>Phi = 0</li><li>Q = 1·10<sup>8</sup> </li></ul><br> <br><b><font color='#C00000'>Note:</font></b> Receiver Coordinates Q = Po = 1·10<sup>8</sup> correspond to (10 km)<sup>2</sup> <br> Receiver Clock Q = Po = 9·10<sup>10</sup> correspond to (1 msec)<sup>2</sup>",
                           
                       #Other options
                           self.BackwardCB.GetId():u"<b>Backward filtering</b><br><br>Select this option to perform a forward-backward processing. This kind of processing reverses the input observation RINEX file when it reaches the end of the file, and processes it backwards. This is also called <i>smoothing</i> and it allows to have good estimation of the parameters (such as the troposphere, and the position in a kinematic receiver) in the beginning of the file, what would reduce the convergence period.<br><br><b><font color='#C00000'>NOTE:</font></b> This kind of process is not considered a real-time emulation, as for the backward filtering it is using future information.",

                       #Selection
                           self.PseudoRangeCarrierPhaseRB.GetId():u"<b>Pseudorange and carrier phase measurements</b><br><br>Use both pseudorange and carrier phase measurements for the processing. For conditions with the rest of the values of this window, see <i>SPP or PPP Template</i> tooltips.<br>",
                           self.PseudoRangeRB.GetId():u"<b>Pseudorange and measurements</b><br><br>Use only pseudorange measurements for the processing. The ambiguity estimation in the filter will be disconnected with this option set. For conditions with the rest of the values of this window, see <i>SPP or PPP Template</i> tooltips.<br>",
                           self.PseudoRangeSmoothingCB.GetId():u"<b>Pseudorange smoothing</b><br><br>Use the Hatch filter to smooth the pseudorange measurements with carrier phase one. This option will decrease the noise of the measurements included in the filter. This should only be activated when processing with pseudorange only (no carrier phase), as the carrier phase is better included in the filter in that other way better than using smoothing. The use of smoothing allows to enhance the pseudorange without the cost of the increased filter complexity for carrier phase ambiguities estimations.<br>The Hatch filter is defined as:<br>P<fint size=-2>i</font><font size=-3>smoothed</font> = mean(P-L)<font size=-2>i</font> + L<font size=-2>i</font><br>Being the mean() a function that computes the mean of pseudorange and carrier phase measurements. This mean at epoch <i>i</i> is obtained from:<br>mean(P-L)<font size=-2>i</font> = ((n-1)*mean(P-L)<font size=-2>i-1</font> + (P-L)<font size=-2>i</font>)/n<br>Being <i>n</i> the arc length at epoch <i>i</i> limited to a maximum value. This limitation is to avoid problems with ionospheric divergence between pseudorange and carrier phase measurements. If both measurements do not have ionospheric divergence (i.e. Pc and Lc), the parameter can be as high as desired.<br><br>The smoothing window in terms of time is the multiplication of the smoothing samples (selected with this parameter) by the data interval of the RINEX file. When smoothing C1C with L1P getting data separated more than about 300 seconds will be affected from ionospheric divergence. In this case good values for smoothing are 100 samples for 1 second data rate, and 10 samples for 30 seconds data rate.",
                       #Configuration
                           self.PseudoRangeComboBox.GetId():u"<b>Pseudo Range </b><br><br>This sets the measurements used in the filter. More information about each RINEX version measurements can be found in the html files provided in the gLAB suite. <br> Depending if a single or double frequency is used, different options can be selected: <br><ul><li>Single-Freq : C1C, C2C, C1P, C2P measurements </li><li>Double Freq : PC combinations</li></ul>  The <b>PC</b> measurements combination is computed as follows:<br><ul><li>[Iono-Free (P1,P2)] PC = ( f<sub>1</sub><sup>2</sup> · C1P - f<sub>2</sub><sup>2</sup> · C2P ) / ( f<sub>1</sub><sup>2</sup> - f<sub>2</sub><sup>2</sup> )</li> </ul>",

                           self.PseudoRangeCarrierPhaseComboBox.GetId():u"<b>Pseudo Range + Carrier Phase</b><br><br>This sets the measurements used in the filter. More information about each RINEX version measurements can be found in the html files provided in the gLAB suite. <br> Depending if a single or double frequency is used, different options can be selected: <br><ul><li>Single-Freq: L1C, L2C, L1P, L2P measurements </li><li>Double Freq : G1C, LC combinations</li></ul>  The  <b>LC</b> and <b>G1C</b> measurements combinations are computed as follows:<br><ul><li>[Iono-Free] LC = ( f<sub>1</sub><sup>2</sup> · L1C - f<sub>2</sub><sup>2</sup> · L2C ) / ( f<sub>1</sub><sup>2</sup> - f<sub>2</sub><sup>2</sup> )</li><li>[GRAPHIC] G1C = ( C1C + L1C ) / 2 </li></ul>",

                           self.FixedWeightPseudoRangeCarrierPhaseRB.GetId():u"<b>Fixed carrier phase standard deviation</b><br><br>This sets the standard deviation of carrier phase measurements to be used as weight in the filter. The weight is computed as:<br>W = 1/(stdDev)^2<br>",
                           self.ElevationWeightPseudoRangeCarrierPhaseRB.GetId():u"<b>Elevation-dependant carrier phase standard deviation</b><br><br>This sets the standard deviation of carrier phase measurements to be used as weight in the filter as a function of the elevation.<br>The standard deviation is computed as:<br>stdDev = <b>a</b>+<b>b</b>*e^(-elev/<b>c</b>)<br>Being <b>a</b>, <b>b</b> and <b>c</b> the three parameters, and elev the elevation in degrees of the satellite.<br>The filter weight is finally computed as:<br>W = 1/(stdDev)^2<br><b><font color='#C00000'>IMPORTANT:</font></b> Carrier phase measurement errors do not usually have a very pronounced dependence on elevation.<br>",
                           self.FixedWeightPseudoRangeRB.GetId():u"<b>Fixed pseudorange standard deviation</b><br><br>This sets the standard deviation of pseudorange measurements to be used as weight in the filter. The weight is computed as:<br>W = 1/(stdDev)^2<br>",
                           self.ElevationWeightPseudoRangeRB.GetId():u"<b>Elevation-dependant pseudorange standard deviation</b><br><br>This sets the standard deviation of pseudorange measurements to be used as weight in the filter as a function of the elevation.<br>The standard deviation is computed as:<br>stdDev = <b>a</b>+<b>b</b>*e^(-elev/<b>c</b>)<br>Being <b>a</b>, <b>b</b> and <b>c</b> the three parameters, and elev the elevation in degrees of the satellite.<br>The filter weight is finally computed as:<br>W = 1/(stdDev)^2<br>",

                       #Parameters
                           self.PhiText.GetId() :u"<b>Phi </b><br><br><b>Phi</b> sets the propagation of parameters between epochs. '1' means that the value estimated in the epoch <i>i+1</i> is used as a priori value in the epoch <i>i</i>, '0', means that the a priori value is always '0'.<br>",
                           self.NoiseText.GetId() :u"<b>Noise </b><br><br>The process noise parameter (<b>Q</b>) sets the stability of a parameter along time. The process noise is included after the estimation when propagating the parameters to the next epoch, and is an increase in the covariance of the parameter. A process noise of '0' means that the parameter is a constant, and thence its estimated covariance (uncertainty) will decrease along time, a process noise different than '0' can be understood as a minimum covariance, thence leaving its uncertainty always above a specific value, and allowing the parameter to vary along time.<br>",
                           self.PoText.GetId() :u"<b>Initial covariance (P0)</b><br><br>The Kalman filter requires initial values for all its parameters: <ul><li><b>Position:</b> The <i>A priori receiver position</i> in the <i>Input</i> section is used</li><li><b>Clock:</b> A '0' value is assigned due to its high variability</li><li><b>Troposphere:</b> Due to the fact that about 90% of the troposphere is corrected by a proper modelling, and only a 10% has to be estimated (and is usually no higher than about 10 cm), a '0' value is used for a priori.</li><li><b>Carrier phase ambiguities:</b>As carrier phase is prealigned with code, the ambiguities are not far from '0', thence this value is used as a priori.</li></ul>The parameter <b>P0</b> sets the initial uncertainty of these a priori values.<br>",
                           self.PositionText.GetId() :u"<b>Receiver coordinates</b><br><br>This comprises the 3 parameters with the 3D position [XYZ] of the receiver. Typical values:<br><ul><li><b>A priori value:</b> A priori receiver position, configured in <i>Input</i> section</li><li><b>Phi: </b> '1' [Static positioning], '0' [Kinematic positioning]</li><li><b>Process noise:</b> '0' [Static positioning], 1·10<sup>8</sup> [Kinematic positioning]</li><li><b>Initial covariance: </b> 1·10<sup>8</sup> (A smaller value for the initial covariance will increase the convergence time, but would require a good initial position)</li></ul><br> <br> <br><b><font color='#C00000'>Note:</font></b> Receiver Coordinates Q = Po = 1·10<sup>8</sup> correspond to (10 km)<sup>2</sup> <br> Receiver Clock Q = Po = 9·10<sup>10</sup> correspond to (1 msec)<sup>2</sup>",
                           self.ReceiverText.GetId() :u"<b>Receiver clock error</b><br><br>This parameter is the receiver clock synchronism error referring to GPS time scale. Typical values:<br><ul><li><b>A priori value:</b> '0'</li><li><b>Phi:</b> '0'</li><li><b>Process noise: </b>9·10<sup>10</sup> </li><li><b>Initial covariance: </b>9·10<sup>10</sup></li></ul><br> <br> <br><b><font color='#C00000'>Note:</font></b> Receiver Coordinates Q = Po = 1·10<sup>8</sup> correspond to (10 km)<sup>2</sup> <br> Receiver Clock Q = Po = 9·10<sup>10</sup> correspond to (1 msec)<sup>2</sup>",
                           self.TroposphereText.GetId() :u"<b>Troposphere</b><br><br>This parameter estimates the 10% of the troposphere that is not taken into account in the nominal model (see <i>Modelling</i> section). This 10% is associated with the wet component of the troposphere, which is due to the water vapour of the atmosphere. The mapping m<i><font size=-2>wet</font></i>(elev) function described in the <i>Modelling</i> section is used in the troposphere estimation. Typical values:<br><ul><li><b>A priori value:</b> '0'</li><li><b>Phi:</b> '1'</li><li><b>Process noise:</b> '1e-4' in units of [m²/h] (i.e, 1 cm<sup>2</sup>/h). Contrary to the rest of parameters, troposphere process noise is given as increase of <b>Q</b> per time unit</li><li><b>Initial covariance:</b> '0.25' (equivalent to 0.5 metres of uncertainty)</li></ul><br>",
                           self.PhaseAmbiguitiesText.GetId() :u"<b>Carrier phase ambiguities</b><br><br>Carrier phase ambiguities are the parameters which estimates the ambiguities of the carrier phase measurements (or combinations) used in the filter. Each epoch, there is one parameter for each satellite. Additionally, when a cycle-slip is detected, the carrier phase ambiguity is reset (by providing a new initial value and covariance).Typical values:<br><ul><li><b>Phi:</b> '1'</li><li><b>Process noise:</b> '0'</li><li><b>Initial covariance:</b> '400' (equivalent to 20 metres in relation to the pseudorange measurement used to prealign the carrier phase)</li></ul><br>",
                  #Output Tooltips:                                   
                          self.OutputButton.GetId():u"<b>Output section</b><br><br>This section provides all the configuration options to select which messages are output.<br>",

                      #Output Files
                          self.OutputDestinationText.GetId():u"<b>Output file</b><br><br>Sets the output file where all messages will be written.<br>",

                      #Output Messages
                          self.InfoMessage.GetId():u"<b>INFO messages</b><br><br>INFO messages are shown at several points in the program and provide information on the program configuration and problems it may encounter.<br>",
                          self.CSMessage.GetId():u"<b>CS messages</b><br><br>CS messages are shown when a cycle-slip is found by any detector. The first 5 fields are fixed, afterwards, there are three possible blocks depending on the cycle-slip detector activated.<br><ul><li><b>Field 1:</b> 'CS'</li><li><b>Field 2:</b>  Year</li><li><b>Field 3:</b> Doy</li><li><b>Field 4:</b> Seconds of day</li><li><b>Field 5:</b> GNSS System (GPS, GAL, GLO or GEO)</li><li><b>Field 6:</b> PRN satellite identifier</li><li><b>Block 1:</b> Geometry-free carrier phase cycle-slip detector (Li)</li><ul><li><b>Field 1:</b> 'Li:x', being x, 0 or 1 (0-> Li did not detect the cycle-slip, 1-> Li did)</li><li><b>Field 2:</b>  Li value</li><li><b>Field 3:</b> Li Threshold</li></ul><li><b>Block 2:</b> Melbourne-Wübbena cycle-slip detector (Bw)</li><ul><li><b>Field 1:</b> 'Bw:x', being x, 0 or 1 (0-> Bw did not detect the cycle-slip, 1-> Bw did)</li><li><b>Field 2:</b> Bw value</li><li><b>Field 3:</b> Bw Threshold</li></ul><li><b>Block 3:</b> L1-C1 average cycle-slip detector (L1C1)</li><ul><li><b>Field 1:</b> 'L1C1:x', being x, 0 or 1 (0-> L1C1 did not detect the cycle-slip, 1-> L1C1 did)</li><li><b>Field 2:</b> L1C1 value</li><li><b>Field 3:</b> L1C1 Threshold</li></ul><li><b>Sample:</b> CS 2006 200  4500.00 27 Li:1 (0.066968 [0.052100]) Bw:0 (0.360430 [5.775316])</li></ul><br>",
                          self.InputMessage.GetId():u"<b>INPUT messages</b><br><br>Input data message. It is shown after an epoch is read (and decimated). It contains the measurements for each satellite for this epoch <br><ul><li><b>Field 1:</b> 'INPUT' </li><li><b>Field 2:</b> Year </li><li><b>Field 3:</b> Doy </li><li><b>Field 4:</b> Seconds of day </li><li><b>Field 5:</b> GNSS System (GPS, GAL, GLO or GEO) </li><li><b>Field 6:</b> PRN satellite identifier </li><li><b>Field 7:</b> Arc length (number of undecimated epochs after the last cycle-slip) </li><li><b>For GPS:</b></li><ul><li><b>Field 8:</b> C1 [C1C] </li><li><b>Field 9:</b> P1 [C1P] </li><li><b>Field 10:</b> P2 [C2P] </li><li><b>Field 11:</b> L1 [L1P] (prealigned, in metres) </li><li><b>Field 12:</b> L2 [L2P] (prealigned, in metres) </li></ul><li><b>For Galileo (GAL):</b></li><ul><li><b>Field 8:</b> C1A </li><li><b>Field 9:</b> C1B </li><li><b>Field 10:</b> C1C </li><li><b>Field 11:</b> C7Q </li><li><b>Field 12:</b> C8Q </li><li><b>Field 13:</b> L1A (prealigned, in metres) </li><li><b>Field 14:</b> L1B (prealigned, in metres) </li><li><b>Field 15:</b> L1C (prealigned, in metres) </li><li><b>Field 16:</b> L7Q (prealigned, in metres) </li><li><b>Field 17:</b> L8Q (prealigned, in metres) </li></ul><li><b>For GLONASS (GLO):</b></li><ul><li><b>Field 8:</b> C1 [C1C] </li><li><b>Field 9:</b> C2 [C2C] </li><li><b>Field 10:</b> L1 [L1P] (prealigned, in metres) </li><li><b>Field 11:</b> L2 [L2P] (prealigned, in metres) </li></ul><li><b>For GEO: </b></li><ul><li><b>Field 8:</b> C1 [C1C] </li></ul><li><b>Sample:</b> INPUT 2006 200     0.00 GPS 19      1  23119003.9020  23119002.6110  23119004.0750  23119002.7507  23119004.0925</li></ul><br>",
                          self.ModelMessage.GetId():u"<b>MODEL messages</b><br><br>Model break down message. It is shown when a model can be fully computed for each measurement. <br><ul><li><b>Field 1:</b> 'MODEL' </li><li><b>Field 2:</b> Year </li><li><b>Field 3:</b> Doy </li><li><b>Field 4:</b> Seconds of day </li><li><b>Field 5:</b> GNSS System (GPS, GAL, GLO or GEO) </li><li><b>Field 6:</b> PRN satellite identifier </li><li><b>Field 7:</b> Measurement identifier (as string) </li><li><b>Field 8:</b> Signal flight time [sec] </li><li><b>Field 9:</b> Measured value [m] </li><li><b>Field 10:</b> Full model value [m] </li><li><b>Field 11:</b> Satellite X position [m] </li><li><b>Field 12:</b> Satellite Y position [m] </li><li><b>Field 13:</b> Satellite Z position [m] </li><li><b>Field 14:</b> Satellite X velocity [m] </li><li><b>Field 15:</b> Satellite Y velocity [m] </li><li><b>Field 16:</b> Satellite Z velocity [m] </li><li><b>Field 17:</b> Satellite-receiver geometric distance [m] </li><li><b>Field 18:</b> Satellite clock correction [m] </li><li><b>Field 19:</b> Satellite mass centre to antenna phase centre projection [m] </li><li><b>Field 20:</b> Receiver phase centre projection [m] </li><li><b>Field 21:</b> Receiver Antenna Reference Point (ARP) projection [m] </li><li><b>Field 22:</b> Relativistic clock correction [m] </li><li><b>Field 23:</b> Wind-up correction [m] (for carrier phase measurements) </li><li><b>Field 24:</b> Troposphere nominal correction [m] </li><li><b>Field 25:</b> Ionosphere correction [m] </li><li><b>Field 26:</b> Relativistic path range correction [m] </li><li><b>Field 27:</b> Total Group Delay (TGD) correction [m] </li><li><b>Field 28:</b> Solid tides correction [m] </li> <li><b>Field 29:</b> Satellite Elevation [degrees] </li> <li><b>Field 30:</b> Satellite Azimuth [degrees] </li><li><b>Sample:</b> MODEL 2006 200     0.00 GPS 19 L1P     0.07712  23119002.7507  23119008.7501   8811456.7780 -21033910.1687  13675922.8867      1828.7339      2353.7679      2467.3576  23119457.7652     -456.31787   0.00000  -0.04936  -0.01140   2.32333   0.10671   4.85412  -0.00000   0.01544   0.00000   0.06394   9.16487738221  -79.27496674531</li></ul>Field 9 is the direct measurement (as in the RINEX file), but scaled to metres for carrier phase </li>measurements. Field 10 is the model computed for this measurement. Field 10 is the direct sum of fields 17 to 28.<br>",

			  self.SatMEASMessage.GetId():u"<b>MEAS messages</b><br><br>It provides the MEASurement values. It is shown after an epoch is read (and decimated). It contains the measurements for each satellite for this epoch. <br><ul><li><b>Field 1:</b> 'MEAS' </li><li><b>Field 2:</b> Year </li><li><b>Field 3:</b> Doy </li><li><b>Field 4:</b> Seconds of day </li><li><b>Field 5:</b> GNSS System (GPS, GAL, GLO or GEO) </li><li><b>Field 6:</b> PRN satellite identifier </li><li><b>Field 7:</b> Elevation of the satellite [degrees] </li><li><b>Field 8:</b> Azimuth of the satellite [degrees] </li><li><b>Field 9:</b> Number of Measurement(s) </li><li><b>Field 10:</b> Measurement identifier (as string) </li><li><b>Field 11:</b> Measurement(s) value [m] </li><li><b>Sample: </b>MEAS     2010 081   300.00 GPS 30   30.00  240.00  6  C1C:L1C:C1P:L1P:C2P:L2P  20228715.3270         0.0000         0.0000  20228715.2722  20228714.8230  20228714.7005</li></ul><br>",

                          self.EpochSatMessage.GetId():u"<b>EPOCHSAT messages</b><br><br>Message with the satellites used to compute the solution. It is shown when the filter is run, and is given for each measurement <br><ul><li><b>Field 1:</b> 'EPOCHSAT' </li><li><b>Field 2:</b> Year </li><li><b>Field 3:</b> Doy </li><li><b>Field 4:</b> Seconds of day </li><li><b>Field 5:</b> Measurement identifier (as string) </li><li><b>Field 6:</b> Number of satellites in view </li><li><b>Field 7-:</b> Each following column is the identifier of a satellite </li><li><b>Sample: </b>EPOCHSAT 2006 200 300.00 PC 7 15 3 19 16 18 21 22</li></ul><br>",
                          self.PreFitMessage.GetId():u"<b>PREFIT messages</b><br><br>Prefilter values message. It provides the measurement-model values. It is shown in each filter execution. <br><ul><li><b>Field 1:</b> 'PREFIT' (if the satellite is used in the computation) or 'PREFIT*' (if it is not) </li><li><b>Field 2:</b> Year </li><li><b>Field 3:</b> Doy </li><li><b>Field 4:</b> Seconds of day </li><li><b>Field 5:</b> GNSS System (GPS, GAL, GLO or GEO) </li><li><b>Field 6:</b> PRN satellite identifier </li><li><b>Field 7:</b> Measurement identifier (as string) </li><li><b>Field 8:</b> Measurement-model value (prefit) [m] </li><li><b>Field 9:</b> Measurement value [m] </li><li><b>Field 10:</b> Model value [m] </li><li><b>Field 11:</b> X-partial derivative (-X component of the satellite line-of-sight vector) </li><li><b>Field 12:</b> Y-partial derivative (-Y component of the satellite line-of-sight vector) </li><li><b>Field 13:</b> Z-partial derivative (-Z component of the satellite line-of-sight vector) </li><li><b>Field 14:</b> T-partial derivative </li><li><b>Field 15:</b> Elevation of the satellite [degrees] </li><li><b>Field 16:</b> Azimuth of the satellite [degrees] </li><li><b>Field 17:</b> Standard deviation of the measurement  (for the filter) [m]</li><li><b>Sample: </b>PREFIT   2006 200   300.00 GPS 19  LC        -7.3029  22982271.7155  22982279.0184  28.28  0.3931  -0.4834  0.7822  1.0000  77.912   130.010</li></ul>In general Field 8 = Field 9 - Field 10, but this is no longer true when using smoothing, as the Field 9 is the raw measurement without smoothing, but the Field 8 computation takes smoothing into account.<br>",
                          self.PostFitMessage.GetId():u"<b>POSTFIT messages</b><br><br>Postfilter values message. It provides the corrected prefits with the filter estimation. It is shown in each filter execution. <br><ul><li><b>Field 1:</b> 'POSTFIT' </li><li><b>Field 2:</b> Year </li><li><b>Field 3:</b> Doy </li><li><b>Field 4:</b> Seconds of day </li><li><b>Field 5:</b> GNSS System (GPS, GAL, GLO or GEO) </li><li><b>Field 6:</b> PRN satellite identifier </li><li><b>Field 7:</b> Measurement identifier (as string) </li><li><b>Field 8:</b> Measurement-corrected model value (postfit) [m] </li><li><b>Field 9:</b> Measurement value [m] </li><li><b>Field 10:</b> Corrected model value with the filter estimations[m] </li><li><b>Field 11:</b> Elevation of the satellite [degrees] </li><li><b>Field 12:</b> Azimuth of the satellite [degrees] </li><li><b>Field 13:</b> Only given for carrier phase measurements. It is the estimated carrier phase ambiguity. [m] </li><li><b>Sample: </b>POSTFIT  2006 200   300.00 GPS 19  LC         0.0000  22982271.7155  22982271.7155   28.28  -77.91         0.3029</li></ul>As with PREFIT messages, Field 8 is not necessary Field 9 - Field 10 when using smoothing.<br>",
                          self.FilterMessage.GetId():u"<b>FILTER messages</b><br><br>Filter solution message. This message provides direct information on the filter estimates. It is shown in each filter execution. <br><ul><li><b>Field 1:</b> 'FILTER' </li><li><b>Field 2:</b> Year </li><li><b>Field 3:</b> Doy </li><li><b>Field 4:</b> Seconds of day </li><li><b>Field 5-:</b> Filter estimates. The order is: 3D estimated position, clock, troposphere and ambiguities. The number of fields is variable in this message. With a full filter (troposphere and ambiguities estimation), the Fields are as follows: </li><ul><li><b>Field 5:</b> Receiver X position [m] </li><li><b>Field 6:</b> Receiver Y position [m] </li><li><b>Field 7:</b> Receiver Z position [m] </li><li><b>Field 8:</b> Receiver clock [m] </li><li><b>Field 9:</b> Zenith Tropospheric Delay [m] </li><li><b>Field 10:</b> Carrier phase ambiguities [m] </li></ul><li><b>Sample: </b>FILTER 2006 200   300.00   4849203.0770   -360328.5730   4114913.9184        -7.4867         2.1946        -0.0001         1.1079        -1.1073         0.3029         0.4555         0.0897         0.0001         0.3845         0.2136</li></ul><br>",
                          self.OutputMessage.GetId():u"<b>OUTPUT messages</b><br><br>Receiver solution message. This message provides the estimated receiver position. It is shown in each filter execution. <br><ul><li><b>Field 1:</b> 'OUTPUT' </li><li><b>Field 2:</b> Year </li><li><b>Field 3:</b> Doy </li><li><b>Field 4:</b> Seconds of day </li><li><b>Field 5:</b> 3D Formal Error: (&sigma;<sub>X</sub><sup>2</sup> + &sigma;<sub>Y</sub><sup>2</sup> + &sigma;<sub>Z</sub><sup>2</sup>)<sup>1/2</sup>. </li><li><b>Field 6:</b> Receiver X position [m] </li><li><b>Field 7:</b> Receiver Y position [m] </li><li><b>Field 8:</b> Receiver Z position [m] </li><li><b>Field 9:</b> Receiver X position - Nominal a priori X position [m] </li><li><b>Field 10:</b> Receiver Y position - Nominal a priori Y position [m] </li><li><b>Field 11:</b> Receiver Z position - Nominal a priori Z position [m] </li><li><b>Field 12:</b> Receiver X formal error [m] </li><li><b>Field 13:</b> Receiver Y formal error [m] </li><li><b>Field 14:</b> Receiver Z formal error [m] </li><li><b>Field 15:</b> Receiver latitude [degrees] </li><li><b>Field 16:</b> Receiver longitude [degrees] </li><li><b>Field 17:</b> Receiver height [m] </li><li><b>Field 18:</b> Receiver North difference in relation to nominal a priori position [m] </li><li><b>Field 19:</b> Receiver East difference in relation to nominal a priori position [m] </li><li><b>Field 20:</b> Receiver Up difference in relation to nominal a priori position [m] </li><li><b>Field 21:</b> Receiver formal error in North direction [m] </li><li><b>Field 22:</b> Receiver formal error in East direction [m] </li><li><b>Field 23:</b> Receiver formal error in Up direction [m] </li><li><b>Field 24:</b> Geometric Dilution of Precision (GDOP) </li><li><b>Field 25:</b> Position Dilution of Precision (PDOP) </li><li><b>Field 26:</b> Time Dilution of Precision (TDOP) </li><li><b>Field 27:</b> Horizontal Dilution of Precision (HDOP) </li><li><b>Field 28:</b> Vertical Dilution of Precision (VDOP)</li><li><b>Field 29:</b> Zenith Tropospheric Delay (including nominal value) [m]</li><li><b>Field 30:</b> Zenith Tropospheric Delay (excluding nominal value) [m]</li><li><b>Field 31:</b> Zenith Tropospheric Delay formal error [m]</li><li><b>Sample: </b>OUTPUT 2006 200   300.00    2.6219   4849203.1236   -360328.5229   4114913.9535     0.7693     0.4145     0.7580     1.9353     0.6998     1.6246   40.429162956   -4.249653155  830.480629026     0.0993     0.4704     1.0522     1.1365     0.6772     2.2637     5.0394     5.5472     6.3482    2.4261    2.2142     2.1982     0.0097     0.4995</li></ul>Providing a nominal a priori position is optional of the processing, but if it is given, fields 9, 10, 11, 18, 19 and 20 will be given in relation to this a priori position. See the option 'A priori Receiver Position' in the <i>Input</i> section for more details.<br>",
                          self.SatDifMessage.GetId():u"<b>SATDIFF messages (only products comparison)</b><br><br>Satellite difference message for products comparison mode. This message shows the difference between both orbit and clocks for two different OC products fonts for a specific time. It is shown each time step. The SISRE fields are the Signal-in-Space Range Error, it is a measure of how the error impact a general receiver found in the Earth's surface. It is computed as:<br>SISRE = sqrt((radDiff-clkDiff)<sup>2</sup> + 1/49*(ctDiff<sup>2</sup> + atDiff<sup>2</sup>))<br>Being:<br>radDiff, the difference of satellite positions in the radial component<br>clkDiff, the difference of satellite clock offsets<br>ctDiff, the difference of satellite positions in the cross-track component<br>atDiff, the difference of satellite positions in the along-track component<ul><li><b>Field 1:</b> 'SATDIFF' </li><li><b>Field 2:</b> Year </li><li><b>Field 3:</b> Doy </li><li><b>Field 4:</b> Seconds of day </li><li><b>Field 5:</b> PRN satellite identifier </li><li><b>Field 6:</b> SISRE difference </li><li><b>Field 7:</b> SISRE orbit-only difference </li><li><b>Field 8:</b> 3D orbit difference </li><li><b>Field 9:</b> Clock difference </li><li><b>Field 10:</b> Radial position difference</li><li><b>Field 11:</b> Along-track position difference</li><li><b>Field 12:</b> Cross-track position difference</li></ul><br>",
                          self.SatStatMessage.GetId():u"<b>SATSTAT messages (only products comparison)</b><br><br>Satellite difference summary message for products comparison mode. This message shows the differences between orbit and clocks for two different OC products fonts for a specific satellite. It is shown after all the comparisons are done, and presents a summary for each satellite. <br><ul><li><b>Field 1:</b> 'SATSTAT'</li><li><b>Field 2:</b> PRN satellite identifier </li><li><b>Field 3:</b> Number of samples</li><li><b>Field 4-6:</b> SISRE comparisons [mean,RMS,stdDev] </li><li><b>Field 7-9:</b> SISRE orbit-only comparisons [mean,RMS,stdDev] </li><li><b>Field 10-12:</b> 3D orbit comparisons [mean,RMS,stdDev]</li><li><b>Field 13-15:</b> Clock comparisons [mean,RMS,stdDev] </li><li><b>Field 16-18:</b> Radial position comparisons [mean,RMS,stdDev]</li><li><b>Field 19-21:</b> Along-track position comparisons [mean,RMS,stdDev]</li><li><b>Field 22-24:</b> Cross-track position comparisons [mean,RMS,stdDev]</li></ul><br>",
                          self.SatStatTotMessage.GetId():u"<b>SATSTATTOT messages (only products comparison)</b><br><br>Difference summary message for products comparison mode. This message shows the statistics of the differences between orbit and clocks for two different OC products fonts. It is shown after all the comparisons are done, and presents a summary for all of them. <ul><li><b>Field 1:</b> 'SATSTATTOT'</li><li><b>Field 2:</b> Number of samples</li><li><b>Field 3-5:</b> SISRE comparisons [mean,RMS,stdDev] </li><li><b>Field 6-8:</b> SISRE orbit-only comparisons [mean,RMS,stdDev] </li><li><b>Field 9-11:</b> 3D orbit comparisons [mean,RMS,stdDev] </li><li><b>Field 12-14:</b> Clock comparisons [mean,RMS,stdDev] </li><li><b>Field 15-17:</b> Radial position comparisons [mean,RMS,stdDev]</li><li><b>Field 18-20:</b> Along-track position comparisons [mean,RMS,stdDev]</li><li><b>Field 21-23:</b> Cross-track position comparisons [mean,RMS,stdDev]</li></ul><br>",
                  #Buttons
                          self.SaveCfgButton.GetId():u"<b>Save configuration</b><br><br>This button saves all the parameters from the gLAB-GUI to a .cfg file, which can be read by the processing core by means of the '-input:cfg' parameter.<br><br>In Linux:<br>&nbsp;./gLAB_linux -input:cfg gLAB.cfg<br><br>In Windows:<br>&nbsp;gLAB.exe -input:cfg gLAB.cfg",
                          self.RunButton.GetId():u"<b>Execute gLAB</b><br><br>This button executes the processing core of gLAB with the options specified in the gLAB-GUI.",
                          self.ShowOutputButton.GetId():u"<b>Show last output file</b><br><br>This button opens a text editor with the output of the last execution of gLAB.",
                          self.ShowConfigButton.GetId():u"<b>Show last configuration file saved by the user</b><br><br>This button opens a text editor with the last options specified in the gLAB-GUI"
                          }
    
                
##########################################################################
# Name        : SetTooltipBinds
# Description : Every object in the tooltips dictionary, is binded to two
#               different mouse actions: entering and leaving, which show 
#               or hide a small help text (a tooltip)
##########################################################################
    def SetTooltipBinds(self):
        for item in self.TooltipTextDictionary:
            self.FindWindowById(item).Bind(wx.EVT_ENTER_WINDOW, self.SetToolTip, id=item)
            self.FindWindowById(item).Bind(wx.EVT_LEAVE_WINDOW, self.HideToolTip)                              
    
##########################################################################
# Name        : ShowToolTip
# Description : Function to synchronise events for GUIShowToolTip
##########################################################################
    def ShowToolTip(self):
        wx.CallAfter(self.GUIShowToolTip)

##########################################################################
# Name        : GUIShowToolTip
# Description : A small frame is showed with some text inside 
##########################################################################
    def GUIShowToolTip(self):
        self.threadLock.acquire()
        TooltipText = self.TooltipTextDictionary[self.IdToShowTooltip]

        MousePosX, MousePosY = wx.GetMousePosition()
        ZeroPointX, ZeroPointY, ScreenSizeX, ScreenSizeY = wx.ClientDisplayRect()
        
        self.ToolTipFrame.html.SetSize(self.formerSize)
        self.ToolTipFrame.SetSize(self.formerSize)
        self.ToolTipFrame.html.SetPage(TooltipText)
        self.ToolTipFrame.html.EnableScrolling(False, False)
        try:
            self.ToolTipFrame.html.SetSize(self.ToolTipFrame.html.GetVirtualSize())
            self.ToolTipFrame.SetSize(self.ToolTipFrame.html.GetVirtualSize())
            self.ToolTipFrame.SetBackgroundColour((255, 255, 197))
            self.ToolTipFrame.html.SetBackgroundColour((255, 255, 197))
            while (self.ToolTipFrame.html.GetSize() != self.ToolTipFrame.html.GetVirtualSize()):
                self.ToolTipFrame.html.SetSize(self.ToolTipFrame.html.GetVirtualSize())
            self.ToolTipFrame.Show()
        except:
            pass
        
        ToolTipX, ToolTipY = self.ToolTipFrame.html.GetSize()
        
        self.CalculateTooltipPosition(MousePosX, MousePosY, ScreenSizeX, ScreenSizeY, ToolTipX, ToolTipY, ZeroPointX, ZeroPointY)
        
        self.threadLock.release()

##########################################################################
# Name        : CalculateTooltipPosition
# Description : Computes where the tooltip is going to be displayed 
##########################################################################    
    def CalculateTooltipPosition(self, MouseX, MouseY, ScreenX, ScreenY, ToolTipSizeX, ToolTipSizeY, ZeroX, ZeroY):
        changes = 0
        Xpos = MouseX + 10
        if (Xpos + ToolTipSizeX + ZeroX) > ScreenX :
            Xpos = ScreenX - ToolTipSizeX
            changes += 1
        Ypos = MouseY + 10
        if (Ypos + ToolTipSizeY + ZeroY) > ScreenY :
            Ypos = ScreenY - ToolTipSizeY
            changes += 1
            
        if changes == 2:
            Xpos = MouseX - 20 - ToolTipSizeX 
        self.ToolTipFrame.SetPosition(wx.Point(Xpos, Ypos))
        
##########################################################################
# Name        : SetToolTip
# Description : Sets a timer to show a tooltip of a specific GUI object.
##########################################################################
    def SetToolTip(self, event):
        global tooltips, tooltipDelay
        if (tooltips == True):
            self.threadLock.acquire()
            try:
                self.tooltipthread.cancel()
            except:
                pass
            if (self.ToolTipFrame.IsShown()):
                self.ToolTipFrame.Hide()
            
            try:                
                self.IdToShowTooltip = event.GetId()
                self.tooltipthread = threading.Timer(tooltipDelay, function=self.ShowToolTip)
                self.tooltipthread.start()
            except:
                pass
            self.threadLock.release()
        event.Skip()
            
            
##########################################################################
# Name        : HideToolTip
# Description : This functions destroys the tooltip once the mouse leaves 
#               the object that created it 
##########################################################################
    def HideToolTip(self, event):
        global tooltips, tooltipDelay
        
        if (tooltips == True):
            self.threadLock.acquire()
            try:
                self.tooltipthread.cancel()
            except:
                pass    
            
            if (self.ToolTipFrame.IsShown()):
                self.ToolTipFrame.Hide()
            self.threadLock.release()
        event.Skip()
             
##########################################################################
# Name        : AnalysisPage
# Description : This is the tab where the user can configure what the  
#               graphic part will plot. User can configurates plot's 
#               title, labels, source file to be plotted and which 
#               columns which meet some user requirements
#  
#               There are 8 different buttons which are templates
#
#               NEU position error
#               Horizontal position error
#               Model Components
#               Ionospheric combinations
#               Zenith Tropospheric Delay
#               Measurement postfits/elevation
#               Satellite skyplot
#               Carrier Phase ambiguities
#
########################################################################## 
class AnalysisPage(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)

  #Create Analysis TAB Panel    
        self.AnalysisPanel = wx.Panel(self, -1, size=(TabXSize, TabYSize), style=wx.ALIGN_CENTER)
        XAnalysisPanel, YAnalysisPanel = self.AnalysisPanel.GetSize()
        self.PlotButton = wx.Button(self.AnalysisPanel, -1, '&Plot')
        self.PlotButton.Bind(wx.EVT_LEFT_DOWN, self.ShowGraphic)
        LinedFont = wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL)
        LinedFontColour = (wx.BLUE)
        
  #Templates Panel
        TemplatesPanel = wx.Panel(self.AnalysisPanel, -1, size=wx.Size(1.0 * XAnalysisPanel, 0.10 * YAnalysisPanel))
        XTemplatesPanel, YTemplatesPanel = TemplatesPanel.GetSize()
        
        TemplatesPanelCore = wx.Panel(TemplatesPanel, -1, size=wx.Size(0.95 * XTemplatesPanel, 0.95 * YTemplatesPanel))
        XTemplatesPanelCore, YTemplatesPanelCore = TemplatesPanelCore.GetSize()
    
        TemplatesButtonXsize = 0.235 * XTemplatesPanelCore
#        TemplatesButtonXsize = 0.1875 * XTemplatesPanelCore
        TemplatesButtonYsize = 0.80 * YTemplatesPanelCore
        
        self.TemplateNumButtons = 12
        TemplateButtonLabels = ['NEU positioning error', 'Horizontal positioning error', 'Zenith Tropospheric Delay', 'Ionospheric combinations', 'Dilution Of Precision','Satellite skyplot',  'Carrier phase ambiguities', 'Measur. Multipath/Noise','Model components','Prefit residuals', 'Postfit residuals','Orbit and Clock comparison']

        global OutputPath
        self.TemplatePlotParameters = [
                [['NEU positioning error', 'time (s)', 'error (m)', '', '', '', ''], [OutputPath, OutputPath, OutputPath, ''], ['($1=="OUTPUT")', '($1=="OUTPUT")', '($1=="OUTPUT")', ''], ['4', '4', '4', ''], ['18', '19', '20', ''], ['North error', 'East error', 'Up error', ''], [8, 8, 8, 0],[0, 1, 2, 3]],
                [['Horizontal positioning error', 'East error (m)', 'North error(m)', '', '', '', ''], [OutputPath, '', '', ''], ['($1=="OUTPUT")', '', '', ''], ['19', '', '', ''], ['18', '', '', ''], ['', '', '', ''], [1, 0, 0, 0], [0, 1, 2, 3]],
		[['Zenith Tropospheric Delay', 'time (s)', 'troposphere (m)', '', '', '', ''], [OutputPath, '', '', ''], ['($1=="FILTER")', '', '', ''], ['4', '', '', ''], ['9', '', '', ''], ['', '', '', ''], [8, 0, 0, 0], [0, 1, 2, 3]],                
		[['Ionospheric combinations', 'time (s)', 'metres of L1-L2 delay (m)', '', '', '-10', '10'], [OutputPath, OutputPath, '', ''], ['($1=="INPUT") ', '($1=="INPUT") &  ($6==13)', '', ''], ['4', '4', '', ''], ['($10-$9)', '($10-$9)', '', ''], ['All PRN : PI=(P2-P1)', 'PRN 13 : PI=(P2-P1)', '', ''], [3, 3, 0, 0],[0, 2, 1, 3]],



		[['Dilution Of Precision', 'time (s)', 'DOP (m)', '', '', '', ''], [OutputPath, OutputPath, OutputPath, OutputPath], ['($1=="OUTPUT")', '($1=="OUTPUT")', '($1=="OUTPUT")', '($1=="OUTPUT")'], ['4', '4', '4', '4'], ['24', '27', '28', '26'],['Geometric Dilution Of Precision','Horizontal Dilution Of Precision','Vertical Dilution Of Precision','Time Dilution Of Precision'], [8, 8, 8, 8],[0, 1, 2, 3]],
                [['Satellite skyplot', '', '', '-1', '+1', '-1', '+1'], [OutputPath, OutputPath, '', ''], ['($1=="PREFIT")', '($1=="PREFIT") & ($6==3)', '', ''], ['(math.sin($16*math.pi/180)*(90-$15)/90)', '(math.sin($16*math.pi/180)*(90-$15)/90)', '', ''], ['(math.cos($16*math.pi/180)*(90-$15)/90)', '(math.cos($16*math.pi/180)*(90-$15)/90)', '', ''], ['All PRN', 'PRN3', '', ''], [3, 3, 0, 0], [0, 2, 1, 3]],
                [['Carrier phase ambiguities estimations', 'time (s)', 'm', '', '', '', ''], [OutputPath, '', '', ''], ['($1=="POSTFIT") & ($7[0]=="L")', '', '', ''], ['4', '', '', ''], ['13', '', '', ''], ['', '', '', ''], [1, 0, 0, 0], [0, 1, 2, 3]],
		[['Measure Multipath - Noise', 'time (s)', 'multipath', '', '', '-200', '200'], [OutputPath, OutputPath, OutputPath,''], ['($1=="INPUT") & ($6==31)', '($1=="INPUT") & ($6==31)', '($1=="INPUT") & ($6==31)', ''], ['4', '4', '4', ''], ['($8-$11-3.09*($11-$12))', '( ( 154*154 ) * $9  - ( 120*120 )*$10 ) / ( ( 154*154 ) - ( 120*120 ) ) - ( ( 154*154 ) * $11 - ( 120*120 )*$12 ) / ( ( 154*154 ) - ( 120*120 ) )', '( 154 * $9 + 120 * $10 ) / ( 154 + 120 ) - ( 154 * $11  - 120 * $12 ) / ( 154 - 120 )', ''], ['C1 Multipath', 'PC Multipath', 'MW Multipath',''], [1, 1, 1, 1], [0, 1, 2, 3]],




		[['Model components', 'time (s)', 'model (m)', '', '', '', ''], [OutputPath, '', '', ''], ['($1=="MODEL")', '', '', ''], ['4', '', '', ''], ['22', '', '', ''], ['', '', '', ''], [1, 0, 0, 0], [0, 1, 2, 3]],		
		[['Measurement prefits vs elevation', 'elevation (degrees)', 'prefits (m)', '', '', '', ''], [OutputPath, OutputPath, '', ''], ['($1=="PREFIT") & ($7[0]=="P")', '($1=="PREFIT") & ($7[0]=="L")', '', ''], ['15', '15', '', ''], ['8', '8', '', ''], ['Pseudorange prefit residuals','Carrier phase prefit residuals' , '', ''], [1, 1, 0, 0],[0, 1, 2, 3]],				
		[['Measurement postfits vs elevation', 'elevation (degrees)', 'postfit (m)', '', '', '', ''], [OutputPath, OutputPath, '', ''], ['($1=="POSTFIT") & ($7[0]=="P")', '($1=="POSTFIT") & ($7[0]=="L")', '', ''], ['11', '11', '', ''], ['8', '8', '', ''], ['Pseudorange postfit residuals', 'Carrier phase postfit residuals', '', ''], [1, 1, 0, 0],[0, 1, 2, 3]],                
		[['Orbits and Clock comparison', 'time (s)', 'Discrepancy (m)', '', '', '', ''], ['dif.out', 'dif.out', 'dif.out',''], ['($1=="SATDIFF")', '($1=="SATDIFF")', '($1=="SATDIFF")',''], ['4', '4', '4', ''], ['11', '12', '13', ''], ['Radial', 'Along-Track', 'Cross-Track', ''], [1, 1, 1, 1], [0, 1, 2, 3]],  
                
            ]

	"""
	C1: 8
	P1: 9
	P2:10
	L1:11
	L2:12 
	f1 = 154
	f2 = 120
        Mc1 = ($8-$11-3.09*($11-$12))
	Mpc = ( ( 154*154 ) * $9  - ( 120*120 )*$10 ) / ( ( 154*154 ) - ( 120*120 ) ) - ( ( 154*154 ) * $11 - ( 120*120 )*$12 ) / ( ( 154*154 ) - ( 120*120 ) )
	Mmw = ( 154 * $9 + 120 * $10 ) / ( 154 + 120 ) - ( 154 * $11  - 120 * $12 ) / ( 154 - 120 )
	Mgc = ($9+$11)/2 - ( ( 154*154 ) * $11 - ( 120*120 )*$12 ) / ( ( 154*154 ) - ( 120*120 ) )
	"""
        self.TemplateSelectedConditions = [
                [[8, 8, 8, 0], [0, 0, 0, 0], [ 4, 4, 4, 0], [18,19,20, 0]],
                [[8, 0, 0, 0], [0, 0, 0, 0], [19, 0, 0, 0], [18, 0, 0, 0]],
		[[7, 0, 0, 0], [0, 0, 0, 0], [ 4, 4, 0, 0], [ 9, 9, 0, 0]],
                [[1, 1, 0, 0], [0,13, 0, 0], [ 4, 4, 0, 0], [ 0, 0, 0, 0]], 

                [[8, 8, 8, 8], [ 0, 0, 0, 0], [ 4, 4, 4, 4], [24,27,28,26]], 
                [[5, 5, 0, 0], [ 0, 3, 0, 0], [ 0, 0, 0, 0], [ 0, 0, 0, 0]],            
                [[6, 0, 0, 0], [ 0, 0, 0, 0], [ 4, 0, 0, 0], [13, 0, 0, 0]],
                [[1, 1, 1, 0], [31,31,31,31], [ 4, 4, 4, 0], [ 0, 0, 0, 0]],

		[[3, 0, 0, 0], [0, 0, 0, 0], [ 4, 0, 0, 0], [22, 0, 0, 0]],   
		[[5, 5, 0, 0], [0, 0, 0, 0], [15,15, 0, 0], [ 8, 8, 0, 0]],
		[[6, 6, 0, 0], [0, 0, 0, 0], [11,11, 0, 0], [ 8, 8, 0, 0]],
		[[9, 9, 9, 0], [0, 0, 0, 0], [ 4, 4, 4, 0], [11,12,13, 0]],
 
            ]

        TemplatesPanelCoreGrill = wx.GridBagSizer()
        self.TemplateButtons = []
        for i in range(self.TemplateNumButtons):
            self.TemplateButtons.append(wx.Button(TemplatesPanelCore, -1, TemplateButtonLabels[i], size=wx.Size(TemplatesButtonXsize, TemplatesButtonYsize)))
            self.TemplateButtons[i].Bind(wx.EVT_LEFT_DOWN, self.SetTemplate)
            if (i < 9 ): 
		TemplatesPanelCoreGrill.Add(self.TemplateButtons[i], (int(i / 4), i % 4), (1, 1), wx.TOP|wx.LEFT|wx.RIGHT, 5)
	    else:
		TemplatesPanelCoreGrill.Add(self.TemplateButtons[i], (int(i / 4), i % 4), (1, 1), wx.TOP|wx.LEFT|wx.RIGHT|wx.BOTTOM, 5)	
 
        TemplatesPanelCoreGrill.SetMinSize(size=wx.Size(0.99 * XTemplatesPanelCore, 0.95 * YTemplatesPanelCore))
       
        TemplatesPanelCore.SetSizer(TemplatesPanelCoreGrill)
       
      #Templates Panel Group 
        TemplatesPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        TemplatesPanel.SetSizer(TemplatesPanelGroupBox)
        TemplatesPanelGroupSB = wx.StaticBox(parent=TemplatesPanel, label="Templates")
        TemplatesPanelGroupSB.SetFont(LinedFont)
        TemplatesPanelGroupSB.SetForegroundColour(LinedFontColour)            
        TemplatesPanelGroupSizer = wx.StaticBoxSizer(TemplatesPanelGroupSB, wx.VERTICAL)
        TemplatesPanelGroupSizer.Add(TemplatesPanelCore, wx.ALL, 5)
        TemplatesPanelGroupSizer.SetMinSize(wx.Size(0.95 * XTemplatesPanel, 0.95 * YTemplatesPanel))
        TemplatesPanelGroupBox.Add(TemplatesPanelGroupSizer)
        
     
     
     
 #Graphic Details Panel
    
        GraphicDetailsPanel = wx.Panel(self.AnalysisPanel, -1, size=wx.Size(1.0 * XAnalysisPanel, 0.45 * YAnalysisPanel))
        XGraphicDetailsPanel, YGraphicDetailsPanel = GraphicDetailsPanel.GetSize()
        Xsize, Ysize = GraphicDetailsPanel.GetSize()
        Xsize = 0.20 * Xsize
        Ysize = 0.12 * Ysize
        ButtonXsize = 0.45 * Xsize
        ButtonYsize = Ysize
        ComboBoxXsize = 0.5*Xsize
        ComboBoxYsize = Ysize

        GraphicDetailsPanelCore = wx.Panel(GraphicDetailsPanel, -1, size=wx.Size(0.95 * XGraphicDetailsPanel, 0.95 * YGraphicDetailsPanel))
        XGraphicDetailsPanelCore, YGraphicDetailsPanelCore = GraphicDetailsPanelCore.GetSize()
    
    #Upper Panel
        CommonGraphicsPanel = wx.Panel(GraphicDetailsPanelCore, -1, size=wx.Size(0.95 * XGraphicDetailsPanelCore, 0.15 * YGraphicDetailsPanelCore))  
        
        self.TitleTxt = wx.lib.stattext.GenStaticText(CommonGraphicsPanel, -1, 'Title :')
        self.Title = wx.TextCtrl(CommonGraphicsPanel, -1, size=wx.Size(Xsize, Ysize), style=wx.TE_PROCESS_ENTER)
        self.XlabelTxt = wx.lib.stattext.GenStaticText(CommonGraphicsPanel, -1, 'X-label :')
        self.Xlabel = wx.TextCtrl(CommonGraphicsPanel, -1, size=wx.Size(Xsize, Ysize), style=wx.TE_PROCESS_ENTER)
        self.YlabelTxt = wx.lib.stattext.GenStaticText(CommonGraphicsPanel, -1, 'Y-label :') 
        self.Ylabel = wx.TextCtrl(CommonGraphicsPanel, -1, size=wx.Size(Xsize, Ysize), style=wx.TE_PROCESS_ENTER)
        self.ClearButton = wx.Button(CommonGraphicsPanel, -1, size=wx.Size(ButtonXsize, ButtonYsize) , label = 'Clear')
        
        self.ClearButton.Bind(wx.EVT_LEFT_DOWN, self.ErasePlotOptions)
        self.Title.Bind(wx.EVT_TEXT, self.StoreTitle)
        self.Xlabel.Bind(wx.EVT_TEXT, self.StoreXlabel)
        self.Ylabel.Bind(wx.EVT_TEXT, self.StoreYlabel)

        CommonGraphicsPanelBox = wx.GridBagSizer()
        CommonGraphicsPanelBox.Add(self.TitleTxt, (0, 0), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 5) 
        CommonGraphicsPanelBox.Add(self.Title, (0, 1), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 5)  
        CommonGraphicsPanelBox.Add(self.XlabelTxt, (0, 2), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 15) 
        CommonGraphicsPanelBox.Add(self.Xlabel, (0, 3), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 5)  
        CommonGraphicsPanelBox.Add(self.YlabelTxt, (0, 4), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 15)  
        CommonGraphicsPanelBox.Add(self.Ylabel, (0, 5), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 5)  
        CommonGraphicsPanelBox.Add(self.ClearButton, (0, 8), (1, 1), wx.ALIGN_RIGHT) 

        CommonGraphicsPanelBox.SetMinSize(size=wx.Size(0.99 * XGraphicDetailsPanelCore, 0.15 * YGraphicDetailsPanelCore))
        CommonGraphicsPanel.SetSizer(CommonGraphicsPanelBox)
        
    #Graphic Limits Panel
        GraphicLimitsPanel = wx.Panel(GraphicDetailsPanelCore, -1, size=wx.Size(0.95 * XGraphicDetailsPanelCore, 0.10 * YGraphicDetailsPanelCore))
        XGraphicLimitsPanel, YGraphicLimitsPanel = GraphicLimitsPanel.GetSize()
        #GraphicLimitsPanel.SetBackgroundColour(wx.RED)
        
        self.AutomaticLimitsCB = wx.CheckBox(GraphicLimitsPanel, -1 , 'Automatic Limits', style=wx.ALIGN_LEFT)
        self.AutomaticLimitsCB.Bind(wx.EVT_CHECKBOX, self.SetGraphicLimits)
        
        XMinMaxSize= 0.1*XGraphicLimitsPanel 
        YMinMaxSize = 0.99*Ysize
        
        self.XminlabelTxt = wx.lib.stattext.GenStaticText(GraphicLimitsPanel, -1, 'X-min :' , style=wx.ALIGN_CENTER)
        self.Xminlabel = wx.TextCtrl(GraphicLimitsPanel, -1, size=wx.Size(XMinMaxSize, YMinMaxSize), style=wx.TE_PROCESS_ENTER)
        self.XmaxlabelTxt = wx.lib.stattext.GenStaticText(GraphicLimitsPanel, -1, 'X-max :')
        self.Xmaxlabel = wx.TextCtrl(GraphicLimitsPanel, -1, size=wx.Size(XMinMaxSize, YMinMaxSize), style=wx.TE_PROCESS_ENTER)
        self.YminlabelTxt = wx.lib.stattext.GenStaticText(GraphicLimitsPanel, -1, 'Y-min :')
        self.Yminlabel = wx.TextCtrl(GraphicLimitsPanel, -1, size=wx.Size(XMinMaxSize, YMinMaxSize), style=wx.TE_PROCESS_ENTER)
        self.YmaxlabelTxt = wx.lib.stattext.GenStaticText(GraphicLimitsPanel, -1, 'Y-max :')
        self.Ymaxlabel = wx.TextCtrl(GraphicLimitsPanel, -1, size=wx.Size(XMinMaxSize, YMinMaxSize), style=wx.TE_PROCESS_ENTER)
        self.GraphicLimitsLabelVector=['X-min','X-max','Y-min','Y-max' ]
        
        self.Xminlabel.Bind(wx.EVT_TEXT, self.StoreXminlabel)
        self.Xmaxlabel.Bind(wx.EVT_TEXT, self.StoreXmaxlabel)
        self.Yminlabel.Bind(wx.EVT_TEXT, self.StoreYminlabel)
        self.Ymaxlabel.Bind(wx.EVT_TEXT, self.StoreYmaxlabel)
       
        GraphicLimitsPanelBox = wx.GridBagSizer()
        GraphicLimitsPanel.SetSizer(GraphicLimitsPanelBox)
        GraphicLimitsPanelBox.Add(self.AutomaticLimitsCB, (0, 0), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 5) 
        GraphicLimitsPanelBox.Add(self.XminlabelTxt, (0, 1), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 25)
        GraphicLimitsPanelBox.Add(self.Xminlabel, (0, 2), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 5)
        GraphicLimitsPanelBox.Add(self.XmaxlabelTxt, (0, 3), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 20)        
        GraphicLimitsPanelBox.Add(self.Xmaxlabel, (0, 4), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 5)
        GraphicLimitsPanelBox.Add(self.YminlabelTxt, (0, 5), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 20)
        GraphicLimitsPanelBox.Add(self.Yminlabel, (0, 6), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 5)
        GraphicLimitsPanelBox.Add(self.YmaxlabelTxt, (0, 7), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 20)        
        GraphicLimitsPanelBox.Add(self.Ymaxlabel, (0, 8), (1, 1), wx.ALIGN_CENTER | wx.ALL, 5)
        GraphicLimitsPanelBox.SetMinSize(size=wx.Size(0.95 * XGraphicLimitsPanel, 0.95 * YGraphicLimitsPanel))

    

    #Plots Panel
        PlotsPanel = wx.Panel(GraphicDetailsPanelCore, -1, size=wx.Size(0.99 * XGraphicDetailsPanel, 0.85 * YGraphicDetailsPanel))
        XPlotsPanel, YPlotsPanel = PlotsPanel.GetSize()
        
        PlotsPanelCore = wx.Panel(PlotsPanel, -1, size=wx.Size(0.95 * XPlotsPanel, 0.95 * YPlotsPanel))
        XPlotsPanelCore, YPlotsPanelCore = PlotsPanelCore.GetSize()
    
        #Radio Buttons
        RBPlotsPanelCore = wx.Panel(PlotsPanelCore, -1, size=wx.Size(0.95 * XPlotsPanel, 0.25 * YPlotsPanel))
        self.Plot1RB = wx.RadioButton(RBPlotsPanelCore, -1, 'Plot Nr. 1', style=wx.RB_GROUP)
        self.Plot2RB = wx.RadioButton(RBPlotsPanelCore, -1, 'Plot Nr. 2')
        self.Plot3RB = wx.RadioButton(RBPlotsPanelCore, -1, 'Plot Nr. 3')
        self.Plot4RB = wx.RadioButton(RBPlotsPanelCore, -1, 'Plot Nr. 4')
        
        RadButtonsPlotsPanelBox = wx.BoxSizer(wx.HORIZONTAL)
        RadButtonsPlotsPanelBox.Add(self.Plot1RB, wx.LEFT, 15)
        RadButtonsPlotsPanelBox.Add(self.Plot2RB, wx.LEFT, 15)
        RadButtonsPlotsPanelBox.Add(self.Plot3RB, wx.LEFT, 15)
        RadButtonsPlotsPanelBox.Add(self.Plot4RB, wx.LEFT, 15)        
        
        RadButtonsPlotsPanelBox.SetMinSize(size=wx.Size(0.99 * XPlotsPanelCore, 0.25 * YPlotsPanelCore))
        RBPlotsPanelCore.SetSizer(RadButtonsPlotsPanelBox)  
        
        self.Plot1RB.Bind(wx.EVT_RADIOBUTTON, self.StoreUpdatePlot)
        self.Plot2RB.Bind(wx.EVT_RADIOBUTTON, self.StoreUpdatePlot)
        self.Plot3RB.Bind(wx.EVT_RADIOBUTTON, self.StoreUpdatePlot)
        self.Plot4RB.Bind(wx.EVT_RADIOBUTTON, self.StoreUpdatePlot)

        
        #Source File
        SourceFilePlotsPanelCore = wx.Panel(PlotsPanelCore, -1, size=wx.Size(0.95 * XPlotsPanel, 0.25 * YPlotsPanel))
        #SourceFilePlotsPanelCore.SetBackgroundColour((120,210,169))
        
        self.SourceFileButton = wx.Button(SourceFilePlotsPanelCore, 1, size=wx.Size(ButtonXsize, ButtonYsize) , label='Examine' )
        self.SourceFileTxt = wx.lib.stattext.GenStaticText(SourceFilePlotsPanelCore, -1, 'Source File  : ', size=wx.Size(0.6 * Xsize, Ysize), style=wx.ST_NO_AUTORESIZE )        
        self.SourceFileTextCtrl = wx.TextCtrl(SourceFilePlotsPanelCore, -1, '', size=wx.Size(3 * Xsize, Ysize), style=wx.TE_PROCESS_ENTER)
        LinesOrDotsList = ['Lines', 'Dots', 'Dashes','Circles','Squares','Pentagons','Pluses','Crosses','Dotted Line']
        self.PlotStyle  = ['-', '.','--','o','s','p','+','x','.-']

        self.LinesOrDotsComboBox = wx.ComboBox(SourceFilePlotsPanelCore, -1, value=LinesOrDotsList[0], choices=LinesOrDotsList, style=wx.CB_READONLY, size=wx.Size(0.12 * XPlotsPanelCore, 0.035 * XPlotsPanelCore))
        
        SourceFileGrid = wx.GridBagSizer()
        SourceFileGrid.Add(self.SourceFileTxt, (0, 1), (1, 1), flag=wx.ALIGN_CENTER_VERTICAL)
        SourceFileGrid.Add(self.SourceFileTextCtrl, (0, 2), (1, 5), flag=wx.ALIGN_CENTER)
        SourceFileGrid.Add(self.SourceFileButton, (0, 8), (1, 1), wx.ALIGN_CENTER | wx.ALIGN_RIGHT | wx.RIGHT , 27)
        SourceFileGrid.Add(self.LinesOrDotsComboBox, (0, 9), (1, 1), flag=wx.ALIGN_CENTER)        

        SourceFileGrid.SetMinSize(size=wx.Size(0.99 * XPlotsPanelCore, 0.25 * YPlotsPanelCore))
        SourceFilePlotsPanelCore.SetSizer(SourceFileGrid)
         
        self.SourceFileButton.Bind(wx.EVT_LEFT_DOWN, self.SavePlotPathByButton)
        self.SourceFileTextCtrl.Bind(wx.EVT_TEXT, self.SavePlotPathByTextCtrl)        
        
        #Conditions
        ConditionsPlotsPanelCore = wx.Panel(PlotsPanelCore, -1, size=wx.Size(0.95 * XPlotsPanel, 0.25 * YPlotsPanel))
        #ConditionsPlotsPanelCore.SetBackgroundColour((120,010,169))
        
        self.ConditionsPlotsTxt = wx.lib.stattext.GenStaticText(ConditionsPlotsPanelCore, -1, 'Condition  :  ', size=wx.Size(0.6 * Xsize, Ysize), style=wx.ST_NO_AUTORESIZE)
        self.ConditionsPlotsTextCtrl = wx.TextCtrl(ConditionsPlotsPanelCore, -1, '', size=wx.Size(2 * Xsize, Ysize), style=wx.TE_PROCESS_ENTER)

        self.FirstCondition = ['', 'INPUT', 'CS', 'MODEL', 'EPOCHSAT', 'PREFIT', 'POSTFIT', 'FILTER', 'OUTPUT', 'SATDIFF', 'SATSTAT', 'SATSTATTOT', 'SATPVT']
        i = 1;
        PRNList = ['']
        while (i <= 32):
            PRNList.append('PRN ' + str(i))
            i += 1
        self.EmptyCondition = ['']

        self.ColumnsList = ['',
            ['', 'INPUTID', 'YEAR', 'DOY', 'SEC', 'GNSS', 'PRN', 'ARCLENGTH', 'C1 [C1C]', 'P1 [C1P]', 'P2 [C2P]', 'L1 [L1P]', 'L2 [L2P]'],
            ['', 'CSID', 'YEAR', 'DOY', 'SEC', 'GNSS', 'PRN' ] ,
            ['', 'MODELID', 'YEAR', 'DOY', 'SEC', 'GNSS', 'PRN', 'MEASUREMENT', 'FLIGHTTIME', 'MEASVAL', 'MODELVAL', 'SATXPOS', 'SATYPOS', 'SATZPOS', 'SATXVEL', 'SATYVEL', 'SATZVEL', 'SATRECGEOMDIST', 'SATCLK', 'SATPCOPROJ', 'RECPCOPROJ', 'RECARPPROJ', 'RELCLOCK', 'WINDUP', 'TROP', 'IONO', 'RELPATH', 'TGD', 'SOLIDTIDES', 'ELEVATION', 'AZIMUTH'] ,
            ['', 'EPOCHSATID', 'YEAR', 'DOY', 'SEC', 'MEASUREMENTID', 'NSATINVIEW', 'SAT1', 'SAT2', 'SAT3', 'SAT4', 'SAT5', 'SAT6', 'SAT7', 'SAT8', 'SAT9', 'SAT10', 'SAT11', 'SAT12'] ,
            ['', 'PREFITID', 'YEAR', 'DOY', 'SEC', 'GNSS', 'PRN', 'MEASUREMENTID', 'PREFIT', 'MEASVAL', 'MODELVAL', 'X-LoS', 'Y-LoS', 'Z-LoS', 'T-LoS', 'ELEVATION', 'AZIMUTH','STDDEV'],
            ['', 'POSTFITID', 'YEAR', 'DOY', 'SEC', 'GNSS', 'PRN', 'MEASUREMENTID', 'POSTFIT', 'MEASVAL', 'ESTMODELVAL', 'ELEVATION', 'AZIMUTH', 'CPAMB'],
            ['', 'FILTERID', 'YEAR', 'DOY', 'SEC', 'F1-STAX', 'F2-STAY', 'F3-STAZ', 'F4-STACLK', 'F5-TROP'] ,
            ['', 'OUTPUTID', 'YEAR', 'DOY', 'SEC', 'COV', 'STAX', 'STAY', 'STAZ', 'DSTAX', 'DSTAY', 'DSTAZ', 'ERRSTAX', 'ERRSTAY', 'ERRSTAZ', 'STALAT', 'STALON', 'STAHGT', 'DSTAN', 'DSTAE', 'DSTAU', 'ERRSTAN', 'ERRSTAE', 'ERRSTAU','GDOP','PDOP','TDOP','HDOP','VDOP','ZTD_TOT','ZTD_EST','ERRZTD'],
            ['', 'SATDIFFID', 'YEAR', 'DOY', 'SEC', 'GNSS', 'PRN', 'SISRE', 'SISREORB', '3DORB', 'CLK', 'RADIAL', 'ALONG-TRACK', 'CROSS-TRACK'],
            ['', 'SATSTATID', 'PRN', 'NSAMPLES', 'SISREMEAN', 'SISRERMS', 'SISRESTDDEV', 'SISREORBMEAN', 'SISREORBRMS', 'SISREORBSTDDEV', '3DORBMEAN', '3DORBRMS', '3DORBSTDDEV', 'CLKMEAN', 'CLKRMS', 'CLKSTDDEV', 'RADMEAN', 'RADRMS', 'RADSTDDEV', 'ALONGMEAN', 'ALONGRMS', 'ALONGSTDDEV', 'CROSSMEAN', 'CROSSRMS', 'CROSSSTDDEV'],
            ['', 'SATSTATTOTID', 'NSAMPLES', 'SISREMEAN', 'SISRERMS', 'SISRESTDDEV', 'SISREORBMEAN', 'SISREORBRMS', 'SISREORBSTDDEV', '3DORBMEAN', '3DORBRMS', '3DORBSTDDEV', 'CLKMEAN', 'CLKRMS', 'CLKSTDDEV', 'RADMEAN', 'RADRMS', 'RADSTDDEV', 'ALONGMEAN', 'ALONGRMS', 'ALONGSTDDEV', 'CROSSMEAN', 'CROSSRMS', 'CROSSSTDDEV'],
            ['', 'SATPVTID', 'YEAR', 'DOY', 'SEC', 'GNSS', 'PRN', 'SATRX', 'SATRY', 'SATRZ', 'SATVX', 'SATVY', 'SATVZ', 'SATCLK']]
        self.SecondList = [[''], PRNList, PRNList, PRNList, [''], PRNList, PRNList, [''], [''], PRNList, [''], [''], PRNList]
        self.SecondListPos = [0, 6, 6, 6, 0, 6, 6, 0, 0, 6, 0, 0, 6]
 

        
        self.FirstConditionComboBox = wx.ComboBox(ConditionsPlotsPanelCore, -1, value=self.FirstCondition[0], choices=self.FirstCondition, style=wx.CB_READONLY, size=wx.Size(0.15 * XPlotsPanelCore, Ysize))    
        self.SecondConditionComboBox = wx.ComboBox(ConditionsPlotsPanelCore, -1, value=self.EmptyCondition[0], choices=self.EmptyCondition, style=wx.CB_READONLY, size=wx.Size(0.15 * XPlotsPanelCore, Ysize))


        self.FirstConditionComboBox.Bind(wx.EVT_COMBOBOX, self.SelectConditions)
        self.SecondConditionComboBox.Bind(wx.EVT_COMBOBOX, self.SelectConditions)
        self.ConditionsPlotsTextCtrl.Bind(wx.EVT_KEY_UP, self.SetConditionsText) 
        
        
        self.ColorList = ['Blue','Green','Red','Cyan','Magenta','Yellow','Black']
        self.GraphColorList = ['blue','green','red','cyan','magenta','yellow','black']
        self.ColorComboBox = wx.ComboBox(ConditionsPlotsPanelCore, -1, value=self.ColorList[0], choices=self.ColorList, style=wx.CB_READONLY, size=wx.Size(0.12 * XPlotsPanelCore, Ysize))
        
        ConditionsGrid = wx.GridBagSizer()
        ConditionsGrid.Add(self.ConditionsPlotsTxt, (0, 1), (1, 1), wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT)
        ConditionsGrid.Add(self.FirstConditionComboBox, (0, 2), (1, 1), wx.ALIGN_CENTER)
        ConditionsGrid.Add(self.SecondConditionComboBox, (0, 3), (1, 1), wx.ALIGN_CENTER | wx.RIGHT, 15)
        ConditionsGrid.Add(self.ConditionsPlotsTextCtrl, (0, 4), (1, 9), wx.ALIGN_CENTER | wx.EXPAND | wx.RIGHT, 20)
        ConditionsGrid.Add(self.ColorComboBox, (0, 14), (1, 1), wx.ALIGN_CENTER)
        
        ConditionsGrid.SetMinSize(size=wx.Size(0.99 * XPlotsPanelCore, 0.25 * YPlotsPanelCore))
        ConditionsPlotsPanelCore.SetSizer(ConditionsGrid)

        PlotsPanelCoreBox = wx.BoxSizer(wx.VERTICAL)
        PlotsPanelCoreBox.Add(RBPlotsPanelCore)
        PlotsPanelCoreBox.Add(SourceFilePlotsPanelCore)
        PlotsPanelCoreBox.Add(ConditionsPlotsPanelCore)
      
        PlotsPanelCoreBox.SetMinSize(size=wx.Size(0.99 * XPlotsPanelCore, 0.99 * YPlotsPanelCore))
        PlotsPanelCore.SetSizer(PlotsPanelCoreBox)         
        
        #Column Conditions
        ColumnConditionsPlotsPanelCore = wx.Panel(PlotsPanelCore, -1, size=wx.Size(0.95 * XPlotsPanel, 0.25 * YPlotsPanel))

        self.XColumnConditionsPlotsTxt = wx.lib.stattext.GenStaticText(ColumnConditionsPlotsPanelCore, -1, 'X Column  : ', size=wx.Size(0.6 * Xsize, Ysize), style=wx.ST_NO_AUTORESIZE)
        self.XFirstColumnCondition = ['']
        self.XFirstConditionComboBox = wx.ComboBox(ColumnConditionsPlotsPanelCore, -1, value=self.XFirstColumnCondition[0], choices=self.XFirstColumnCondition, style=wx.CB_READONLY, size=wx.Size(0.15 * XPlotsPanelCore, Ysize))    
        self.XFirstConditionTextCtrl = wx.TextCtrl(ColumnConditionsPlotsPanelCore, -1, '', style=wx.TE_PROCESS_ENTER)
        self.XFirstConditionTextCtrl.SetSize(wx.Size(0.5 * Xsize, Ysize))

        self.YColumnConditionsPlotsTxt = wx.lib.stattext.GenStaticText(ColumnConditionsPlotsPanelCore, -1, 'Y Column : ')        
        YFirstColumnCondition = self.XFirstColumnCondition
        self.YFirstConditionComboBox = wx.ComboBox(ColumnConditionsPlotsPanelCore, -1, value=YFirstColumnCondition[0] , choices=YFirstColumnCondition, style=wx.CB_READONLY, size=wx.Size(0.15 * XPlotsPanelCore, Ysize))    
        self.YFirstConditionTextCtrl = wx.TextCtrl(ColumnConditionsPlotsPanelCore, -1, '', style=wx.TE_PROCESS_ENTER)
        self.YFirstConditionTextCtrl.SetSize(wx.Size(0.5 * Xsize, Ysize))

        self.LabelTxt = wx.lib.stattext.GenStaticText(ColumnConditionsPlotsPanelCore, -1, 'Label :  ')        
        self.LabelTextCtrl = wx.TextCtrl(ColumnConditionsPlotsPanelCore, -1, '', size=wx.Size(0.7 * Xsize, Ysize), style=wx.TE_PROCESS_ENTER)
        
        self.LabelTextCtrl.Bind(wx.EVT_TEXT, self.SaveLabelByTextCtrl)
        self.XFirstConditionComboBox.Bind(wx.EVT_COMBOBOX, self.XFirstConditionComboBoxCallback)
        self.YFirstConditionComboBox.Bind(wx.EVT_COMBOBOX, self.YFirstConditionComboBoxCallback)
        self.XFirstConditionTextCtrl.Bind(wx.EVT_KEY_UP, self.SaveXFirstConditionByTextCtrl)
        self.YFirstConditionTextCtrl.Bind(wx.EVT_KEY_UP, self.SaveYFirstConditionByTextCtrl)
        
        ColumnConditionsGrid = wx.GridBagSizer()
        ColumnConditionsGrid.Add(self.XColumnConditionsPlotsTxt, (0, 1), (1, 1), wx.ALIGN_CENTER)
        ColumnConditionsGrid.Add(self.XFirstConditionComboBox, (0, 2), (1, 1), wx.ALIGN_CENTER)
        ColumnConditionsGrid.Add(self.XFirstConditionTextCtrl, (0, 3), (1, 1))
        ColumnConditionsGrid.Add(self.YColumnConditionsPlotsTxt, (0, 4), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 20)
        ColumnConditionsGrid.Add(self.YFirstConditionComboBox, (0, 5), (1, 1), wx.ALIGN_CENTER)
        ColumnConditionsGrid.Add(self.YFirstConditionTextCtrl, (0, 6), (1, 1))
        ColumnConditionsGrid.Add(self.LabelTxt, (0, 7), (1, 1), wx.ALIGN_CENTER | wx.LEFT, 30)
        ColumnConditionsGrid.Add(self.LabelTextCtrl, (0, 8), (1, 1), wx.ALIGN_CENTER)        

        
        ColumnConditionsGrid.SetMinSize(size=wx.Size(0.99 * XPlotsPanelCore, 0.25 * YPlotsPanelCore))
        ColumnConditionsPlotsPanelCore.SetSizer(ColumnConditionsGrid)

        PlotsPanelCoreBox = wx.BoxSizer(wx.VERTICAL)
        PlotsPanelCoreBox.Add(RBPlotsPanelCore)
        PlotsPanelCoreBox.Add(SourceFilePlotsPanelCore)
        PlotsPanelCoreBox.Add(ConditionsPlotsPanelCore)
        PlotsPanelCoreBox.Add(ColumnConditionsPlotsPanelCore)
      
        PlotsPanelCoreBox.SetMinSize(size=wx.Size(0.99 * XPlotsPanelCore, 0.99 * YPlotsPanelCore))
        PlotsPanelCore.SetSizer(PlotsPanelCoreBox)
        
      #Plots Panel Group 
        PlotsPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        PlotsPanel.SetSizer(PlotsPanelGroupBox)    
        PlotsPanelGroupSB = wx.StaticBox(parent=PlotsPanel, label="Individual Plot(s) Configuration")
        PlotsPanelGroupSB.SetFont(LinedFont)
        PlotsPanelGroupSB.SetForegroundColour(LinedFontColour)         
        PlotsPanelGroupSizer = wx.StaticBoxSizer(PlotsPanelGroupSB, wx.VERTICAL)
        PlotsPanelGroupSizer.Add(PlotsPanelCore, wx.ALL, 5)
        PlotsPanelGroupSizer.SetMinSize(wx.Size(0.95 * XPlotsPanel, 0.95 * YPlotsPanel))
        PlotsPanelGroupBox.Add(PlotsPanelGroupSizer)        
                
      #Graphic Details CorePanel Group   
        GraphicDetailsPanelCoreBox = wx.BoxSizer(wx.VERTICAL)
        GraphicDetailsPanelCoreBox.Add(CommonGraphicsPanel)
        GraphicDetailsPanelCoreBox.Add(GraphicLimitsPanel) 
        GraphicDetailsPanelCoreBox.Add(PlotsPanel)        
        GraphicDetailsPanelCoreBox.SetMinSize(size=wx.Size(0.99 * XGraphicDetailsPanelCore, 0.95 * YGraphicDetailsPanelCore))       
        GraphicDetailsPanelCore.SetSizer(GraphicDetailsPanelCoreBox)
       
      #Graphic Details Panel Group 
        GraphicDetailsPanelGroupBox = wx.BoxSizer(wx.VERTICAL)
        GraphicDetailsPanel.SetSizer(GraphicDetailsPanelGroupBox)
        GraphicDetailsPanelGroupSB = wx.StaticBox(parent=GraphicDetailsPanel, label="Global Graphic Parameters")
        GraphicDetailsPanelGroupSB.SetFont(LinedFont)
        GraphicDetailsPanelGroupSB.SetForegroundColour(LinedFontColour)             
        GraphicDetailsPanelGroupSizer = wx.StaticBoxSizer(GraphicDetailsPanelGroupSB, wx.VERTICAL)
        GraphicDetailsPanelGroupSizer.Add(GraphicDetailsPanelCore, wx.ALL, 5)
        GraphicDetailsPanelGroupSizer.SetMinSize(wx.Size(0.95 * XGraphicDetailsPanel, 0.95 * YGraphicDetailsPanel))
        GraphicDetailsPanelGroupBox.Add(GraphicDetailsPanelGroupSizer)        
        
     #Analysis Panel Layout   
        AnalisiPanelGrill = wx.GridBagSizer()
        AnalisiPanelGrill.Add(TemplatesPanel, (0, 0), (1, 5), wx.ALL | wx.ALIGN_CENTER, 5) 
        AnalisiPanelGrill.Add(GraphicDetailsPanel, (1, 0), (1, 5), wx.ALL, 5)  
        AnalisiPanelGrill.Add(self.PlotButton , (2, 0), (1, 5), wx.ALIGN_RIGHT | wx.ALL, 5)
        AnalisiPanelGrill.SetMinSize(size=wx.Size(0.99 * XAnalysisPanel, 0.95 * YAnalysisPanel))
        self.AnalysisPanel.SetSizer(AnalisiPanelGrill)
        self.SetSizer(AnalisiPanelGrill)
     
     #Variables needed for callbacks:
         
        self.PlotParameters = [['', '', '','', '', '', ''], ['', '', '', ''], ['', '', '', ''], ['', '', '', ''], ['', '', '', ''], ['', '', '', ''], [0, 0, 0, 0],[0, 0, 0, 0]]  
        #self.PlotParameters=[['tile','Xlabel','Ylabel','Xmin','Xmax','Ymin','Ymax'],['Source1','Source2','Source3','Source4'],['WritenCondition1','WritenCondition2','WritenCondition3','WritenCondition4'],
        #['XCol1','XCol2','XCol3','XCol4'],['YCol1','YCol2','YCol3','YCol4'],['Label1','Label2','Label3','Labe4'],[LinesOrDots1,LinesOrDots2,LinesOrDots3,LinesOrDots4],[PlotColor1,PlotColor2,PlotColor3,PlotColor4] ] 
        
        self.SelectedConditions = [[0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0]]
        #self.SelectedConditions=[[FirstCondition1,FirstCondition2,FirstCondition3,FirstCondition4],[SecondCondition1,SecondCondition2,SecondCondition3,SecondCondition4],
        #[XColCondition1,XColCondition2,XColCondition3,XColCondition4],[YColCondition1,YColCondition2,YColCondition3,YColCondition4]]             
             
        self.Plot1RB.SetValue(True)
        self.PlotNumber = 0
        self.PlotPath = ''
        self.PlotLabel = ''

        # Set tooltips
        self.SetTooltipDictionary()
        # Thread lock
        self.threadLock = threading.RLock()
        # Tooltip creation
        self.ToolTipFrame = WindowHTML(None)
        self.formerSize = wx.Size(tooltipXsize, tooltipYsize)

        # Clear
        self.ErasePlotOptionsFunc()
        

    '''
    Definition of functions:
    '''        
    
##########################################################################
# Name        : SetTooltipDictionary
# Description : In this function, the text for the tooltips of the analysis tab
#               is stored in a dictionary
##########################################################################
    def SetTooltipDictionary(self):
        self.TooltipTextDictionary = {

                  #Templates:
                          self.TemplateButtons[0].GetId():u"<b>Templates: NEU positioning error</b><br><br>The templates are a set of preconfigured plotting options for the <i>Graphic Details</i> section. Clicking on any button will load the options in that section, allowing to modify them or plot them directly.<br><br>The <i>NEU position error</i> template sets the options to print the three components (North, East and Up) of the error of the receiver positioning obtained by the filter. This error is computed by the difference between the direct filter estimation and the <i>A priori Receiver Position</i> in the <i>Input</i> section of <i>Calculus</i>. Thence to obtain a reliable error estimation, this a priori position should be precise.<br><br>If you do not have any precise position of the receiver, you can obtain it by doing a PPP processing (<i>SP3</i>/<i>Static</i>/<i>Dual-frequency</i>/<i>Pseudorange+Carrier phase</i>). The XYZ position (fields 6 to 8 of <i>OUTPUT</i> message) of the last epoch should have an accuracy in the order of the centimetre, and can be used as a good reference position.",
                          self.TemplateButtons[1].GetId():u"<b>Templates: Horizontal positioning error</b><br><br>The templates are a set of preconfigured plotting options for the <i>Graphic Details</i> section. Clicking on any button will load the options in that section, allowing to modify them or plot them directly.<br><br>The <i>Horizontal positioning error</i> template sets the options to print the North vs East position error components. This provides an insight of the horizontal dispersion and bias of the errors. To see more details on how the error is computed, see the <i>NEU positioning error</i> button template.",
                          self.TemplateButtons[2].GetId():u"<b>Templates: Zenith Tropospheric Delay</b><br><br>The templates are a set of preconfigured plotting options for the <i>Graphic Details</i> section. Clicking on any button will load the options in that section, allowing to modify them or plot them directly.<br><br> The <i>Zenith Tropospheric Delay</i> template sets the options to print the <b>tropospheric estimations</b> as a function of time. It includes the nominal part corrected in the modelling and the estimated part computed in the filter.",
                          self.TemplateButtons[3].GetId():u"<b>Templates: Ionospheric combinations</b><br><br>The templates are a set of preconfigured plotting options for the <i>Graphic Details</i> section. Clicking on any button will load the options in that section, allowing to modify them or plot them directly.<br><br>The <i>Ionospheric combinations</i> template sets the options to print the two ionospheric (geometry-free) combinations: PI (P2-P1) [pseudorange] and LI (L1-L2) [carrier phase].",


                          self.TemplateButtons[4].GetId():u"<b>Templates: Dilution Of Precision (DOP)</b><br><br>The templates are a set of preconfigured plotting options for the <i>Graphic Details</i> section. Clicking on any button will load the options in that section, allowing to modify them or plot them directly.<br><br>The <i>DOP</i> template sets the options to print Geometric Dilution of Precision (GDOP), Horizontal Dilution of Precision (HDOP), Vertical Dilution of Precision (VDOP), Time Dilution of Precision (TDOP). <br><br>This computations take into account the geometry matrix (G), which is linked only to the receiver-satellite(s) ray geometry. This factor impacts the accuracy of the filter solution.",
                          self.TemplateButtons[5].GetId():u"<b>Templates: Satellite skyplot</b><br><br>The templates are a set of preconfigured plotting options for the <i>Graphic Details</i> section. Clicking on any button will load the options in that section, allowing to modify them or plot them directly.<br><br>The <i>Satellite skyplot</i> template sets the options to print the elevation/azimuth of the satellites in a skyplot, being the centre of the plot the zenith of the satellite, and the extremes, the lower elevations.<br><br>This is a special plot which makes use of sin and cos in the <i>X column</i> and <i>Y column</i> to obtain a polar plot. You can use it as example for other operations in the columns.",
                          self.TemplateButtons[6].GetId():u"<b>Templates: Carrier phase ambiguities</b><br><br>The templates are a set of preconfigured plotting options for the <i>Graphic Details</i> section. Clicking on any button will load the options in that section, allowing to modify them or plot them directly.<br><br>The <i>Carrier phase ambiguities</i> template sets the options to print the estimation of carrier phase measurements in the filter. This is only possible in a processing with the <i>Pseudorange + carrier phase</i> measurements (in the <i>Filter</i> section).",
                          self.TemplateButtons[7].GetId():u"<b>Templates: Measurements Multipath/Noise</b><br><br>The templates are a set of preconfigured plotting options for the <i>Graphic Details</i> section. Clicking on any button will load the options in that section, allowing to modify them or plot them directly.<br><br>The <i>Measurements Multipath/Noise</i> template sets the options to print the measurement multipath for: <i>C1 code measurement, the Geometry-Free combination, the Melbourne-Wübenna and the GRAPHIC combination.</i> In addition to these combinations, raw measurements present in the file can also be plotted.",
                         

                          self.TemplateButtons[8].GetId():u"<b>Templates: Model components</b><br><br>The templates are a set of preconfigured plotting options for the <i>Graphic Details</i> section. Clicking on any button will load the options in that section, allowing to modify them or plot them directly.<br><br>The <i>Model components</i> template sets the options to print a component of the model as a function of time. By default, it selects the <b>Relativity effect</b>, but it can easily be chosen which model to print, by choosing it in the <i>Y Column</i> option.",
                          self.TemplateButtons[9].GetId():u"<b>Templates: Prefit residuals</b><br><br>The templates are a set of preconfigured plotting options for the <i>Graphic Details</i> section. Clicking on any button will load the options in that section, allowing to modify them or plot them directly.<br><br>The <i>Prefit residuals</i> template sets the options to print the measurement-model values <b>prefits</b>, which is the input to each filter execution as a function of time. It prints both pseudorange and carrier phase prefits.<br> <br> In general Prefit = Measurement - Model, but this is no longer true when using smoothing, as the first is the raw measurement without smoothing, but the second term takes smoothing into account." ,
                          self.TemplateButtons[10].GetId():u"<b>Templates: Postfit residuals</b><br><br>The templates are a set of preconfigured plotting options for the <i>Graphic Details</i> section. Clicking on any button will load the options in that section, allowing to modify them or plot them directly.<br><br>The <i>Postfit residuals</i> template sets the options to print the <b>filter residuals</b> (postfits) as a function of the satellite elevation. It prints both pseudorange and carrier phase postfits. This plot allows to observe the dependence of residuals from elevation. In general, carrier phase residuals will be quite independent from elevation, and pseudorange residuals can have a large dependence on elevation.",
                          self.TemplateButtons[11].GetId():u"<b>Templates: Orbits and Clock comparison</b><br><br>The templates are a set of preconfigured plotting options for the <i>Graphic Details</i> section. Clicking on any button will load the options in that section, allowing to modify them or plot them directly.<br><br>The <i>Orbits and Clock comparison</i> template sets the options to print the <b>discrepancies</b> between gLAB output solution (from a gLAB.out file) against the reference coordinates (from an sp3 file). <br> <br><b><font color='#C00000'>Note:</font></b> The comparison between these two files is excluded from the GUI, but gLAB Data Processing Core can be easily executed from the command line, using a dif.cfg configuration file.",

                  #Plots:
                          self.TitleTxt.GetId():u"<b>Graphic Title</b><br><br>Sets the title of the graphic.",
                          self.XlabelTxt.GetId():u"<b>Graphic X label</b><br><br>Sets the X label of the graphic.",
                          self.YlabelTxt.GetId():u"<b>Graphic Y label</b><br><br>Sets the Y label of the graphic.",
                          self.AutomaticLimitsCB.GetId():u"<b>Graphic Automatic Limits </b><br><br>Allows to change the X and Y axis plot range.",
                          self.XminlabelTxt.GetId():u"<b>Graphic X-min </b><br><br>Sets the minimum value of X graphic axis.",
                          self.XmaxlabelTxt.GetId():u"<b>Graphic X-max </b><br><br>Sets the maximum value of X graphic axis.",
                          self.YminlabelTxt.GetId():u"<b>Graphic Y-min </b><br><br>Sets the minimum value of Y graphic axis.",
                          self.YmaxlabelTxt.GetId():u"<b>Graphic Y-max </b><br><br>Sets the maximum value of Y graphic axis.",
                          self.ClearButton.GetId():u"<b>Clear graphic</b><br><br>Clear all the options in the <i>Graphic Details</i> section.",

                          self.Plot1RB.GetId():u"<b>Plot 1</b><br><br>Options for the first plot of the graphic.",
                          self.Plot2RB.GetId():u"<b>Plot 2</b><br><br>Options for the second plot of the graphic.",
                          self.Plot3RB.GetId():u"<b>Plot 3</b><br><br>Options for the third plot of the graphic.",
                          self.Plot4RB.GetId():u"<b>Plot 4</b><br><br>Options for the fourth plot of the graphic.",

                          self.SourceFileTxt.GetId():u"<b>Source File</b><br><br>Selects the source input file for the plot.",
                          self.LinesOrDotsComboBox.GetId():u"<b>Style selection</b><br><br>Allows to change the style of the plot marker: <i>Lines</i> or <i>Dots</i> markers are available together with some others.",
                          self.ColorComboBox.GetId():u"<b>Color selection</b><br><br>Allows to change the color of the plot: <i>Blue</i>, <i>Red</i>, <i>Green </i>, <i>Cyan</i>, <i>Magenta</i>, <i>Yellow</i>,  and <i>Black</i>.",
                          self.ConditionsPlotsTxt.GetId():u"<b>Condition</b><br><br>The condition is a way to insert one or more conditions in order to select which lines of the source file will be used in the plotting. This has a space to write the required conditions and two comboboxes, which automatically sets the condition text. The text is the only that matters, and the combo boxes are only configurators for the text.<br><br>To specify a column, it should be done as '$x' being <i>x</i> the number of the column. For example if you would like to specify the condition that the sixth column must be greater or equal than 10, you should do it as: <i>($6>=10)</i><br><br>You can specify a set of conditions by using AND [&] and OR [|] operators, such as: <i>($6&gt;=10)&($6&lt;=20)|($8==2)</i><br><br>You can specify that a column matches a specific string by surrounding the string by \", such as: <i>($1==\"OUTPUT\")</i><br><br>You can specify mathematical operations and constants, as: math.pi, math.e, math.sin() and math.cos(), as example: <i>($1==\"POSTFIT\")&(math.cos($11+5)&gt;0.707)</i><br><br>You can specify a specific character inside a column, by using $x[y], being <i>x</i> the column and <i>y</i> the character position beginning by 0, such as: <i>($1==\"POSTFIT\")&($7[0]==\"P\")</i> [First character of seventh column is 'P']<br><br>You can also operate between columns, such as: <i>(($9-$11)&lt;2)</i>",
                          self.FirstConditionComboBox.GetId():u"<b>First condition</b><br><br>This condition allows to specify the specific output message. Selection a value, will automatically refresh the rest of the combo boxes (second condition and x and y columns) with its specific list for the condition.",
                          self.SecondConditionComboBox.GetId():u"<b>Second condition</b><br><br>This condition depends on the first one, and in general allows you to select a specific PRN satellite.",
                          self.XColumnConditionsPlotsTxt.GetId():u"<b>X Column</b><br><br>This allows to specify the column for the x values for the selected plot. The combo box will be updated by the <i>First condition</i>, but the text part is the one that matters. As in the conditions, you can specify several things. In general if you want to plot a column you can do it by putting its column position, such as: <i>4</i>. You can also use the $4 to plot the forth column. As before, you can do operations between scalars and columns (and even use functions), such as: ($9*2-$1F1*3-1.5). Use the templates buttons to obtain some examples.",
                          self.YColumnConditionsPlotsTxt.GetId():u"<b>Y column</b><br><br>This allows to specify the column for the y values for the selected plot. Please see <i>X column</i> and <i>Condition</i> for more info on how to fill the text.",
                          self.LabelTxt.GetId():u"<b>Plot Label</b><br><br>Sets the label for the selected plot.",
                          self.PlotButton.GetId():u"<b>Execute the plotting tool</b><br><br>This button executes the plotting tool of gLAB with the specified options.",
                      
                          }
        self.SetTooltipBinds()
  
#########################################################################
# Name        : SetTooltipBinds
# Description : Every object in the tooltips dictionary, is binded to two
#               different mouse actions: entering and leaving, which show 
#               or hide a small help text (a tooltip)
##########################################################################
    def SetTooltipBinds(self):
        for item in self.TooltipTextDictionary:
            self.FindWindowById(item).Bind(wx.EVT_ENTER_WINDOW, self.SetToolTip, id=item)
            self.FindWindowById(item).Bind(wx.EVT_LEAVE_WINDOW, self.HideToolTip)                              
    
##########################################################################
# Name        : ShowToolTip
# Description : Function to synchronise events for GUIShowToolTip
##########################################################################
    def ShowToolTip(self):
        wx.CallAfter(self.GUIShowToolTip)

##########################################################################
# Name        : GUIShowToolTip
# Description : A small frame is showed with some text inside 
##########################################################################
    def GUIShowToolTip(self):
        self.threadLock.acquire()
        TooltipText = self.TooltipTextDictionary[self.IdToShowTooltip]

        MousePosX, MousePosY = wx.GetMousePosition()
        ZeroPointX, ZeroPointY, ScreenSizeX, ScreenSizeY = wx.ClientDisplayRect()
        
        self.ToolTipFrame.html.SetSize(self.formerSize)
        self.ToolTipFrame.SetSize(self.formerSize)
        self.ToolTipFrame.html.SetPage(TooltipText)
        self.ToolTipFrame.html.EnableScrolling(False, False)
        try:
            self.ToolTipFrame.html.SetSize(self.ToolTipFrame.html.GetVirtualSize())
            self.ToolTipFrame.SetSize(self.ToolTipFrame.html.GetVirtualSize())
            self.ToolTipFrame.SetBackgroundColour((255, 255, 197))
            self.ToolTipFrame.html.SetBackgroundColour((255, 255, 197))
            while (self.ToolTipFrame.html.GetSize() != self.ToolTipFrame.html.GetVirtualSize()):
                self.ToolTipFrame.html.SetSize(self.ToolTipFrame.html.GetVirtualSize())
            self.ToolTipFrame.Show()
        except:
            pass
        
        ToolTipX, ToolTipY = self.ToolTipFrame.html.GetSize()
        
        self.CalculateTooltipPosition(MousePosX, MousePosY, ScreenSizeX, ScreenSizeY, ToolTipX, ToolTipY, ZeroPointX, ZeroPointY)
        
        self.threadLock.release()

##########################################################################
# Name        : CalculateTooltipPosition
# Description : Computes where the tooltip is going to be displayed 
##########################################################################    
    def CalculateTooltipPosition(self, MouseX, MouseY, ScreenX, ScreenY, ToolTipSizeX, ToolTipSizeY, ZeroX, ZeroY):
        changes = 0
        Xpos = MouseX + 10
        if (Xpos + ToolTipSizeX + ZeroX) > ScreenX :
            Xpos = ScreenX - ToolTipSizeX
            changes += 1
        Ypos = MouseY + 10
        if (Ypos + ToolTipSizeY + ZeroY) > ScreenY :
            Ypos = ScreenY - ToolTipSizeY
            changes += 1
            
        if changes == 2:
            Xpos = MouseX - 20 - ToolTipSizeX 
        self.ToolTipFrame.SetPosition(wx.Point(Xpos, Ypos))
        
##########################################################################
# Name        : SetToolTip
# Description : Sets a timer to show a tooltip of a specific GUI object.
##########################################################################
    def SetToolTip(self, event):
        global tooltips, tooltipDelay
        if (tooltips == True):
            self.threadLock.acquire()
            try:
                self.tooltipthread.cancel()
            except:
                pass
            if (self.ToolTipFrame.IsShown()):
                self.ToolTipFrame.Hide()
            
            try:                
                self.IdToShowTooltip = event.GetId()
                self.tooltipthread = threading.Timer(tooltipDelay, function=self.ShowToolTip)
                self.tooltipthread.start()
            except:
                pass
            self.threadLock.release()
        event.Skip()
            
            
##########################################################################
# Name        : HideToolTip
# Description : This functions destroys the tooltip once the mouse leaves 
#               the object that created it 
##########################################################################
    def HideToolTip(self, event):
        global tooltips, tooltipDelay
        
        if (tooltips == True):
            self.threadLock.acquire()
            try:
                self.tooltipthread.cancel()
            except:
                pass    
            
            if (self.ToolTipFrame.IsShown()):
                self.ToolTipFrame.Hide()
            self.threadLock.release()
        event.Skip()
             
##########################################################################
# Name        : EnablePlotButton
# Description : This function simply reactivates the plot button 
##########################################################################
    def EnablePlotButton(self):
        self.PlotButton.Enable()

###########################################################################
# Name        : WaitPlot
# Description : This functions scans if there is a lock to the plotting
#               once the lock is released, it enables the plot button again
###########################################################################
    def WaitPlot (self):      
        start = time.time()
        while ((os.path.isfile(".plot.lock") == True) or ((os.path.isfile(DireName + "/.plot.lock") == True)))and (time.time() < start + 60):
            time.sleep(1)
        
        wx.CallAfter(self.EnablePlotButton)
    
############################################################################################################
# Name        : ShowGraphic
# Description : This function loads all the parameters from the GUI and 
#               opens a graphic up to 4 different plots. These paramters are:
#               self.PlotParameters=[['tile','Xlabel','Ylabel','Xmin','Xmax','Ymin','Ymax'],
#               ['Source1','Source2','Source3','Source4'],
#               ['WritenCondition1','WritenCondition2','WritenCondition3','WritenCondition4'],
#               ['XCol1','XCol2','XCol3','XCol4'],['YCol1','YCol2','YCol3','YCol4'],
#               ['Label1','Label2','Label3','Labe4'],[LinesOrDots1,LinesOrDots2,LinesOrDots3,LinesOrDots4],
#               [PlotColor1,PlotColor2,PlotColor3,PlotColor4] ]  
#############################################################################################################
    def ShowGraphic(self, event):
        errorstr = ""
        
        # Change directory to the DireName (in order to provide an input directory if no absolute file has been provided)
        os.chdir(DireName)
        
        self.StoreUpdatePlot(event)
        
        if WIN:
            i = 0
            for item in self.PlotParameters[2]:
                if (self.PlotParameters[2][i] != '') :
                    self.PlotParameters[2][i] = self.PlotParameters[2][i].replace(u'\u0022', u'\u0027')
                i += 1
            
        GraphicParametersList = []                            
        GraphicParametersList.extend(['--title', self.PlotParameters[0][0] ])
        GraphicParametersList.extend(['--Xlabel', self.PlotParameters[0][1] ])
        GraphicParametersList.extend(['--Ylabel', self.PlotParameters[0][2] ])
        
        i=3
        while i < len( self.PlotParameters[0] ):
            if (self.PlotParameters[0][i] == '' ): 
                self.PlotParameters[0][i] = '*'
            else:
                if( isFloat(self.PlotParameters[0][i]) == False  and isInt(self.PlotParameters[0][i]) == False  and (self.PlotParameters[0][i] == '*') == False):
                    errorstr += 'Plotting limit: '+ self.GraphicLimitsLabelVector[i-3]+' '+self.PlotParameters[0][i]+' must be a number.\n'
            i += 1
                        
        GraphicParametersList.extend(['--Xmin', self.PlotParameters[0][3] ])
        GraphicParametersList.extend(['--Xmax', self.PlotParameters[0][4] ])
        GraphicParametersList.extend(['--Ymin', self.PlotParameters[0][5] ])
        GraphicParametersList.extend(['--Ymax', self.PlotParameters[0][6] ])        

        try:
            if (float(self.PlotParameters[0][3]) >= float(self.PlotParameters[0][4])):
                errorstr += 'X maximum value must be greater than the X minimum value.\n'
        except:
            errorstr+=''
       
        try:
            if (float(self.PlotParameters[0][5]) >= float(self.PlotParameters[0][6])):
                errorstr += 'Y maximum value must be greater than the Y minimum value.\n'
        except:
            errorstr+=''

        #LOAD PARAMETERS
        i = 0
        while i < len(self.PlotParameters[1]):
            if (self.PlotParameters[1][i] != '') :
                GraphicParametersList.extend(['--file' , self.PlotParameters[1][i] ])
                if (self.PlotParameters[2][i] != '') :
                    GraphicParametersList.extend(['--cond' , self.PlotParameters[2][i] ])
                
                if (self.PlotParameters[3][i] != '') :    
                    GraphicParametersList.extend(['--Xcol' , self.PlotParameters[3][i] ])
                
                if (self.PlotParameters[4][i] != '') :
                    GraphicParametersList.extend(['--Ycol' , self.PlotParameters[4][i] ])
               
                if (self.PlotParameters[5][i] != '') :
                    GraphicParametersList.extend(['--label', self.PlotParameters[5][i] ])
                
                
                GraphicParametersList.extend(['-s', self.PlotStyle[self.PlotParameters[6][i]] ])
                GraphicParametersList.extend(['--cl', self.GraphColorList[self.PlotParameters[7][i]] ])    

                if (os.path.isfile(self.PlotParameters[1][i]) == False):
                    errorstr += 'PLOT ' + str(i + 1) + ': Source file ' + self.PlotParameters[1][i] + ' not found.\n'
                
                if (self.PlotParameters[3][i] == ""):
                    errorstr += 'PLOT ' + str(i + 1) + ': X Column empty.\n'
                
                if (self.PlotParameters[4][i] == ""):
                    errorstr += 'PLOT ' + str(i + 1) + ': Y Column empty.\n'
            i += 1 
        
        if (errorstr != ""):
            showstr = "gLAB has found the following errors, please correct them before processing again:\n\n" + errorstr
            dial = wx.MessageDialog(None, showstr, 'Errors found', wx.OK | wx.ICON_HAND)
            dial.ShowModal() 
        else:
            self.PlotButton.Disable()
            self.PlotLockFile = open(".plot.lock", "w")
            self.PlotLockFile.close()
            if WIN:
                if PYTHON:
                    # auxstr = 'python ' + DireMain + '\graph.py '
                    
                    auxstr = u''
		    auxstr += 'python "'
		    auxstr += DireMain.encode('utf-8')
                    auxstr += '\graph.py" '
                
                else:
                    auxstr = DireMain + '\graph.exe '            
                for i in GraphicParametersList:
                    auxstr = auxstr + '"' + i + '" '
                

                subprocess.Popen(auxstr.encode('iso-8859-1'))
                
            else:
                ParameterList = ['python', gBinElements[4] ]
                ParameterList.extend(GraphicParametersList)
                subprocess.Popen(ParameterList)

            threadplotting = threading.Timer(0, function=self.WaitPlot)
            threadplotting.start()
        
        os.chdir(DireMain)
        #event.Skip()
            
    
##########################################################################
# Name        : SavePlotPathByButton
# Description : The user can select the path to open the plot from the 
#               dialogue that will be opened when pressed the Examine button 
##########################################################################
    def SavePlotPathByButton(self, event):
        global DireName
        
        if WIN:
            wildcard = ".OUT files (*.out)|*.out*|.DAT files (*.dat)|*.dat*|All files|*.*"
        else:
            wildcard = ".OUT files (*.out)|*.[o,O][u,U][t,T]*|.DAT files (*.dat)|*.[d,D][a,A][t,T]*|All files|*"
        dlg = wx.FileDialog(self, "Choose file to be plotted", DireName, "", wildcard, wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            path = dlg.GetPath()
            FileName = dlg.GetFilename()
            DireName = dlg.GetDirectory()
            Aux = os.path.join(DireName, path)
            self.SourceFileTextCtrl.Clear()
            self.SourceFileTextCtrl.WriteText(Aux)  
        dlg.Destroy()    
    
##########################################################################
# Name        : XFirstConditionComboBoxCallback
# Description : When the combobox is modified, the selection is stored
#               in the text controls 
##########################################################################
    def XFirstConditionComboBoxCallback(self, event):
        self.XFirstConditionTextCtrl.Clear()
        
        if (self.XFirstConditionComboBox.GetSelection()) != 0 :
            self.XFirstConditionTextCtrl.WriteText(str (self.XFirstConditionComboBox.GetSelection()))
        else :
            self.XFirstConditionTextCtrl.WriteText('')
            
##########################################################################
# Name        : YFirstConditionComboBoxCallback
# Description : When the combobox is modified, the selection is stored
#               in the text controls 
##########################################################################
    def YFirstConditionComboBoxCallback(self, event):
        self.YFirstConditionTextCtrl.Clear()
        
        if (self.YFirstConditionComboBox.GetSelection()) != 0 :
            self.YFirstConditionTextCtrl.WriteText(str (self.YFirstConditionComboBox.GetSelection()))
        else :
            self.YFirstConditionTextCtrl.WriteText('')       
            
##########################################################################
# Name        : SavePlotPathByTextCtrl
# Description : When the text control is modified, the path is stored 
##########################################################################
    def SavePlotPathByTextCtrl(self, event):
        self.PlotPath = self.SourceFileTextCtrl.GetValue()   
        
##########################################################################
# Name        : SaveLabelByTextCtrl
# Description : When the text control is modified, the label is stored
##########################################################################
    def SaveLabelByTextCtrl(self, event):
        self.PlotLabel = self.LabelTextCtrl.GetValue()       
        
##########################################################################
# Name        : ErasePlotOptionsFunc
# Description : When the clear button is pressed, all of stored options are 
#               erased:  the GUI and the variables. 
##########################################################################
    def ErasePlotOptionsFunc(self):
        global prevPlotFirstCondition
        
        self.Plot1RB.SetValue(True) 
	self.PlotNumber = 0
        
        self.Title.Clear()
        self.Xlabel.Clear()
        self.Ylabel.Clear()
        self.SourceFileTextCtrl.Clear() 
        self.XFirstConditionTextCtrl.Clear()
        self.YFirstConditionTextCtrl.Clear()
        self.ConditionsPlotsTextCtrl.Clear()
        self.LabelTextCtrl.Clear()
        self.XFirstConditionComboBox.Clear()
        self.YFirstConditionComboBox.Clear()
        self.SecondConditionComboBox.Clear()
        
        self.Xminlabel.Clear()
        self.Xmaxlabel.Clear()
        self.Yminlabel.Clear()
        self.Ymaxlabel.Clear()
        
        i=0
        j=0
        while j < len( self.PlotParameters[:] ):
            i=0
            while i < len( self.PlotParameters[j] ):
                 self.PlotParameters [j][i]= ""
                 if ( j==6 ): self.PlotParameters [j][i]= 0
                 if ( j==7 ): self.PlotParameters [j][i]= i
                 i = i+1
            j= j + 1     
          

        i=0
        j=0
        while j < len( self.SelectedConditions[:] ):
            i=0
            while i < len( self.SelectedConditions[j] ):
                 self.SelectedConditions [j][i]= 0
                 i = i+1
            j= j + 1     
                           
   
        self.XFirstConditionComboBox.SetValue('')
        self.YFirstConditionComboBox.SetValue('')
        self.FirstConditionComboBox.SetValue('')
        self.SecondConditionComboBox.SetValue('')      
        
        self.LinesOrDotsComboBox.SetSelection(0)
        self.ColorComboBox.SetSelection(0)

        prevPlotFirstCondition = 0
                
##########################################################################
# Name        : ErasePlotOptions
# Description : When the clear button is pressed, all of stored options are 
#               erased:  the GUI and the variables. 
##########################################################################
    def ErasePlotOptions(self, event):
        self.ErasePlotOptionsFunc()
        event.Skip()
        
##########################################################################
# Name        : SetConditionsText
# Description : When the TextCtrl is changed, the combo boxes are reseted 
##########################################################################   
    def SetConditionsText(self, event):
        global prevPlotFirstCondition

        self.SecondConditionComboBox.SetSelection(0)
        
        expected = "($1==\"" + self.FirstConditionComboBox.GetValue() + "\")"
        
        if (self.ConditionsPlotsTextCtrl.GetValue()[0:len(expected)] != expected):
            prevPlotFirstCondition = 0
        
            self.FirstConditionComboBox.SetSelection(0)
            self.YFirstConditionComboBox.SetSelection(0)
            self.XFirstConditionComboBox.SetSelection(0) 
            self.XFirstConditionComboBox.Clear()
            self.YFirstConditionComboBox.Clear()
   
##########################################################################
# Name        : UpdateComboBoxes
# Description : Update the combo boxes of the analysis tab
##########################################################################     
    def UpdateComboBoxes (self, Selection):
        self.SecondConditionComboBox.SetSelection(0)
        self.XFirstConditionComboBox.SetSelection(0)        
        self.YFirstConditionComboBox.SetSelection(0) 
        
        self.XFirstConditionComboBox.Clear()
        self.YFirstConditionComboBox.Clear()
        self.SecondConditionComboBox.Clear()
        #Need to erase the Text Control where the conditions are written to load to the plotting tool
        self.ConditionsPlotsTextCtrl.Clear() 
        
        self.XFirstConditionComboBox.AppendItems(self.ColumnsList[Selection]);
        self.YFirstConditionComboBox.AppendItems(self.ColumnsList[Selection]);
        self.SecondConditionComboBox.AppendItems(self.SecondList[Selection]);
        
##########################################################################
# Name        : SelectConditions
# Description : This function gets the comboboxes conditions and modifies 
#               associated text controls
##########################################################################        
    def SelectConditions(self, event):
        global prevPlotFirstCondition
        
        Condition = self.FirstConditionComboBox.GetSelection()
        
        if (self.FirstConditionComboBox.GetValue() != prevPlotFirstCondition):
            self.XFirstConditionTextCtrl.Clear()
            self.YFirstConditionTextCtrl.Clear()
 
            aux = self.SecondConditionComboBox.GetSelection()

            self.UpdateComboBoxes(Condition)
            self.SecondConditionComboBox.SetSelection(aux)

        if (Condition != 0):
            auxStr = "($1==\"" + self.FirstConditionComboBox.GetValue() + "\")"
            if (self.SecondConditionComboBox.GetValue() != ""):
                auxStr = auxStr + " & ($" + str(self.SecondListPos[Condition]) + "==" + str(self.SecondConditionComboBox.GetSelection()) + ")"

            self.ConditionsPlotsTextCtrl.SetValue(auxStr)
        
        prevPlotFirstCondition = self.FirstConditionComboBox.GetValue()

##########################################################################
# Name        : SaveXFirstConditionByTextCtrl
# Description : This function gets the X conditions from the user when its 
#               text control is modified 
##########################################################################
    def SaveXFirstConditionByTextCtrl(self, event):
        self.XFirstConditionComboBox.SetSelection(0) 
        if (self.XFirstConditionTextCtrl.GetValue() != ''):
            try:
                 if int(self.XFirstConditionTextCtrl.GetValue()) < len (self.ColumnsList[ self.FirstConditionComboBox.GetSelection() ]) :
                     self.XFirstConditionComboBox.SetSelection(int(self.XFirstConditionTextCtrl.GetValue()))
                 else:
                     self.XFirstConditionComboBox.SetSelection(0) 
            except:
                     self.XFirstConditionComboBox.SetSelection(0) 

##########################################################################
# Name        : SaveYFirstConditionByTextCtrl
# Description : This function gets the Y conditions from the user when its 
#               text control is modified 
##########################################################################
    def SaveYFirstConditionByTextCtrl(self, event):
        self.YFirstConditionComboBox.SetSelection(0)
        if (self.YFirstConditionTextCtrl.GetValue() != ''):
            try:
                 if int(self.YFirstConditionTextCtrl.GetValue()) < len (self.ColumnsList[ self.FirstConditionComboBox.GetSelection() ]) :
                     self.YFirstConditionComboBox.SetSelection(int(self.YFirstConditionTextCtrl.GetValue()))
                 else:
                     self.YFirstConditionComboBox.SetSelection(0) 
            except:
                     self.YFirstConditionComboBox.SetSelection(0)                      
                     
##########################################################################
# Name        : StoreTitle
# Description : Get the value of the title of the graphic
##########################################################################
    def StoreTitle(self, event):
        self.PlotParameters[0][0] = self.Title.GetValue()  

##########################################################################
# Name        : StoreXlabel
# Description : Get the value of the label for the X axis
##########################################################################
    def StoreXlabel(self, event):
        self.PlotParameters[0][1] = self.Xlabel.GetValue()  
        
##########################################################################
# Name        : StoreYlabel
# Description : Get the value of the label for the Y axis
##########################################################################
    def StoreYlabel(self, event):
        self.PlotParameters[0][2] = self.Ylabel.GetValue()  
   
##########################################################################
# Name        : StoreXminlabel
# Description : Get the minimum value to plot for the X axis
##########################################################################
    def StoreXminlabel(self, event):
        self.PlotParameters[0][3] = self.Xminlabel.GetValue()  

##########################################################################
# Name        : StoreXmaxlabel
# Description : Get the max value to plot for the X axis
##########################################################################
    def StoreXmaxlabel(self, event):
        self.PlotParameters[0][4] = self.Xmaxlabel.GetValue()    
        
##########################################################################
# Name        : StoreYminlabel
# Description : Get the minimum value to plot for the X axis
##########################################################################
    def StoreYminlabel(self, event):
        self.PlotParameters[0][5] = self.Yminlabel.GetValue()  

##########################################################################
# Name        : StoreYmaxlabel
# Description : Get the max value to plot for the X axis
##########################################################################
    def StoreYmaxlabel(self, event):
        self.PlotParameters[0][6] = self.Ymaxlabel.GetValue()                     

##########################################################################
# Name        : SetGraphicLimits
# Description : # Description : Controls automatic limits for to plot for 
#               the X and Y axis: this handles introduced events.
##########################################################################
    def SetGraphicLimits(self, event):
        self.SetGraphicLimitsFunc()
        event.Skip()

##########################################################################
# Name        : SetGraphicLimits
# Description : Controls automatic limits for to plot for the X and Y axis
##########################################################################
    def SetGraphicLimitsFunc(self):
        if (self.AutomaticLimitsCB.GetValue() == False):
            self.Xminlabel.Enable(True)   
            self.Xmaxlabel.Enable(True)    
            self.Yminlabel.Enable(True)   
            self.Ymaxlabel.Enable(True)   
        else:    
            self.Xminlabel.Enable(False)   
            self.Xmaxlabel.Enable(False)    
            self.Yminlabel.Enable(False)   
            self.Ymaxlabel.Enable(False)
            
            self.Xminlabel.Clear()   
            self.Xmaxlabel.Clear()      
            self.Yminlabel.Clear()      
            self.Ymaxlabel.Clear()     
 
##########################################################################
# Name        : SetTemplate
# Description : Some default configuration to make easier use of the  
#               plots tool, in the analysis tab 
##########################################################################
    def SetTemplate (self, event):
        self.ErasePlotOptions(event)
        global OutputPath
        
        self.AutomaticLimitsCB.SetValue(True)  
        self.SetGraphicLimitsFunc()
        
        del self.PlotParameters[:]
        del self.SelectedConditions[:]
        
        id = event.GetId()
        for i in range (self.TemplateNumButtons):
            if (self.TemplateButtons[i].GetId() == id):
                idTemplate = i
                break
        i = 0
        for item in self.TemplatePlotParameters[idTemplate][1]:
            if (self.TemplatePlotParameters[idTemplate][1][i] != ''):
#                Except the Orbits and Clocks Template
                 if (idTemplate!= 11):
                      self.TemplatePlotParameters[idTemplate][1][i] = OutputPath
                 else:
                      self.TemplatePlotParameters[idTemplate][1][i] = self.TemplatePlotParameters[idTemplate][1][i]
            i += 1       
        for item in self.TemplatePlotParameters[idTemplate]:
            self.PlotParameters.append(item[:])
        for item in self.TemplateSelectedConditions[idTemplate]:
            self.SelectedConditions.append(item[:])
        self.LoadNewTemplate() 
        event.Skip()
        
##########################################################################
# Name        : LoadNewTemplate
# Description : When selected, the path, conditions and labels are load
#               from the different templates 
##########################################################################
    def LoadNewTemplate(self):
        global prevPlotFirstCondition
        
        self.FirstConditionComboBox.SetSelection(self.SelectedConditions[0][0])
        prevPlotFirstCondition = ''
        self.SelectConditions(None)
        self.SecondConditionComboBox.SetSelection(self.SelectedConditions[1][0])
        self.XFirstConditionComboBox.SetSelection(self.SelectedConditions[2][0])
        self.YFirstConditionComboBox.SetSelection(self.SelectedConditions[3][0])
        
        self.Plot1RB.SetValue(True)
        self.PlotNumber = 0
        
        self.Title.WriteText(self.PlotParameters[0][0])  
        self.Xlabel.WriteText(self.PlotParameters[0][1])
        self.Ylabel.WriteText(self.PlotParameters[0][2])
        self.Xminlabel.WriteText(self.PlotParameters[0][3])  
        self.Xmaxlabel.WriteText(self.PlotParameters[0][4])
        self.Yminlabel.WriteText(self.PlotParameters[0][5])   
        self.Ymaxlabel.WriteText(self.PlotParameters[0][6])                   
        self.SourceFileTextCtrl.WriteText(self.PlotParameters[1][0])  
        self.ConditionsPlotsTextCtrl.SetValue(self.PlotParameters[2][0])
        self.XFirstConditionTextCtrl.WriteText(self.PlotParameters[3][0])
        self.YFirstConditionTextCtrl.WriteText(self.PlotParameters[4][0])
        self.LabelTextCtrl.WriteText(self.PlotParameters[5][0])
        self.LinesOrDotsComboBox.SetSelection(self.PlotParameters[6][0])
        
        if ( (self.Xminlabel.GetValue() != '') or (self.Xmaxlabel.GetValue() != '') or (self.Yminlabel.GetValue() != '') or (self.Ymaxlabel.GetValue() != '') ): 
            self.AutomaticLimitsCB.SetValue(False)
            self.Xminlabel.Enable(True)   
            self.Xmaxlabel.Enable(True)    
            self.Yminlabel.Enable(True)   
            self.Ymaxlabel.Enable(True)   
            
##########################################################################
# Name        : StoreUpdatePlot
# Description : This function sets the path, the conditions and the labels
#               of the plots when changing from one plot to a different plot
##########################################################################
    def StoreUpdatePlot(self, event):
        self.PlotParameters[1][self.PlotNumber] = self.PlotPath
        self.PlotParameters[2][self.PlotNumber] = self.ConditionsPlotsTextCtrl.GetValue()
        self.PlotParameters[3][self.PlotNumber] = self.XFirstConditionTextCtrl.GetValue()
        self.PlotParameters[4][self.PlotNumber] = self.YFirstConditionTextCtrl.GetValue()
        self.PlotParameters[5][self.PlotNumber] = self.LabelTextCtrl.GetValue()
        self.PlotParameters[6][self.PlotNumber] = self.LinesOrDotsComboBox.GetSelection()
        self.PlotParameters[7][self.PlotNumber] = self.ColorComboBox.GetSelection()
        
        self.SelectedConditions[0][self.PlotNumber] = self.FirstConditionComboBox.GetSelection()
        self.SelectedConditions[1][self.PlotNumber] = self.SecondConditionComboBox.GetSelection()
        self.SelectedConditions[2][self.PlotNumber] = self.XFirstConditionComboBox.GetSelection()
        self.SelectedConditions[3][self.PlotNumber] = self.YFirstConditionComboBox.GetSelection()

        if (self.Plot1RB.GetValue() == True):
            self.PlotNumber = 0
        elif(self.Plot2RB.GetValue() == True):
            self.PlotNumber = 1
        elif (self.Plot3RB.GetValue() == True):
            self.PlotNumber = 2
        elif(self.Plot4RB.GetValue() == True):
            self.PlotNumber = 3
        
        AuxPath = self.PlotParameters[1][self.PlotNumber]
        AuxConditions = self.PlotParameters[2][self.PlotNumber]
        AuxConditionX = self.PlotParameters[3][self.PlotNumber]
        AuxConditionY = self.PlotParameters[4][self.PlotNumber]
        AuxLabel = self.PlotParameters[5][self.PlotNumber]
        AuxStyle = self.PlotParameters[6][self.PlotNumber]
        AuxPlotColor = self.PlotParameters[7][self.PlotNumber]
        
        AuxFirstConditionComboBox = self.SelectedConditions[0][self.PlotNumber]
        AuxSecondConditionComboBox = self.SelectedConditions[1][self.PlotNumber]
        AuxXFirstConditionComboBox = self.SelectedConditions[2][self.PlotNumber]
        AuxYFirstConditionComboBox = self.SelectedConditions[3][self.PlotNumber]
        
        self.SourceFileTextCtrl.Clear()  
        self.ConditionsPlotsTextCtrl.Clear()
        self.XFirstConditionTextCtrl.Clear()
        self.YFirstConditionTextCtrl.Clear()
        self.LabelTextCtrl.Clear()
        
        self.FirstConditionComboBox.SetSelection(0) 
        self.SecondConditionComboBox.SetSelection(0) 
        self.XFirstConditionComboBox.SetSelection(0) 
        self.YFirstConditionComboBox.SetSelection(0) 
        self.LinesOrDotsComboBox.SetSelection(0)
        self.ColorComboBox.SetSelection(0)
            
        self.SourceFileTextCtrl.WriteText(AuxPath)  
        self.ConditionsPlotsTextCtrl.WriteText(AuxConditions)
        self.XFirstConditionTextCtrl.WriteText(AuxConditionX)
        self.YFirstConditionTextCtrl.WriteText(AuxConditionY)
        self.LabelTextCtrl.WriteText(AuxLabel)
        
        self.FirstConditionComboBox.SetSelection(AuxFirstConditionComboBox)
        self.SecondConditionComboBox.SetSelection(AuxSecondConditionComboBox)
        self.XFirstConditionComboBox.SetSelection(AuxXFirstConditionComboBox)
        self.YFirstConditionComboBox.SetSelection(AuxYFirstConditionComboBox)
        self.LinesOrDotsComboBox.SetSelection(AuxStyle)
        self.ColorComboBox.SetSelection(AuxPlotColor)

        
##########################################################################
# Name        : InitializeControls
# Description : This function is called just once when the program is 
#               initiating, this hides some objects after the 
#               interface has calculated their position, size... (layout) 
##########################################################################
    def InitializeControls(self):
        # Activate limits                      
        self.AutomaticLimitsCB.SetValue(True)
        self.SetGraphicLimitsFunc()

##########################################################################
# Name        : WindowHTML
# Description : This frame is where the tooltips are written, enabling
#               a friendly lecture, as they are written in html style
##########################################################################     
class WindowHTML(wx.Frame):
    def __init__(self, parent): 
        wx.Frame.__init__(self, parent, -1, style=wx.FRAME_NO_TASKBAR | wx.NO_BORDER)        
        self.html = wx.html.HtmlWindow(self) 
        self.html.SetStandardFonts() 
        
##########################################################################
# Name        : PreferencesFrame
# Description : This frame is shown when the user wants to change the
#               visibility or non visibility of the tooltips
#               the default configuration is tooltips are active 
##########################################################################    
class PreferencesFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, -1, title="Preferences", style=wx.FRAME_NO_TASKBAR | wx.STAY_ON_TOP)
        global TabXSize, TabYSize
        self.SetSize(wx.Size(0.4 * TabXSize, 0.45 * TabYSize))
        PreferencesFramePanel = wx.Panel(self, -1)
        XPreferencesFrame, YPreferencesFrame = self.GetSize() 
        
        TooltipsRadioBox = wx.RadioBox(PreferencesFramePanel, 50, "Select Tooltip Options", wx.Point(20, 210), wx.Size(0.95 * XPreferencesFrame, 0.2 * YPreferencesFrame) , ['Tooltips ON', 'Tooltips OFF'], 3, wx.RA_SPECIFY_COLS)

        ScrollbarsRadioBox = wx.RadioBox(PreferencesFramePanel, 50, "Activate Scrollbars Options", wx.Point(20, 210), wx.Size(0.95 * XPreferencesFrame, 0.2 * YPreferencesFrame) , ['Scrollbars OFF', 'Scrollbars ON'], 3, wx.RA_SPECIFY_COLS)

        CheckButton = wx.Button(PreferencesFramePanel, -1, label="Check for update")
        HideButton = wx.Button(PreferencesFramePanel, -1, label="Close Preferences")     
        
        #Preferences Frame Layout
        PreferencesFrameGrill = wx.GridBagSizer()
        PreferencesFrameGrill.Add(TooltipsRadioBox, (0, 0), (1, 2), wx.EXPAND | wx.ALL, 10)
        PreferencesFrameGrill.Add(ScrollbarsRadioBox, (1, 0), (1, 2), wx.EXPAND | wx.ALL, 10)
        PreferencesFrameGrill.Add(CheckButton, (2, 0), (1, 2), wx.EXPAND | wx.ALL, 10)
        PreferencesFrameGrill.Add(HideButton, (3, 0), (1, 2), wx.EXPAND | wx.ALL, 10)
        PreferencesFrameGrill.SetMinSize(wx.Size(XPreferencesFrame, YPreferencesFrame))
        PreferencesFramePanel.SetSizer(PreferencesFrameGrill) 
        PreferencesFrameGrill.Fit(self)
        
        #Link diferent bindings for all buttons
        HideButton.Bind(wx.EVT_LEFT_DOWN, self.HidePreferencesFrame)
        CheckButton.Bind(wx.EVT_LEFT_DOWN, self.CheckVersion)
        TooltipsRadioBox.Bind(wx.EVT_RADIOBOX, self.EvtRadioBox)
        ScrollbarsRadioBox.Bind(wx.EVT_RADIOBOX, self.EvtScrollRadioBox)        


##########################################################################
# Name        : EvtRadioBox
# Description : The user can chose to have the tooltips or not being  
#               displayed when hovering with the mouse on different options 
##########################################################################
    def EvtRadioBox(self, event):
        global tooltips
        if event.GetSelection() == 0:
            tooltips = True
        else:
            tooltips = False

##########################################################################
# Name        : EvtScrollRadioBox
# Description : The user can chose to have the scrollbars or not being  
#               displayed when working on a small screen.
##########################################################################
    def EvtScrollRadioBox(self, event):
        global scrollbars
        if event.GetSelection() == 0:
            scrollbars = False
            MF.MainFrameScrolled.SetScrollbars(0, 0, 2*MF.MainFrameXsize, 2*MF.MainFrameYsize)   
        else:
            scrollbars = True
            MF.MainFrameScrolled.SetScrollbars(1, 1, 2*MF.MainFrameXsize, 2*MF.MainFrameYsize)   

##########################################################################
# Name        : HidePreferencesFrame
# Description : It will hide the preferences frame 
##########################################################################
    def HidePreferencesFrame(self, event):
        self.Hide()


##########################################################################
# Name        : CheckVersion
# Description : Checks if the software has a new version available
##########################################################################
    def CheckVersion (self, event):
        try:
            f = urllib.urlopen("http://www.gage.es/gLAB/version") 
            s = f.read()
            f.close()
        except:
            dial = wx.MessageDialog(None, 'Cannot connect to server. Is Internet available?', 'Connection problem', wx.OK | wx.ICON_HAND)
            dial.SetIcon(wx.Icon( gBinElements[2] , wx.BITMAP_TYPE_PNG))
            dial.ShowModal()
            return
        p1 = s.find("<version>")
        p2 = s.find("</version>")
        p3 = s.find("<revision>")
        p4 = s.find("</revision>")
        if (p1 == -1 or p2 == -1 or p3 == -1 or p4 == -1):
            dial = wx.MessageDialog(None, 'Parsing problem. Please check  http://www.gage.es/gLAB/', 'Parsing problem', wx.OK | wx.ICON_HAND)
            dial.SetIcon(wx.Icon( gBinElements[2] , wx.BITMAP_TYPE_PNG))
            dial.ShowModal()
            return
        newVersion = float(s[p1 + 9:p2])
        newRevision = float(s[p3 + 10:p4])
        if (float(Version) > newVersion or (float(Version) == newVersion and float(Revision) >= newRevision)):
            dial = wx.MessageDialog(None, 'Current version (' + Version + '.' + Revision + ') is the last one available', 'Version check', wx.OK)
            dial.SetIcon(wx.Icon( gBinElements[2] , wx.BITMAP_TYPE_PNG))
            dial.ShowModal()
        else:
            dial = wx.MessageDialog(None, 'New version available: (' + s[p1 + 9:p2] + '.' + s[p3 + 10:p4] + '). Please check http://www.gage.es/gLAB/ to obtain it.', 'Version check', wx.OK)
            dial.SetIcon(wx.Icon( gBinElements[2] , wx.BITMAP_TYPE_PNG))
            dial.ShowModal()
            
        
##########################################################################
# Name        : MainFrame
# Description : This frame is the main window of the GUI, it is divided in
#               three main zones: the header, with ESA and gAGE logos 
#               the preferences and about button, and the notebook where
#               the different tabs of the Calculus and Analysis options
#               can be found and configured
##########################################################################    
class MainFrame(wx.Frame):
    def __init__(self):
        global Version
        global Revision

        # Original x=980 y=715      up to 25/11/2015 x=980,y=725
        self.MainFrameXsize = 980
        self.MainFrameYsize = 780


	# Initialise the Main Frame (window)
        if WIN : 
        	wx.Frame.__init__(self, None, title="gLAB - Version " + Version + "." + Revision, pos=wx.DefaultPosition, style=wx.SYSTEM_MENU | wx.CAPTION | wx.CLOSE_BOX | wx.CLIP_CHILDREN  , size = (1.005*self.MainFrameXsize, self.MainFrameYsize) )

	else:
        	wx.Frame.__init__(self, None, title="gLAB - Version " + Version + "." + Revision, pos=wx.DefaultPosition, style=wx.SYSTEM_MENU | wx.CAPTION | wx.CLOSE_BOX | wx.MINIMIZE_BOX, size = (self.MainFrameXsize, self.MainFrameYsize) )

		# Check if gLAB_GUI is being executed from the same directory, and manage icons, images, programs and staff [Linux only].
		self.OsCheckList()

	# Harmonise Fonts
        self.SetFont(wx.Font(11, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))

	# Initialise the scrollbars (depending its visibility on the preferences)
        self.MainFrameScrolled = wx.ScrolledWindow( self, -1, size = (self.MainFrameXsize, self.MainFrameYsize) )
        self.MainFrameScrolled.SetScrollbars(0, 0, 2*self.MainFrameXsize, 2*self.MainFrameYsize)       
        self.MainFrameScrolled.EnableScrolling(True,True)

	# Create the panel where the logo + notebook will be allocated
        MainFramePanel = wx.Panel(self.MainFrameScrolled, -1)
#        MainFramePanel = wx.Panel(self, -1)
        XMainFramePanel, YMainFramePanel = self.MainFrameXsize, self.MainFrameYsize
        MainFramePanel.SetSize(wx.Size(XMainFramePanel, YMainFramePanel))
        
	# Set the tab size of the notebook      
        global TabXSize, TabYSize
        TabXSize, TabYSize = (0.991 * self.MainFrameXsize, 0.745 * self.MainFrameYsize)
        
        # Create main frame elements
        panel = wx.Panel(MainFramePanel)
        notebook = wx.Notebook(panel)
	notebook.SetSize(wx.Size(TabXSize, TabYSize))
        
        GAGEESALogo = wx.StaticBitmap(MainFramePanel, -1, wx.Bitmap(gBinElements[0]))
        ButtonsPanel = wx.Panel(MainFramePanel , size=wx.Size(self.MainFrameXsize, 0.04 * self.MainFrameYsize))
        
        PreferencesButton = wx.Button(ButtonsPanel, -1, label="Preferences")
        PreferencesButton.SetMinSize(wx.Size(0.2 * self.MainFrameXsize, 0.04 *self. MainFrameYsize))
        
        AboutButton = wx.Button(ButtonsPanel, -1, label="About")
        AboutButton.SetMinSize(wx.Size(0.2 * self.MainFrameXsize, 0.04 * self.MainFrameYsize))
        
        ButtonsPanelGrid = wx.GridBagSizer()
        ButtonsPanelGrid.SetEmptyCellSize((0.2 * self.MainFrameXsize, 0))
        ButtonsPanelGrid.Add(PreferencesButton, (0, 0), (1, 1), wx.ALIGN_CENTER | wx.TOP , 10)
        ButtonsPanelGrid.Add(AboutButton, (0, 2), (1, 1), wx.ALIGN_CENTER | wx.TOP , 10)
        ButtonsPanelGrid.SetMinSize(wx.Size(0.6 * self.MainFrameXsize, 0.04 * self.MainFrameYsize))    
        ButtonsPanel.SetSizer(ButtonsPanelGrid) 

        
        
        self.PreferencesFrame = PreferencesFrame()
        
        # Create the page windows as children of the notebook
        self.page2 = CalculusPage(notebook)
        self.page3 = AnalysisPage(notebook)
        
        # Add the pages to the notebook with the label to show on the tab
        notebook.AddPage(self.page2, "Positioning")
        notebook.AddPage(self.page3, "Analysis")

        # Pack the notebook on top a panel
        sizer = wx.BoxSizer()
        sizer.Add(notebook, 1, wx.EXPAND)
        panel.SetSizer(sizer)
        
        MainFrameGrill = wx.GridBagSizer()
        MainFrameGrill.Add(GAGEESALogo, pos=(0, 0), span=(2, 2), flag=wx.ALIGN_CENTER_HORIZONTAL)
        MainFrameGrill.Add(ButtonsPanel, (2, 0), (1, 2), flag=wx.ALIGN_CENTER_HORIZONTAL) 
        MainFrameGrill.Add(panel, pos=(3, 0), span=(2, 2), flag=wx.EXPAND)
        MainFrameGrill.SetMinSize(wx.Size(self.MainFrameXsize, self.MainFrameYsize))
        MainFramePanel.SetSizer(MainFrameGrill)
	self.SetSizer(MainFrameGrill)


	"""
        MainFrameScrolledGrill = wx.GridBagSizer()
        MainFrameScrolledGrill.Add(MainFramePanel, pos=(0, 0), span=(1, 1) )
        MainFrameScrolledGrill.SetMinSize(wx.Size(self.MainFrameXsize, self.MainFrameYsize))
#        self.SetSizer(MainFrameScrolledGrill)
        self.MainFrameScrolled.SetSizer(MainFrameScrolledGrill)	
        self.SetSizer(MainFrameScrolledGrill)
	"""

        '''
        if WIN :
            MainFrameGrill = wx.GridBagSizer()
            MainFrameGrill.Add(GAGEESALogo,pos=(0,0),span=(2,5), flag= wx.ALIGN_CENTER_HORIZONTAL)
            MainFrameGrill.Add(PreferencesButton, (2,1),(1,1), wx.ALIGN_CENTER |wx.ALL , 10)
            MainFrameGrill.Add(AboutButton, (2,3),(1,1), wx.ALIGN_CENTER |wx.ALL , 10)   
            MainFrameGrill.Add(panel,pos=(3,0),span=(2,5), flag=wx.EXPAND)
            MainFrameGrill.SetMinSize(wx.Size(MainFrameXsize,MainFrameYsize))
            
            MainFramePanel.SetSizer(MainFrameGrill) 
            MainFrameGrill.Fit(self)

        else :
            MainFrameGrill = wx.GridBagSizer()
            MainFrameGrill.Add(GAGEESALogo,pos=(0,0),span=(2,2), flag= wx.ALIGN_CENTER_HORIZONTAL)
            MainFrameGrill.Add(PreferencesButton, (2,0),(1,1), wx.ALIGN_CENTER | wx.ALL,10)
            MainFrameGrill.Add(AboutButton, (2,1),(1,1), wx.ALIGN_CENTER| wx.ALL,10)   
            MainFrameGrill.Add(panel,pos=(3,0),span=(2,2), flag=wx.EXPAND)
            MainFrameGrill.SetMinSize(wx.Size(MainFrameXsize,MainFrameYsize))
            self.SetSizer(MainFrameGrill)
        '''
        #Link different bindings for all buttons
        PreferencesButton.Bind(wx.EVT_LEFT_DOWN, self.ShowPreferences)
        AboutButton.Bind(wx.EVT_LEFT_DOWN, self.ShowAbout)
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        self.Bind(wx.EVT_KEY_DOWN,self.OnKeyPress)
           

#########################################9#################################
# Name        : ShowAlert
# Description : Shows an alert window
##########################################################################
    def ShowAlert(self, Title, Message):
        dial = wx.MessageDialog(None, Message, Title, wx.OK)
        dial.SetIcon(wx.Icon( gBinElements[2] , wx.BITMAP_TYPE_PNG))        

#########################################9#################################
# Name        : OnClose
# Description : It displays a dialog to confirm the user wants to close  
#               gLAB when the user exits the program
##########################################################################
    def OnClose(self, event):
        dial = wx.MessageDialog(None, 'You are about to close gLAB. Are you sure you want to continue?', 'Confirm Exit', wx.YES_NO | wx.NO_DEFAULT)
        dial.SetIcon(wx.Icon( gBinElements[2] , wx.BITMAP_TYPE_PNG))

        ret = dial.ShowModal()
        if ret == wx.ID_YES:
            self.Destroy()
            sys.exit()
        else:
            pass
        
##########################################################################
# Name        : OnKeyPress
# Description : Event handler for key preses:
#               F1 => Unblocks blocked buttons
########################################################################## 
    def OnKeyPress (self,event):
        keycode = event.GetKeyCode()
        if (keycode==wx.WXK_F1):
            self.page2.EnableCalculusButtons(1, 1)
            self.page3.EnablePlotButton
                    
##########################################################################
# Name        : ShowPreferences
# Description : It displays the preferences frame, when the user can  
#               change from some options 
##########################################################################
    def ShowPreferences(self, event):
        ZeroPointX, ZeroPointY, ScreenSizeX, ScreenSizeY = wx.ClientDisplayRect()
        PreferencesFrameX, PreferencesFrameY = self.PreferencesFrame.GetSize()
        XFrame = 0.5 * ScreenSizeX + ZeroPointX - 0.5 * PreferencesFrameX
        YFrame = 0.5 * ScreenSizeY + ZeroPointY - 0.5 * PreferencesFrameY
        self.PreferencesFrame.SetPosition(wx.Point(XFrame, YFrame))
        self.PreferencesFrame.Show()
	


##########################################################################
# Name        : ShowAbout
# Description : This shows a common about-type dialog, where some 
#               information about gLAB can be easily found: its version, 
#               authors, gAGE's website, copyright, and the license 
##########################################################################    
    def ShowAbout(self, event):
        global Version
        global Revision
        info = wx.AboutDialogInfo()

        info.SetIcon(wx.Icon( gBinElements[2] , wx.BITMAP_TYPE_PNG))
        info.SetName('gLAB')
        info.SetVersion('Version: ' + Version + '.' + Revision)
        info.SetDescription("""
gLAB is a complete GNSS analysis tool for both educational 
and professional purposes. gLAB is given as is for free (please
see the license). gLAB allows a full customisation of its
options and provides precise point positioning capabilities
on the centimetre level.

gLAB is divided in three different modules:
Data Processing Core (DPC) - For all the processing
Graphic User Interface (GUI) - To customise options
Data Analysis Tool (DAT) - Graphics

Both the processing core and the plotting tool can be executed 
independently from the GUI

Please e-mail any bugs, comments or requests to:
glab.gage @ upc.edu   
""")
        info.SetCopyright(u'\u00a9' + ' 2009 - 2017 gAGE/UPC ' + u'\u0026' + ' ESA')
        info.SetWebSite('http://www.gage.es')
        info.SetLicence("""
Copyright 2009-2017 gAGE/UPC """ + u'\u0026' + """ ESA
   
Licensed under the Apache License, Version 2.0 (the "License").
You may obtain a copy of the License at
  
http://www.apache.org/licenses/LICENSE-2.0
  
Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
""")
        
        info.AddDeveloper('Pere Ramos-Bosch [DPC, GUI, DAT]')
        info.AddDeveloper('\nAdria Rovira-Garcia [GUI, DAT]')
	if WIN:
		info.AddDeveloper('\nDeimos Ib' + u"á" + u'\xf1' + 'ez Segura [DPC, GUI, DAT]')
	else:
		info.AddDeveloper('\nDeimos Ib' + u"á" + u'\xf1' + 'ez Segura [DPC, GUI, DAT]')
	if WIN:
		info.AddDeveloper('\nJes' + u"ú" +'s Romero S' +  u"á" + 'nchez [DPC, GUI]')
	else:
		info.AddDeveloper('\nJes' + u"ú" +'s Romero S' +  u"á" + 'nchez [DPC, GUI]')
        info.AddDeveloper('\nYixie Shao [DAT]')
        info.AddDeveloper('\nJaume Sanz [Supervisor]')

        a = wx.AboutBox(info)

##########################################################################
# Name        : Init
# Description : This forces de Initialise Controls function to occur 
#               and the GUI is ready to be displayed 
##########################################################################
    def Init (self):
        self.page2.InitializeControls()
        self.page3.InitializeControls()

##########################################################################
# Name        : OsCheckList
# Description : This functions permits gLAB to be executed from another 
# 		folder different from the source one. There are several
#               paths defined in gBinPaths list to look for, if some of
#               the auxiliary files are not found, it fails to open.
##########################################################################
    def OsCheckList (self):
	global gBinPaths
	global gBinElements

	for Element in gBinElements:

		ElementFound = False

		for Path in gBinPaths:	

			if os.path.isfile( os.path.join( Path , Element ) ) == True and ElementFound == False:

				gBinElements[ gBinElements.index(Element) ] = os.path.join( Path , Element )

				ElementFound = True 

		if ElementFound == False:

			print 'gLAB - Graphical User Interface has found an error which prevent it from opening. Please correct it before executing gLAB - GUI again.'
			print 'Error: file ' , Element, ' not found.'
			print 'You need gLAB to be installed in some of the following directories: '
			for Path in gBinPaths:	
				print Path

		    	self.Destroy()
		    	sys.exit()

##########################################################################
# Name        : handleSIGCHLD
# Description : Children death handling routine 
##########################################################################
def handleSIGCHLD(self, event):
    try:
        os.waitpid(-1, os.WNOHANG)
    except:
        pass

##########################################################################
# Name        : 
# Description : Main execution thread
##########################################################################
WIN = False
if sys.platform[:3].lower() == "win":
    WIN = True

if WIN:
    import win32pipe
    popen = win32pipe.popen
else:
    signal.signal(signal.SIGCHLD, handleSIGCHLD)
    popen = os.popen

if (os.path.splitext(sys.argv[0])[1] == ".py"):
    PYTHON = True
else:
    PYTHON = False

app = wx.App(0)

# Create the gLAB-GUI:
MF = MainFrame()

# Get where the GUI is being executed!!
DireName = os.getcwd()
DireMain = os.getcwd()

#Erase any .plot.lock file if they exist
if (os.path.isfile(".plot.lock") == True):
    os.remove(".plot.lock")

if (os.path.isfile(DireName + "/.plot.lock") == True):
    os.remove(DireName + ".plot.lock")

# Show gLAB-GUI:
MF.Show()
MF.Init()
MF.SetIcon(wx.Icon(gBinElements[3] , wx.BITMAP_TYPE_ICO))

# Wait patiently while GUI is being displayed:
app.MainLoop()

